;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;British Crown Copyright (c) 2012, the Met Office All rights reserved.
;
;Redistribution and use in source and binary forms, with or without 
;modification, are permitted provided that the following conditions 
;are met:
;
;Redistributions of source code must retain the above copyright notice, 
;this list of conditions and the following disclaimer. 
;
;Redistributions in binary form must reproduce the above copyright 
;notice, this list of conditions and the following disclaimer in the 
;documentation and/or other materials provided with the distribution. 
;
;Neither the name of the Met Office nor the names of its contributors 
;may be used to endorse or promote products derived from this software 
;without specific prior written permission. 
;
;THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
;"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
;LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
;A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
;HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
;SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
;LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
;DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
;THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
;(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
;OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

function bico, n, k
;+
; NAME:
;   BICO
; PURPOSE:
;   Return the binomial coefficient n choose k
;
; CALLING SEQUENCE:
;   result=bico(n,k)
;
; INPUTS:
;   N,K -- arrays of integers.  Works for all values up to N=1029
;
; KEYWORD PARAMETERS:
;   none
;
; OUTPUTS:
;   N choose K with data type corresponding to the smallest that can
;   hold the value.
;
; MODIFICATION HISTORY:
;       Adapted from Numerical Recipes --
;       Fri Jun 22 10:57:21 2001, Erik Rosolowsky <eros@cosmic>
;       Downloaded, 21 Feb 2011, Robert Dunn
;-
on_error, 2
if n_elements(n) ne n_elements(k) then message, $
  'Input vectors must be the same size'

if total(k gt n) gt 0 then message, $
  'Elements in second vector must be smaller than those in first.'

L64 = 0b
result = (lngamma(n+1)-lngamma(k+1)-lngamma(n-k+1))

if max(result, /nan) gt 21 then L64 = 1b
if max(result, /nan) gt 88 then return, exp(double(result))
if max(result, /nan) gt 43 then return, exp(result)

  return, floor(0.5+exp(result), L64 = L64) ; ensure it is integer
end

;*************************************************************
;*************************************************************
;*************************************************************
;*************************************************************



PRO BINOMIALFILTER, input_ts, output_ts, point, mdi, extra=extra
;*************************************************************
; NAME:
;     BINOMIALFILTER
;
; PURPOSE:
;     To apply a binomial filter to a timeseries to smooth.
;
; CALLING SEQUENCE:
;     binomialfilter, input_ts, output_ts, point, mdi, extra=extra
;
; INPUTS:
;     input_ts - input time series
;
;     output_ts - output time series
;
;     point - the number of points to use in the filter (odd integer)
;
;     mdi - missing data indicator
;
;     extra - keyword to select whether extra data is included.
;
; METHOD:
;     Uses binomial coefficients (n choose k) to weight the data
;     surrounding the point of interest.
;
; OUTPUTS:
;
;     None - files written to specified directory
;
; RESTRICTIONS:
;
; HISTORY:
;      34 - created
;           22 February 2011, RJHD
;      ?? - Added "extra" keyword to select whether the padding with
;           missing data is used.
;           Added check to see if final points are missing data and
;           adjust the padding amount appropriately
;           22 February 2011, RJHD
;      218- added to this SVN repository for HadISD data  
;      221- Bugfix from SmallProjects Version 16/9/11 RJHD
;************************************************************************
;                    SVN Info
;$Rev:: 303                                           $:  Revision of last commit
;$Author:: rdunn                                      $:  Author of last commit
;$Date:: 2012-06-14 14:02:44 +0100 (Thu, 14 Jun 2012) $:  Date of last commit
;************************************************************************
;                                 START
;************************************************************************

; Sort the weights for the binomial filter

rawweights=FLTARR(point)
rawweights(*)=0.

ns=INTARR(point)

; 21 point filter is 20th line in Pascal's set
ns(*)=point-1 

ks=INDGEN(point)

rawweights=BICO(ns,ks)

;****************************************
; Using filter comparison.pro found that the effect of padding or
; missing data at the end of a series only comes into play around
; 1-sigma from the mean.  For a 21 point filter, the 1sigma value
; occurs between 8 and 9th point in (11th is middle).  Therefore could
; go up to 70% of way towards middle before stopping smoothing, hence
; only the last 30% of the data is missed (as fractions of one tail of
; the distribution)
; 23 February 2011 RJHD
;****************************************

IF KEYWORD_SET(extra) THEN BEGIN
    ; how much extra data to include over the half-width of the filter
    ; 
    ExtraIncluded=FIX(0.7*(point-1.)/2.)

    pad_input_ts=input_ts

    ; find how many missing data are at the front of the data set
    frontcounter=0
    WHILE 1 EQ 1 DO BEGIN
        IF pad_input_ts[frontcounter] NE mdi THEN BREAK
        frontcounter=frontcounter+1
    ENDWHILE

    ; create a missing value array to pad the data array with
    ; adjust length so have the right number for the padding
    IF frontcounter LT ExtraIncluded THEN BEGIN
        fmdi_array=FLTARR(ExtraIncluded-frontcounter)
        fmdi_array(*)=mdi
        pad_input_ts=[fmdi_array,pad_input_ts]
    ENDIF ELSE IF frontcounter GT ExtraIncluded THEN BEGIN
        pad_input_ts=pad_input_ts[frontcounter-ExtraIncluded:N_ELEMENTS(pad_input_ts)-1]

    ENDIF
    

    ; find how many missing data are at the end of the data set
    backcounter=N_ELEMENTS(pad_input_ts)
    WHILE 1 EQ 1 DO BEGIN
        backcounter=backcounter-1        
        IF pad_input_ts[backcounter] NE mdi THEN BREAK
    ENDWHILE

    ; create a missing value array to pad the data array with
    ; adjust length so have the right number for the padding
    IF N_ELEMENTS(pad_input_ts)-backcounter LT ExtraIncluded THEN BEGIN
        bmdi_array=FLTARR(ExtraIncluded-(N_ELEMENTS(pad_input_ts)-backcounter))
        bmdi_array(*)=mdi
        pad_input_ts=[pad_input_ts,bmdi_array]
    ENDIF ELSE IF N_ELEMENTS(pad_input_ts)-backcounter GT ExtraIncluded THEN BEGIN
        pad_input_ts=pad_input_ts[0:backcounter+ExtraIncluded]
    ENDIF
    
ENDIF ELSE BEGIN
    pad_input_ts=input_ts
ENDELSE


FOR obs_no=0L,N_ELEMENTS(pad_input_ts)-1 DO BEGIN

    IF obs_no GE (point-1.)/2. AND obs_no LE N_ELEMENTS(pad_input_ts)-((point-1.)/2.)-1 THEN BEGIN
        
        ; find the range of data to consider for weighting

        data_chunk=pad_input_ts(obs_no-((point-1.)/2.):obs_no+((point-1.)/2.))

        ; scan for missing data

        non_missing=WHERE(data_chunk NE mdi, mcnt)

        ; if over half the data is present

        IF KEYWORD_SET(extra) THEN BEGIN

            IF mcnt GE ((point-1.)/2.)+1. THEN BEGIN

            ;     calculate the appropriate normalisation and apply the weights

                normalisation=SUM(rawweights[non_missing])            
                weighted_data=SUM(data_chunk[non_missing]*rawweights[non_missing]/normalisation)
                
                ; adjust the output writing by the padding
                output_ts[obs_no-ExtraIncluded]=weighted_data
            ENDIF ELSE output_ts[obs_no-ExtraIncluded]=mdi
        ENDIF ELSE BEGIN
            IF mcnt GE ((point-1.)/2.)+1. THEN BEGIN

            ;     calculate the appropriate normalisation and apply the weights

                normalisation=SUM(rawweights[non_missing])            
                weighted_data=SUM(data_chunk[non_missing]*rawweights[non_missing]/normalisation)
                
                ; adjust the output writing by the padding
                output_ts[obs_no]=weighted_data
            ENDIF ELSE output_ts[obs_no]=mdi
        ENDELSE
    ENDIF
ENDFOR


RETURN

END
;************************************************************************
;                                 END
;************************************************************************
