;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;British Crown Copyright (c) 2012, the Met Office All rights reserved.
;
;Redistribution and use in source and binary forms, with or without 
;modification, are permitted provided that the following conditions 
;are met:
;
;Redistributions of source code must retain the above copyright notice, 
;this list of conditions and the following disclaimer. 
;
;Redistributions in binary form must reproduce the above copyright 
;notice, this list of conditions and the following disclaimer in the 
;documentation and/or other materials provided with the distribution. 
;
;Neither the name of the Met Office nor the names of its contributors 
;may be used to endorse or promote products derived from this software 
;without specific prior written permission. 
;
;THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
;"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
;LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
;A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
;HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
;SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
;LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
;DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
;THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
;(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
;OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

PRO calculate_distances, uk=uk

; Code reads in the station id, lat, lon and elev information from ASCII and calculates the great
; arc distance between all possible neighbour sets. Outputs this as a netcdf file for use in
; neighbour-based QC and homogenisation as well as gridding steps of ISD work.
;
; Open and read in the station information.
;
; Read in the list of stations to be considered.
;
; NAME:
;     CALCULATE_DISTANCES
;
; PURPOSE:
;     Calculate the distances between stations using the latitudes and longitudes
;
; CALLING SEQUENCE:
;     calculate_distances, uk=uk
;
; INPUTS:
;     uk - keyword to create UK specified outputfile
;
; OUTPUTS:
;
;     None - files written to specified directory.  Creates the
;            station_locations_and_distances file.
;
; RESTRICTIONS:
;
;     Assumes netcdf file inputs.  Requires file "kept_station_info.txt" to
;     read in the stations to be analysed.
;   
; HISTORY:
;     NB - not all revisions are noted here.
;     # 157 - contains the form of the program as obtained
;            17 December 2010 RJHD 
;     # 159 - adapted to run for UK stations only 
;            17 December 2010 RJHD 
;     # 194 - UK adaption linked to keyword for ease of use
;            8 March 2011 RJHD 
;     # 237 - SVN information added to header
;             16 November 2011 RJHD
;
;
;************************************************************************
;                    SVN Info
;$Rev:: 303                                           $:  Revision of last commit
;$Author:: rdunn                                      $:  Author of last commit
;$Date:: 2012-06-14 14:02:44 +0100 (Thu, 14 Jun 2012) $:  Date of last commit
;************************************************************************
;                                 START
;************************************************************************
nstation=6103
station_ids=STRARR(nstation)
station_lats=FLTARR(nstation)
station_lons=FLTARR(nstation)
station_elevs=FLTARR(nstation)
id=''
st=0
OPENR,1,'./kept_station_info.txt'
WHILE NOT EOF(1) DO BEGIN
  READF,1,id,FORMAT='(A)'
  station_ids(st)=STRMID(id,0,12)
  station_lats(st)=FLOAT(STRCOMPRESS(STRMID(id,13,9),/remove_all))
  station_lons(st)=FLOAT(STRCOMPRESS(STRMID(id,23,9),/remove_all))
  station_elevs(st)=FLOAT(STRCOMPRESS(STRMID(id,33,9),/remove_all))
  st=st+1
ENDWHILE
CLOSE,1

station_no=0
IF KEYWORD_SET(uk) THEN BEGIN
    FOR st=0,nstation-1 DO BEGIN
        IF STRCMP(station_ids(st), '03',2, /FOLD_CASE) EQ 1 THEN station_no=station_no+1
    ENDFOR
ENDIF ELSE BEGIN
    station_no=6103
ENDELSE

distances=FLTARR(station_no,station_no)  ; Distances array is a square array symmetric about the diagonal
angles=FLTARR(station_no,station_no)

new_station_ids=STRARR(station_no)
new_station_lats=FLTARR(station_no)
new_station_lons=FLTARR(station_no)
new_station_elevs=FLTARR(station_no)

xcnt=0
FOR st=0,nstation-1 DO BEGIN

    IF KEYWORD_SET(uk) THEN BEGIN

        IF STRCMP(station_ids(st), '03',2, /FOLD_CASE) EQ 1 THEN BEGIN
            ycnt=0
        
            FOR ts=0,nstation-1 DO BEGIN
                IF STRCMP(station_ids(ts), '03',2, /FOLD_CASE) EQ 1 THEN BEGIN
                    
                    IF st NE ts THEN BEGIN ; Don't calculate the diagonal - waste of time and the function will screw up with a /0. operand.
                        COMPASS,station_lats(st),station_lons(st),station_lats(ts),station_lons(ts),rng,az
                        distances(xcnt,ycnt)=rng
                        angles(xcnt,ycnt)=az
                    ENDIF
                    ycnt=ycnt+1
                ENDIF
            ENDFOR
            
            new_station_ids[xcnt]=station_ids(st)
            new_station_lats[xcnt]=station_lats(st)
            new_station_lons[xcnt]=station_lons(st)
            new_station_elevs[xcnt]=station_elevs(st)
            xcnt=xcnt+1
        ENDIF
    ENDIF ELSE BEGIN
        ycnt=0
        
        FOR ts=0,nstation-1 DO BEGIN
            
            IF st NE ts THEN BEGIN ; Don't calculate the diagonal - waste of time and the function will screw up with a /0. operand.
                COMPASS,station_lats(st),station_lons(st),station_lats(ts),station_lons(ts),rng,az
                distances(xcnt,ycnt)=rng
                angles(xcnt,ycnt)=az
            ENDIF
            ycnt=ycnt+1
        ENDFOR
        
        new_station_ids[xcnt]=station_ids(st)
        new_station_lats[xcnt]=station_lats(st)
        new_station_lons[xcnt]=station_lons(st)
        new_station_elevs[xcnt]=station_elevs(st)
        xcnt=xcnt+1
    ENDELSE
ENDFOR
;
; Write out to a netcdf file
;
IF KEYWORD_SET(uk) THEN BEGIN
    netcdf_outfile='./uk_station_locations_and_distances.nc'
ENDIF ELSE BEGIN
    netcdf_outfile='./station_locations_and_distances.nc'
ENDELSE
;
wilma=NCDF_CREATE(netcdf_outfile,/clobber)
stationid=NCDF_DIMDEF(wilma,'station',station_no)
charid=NCDF_DIMDEF(wilma, 'character_length', 12)

stationsvar=NCDF_VARDEF(wilma,'station',[charid,stationid],/CHAR)
latsvar=NCDF_VARDEF(wilma,'latitude',[stationid],/FLOAT)
lonsvar=NCDF_VARDEF(wilma,'longitude',[stationid],/FLOAT)
elevsvar=NCDF_VARDEF(wilma,'elevation',[stationid],/FLOAT)
distvar=NCDF_VARDEF(wilma,'distance',[stationid,stationid],/FLOAT)
anglevar=NCDF_VARDEF(wilma,'angle',[stationid,stationid],/FLOAT)

NCDF_ATTPUT,wilma,'station','long_name','Station_identifier'
NCDF_ATTPUT,wilma,'station','units','WMO-WBAN'
NCDF_ATTPUT,wilma,'station','axis','S,C'
NCDF_ATTPUT,wilma,'latitude','long_name','Latitude'
NCDF_ATTPUT,wilma,'latitude','units','degrees_north'
NCDF_ATTPUT,wilma,'latitude','axis','S'  
NCDF_ATTPUT,wilma,'latitude','valid_min',-90.
NCDF_ATTPUT,wilma,'latitude','valid_max',90.
NCDF_ATTPUT,wilma,'longitude','long_name','longitude'
NCDF_ATTPUT,wilma,'longitude','units','degrees_east'
NCDF_ATTPUT,wilma,'longitude','axis','S'  
NCDF_ATTPUT,wilma,'longitude','valid_min',-180.
NCDF_ATTPUT,wilma,'longitude','valid_max',180.
NCDF_ATTPUT,wilma,'elevation','long_name','elevation'
NCDF_ATTPUT,wilma,'elevation','units','metres_above_sea_level'
NCDF_ATTPUT,wilma,'elevation','axis','S'  
NCDF_ATTPUT,wilma,'elevation','valid_min',-300.
NCDF_ATTPUT,wilma,'elevation','valid_max',8000.
NCDF_ATTPUT,wilma,'distance','long_name','great_arc_distance'
NCDF_ATTPUT,wilma,'distance','units','Kilometres'
NCDF_ATTPUT,wilma,'distance','axis','S,S'  
NCDF_ATTPUT,wilma,'distance','valid_min',0.
NCDF_ATTPUT,wilma,'angle','long_name','great_arc_bearing'
NCDF_ATTPUT,wilma,'angle','units','Degrees'
NCDF_ATTPUT,wilma,'angle','axis','S,S'  
NCDF_ATTPUT,wilma,'angle','valid_min',-180.
NCDF_ATTPUT,wilma,'angle','valid_max',180.
;
;
current_time=SYSTIME()
PRINT,current_time
NCDF_ATTPUT,wilma,/GLOBAL,'file_created',STRING(current_time)
NCDF_CONTROL,wilma,/ENDEF
  
NCDF_VARPUT, wilma, stationsvar, new_station_ids
NCDF_VARPUT, wilma, latsvar, new_station_lats
NCDF_VARPUT, wilma, lonsvar, new_station_lons
NCDF_VARPUT, wilma, elevsvar, new_station_elevs
NCDF_VARPUT, wilma, distvar, distances
NCDF_VARPUT, wilma, anglevar, angles

NCDF_CLOSE,wilma

RETURN

END
