;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;British Crown Copyright (c) 2012, the Met Office All rights reserved.
;
;Redistribution and use in source and binary forms, with or without 
;modification, are permitted provided that the following conditions 
;are met:
;
;Redistributions of source code must retain the above copyright notice, 
;this list of conditions and the following disclaimer. 
;
;Redistributions in binary form must reproduce the above copyright 
;notice, this list of conditions and the following disclaimer in the 
;documentation and/or other materials provided with the distribution. 
;
;Neither the name of the Met Office nor the names of its contributors 
;may be used to endorse or promote products derived from this software 
;without specific prior written permission. 
;
;THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
;"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
;LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
;A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
;HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
;SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
;LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
;DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
;THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
;(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
;OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

PRO CHECK_DIURNAL, temperatures, temperatures_mdi,temperatures_fdi, times, diurnal_qc_flags, filter_flags, $
                   dynamic_diurnal=dynamic_diurnal, diagnostics, station_id

; Program reads in the temperature timeseries and time index and
; attempts to make a rational decision about when the peak of the 
; diurnal cycle is by minimising the difference between a sin curve scaled
; by the magnitude of the difference between min and max in that day.
; yields a best guess and an uncertainty that is based on the number of
; timesamples. This yields a population of estimates from which a best guess
; is picked and then continuous strings of >30 days that differ from this best
; guess are removed as having a potentially dubious timestamp.
;
; NAME:
;     CHECK_DIURNAL
;
; PURPOSE:
;     To check that a reasonable/sensible diurnal cycle is present in
;     the data - and that there are no "out-of-sync" periods.
;
; CALLING SEQUENCE:
;     check_diurnal temperatures, temperatures_mdi, temperatures_fdi, times,
;     diurnal_qc_flags, dynamic_diurnal=dynamic_diurnal, diagnostics,
;     station_id 
;
; INPUTS:
;     temperatures - array containing the temperature data
;
;     temperatures_mdi - missing data indicator for the temperatures
;
;     temperatures_fdi - flagged data indicator for the temperatures
;
;     times -  array containing the times
;
;     diurnal_qc_flags - array to keep the flags in
;
;     filter_flags - array containing all flags previously set on values
;
;     dyamic_diurnal - keyword to dynamically calculate the
;                      uncertainties in the position of the diurnal cycle
;
;     diagnostics - keyword to activate diagnostic information and
;                   plots
;
;     station_id - the name of the station - for the plot files
;
;
; METHOD:
;     Uses temperature timeseries to find peak of diurnal cycle.
;     Scaling done by the difference between the max and min for that
;     day.  The peaks from all days are the compared, along with their
;     uncertainties.  The average best fit is then taken as the phase
;     of the diurnal cycle, and outliers from that flagged.
;
;
; OUTPUTS:
;
;     None - arrays returned to control script
;
; RESTRICTIONS:
;
;     Assumes netcdf file inputs.
;   
; HISTORY:
; 18/11/(08?)  - Keyword dynamic_uncertainty added that tries to tweak the uncertainty based upon
;          goodness of fit to the simple first harmonic SIN curve that is being fitted.
;
; 19/8/09  - Only calculated if the range of max and min is 5K or
;            greater.
; #13 - As was received in October 2010 - RJHD
; #14 - Commented out as best possible.  Reset the string length
;              20 (it was 30).  Added diagnostic plot.
; #48 - Reset the string length to 30 - to match previous incarnation
;       of the programme - amended header too.  Email sent to confirm.
;       12 November 2010 RJHD
; #171 - Added filter flags to pass through the total flags set
;        for each value
;        14 January 2011 RJHD
; #173 - Used filtered values to find diurnal cycle, but flag
;        unfiltered values.
;        17 January 2011 RJHD
; #237 - SVN information added to header
;        Added fdi tests to mdi tests
;        16 November 2011 RJHD
; #299 - Changed float arrays to Doubles
;        6 June 2012 RJHD
;
;************************************************************************
;                    SVN Info
;$Rev:: 308                                           $:  Revision of last commit
;$Author:: rdunn                                      $:  Author of last commit
;$Date:: 2012-07-04 14:47:15 +0100 (Wed, 04 Jul 2012) $:  Date of last commit
;************************************************************************
;                                 START
;************************************************************************

;******************************************
; Mask values with past flags
;******************************************
untouched_temperatures=temperatures

past_flags=WHERE(filter_flags GT 0, pfcnt)

IF pfcnt GE 1 THEN BEGIN
    temperatures(past_flags)=temperatures_mdi
ENDIF


;************************************************************************
; Find the number of days between the first and last available obs and set up
; the resulting arrays.
;************************************************************************

diurnal_qc_flags=INTARR(N_ELEMENTS(times))
ntimes=N_ELEMENTS(times) ;assumes hourly data (on the hour)
start_time=times(0)
end_time=times(ntimes-1)
num_days_between=end_time-start_time+1

days=INDGEN(num_days_between)+FIX(start_time)
daily_values=DBLARR(num_days_between,24)
daily_values(*,*)=temperatures_mdi
hours_index=FINDGEN(24)/24.

;************************************************************************
; Fill in values in the daily_values array.
;   VERY SLOW FOR LOOP
;************************************************************************
FOR t=0L,ntimes-1 DO BEGIN
  this_day=FIX(times(t))
  day_loc=WHERE(days EQ this_day, check)
  IF check EQ 1 THEN BEGIN  ; Very occasionally close on 00Z and this catches
    day_loc=day_loc(0)
    this_hour=times(t)-this_day
    hours_diffs=ABS(hours_index-this_hour)
    minval=MIN(hours_diffs)
    hour_loc=WHERE(hours_diffs EQ minval)
    hour_loc=hour_loc(0)
    daily_values(day_loc,hour_loc)=temperatures(t)
  ENDIF
ENDFOR

;************************************************************************
; Set up the arrays to output diurnal fits to.  
;************************************************************************

diurnal_best_fits=INTARR(num_days_between)
diurnal_best_fits(*)=-999
diurnal_uncertainties=INTARR(num_days_between)
diurnal_uncertainties(*)=-999

;************************************************************************
; Set up the SINE curve.
;************************************************************************

angles = 2.*!PI/24. * FINDGEN(24) ; so that it loops
sine_curve=SIN(angles)	; PEAK AT 6am (6), TROUGH AT 6pm (18)


;************************************************************************
; Produce a best fit for each day with at least 4 values, at least one in three of the six hour
; periods, and its uncertainty.
;************************************************************************

IF KEYWORD_SET(diagnostics) THEN BEGIN
    Observation_locations=WHERE(((daily_values NE temperatures_mdi) AND (daily_values NE temperatures_fdi)), nObs)
    WrappedHours=fltarr(2*nObs,/NOZERO)
    WrappedValues=fltarr(2*nObs,/NOZERO)
    pos=0
ENDIF

best_estimates=INTARR(24)
FOR d=0,num_days_between-1 DO BEGIN

    actual_data_points=WHERE(((daily_values(d,*) NE temperatures_mdi) AND (daily_values(d,*) NE temperatures_fdi)), datacount)

    ;************************************************************************
    ; Potentially enough data to constrain(ish) diurnal cycle
    ;************************************************************************
    IF datacount GE 4 THEN BEGIN  
        ;************************************************************************
        ;    Find the max and min values to scale cycle by
        ;                          the daily range
        ;************************************************************************
        minval=MIN(daily_values(d,actual_data_points))
        maxval=MAX(daily_values(d,actual_data_points))
        daily_range=maxval-minval

        ;************************************************************************
        ;    If the daily range is more than 5K
        ;          then try to extract daily cycle
        ;************************************************************************
        IF daily_range GE 5. THEN BEGIN

        ;************************************************************************
        ; Now ascertain whether there are data in at least 3 quartiles of the day
        ;************************************************************************
            quart_count=0
            before6=WHERE(actual_data_points LE 4, b6)
            IF b6 GE 1 THEN quart_count=quart_count+1
            before12=WHERE((actual_data_points GE 5) AND (actual_data_points LE 10), b12)
            IF b12 GE 1 THEN quart_count=quart_count+1
            before18=WHERE((actual_data_points GE 11) AND (actual_data_points LE 16), b18)
            IF b18 GE 1 THEN quart_count=quart_count+1
            before_24=WHERE((actual_data_points GE 17) AND (actual_data_points LE 23), b24)
            IF b24 GE 1 THEN quart_count=quart_count+1
            
            IF quart_count GE 3 THEN BEGIN
            ;************************************************************************
            ;  There is data in at least 3 quartiles of the day
            ;************************************************************************

                ;************************************************************************
                ; scale the Sine curve to the data - NB the diagnostic
                ;  plots scale the data to the curve.
                ;************************************************************************

                curve_fit=((sine_curve+1.)*(daily_range/2.))+minval
                ;************************************************************************
                ;KATE*** THIS was originally  curve_fit=(sine_curve*daily_range)+minval 
                ;    BOTH WORK BUT THE FORMER GIVES SLIGHT DIFFERENCES
                ;************************************************************************
                diffs=DBLARR(24)
                
                ;************************************************************************
                ; calculate summed vertical offsets for each observation 
                ;   shift the sine curve by 1 hour and repeat
                ;   find the minimum offset
                ;************************************************************************
                FOR h=0,23 DO BEGIN
                    diffs(h)=TOTAL(ABS(daily_values(d,actual_data_points)-curve_fit(actual_data_points)))
                    curve_fit=SHIFT(curve_fit,1)
                ENDFOR
                min_diff=MIN(diffs)
                max_diff=MAX(diffs)
                hour_loc=WHERE(diffs EQ  min_diff)
                diurnal_best_fits(d)=hour_loc(0)
                best_estimates(hour_loc(0))=best_estimates(hour_loc(0))+1
                
                
                IF KEYWORD_SET(diagnostics) THEN BEGIN

                    ;************************************************************************
                    ; Sort the scaling limits and apply to these sets of values
                    ;************************************************************************
                    
                    SV_Max=MAX(daily_values(d,actual_data_points))
                    SV_Min=MIN(daily_values(d,actual_data_points))
                 
                    ScaledValues=(daily_values(d,actual_data_points)-((SV_Max+SV_Min)/2.0))/(daily_range/2.0)

                    ;************************************************************************
                    ; counter to increment the wrapped values
                    ;************************************************************************
                    nvec=n_elements(daily_values(d,actual_data_points))-1

                    ;************************************************************************
                    ; fill arrays for plotting
                    ;************************************************************************
                    WrappedHours(pos:pos+nvec)=actual_data_points*2.*!PI/24.
                    WrappedValues(pos:pos+nvec)=ScaledValues
                    pos=pos+nvec+1

                    WrappedHours(pos:pos+nvec)=(actual_data_points*2.*!PI/24.)+2.*!PI
                    WrappedValues(pos:pos+nvec)=ScaledValues
                    pos=pos+nvec+1
                    
                ENDIF
                
                
                IF KEYWORD_SET(dynamic_diurnal) THEN BEGIN
                    ;************************************************************************
                    ; Define the uncertainty based upon the tightness of fit and how this falls
                    ; off on either side.  Think like an inverse Gaussian
                    ;  and the min_diff is the mean, and the difference
                    ;  sort of like the standard deviation
                    ;************************************************************************
;     |                                                         
;     |                             uncertainty                
;     |                            |-----------|                
;     |                                                         
; D   |   *  *  *                                      *  *  *  
; i   |            *  *                          *  *           
; f   |                  *                    *                 
; f   |                     *              *                    
; s   |                                                         
;     |                        *        *                       
;     |                                                         
;     |                                                         
;     |                           *  *  <---- Min Diff                
;     |                                                         
;     |                                                         
;     |                                                         
;     |                                                         
;     |_________________________________________________________________
;                              HOURS
             
                    ;************************************************************************
                    crit_diff=min_diff+(max_diff-min_diff)*0.33
                
                    matched_locs=[0,0]
                    contwhile=1
                    whilecounter=1
                    diffs=SHIFT(diffs,11-hour_loc(0)) ; so minimum is in the centre of the array

                    ;************************************************************************
                    ; Count outwards from the minimum until 
                    ; value is exceeded.
                    ;************************************************************************

                    WHILE contwhile EQ 1 DO BEGIN
                        
                        IF (diffs(11-whilecounter) GE crit_diff) AND (matched_locs(0) EQ 0) THEN matched_locs(0)=1
                        IF (diffs(11+whilecounter) GE crit_diff) AND (matched_locs(1) EQ 0) THEN matched_locs(1)=1
                        IF TOTAL(matched_locs) EQ 2 OR whilecounter EQ 11 THEN BEGIN
                            contwhile=0
                        ENDIF ELSE BEGIN
                            whilecounter=whilecounter+1
                        ENDELSE

                    ENDWHILE
                    ;************************************************************************
                    ; And set the uncertainty to the counter
                    ;************************************************************************
                    diurnal_uncertainties(d)=whilecounter

                    ;************************************************************************
                    ; Make sure can't be shorter than the data time increment.
                    ;   i.e. - can't be <6 if data only read every 6 hours
                    ;************************************************************************
                    
                    IF diurnal_uncertainties(d) LT ROUND(24./FLOAT(datacount)) THEN diurnal_uncertainties(d)=ROUND(24./FLOAT(datacount))
                ENDIF ELSE BEGIN
                    diurnal_uncertainties(d)=ROUND(24./FLOAT(datacount))
                ENDELSE
            ENDIF
        ENDIF
    ENDIF
 
ENDFOR

;************************************************************************
; Print out a scaled and shifted diurnal cycle curve with the sine ontop
;************************************************************************
IF KEYWORD_SET(diagnostics) THEN BEGIN
    PeakLocation=WHERE(best_estimates EQ MAX(best_estimates))
    set_plot,'PS'
    cd, "./", current=outdirplot
    device,filename=outdirplot+'/images/'+station_id+'_DiurnalCurve.eps'$
      ,/color,/encapsulate,/helvetica,/bold,/landscape,ysize=20,xsize=26

    !P.Font=0
    !P.Thick=4
    !P.Multi=[0,1,1]
    plotsym,3,1.2
    plot,[angles,(angles+2.*!PI)],SHIFT([sine_curve,sine_curve],PeakLocation),xstyle=1,ystyle=1,ythick=4,xthick=4,ytitle="Cycle",xtitle="Hour (Z)",title="Diurnal Curve Check", xticks=1, POSITION=[0.10, 0.10, 0.90, 0.90]

    WrappedValues=TEMPORARY(WrappedValues(WHERE(WrappedHours NE 0)))
    WrappedHours=TEMPORARY(WrappedHours(WHERE(WrappedHours NE 0)))

    oplot,(WrappedHours+(RANDOMN(systime_seed,n_elements(WrappedHours))*0.25)),(WrappedValues+(RANDOMN(systime_seed,n_elements(WrappedValues))*0.05)),xstyle=1,ystyle=1,charsize=1,ythick=4,xthick=4,psym=3, POSITION=[0.10, 0.10, 0.90, 0.90]
    AXIS, xAXIS=0, XRANGE=[0,48],XTICKV=INDGEN(24)*2, /SAVE
    device,/close
ENDIF  

;************************************************************************
; Now decide on the best fit overall.
;************************************************************************

best_fits=INTARR(6)   

;************************************************************************
; Best fits is for different hour increments / uncertainty ranges up
; to six hours from the central point (min uncertainty is the hourly
; interval. More uncertain than that are ignored in producing a best
; fit - too uncertain and just set to one for final assignment issue. 
;************************************************************************
best_fits(*)=-9.

FOR h=1,6 DO BEGIN
  ;************************************************************************
  ; find where the uncertainties are small enough
  ;************************************************************************
  points=WHERE(diurnal_uncertainties EQ h, hc)

  IF hc GE 300 THEN best_fits(h-1)=MEDIAN(diurnal_best_fits(points)) 
     ;************************************************************************
     ; if the uncertainties are small enough, set the 
     ; best fit to the median best fit offset for that uncertainty
     ; Should be all very similar if there is a sensible cycle
     ;   "Start" of the cycle is at 0 on SineCurve on increasing section.
     ;   This is the value given by the median.  Peak is ~6h later
     ;************************************************************************

ENDFOR


diurnal_peak=-9
num_estimates=0
hours=INDGEN(24)
hour_matches=INTARR(24)			
;************************************************************************
; an additive which accumulates points for hours that show where the start of diurnal cycle falls (9am?)
;************************************************************************


;************************************************************************
; For each level of uncertainty calculate  the range of hours covered
; by the best_fit+/-uncertainty and do these ranges match up?
; If they do there's a well defined cycle.  If not, then we've a problem
;************************************************************************
FOR h=0,5 DO BEGIN
  IF best_fits(h) NE -9 THEN BEGIN
    ;************************************************************************
    ; If there are any valid best fit estimates
    ;************************************************************************
    IF diurnal_peak EQ -9 THEN BEGIN
        ;************************************************************************
        ; DIURNAL_PEAK contains the original hour value that falls on the maximum
        ;   This is the offset from 00Z to the start of the sine curve
        ;************************************************************************
        diurnal_peak=best_fits(h)

        ;************************************************************************
	    ; Centres the hours array then adds 1 to this point +/- the uncertainty range. 		
        ;************************************************************************
        hours=SHIFT(hours,11-diurnal_peak) 
        hour_matches(11-(h+1):11+(h+1))=1	 
        num_estimates=num_estimates+1
    ENDIF

    ;************************************************************************
    ; Increment the hours overlap count
    ;   Builds up a distribution from the spread of values for each uncertainty
    ;   level, where the start of the cycle falls - so full mean+sigma
    ;   range for each uncertainty.  From this distribution it can be checked
    ;   What the best value is from all days + their uncertainties considered
    ;     together.  The best fit for the population might not be the mode.
    ;************************************************************************
    this_uncertainty=h+1
    centre=WHERE(hours EQ best_fits(h))

    IF centre(0)-(h+1) GE 0 THEN BEGIN
      IF centre(0)+(h+1) LE 23 THEN BEGIN
        hour_matches(centre(0)-(h+1):centre(0)+(h+1))=hour_matches(centre(0)-(h+1):centre(0)+(h+1))+1
      ENDIF ELSE BEGIN
        hour_matches(centre(0)-(h+1):23)=hour_matches(centre(0)-(h+1):23)+1
	left_over=centre(0)+(h+1)-24
	hour_matches(0:left_over)=hour_matches(0:left_over)+1
      ENDELSE
    ENDIF ELSE BEGIN
      hour_matches(0:centre(0)+(h+1))=hour_matches(0:centre(0)+(h+1))+1
      left_over=ABS(centre(0)-(h+1))
      hour_matches(23-left_over:23)=hour_matches(23-left_over:23)+1
    ENDELSE
    num_estimates=num_estimates+1
  ENDIF
ENDFOR


;***************************************
; find where all the estimates overlapped - if they did
;***************************************
IF hour_matches(11) NE num_estimates THEN BEGIN 	
    ;************************************************************************
    ; The finest temporal granuity disagrees with all others. Are there 
    ; any times when all the estimates actually agree?
    ;************************************************************************
    ; Do ALL the estimates on the best_fit (+/- uncertainties) match up?
    ; If yes, use this value, if not wipe the values
    ;************************************************************************
  allmatch=WHERE(hour_matches EQ num_estimates, ac)
  IF ac GE 1 THEN diurnal_peak=hours(allmatch(0)) ELSE diurnal_peak=-9
ENDIF


;******************************************
; Flag on un_filtered values, so revert
;******************************************
temperatures=untouched_temperatures



IF diurnal_peak NE -9 THEN BEGIN

  ;********************
  ; Reset hours again
  ;********************
  hours=INDGEN(24)
  hours=SHIFT(hours,11-diurnal_peak)

  ;*****************************
  ; Set up spurious data array
  ;*****************************
  potentially_spurious=INTARR(num_days_between)
  potentially_spurious(*)=-999

  ;*****************************
  ; Find locations which are outside this best fit 
  ; according to their stated uncertainty
  ;*****************************

  FOR d=0,num_days_between-1 DO BEGIN
    ;*****************************
    ;  For each day, if a diurnal best fit was calculable,
    ;   is the location of the best fit outside of the range expected
    ;   from the analysis of the total population.
    ;*****************************
    IF diurnal_best_fits(d) NE -999 THEN BEGIN
      min_range=11-diurnal_uncertainties(d)
      max_range=11+diurnal_uncertainties(d)
      maxloc=WHERE(hours EQ diurnal_best_fits(d))
      IF (maxloc LT min_range) OR (maxloc GT max_range) THEN potentially_spurious(d)=1 ELSE potentially_spurious(d)=0
    ENDIF
  ENDFOR

  ;*****************************
  ; Find strings in potentially spurious and delete these. 
  ; String must be >30 -  counting in days here.
  ; and not contain a set of 3 consecutive "good" or "missing" days or 6 days mix of these within it
  ; and greater than 50% are non-missing.
  ;*****************************

  num_consecutive_good=0
  num_consecutive_missing=0
  num_consecutive_not_bad=0
  total_points_considered=0
  total_non_missing=0
  to_delete=INTARR(num_days_between)

  FOR d=0,num_days_between-1 DO BEGIN
 
   IF potentially_spurious(d) EQ 1 THEN BEGIN
      ;*****************************
      ; potentially spurious observation 
      ;       - reset the consecutive good, missing & not-bad counters.
      ; increment the total points and non-missing counters
      ;*****************************
      IF num_consecutive_good NE 0 THEN num_consecutive_good=0
      IF num_consecutive_missing NE 0 THEN num_consecutive_missing=0
      IF num_consecutive_not_bad NE 0 THEN num_consecutive_not_bad=0
      total_points_considered=total_points_considered+1
      total_non_missing=total_non_missing+1
    ENDIF ELSE BEGIN

      ;*****************************
      ; not a spurious observation
      ;*****************************
      IF potentially_spurious(d) EQ 0 THEN BEGIN
        ;*****************************
        ; increment the good & not-bad counters
        ; reset the missing counter and increment and the non-missing counters
        ;*****************************
        num_consecutive_good=num_consecutive_good+1
        num_consecutive_not_bad=num_consecutive_not_bad+1
        IF num_consecutive_missing NE 0 THEN num_consecutive_missing=0
        total_non_missing=total_non_missing+1
      ENDIF

      ;*****************************
      ; a missing observation
      ;*****************************
      IF potentially_spurious(d) EQ -999 THEN BEGIN
        ;*****************************
        ; increment the missing & not-bad counters
        ; reset the good counter
        ;*****************************
        num_consecutive_missing=num_consecutive_missing+1
        num_consecutive_not_bad=num_consecutive_not_bad+1
        IF num_consecutive_good NE 0 THEN num_consecutive_good=0
      ENDIF


      ;*****************************
      ; increment the total number of points investigated counter
      ;*****************************
      total_points_considered=total_points_considered+1

      ;*****************************
      ; now test to see if there are enough bad to address.
      ;*****************************
      ; Out of the number considered since the last time are there
      ;     1) only 3 good points in the sequence
      ; or  2) only 3 missing points in the sequence
      ; or  3) only 6 non-bad points in the sequence
      ;*****************************
      IF (num_consecutive_good EQ 3) OR (num_consecutive_missing EQ 3) OR (num_consecutive_not_bad GE 6) THEN BEGIN
          ;*****************************
          ;  If at least 30 in the sequence (so, ~1 month's worth)
          ;    And more than half were observed (non-missing)
          ;    Then this sequence is sufficiently spurious to delete
          ;*****************************
	  IF total_points_considered GE 30 THEN BEGIN
	    dataratio=FLOAT(total_non_missing)/FLOAT(total_points_considered)
	    IF dataratio GE 0.5 THEN to_delete(d-total_points_considered+1:d)=1
	  ENDIF
 
          ;*****************************
          ; reset the counters
          ;*****************************
	  num_consecutive_good=0
	  num_consecutive_missing=0
	  num_consecutive_not_bad=0
	  total_points_considered=0
	  total_non_missing=0
	ENDIF
      ENDELSE
  ENDFOR
  
  ;************************************************************************
  ; If there are any non-zero elements of to_delete then map back onto the
  ; input data as missing and set qc flag values as 'diurnal'
  ;************************************************************************
  
  badpoints=WHERE(to_delete EQ 1, badcount)
  IF badcount GE 1 THEN BEGIN
    whole_days=FIX(times)
    FOR d=0,badcount-1 DO BEGIN
      realtimes=WHERE(whole_days EQ days(badpoints(d)), exists)
      IF exists GE 1 THEN BEGIN
        bads=WHERE(((temperatures(realtimes) NE temperatures_mdi) AND (temperatures(realtimes) NE temperatures_fdi)),count)
	IF (count GT 0) THEN BEGIN
	  diurnal_qc_flags(realtimes(bads))=1
        ENDIF
      ENDIF
    ENDFOR
  ENDIF
ENDIF



;************************************************************************
;  Plot the sequences which have an offset diurnal cycle. 9/page
;************************************************************************
IF KEYWORD_SET(diagnostics) THEN BEGIN
    flag_start=0.
    flag_end=0.
    FOR t=0L,ntimes-1 DO BEGIN

        IF diurnal_qc_flags(t) EQ 1 THEN BEGIN
            IF flag_start EQ 0. THEN BEGIN
                flag_start=t
                IF n_elements(flag_starts) eq 0 then flag_starts=[flag_start] else $
                  flag_starts=[flag_starts,flag_start]
            ENDIF
            good_counter=0
            
        ENDIF
        IF (diurnal_qc_flags(t) EQ 0) and (flag_start NE 0.) THEN BEGIN
            IF good_counter EQ 0 THEN flag_end=t-1
            good_counter=good_counter+1
            IF good_counter GT 3 THEN BEGIN
                IF n_elements(flag_ends) eq 0 then flag_ends=[flag_end] else $
                  flag_ends=[flag_ends,flag_end]
                print, flag_start,flag_end
                flag_start=0.
                flag_end=0.
                good_counter=0
           ENDIF
        ENDIF

    ENDFOR


    number_of_sections=n_elements(flag_starts)
    npages=CEIL(number_of_sections/9.)
    remainder=number_of_sections-FIX(number_of_sections/9.)*9.
    set_plot,'PS'
    !P.font=0
    plotsym,0,1,/fill

    x1=0.04
    x2=0.98
    y1=[0.9,0.79,0.68,0.57,0.46,0.35,0.24,0.13,0.02]
    y2=[0.98,0.87,0.76,0.65,0.54,0.43,0.32,0.21,0.10]
    !P.Multi=[0,1,9]
    for pp=0,npages-1 DO BEGIN ; spin through pages
        device,filename=outdirplot+'/images/'+station_id+'_DiurnalFlag_'+strcompress(pp,/remove_all)+'.eps',/landscape,xsize=26,ysize=20,/helvetica,/color
        for mp=0,8 DO BEGIN	; spin through the nine plots per page
            ;!P.Position=[x1,y1(mp),x2,y2(mp)]
            IF ((pp EQ npages-1) AND (mp GE remainder-1)) THEN continue ; escape the last few months
            elem=(pp*9)+mp
            TempTemps=temperatures(flag_starts(elem)-50:flag_ends(elem)+50)
            TempTimes=times(flag_starts(elem)-50:flag_ends(elem)+50)
            PlotTemps=TempTemps(WHERE(((TempTemps NE temperatures_mdi) AND (TempTemps NE temperatures_fdi))))
            PlotTimes=TempTimes(WHERE(((TempTemps NE temperatures_mdi) AND (TempTemps NE temperatures_fdi))))
            
            ;*****************************
            ; plot the month of data to be removed
            ;  with a sine curve on top - to be added in future
            ;*****************************
            ymax=MAX(PlotTemps)
            ymin=MIN(PlotTemps)
            yrange=ymax-ymin
            xmin=MIN(PlotTimes)
            xmax=MAX(PlotTimes)
            
            tvlct,0,0,0,0 

            dummy=LABEL_DATE(DATE_FORMAT=['%D/%N/%Y'])
            start_date=JULDAY(1, 1, 1973, 0)
                        
            !P.Position=[x1,y1(mp),x2,y2(mp)]

            plot,PlotTimes+start_date,PlotTemps,min_value=-1000,ystyle=1, xstyle=1,yrange=[ymin,ymax],xtickformat='LABEL_DATE',$
                  xrange=[xmin,xmax],ytitle='T deg C',charsize=1,color=0,psym=2,symsize=0.1,/noerase;,Position=[x1,y1(mp),x2,y2(mp)]
            oplot,[times(flag_starts(elem))+start_date,times(flag_starts(elem))+start_date],[ymin,ymax],psym=0,color=240,YLog=1,xtickformat='LABEL_DATE'
            oplot,[times(flag_ends(elem))+start_date,times(flag_ends(elem))+start_date],[ymin,ymax],psym=0,color=240,YLog=1,xtickformat='LABEL_DATE'
                
            PLOTS,[xmin,xmax],[ymin,ymin],color=0
            PLOTS,[xmin,xmax],[ymax,ymax],color=0
        ENDFOR
        device,/close

    ENDFOR

ENDIF




RETURN

END
;************************************************************************
;                                 END
;************************************************************************
