;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;British Crown Copyright (c) 2012, the Met Office All rights reserved.
;
;Redistribution and use in source and binary forms, with or without 
;modification, are permitted provided that the following conditions 
;are met:
;
;Redistributions of source code must retain the above copyright notice, 
;this list of conditions and the following disclaimer. 
;
;Redistributions in binary form must reproduce the above copyright 
;notice, this list of conditions and the following disclaimer in the 
;documentation and/or other materials provided with the distribution. 
;
;Neither the name of the Met Office nor the names of its contributors 
;may be used to endorse or promote products derived from this software 
;without specific prior written permission. 
;
;THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
;"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
;LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
;A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
;HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
;SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
;LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
;DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
;THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
;(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
;OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

PRO check_duplicates, last_year, temperatures, temperatures_mdi, temperatures_fdi, times,$  
                      full_times, full_temperatures, $ 
                      duplicate_qc_flags,  $
                      diagnostics, station_id, flagged_values, full_flagged_values$

; File checks for complete duplication of a given months data within the
; temperature timeseries defined as all matches for the smallest number
; of points in a given month.
;
; Limitations: Assumption on first year is hardwired
;
; NAME:
;     CHECK_DUPLICATES
;
; PURPOSE:
;     Checks for the duplication of a given month within the
;     temperature timeseries - scans all subsequent months for a match
;
; CALLING SEQUENCE:
;     check_duplicates, last_year, temperatures, temperatures_mdi, dewpoints,
;     slp, windspeeds, winddirs, times, full_times,
;     full_temperatures, full_dewpoints, full_slp,
;     full_windspeeds,full_winddirs, duplicate_qc_flags,
;     dewpoints_mdi, slp_mdi, windspeeds_mdi, winddirs_mdi,
;     diagnostics, station_id, flagged_values, full_flagged_values,
;     total_cloud_cover_mdi, low_cloud_cover_mdi, mid_cloud_cover_mdi,
;     high_cloud_cover_mdi 
;
; INPUTS:
;     last_year - the last year of observations
;     
;     temperatures - array containing the temperature data
;     
;     temperatures_mdi - missing data indicator for the temperatures
;
;     temperatures_fdi - flagged data indicator for the temperatures
;
;     times - array containing the times of the observations
;
;     full_times - array containing the times of the observations
;
;     full_temperatures - array containing the uncompressed temperature data 
;
;     duplicate_qc_flags - duplicate of QC flag array
;
;     diagnostics - keyword for turning on diagnostic features (plots)
;
;     station_id - station ID for plot names
;
;     flagged_values - array to store flagged values on masking
;
;     full_flagged_values - uncompressed array to store flagged values
;
; OUTPUTS:
;
;     None - files written to specified directory
;
; RESTRICTIONS:
;
;     Assumption on the first year is hardwired - 00:00 1/1/1973
;
; NOTE - only ever tested on faked data - no duplicates found in the
;        HadISD dataset as yet 8-MAR-2011 RJHD.  Diagnostic capability
;        therefore untested.
;
; HISTORY:
;     NB - not all revisions are noted here.
;     # 1 - contains the form of the program as obtained
;           October 2010 RJHD 
;     # 3 - extra commenting and formatting
;           02 November 2010 RJHD
;     # 4 - adjusted for external masking/flagging
;           03 November 2010 RJHD
;     #40 - added diagnostic plotting capability
;           10 November 2010 RJHD
;     #200- changed last_year from optional keyword to required input
;           to remove the hardwiring of the end date
;           11 March 2011 RJHD
;     #237- added flagged values set up.  SVN information added to
;           header.  Moved monthly set up and full_times array set up
;           to MONTH_SETUP.PRO and CREATE_FULLTIMES.PRO respectively. 
;           16 November 2011 RJHD
;
;************************************************************************
;                    SVN Info
;$Rev:: 308                                           $:  Revision of last commit
;$Author:: rdunn                                      $:  Author of last commit
;$Date:: 2012-07-04 14:47:15 +0100 (Wed, 04 Jul 2012) $:  Date of last commit
;************************************************************************
;                                 START
;************************************************************************

; full_xx arrays set up in CREATE_FULLTIMES.PRO 17/11/11 RJHD

;*************************************************************
; Set the qc_flags keyword as a intarr of the required size (only passing back and not in should speed the process)
;*************************************************************
duplicate_qc_flags=INTARR(N_ELEMENTS(times))


;*************************************************************
; Define segment breaks as being 00UTC on first of each month and fill
; the monthly segments arrays.
;
; NOTE - not using monthly_setup.pro for this run as the first
;        definition of many of the arrays (at the moment 031110)
;*************************************************************
; Removed this bit of code and using MONTH_SETUP.PRO 17/11/11 RJHD

MONTH_SETUP,last_year,nmonths,nyears,times,full_times,monthly_segment_starts,$
  monthly_segment_ends, monthly_segment_lengths,/full

monthly_segments=DBLARR(24*31,nmonths) ; 24 hourly obs * 31 days
monthly_segments(*,*)=temperatures_mdi

; difference between code in CHECK_DUPLICATES and that in MONTH_SETUP
monthly_segment_lengths=monthly_segment_lengths-1

FOR seg=0,nmonths-1 DO BEGIN
   monthly_segments(0:monthly_segment_lengths(seg),seg)=$
                  full_temperatures(monthly_segment_starts(seg):monthly_segment_ends(seg))
ENDFOR


plotcounter=0 ; counter for any plots created
;************************************************************************
; Monthly arrays created.  Now starting the comparison.
;************************************************************************


;*****************************************************************
; Now compare them to each other on an iterative basis.
;*****************************************************************
duplicated=INTARR(nmonths) ; to hold a flag if the month is duplicated
duplicated(*)=0
FOR seg=0,nmonths-2 DO BEGIN
  IF duplicated(seg) NE 1 THEN BEGIN ; No point repeating if already a duplicate 
    FOR inner=seg+1,nmonths-1 DO BEGIN
      ;***************************************************
      ; Consider only the shorter of the common periods.
      ;***************************************************
      these_lengths=[monthly_segment_lengths(seg),monthly_segment_lengths(inner)]
      min_length=MIN(these_lengths)
      ;PRINT,these_lengths,min_length
      pop1=monthly_segments(0:min_length,seg)
      pop2=monthly_segments(0:min_length,inner)

      ;***************************************************
      ; Which is the smallest number of valid points?
      ;***************************************************
      valid1=WHERE(((pop1 NE temperatures_mdi) AND (pop1 NE temperatures_fdi)), pop1count)
      valid2=WHERE(((pop2 NE temperatures_mdi) AND (pop2 NE temperatures_fdi)), pop2count)
      ;***************************************************
      ; Does the larger true data points pop agree in all these positions?
      ;***************************************************
      IF pop1count GE 20 AND pop2count GE 20 THEN BEGIN
        IF pop1count LT pop2count THEN BEGIN
          match_points=WHERE(pop2(valid1) EQ pop1(valid1), match_count)
	  ;PRINT,'pop1',pop1count,pop2count,match_count
          IF match_count EQ pop1count THEN BEGIN
	    duplicated(seg)=1
	    duplicated(inner)=1
	  ENDIF
        ENDIF ELSE BEGIN
          match_points=WHERE(pop1(valid2) EQ pop2(valid2), match_count)
	  ;PRINT,'pop2',pop1count,pop2count,match_count
          IF match_count EQ pop2count THEN BEGIN
	    duplicated(seg)=1
	    duplicated(inner)=1 

            IF KEYWORD_SET(diagnostics) THEN BEGIN
            ;***************************************************
            ;  Start a plot showing the comparison of the two months
            ;    Taken from Coyote's guide to IDL - Ladder plots
            ;***************************************************
                set_plot,'PS'
                !P.Multi=[0,2,1]
                cd, "./", current=outdirplot

                dummy=LABEL_DATE(DATE_FORMAT=['%D/%N/%Y'])

                device,filename=outdirplot+'/images/'+station_id+'_Duplicates_'+plotcounter+'.eps'$
                  ,/color,/encapsulate,/helvetica,/bold,/landscape,ysize=20,xsize=26
                Plot,monthly_segments(0:min_length,seg)+start_date , XStyle=8, Position=[0.15, 0.15, 0.9, 0.50], $
                  XTitle='Time', YTitle='Temperatures',xtickformat='LABEL_DATE'
                Plot,monthly_segments(0:min_length,inner)+start_date , Position=[0.15, 0.52, 0.9, 0.90], $
                  /NoErase, XTickformat='(A1)', YTitle='Temperatures',xtickformat='LABEL_DATE'
                Plots, [0.15, 0.15], [0.50, 0.52], /Normal ; Fix left axis.
                Plots, [0.90, 0.90], [0.50, 0.52], /Normal ; Fix right axis.
                device,/close
                plotcounter=plotcounter+1
            ;***************************************************
            ;  Untested as no duplicate's flags found so far 10/11/2010 RJHD
            ;***************************************************
            ENDIF

	  ENDIF
        ENDELSE 
      ENDIF
    ENDFOR
  ENDIF
ENDFOR

;*****************************************************************
; Flag for deletion months that are entirely duplicated over the 
; common period of record. Actual deletion can be done in control
; script. Otherwise end of program has been reached.
;*****************************************************************
duplicate_months=WHERE(duplicated EQ 1, dupcount)
IF dupcount GE 1 THEN BEGIN
  FOR m=0,dupcount-1 DO BEGIN
    time_locs=WHERE(times GE full_times(monthly_segment_starts(duplicate_months(m))) AND $
                    times LE full_times(monthly_segment_ends(duplicate_months(m))))
    duplicate_qc_flags(time_locs)=1
  ENDFOR 
ENDIF

RETURN

END
;*****************************************************************
;                         END
;*****************************************************************
