;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;British Crown Copyright (c) 2012, the Met Office All rights reserved.
;
;Redistribution and use in source and binary forms, with or without 
;modification, are permitted provided that the following conditions 
;are met:
;
;Redistributions of source code must retain the above copyright notice, 
;this list of conditions and the following disclaimer. 
;
;Redistributions in binary form must reproduce the above copyright 
;notice, this list of conditions and the following disclaimer in the 
;documentation and/or other materials provided with the distribution. 
;
;Neither the name of the Met Office nor the names of its contributors 
;may be used to endorse or promote products derived from this software 
;without specific prior written permission. 
;
;THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
;"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
;LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
;A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
;HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
;SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
;LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
;DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
;THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
;(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
;OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

PRO check_records, temperatures, temperatures_mdi, temperatures_fdi, dewpoints, dewpoints_mdi, dewpoints_fdi, $ 
                   windspeeds, windspeeds_mdi, windspeeds_fdi, slp, slp_mdi, slp_fdi, qc_flags, filter_flags, $
                   this_wmo_id, diagnostics
;
; Checks against either world / regional records (wmo.asu.edu/) or logical limits for variables
; and masks out dubious values. Simple nested loops.
;
; Bring the range assignments up front and make them dependent upon
; the station id through a CASE statement
;
; NAME:
;     CHECK_RECORDS
;
; PURPOSE:
;     To check if there have been any exceedances of climatalogical records
;
; CALLING SEQUENCE:
;     check_records, temperatures, temperatures_fdi, temperatures_mdi, dewpoints,
;     dewpoints_mdi, dewpoints_fdi, windspeeds, windspeeds_mdi, windspeeds_fdi, slp, slp_mdi,
;     slp_fdi,  qc_flags, this_wmo_id, diagnostics
;
; INPUTS:
;     temperatures - array containing the temperature data
;
;     temperatures_mdi - missing data indicator for the temperatures
;
;     temperatures_fdi - flagged data indicator for the temperatures
;
;     dewpoints - array containing the dewpoint data
;
;     dewpoints_mdi - missing data indicator for the dewpoints
;
;     dewpoints_fdi - flagged data indicator for the dewpoints
;
;     windspeeds - array containing the windspeeds data
;
;     windspeeds_mdi - missing data indicator for the windspeed
;
;     windspeeds_fdi - flagged data indicator for the windspeed
;
;     slp - array containing the slp data
;
;     slp_mdi - missing data indicator for the slp's
;
;     slp_fdi - flagged data indicator for the slp's
;
;     qc_flags - array containing the flags from the QC run
;
;     filter_flags - array containing all flags previously set on values
;
;     this_wmo_id - value indicating the location of the station using
;                   the WMO descriptor
;
;     diagnostics - keyword to turn on diagnostics
;
; METHOD:
;     Uses built in list of climatological records for each region and
;     the Rest of the World to compare all values against and flag
;     those which exceed them.
;
;     Source for the records is http://wmo.asu.edu/
;     Temperature min/max set for each region
;     SLP - only 1 value for world, no regional differences
;     Windsp. - only 1 value for world, no regional differences.
;               Includes gust values
;     Dewpoints - no useful records found 11/11/10
;
;
; OUTPUTS:
;
;     None - arrays returned to control script
;
; RESTRICTIONS:
;
;     Needs values updating manually
;   
; HISTORY:
;      #18 - Initial version as received in October 2010 RJHD
;      #19 - Added comments.  
;            5 November 2010 RJHD
;      #40 - Added diagnostic capability - just textual
;            10 November 2010 RJHD
;      #46 - Updated Record values 
;            11 November 2010 RJHD
;      #171 - Added filter flags to pass through the total flags set
;             for each value
;             14 January 2011 RJHD
;      #237 - SVN information added to header
;             Added fdi checks to mdi checks
;             16 November 2011 RJHD
;      #325 - Adjusted record max temperatures from El Fadli et al 2012
;             12 December 2012 RJHD
;
;************************************************************************
;                    SVN Info
;$Rev:: 325                                           $:  Revision of last commit
;$Author:: rdunn                                      $:  Author of last commit
;$Date:: 2012-12-12 18:55:16 +0000 (Wed, 12 Dec 2012) $:  Date of last commit
;************************************************************************
;                                 START
;************************************************************************

;************************************************************************
; First assign which of the 7 WMO regions the station exists within. 
; Source for region assignments is http://weather.noaa.gov/tg/site.shtml
;************************************************************************


region='row'  ; default is rest of world
IF this_wmo_id GE 60000L AND this_wmo_id LE 69999L THEN region='Africa'

IF this_wmo_id GE 20000L AND this_wmo_id LE 20099L THEN region='Asia'
IF this_wmo_id GE 20200L AND this_wmo_id LE 21999L THEN region='Asia'
IF this_wmo_id GE 23000L AND this_wmo_id LE 25999L THEN region='Asia'
IF this_wmo_id GE 28000L AND this_wmo_id LE 32999L THEN region='Asia'
IF this_wmo_id GE 35000L AND this_wmo_id LE 36999L THEN region='Asia'
IF this_wmo_id GE 38000L AND this_wmo_id LE 39999L THEN region='Asia'
IF this_wmo_id GE 40350L AND this_wmo_id LE 48599L THEN region='Asia'
IF this_wmo_id GE 48800L AND this_wmo_id LE 49999L THEN region='Asia'
IF this_wmo_id GE 50000L AND this_wmo_id LE 59999L THEN region='Asia'

IF this_wmo_id GE 80000L AND this_wmo_id LE 88999L THEN region='South_America'

IF this_wmo_id GE 70000L AND this_wmo_id LE 79999L THEN region='North_America'

IF this_wmo_id GE 48600L AND this_wmo_id LE 48799L THEN region='Pacific'
IF this_wmo_id GE 90000L AND this_wmo_id LE 98999L THEN region='Pacific'

IF this_wmo_id LE 19999L THEN region='Europe'
IF this_wmo_id GE 20100L AND this_wmo_id LE 20199L THEN region='Europe'
IF this_wmo_id GE 22000L AND this_wmo_id LE 22999L THEN region='Europe'
IF this_wmo_id GE 26000L AND this_wmo_id LE 27999L THEN region='Europe'
IF this_wmo_id GE 33000L AND this_wmo_id LE 34999L THEN region='Europe'
IF this_wmo_id GE 37000L AND this_wmo_id LE 37999L THEN region='Europe'
IF this_wmo_id GE 40000L AND this_wmo_id LE 40349L THEN region='Europe'

IF this_wmo_id GE 89000L AND this_wmo_id LE 89999L THEN region='Antarctica'


;****************************************************
;  Assign the regional climate records
;      Source: 
;  In some cases these are logical limits - see below
;****************************************************

CASE region OF
  'row': BEGIN    
    ;************************************************************************
    ; Rest of world is any unknown a priori locations 
    ; that would be a pain in the proverbial to locate.
    ;************************************************************************
    temperature_max=56.7
    temperature_min=-89.2
    dewpoint_max=57.8
    dewpoint_min=-100.  ; May cut out real very low dewpoints but unlikely to be
                    ; able to measure this low with standard met equipment anyway.
    windspeed_max=113.2 ;103.3
    windspeed_min=0.
    slp_max=1083.3
    slp_min=870. 
  END
  'Africa': BEGIN
    temperature_max=55.0
    temperature_min=-23.9
    dewpoint_max=57.8
    dewpoint_min=-50.  ;  This is based upon the record min T.
    windspeed_max=113.2 ;103.3
    windspeed_min=0.
    slp_max=1083.3
    slp_min=870. 
  END
  'Asia': BEGIN
    temperature_max=53.9
    temperature_min=-67.8
    dewpoint_max=53.9
    dewpoint_min=-100.  ; May cut out real very low dewpoints but unlikely to be
                    ; able to measure this low with standard met equipment anyway.
    windspeed_max=113.2 ;103.3
    windspeed_min=0.
    slp_max=1083.3
    slp_min=870.
  END
  'South_America': BEGIN
    temperature_max=48.9
    temperature_min=-32.8
    dewpoint_max=48.9
    dewpoint_min=-60.  ; This is based upon the record min T.
    windspeed_max=113.2 ;103.3
    windspeed_min=0.
    slp_max=1083.3
    slp_min=870.
  END
  'North_America': BEGIN
    temperature_max=56.7
    temperature_min=-63.0
    dewpoint_max=56.7
    dewpoint_min=-100.  ; May cut out real very low dewpoints but unlikely to be
                    ; able to measure this low with standard met equipment anyway.
    windspeed_max=113.2 ;103.3
    windspeed_min=0.
    slp_max=1083.3
    slp_min=870.
  END
  'Pacific': BEGIN ; and Australia/NZ
    temperature_max=50.7
    temperature_min=-23.0
    dewpoint_max=50.7
    dewpoint_min=-50.  ; This is based upon the record T min
    windspeed_max=113.2 ;103.3
    windspeed_min=0.
    slp_max=1083.3
    slp_min=870.
  END
  'Europe': BEGIN
    temperature_max=48.0
    temperature_min=-58.1
    dewpoint_max=48.0
    dewpoint_min=-100.  ; May cut out real very low dewpoints but unlikely to be
                    ; able to measure this low with standard met equipment anyway.
    windspeed_max=113.2 ;103.3
    windspeed_min=0.
    slp_max=1083.3
    slp_min=870.
  END
  'Antarctica': BEGIN
    temperature_max=15.0
    temperature_min=-89.2
    dewpoint_max=15.0
    dewpoint_min=-100.  ; May cut out real very low dewpoints but unlikely to be
                    ; able to measure this low with standard met equipment anyway.
    windspeed_max=113.2 ;103.3
    windspeed_min=0.
    slp_max=1083.3
    slp_min=870.
  END
  ELSE: STOP
ENDCASE


;****************************************************
;  Find values outside of records and flag
;****************************************************

;****************************************************
; Only flags where flag should be set - old flags not wiped
;****************************************************
valid_temps=WHERE(((temperatures NE temperatures_mdi) AND (temperatures NE temperatures_fdi)), tcount)
IF tcount GE 1 THEN BEGIN
  too_high=WHERE(temperatures(valid_temps) GT temperature_max, high_count)
  IF high_count GE 1 THEN BEGIN
    qc_flags(8,valid_temps(too_high))=1
    IF KEYWORD_SET(diagnostics) THEN BEGIN
        print,"Too High - Temp"
        print,valid_temps(too_high)
    ENDIF
  ENDIF
  too_low=WHERE(temperatures(valid_temps) LT temperature_min, low_count)
  IF low_count GE 1 THEN BEGIN
    qc_flags(8,valid_temps(too_low))=1
    IF KEYWORD_SET(diagnostics) THEN BEGIN
        print,"Too Low - Temp"
        print,valid_temps(too_low)
    ENDIF
  ENDIF
ENDIF

valid_dews=WHERE(((dewpoints NE dewpoints_mdi) AND (dewpoints NE dewpoints_fdi)), dcount)
IF dcount GE 1 THEN BEGIN
  too_high=WHERE(dewpoints(valid_dews) GT dewpoint_max, high_count)
  IF high_count GE 1 THEN BEGIN
    qc_flags(9,valid_dews(too_high))=1
    IF KEYWORD_SET(diagnostics) THEN BEGIN
        print,"Too High - Dewp"
        print,valid_dews(too_high)
    ENDIF
  ENDIF
  too_low=WHERE(dewpoints(valid_dews) LT dewpoint_min, low_count)
  IF low_count GE 1 THEN BEGIN
    qc_flags(9,valid_dews(too_low))=1
    IF KEYWORD_SET(diagnostics) THEN BEGIN
        print,"Too Low - Dewp."
        print,valid_dews(too_low)
    ENDIF
  ENDIF
ENDIF

valid_winds=WHERE(((windspeeds NE windspeeds_mdi) AND (windspeeds NE windspeeds_fdi)), wcount)
IF wcount GE 1 THEN BEGIN
  too_high=WHERE(windspeeds(valid_winds) GT windspeed_max, high_count)
  IF high_count GE 1 THEN BEGIN
    qc_flags(10,valid_winds(too_high))=1
    IF KEYWORD_SET(diagnostics) THEN BEGIN
        print,"Too High - Wind"
        print,valid_winds(too_high)
    ENDIF
  ENDIF
  too_low=WHERE(windspeeds(valid_winds) LT windspeed_min, low_count)
  IF low_count GE 1 THEN BEGIN
    qc_flags(10,valid_winds(too_low))=1
    IF KEYWORD_SET(diagnostics) THEN BEGIN
        print,"Too Low - Wind"
        print,valid_winds(too_low)
    ENDIF
  ENDIF
ENDIF
 
valid_slps=WHERE(((slp NE slp_mdi) AND (slp NE slp_fdi)), scount)
IF scount GE 1 THEN BEGIN
  too_high=WHERE(slp(valid_slps) GT slp_max, high_count)
  IF high_count GE 1 THEN BEGIN
    qc_flags(11,valid_slps(too_high))=1
    IF KEYWORD_SET(diagnostics) THEN BEGIN
        print,"Too High - SLP"
        print,valid_slps(too_high)
    ENDIF
  ENDIF
  too_low=WHERE(slp(valid_slps) LT slp_min, low_count)
  IF low_count GE 1 THEN BEGIN
    qc_flags(11,valid_slps(too_low))=1
    IF KEYWORD_SET(diagnostics) THEN BEGIN
        print,"Too Low - SLP"
        print,valid_slps(too_low)
    ENDIF
  ENDIF
ENDIF


RETURN

END
;************************************************************************
;                                 END
;************************************************************************

