;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;British Crown Copyright (c) 2012, the Met Office All rights reserved.
;
;Redistribution and use in source and binary forms, with or without 
;modification, are permitted provided that the following conditions 
;are met:
;
;Redistributions of source code must retain the above copyright notice, 
;this list of conditions and the following disclaimer. 
;
;Redistributions in binary form must reproduce the above copyright 
;notice, this list of conditions and the following disclaimer in the 
;documentation and/or other materials provided with the distribution. 
;
;Neither the name of the Met Office nor the names of its contributors 
;may be used to endorse or promote products derived from this software 
;without specific prior written permission. 
;
;THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
;"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
;LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
;A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
;HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
;SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
;LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
;DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
;THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
;(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
;OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

PRO clean_up_months, last_year, times, var, var_mdi, var_fdi,qc_flag_columns, qc_flags, flagged_values, $
                     flagged_column, mask=mask, station_id, label
;
; Program reads in the times index the variable, its mdi and the relevant qc_flag subset and undertakes an assessment on a
; monthly basis as to whether to retain the data based upon flag frequency occurence for the month and the residual reporting
; frequency. Cases with less than 20 legal obs remaining in the month or more than 40% of timesteps flagged are removed.
; If either test fails then the entire month is canned.
; 
; NAME:
;     CLEAN_UP_MONTHS
;
; PURPOSE:
;     
;
; CALLING SEQUENCE:
;     clean_up_months, last_year, times, var, var_mdi, var_fdi, qc_flag_columns, qc_flags,
;     flagged_values, mask=mask, station_id, label
;
; INPUTS:
;     last_year - the final year in the data sequence
;
;     times - array containing the observation times
;
;     var - array containing the data (variable)
;
;     var_mdi - the missing data indicator for the variable
;
;     var_fdi - the flagged data indicator for the variable
;
;     qc_flag_columns - the subset of the qc_flag array which apply to
;                       the variable only
;
;     qc_flags - the full qc_flag array to write the flagging back
;                into
;
;     flagged_values - array of observation values that have been flagged
;
;     flagged_column - which column to work on in the flagged_values array
;
;     mask - keyword to turn the masking of data on
;
;     station_id - station ID for txt summary file
;
;     label - label for txt summary file
;
; METHOD:
;     Compares the number of valid readings with the number of flags.
;     Too few readings or too high a proportion of flags are removed.
;
; OUTPUTS:
;
;     None - files written to specified directory
;
; RESTRICTIONS:
;
;     Assumes netcdf file inputs.
;   
; HISTORY:
;     #60 - Code as received in October 2010
;           15 November 2010 RJHD
;     #139 - Any month flagged by this test is output to a file for
;            later eyeballing if desired.
;           8 December 2010 RJHD
;     #237 - SVN information added to header
;            16 November 2011 RJHD
;     #238 - Added fdi functionality.
;            18 November 2011 RJHD
;
;************************************************************************
;                    SVN Info
;$Rev:: 308                                           $:  Revision of last commit
;$Author:: rdunn                                      $:  Author of last commit
;$Date:: 2012-07-04 14:47:15 +0100 (Wed, 04 Jul 2012) $:  Date of last commit
;************************************************************************
;                                 START
;************************************************************************
cd, "./", current=pwd


;**************************************************************
; Find the number of columns which are being assessed and the total
; number of flags per time interval.
;**************************************************************
nlive_qc_flags=N_ELEMENTS(qc_flag_columns)-1
qctotals=INTARR(N_ELEMENTS(times))
FOR t=0L,N_ELEMENTS(times)-1 DO qctotals(t)=TOTAL(qc_flags(qc_flag_columns(0:nlive_qc_flags-1),t))

;**************************************************************
; Full keyword used, but full_times not sent to match the form of the
; code as it was when received.
;**************************************************************
MONTH_SETUP,last_year,nmonths,nyears,times,times,monthly_segment_starts,$
  monthly_segment_ends, monthly_segment_lengths,/full


;**************************************************************
; Now proceed on a month by month basis.
;**************************************************************
FOR seg=0,nmonths-1 DO BEGIN
    ;**************************************************************
    ; Are there any time points within this calendar month for which any variables were archived?
    ;   combination of N>start and N<end
    ;**************************************************************
    tpoints=WHERE(times GE monthly_segment_starts(seg), cnt1)
    IF cnt1 GE 1 THEN tpoints2=WHERE(times(tpoints) LE monthly_segment_ends(seg),ntimematches)

    IF ntimematches GE 1 THEN BEGIN
        tempvar_array=var(tpoints(tpoints2))
        tempqc_array=qctotals(tpoints(tpoints2))
        points=WHERE(((tempvar_array NE var_mdi) AND (tempvar_array NE var_fdi)), varcnt)

        ;**************************************************************
        ; Are there any recorded values?
        ;**************************************************************
        IF varcnt GE 1 THEN BEGIN
            IF varcnt LE 19 THEN BEGIN
                ;**************************************************************
                ; Too few data in this month (<20) so apply all as missing.
                ;**************************************************************
                IF KEYWORD_SET(mask) THEN BEGIN
                   flagged_values[flagged_column,tpoints(tpoints2(points))]=var(tpoints(tpoints2(points)))
                   var(tpoints(tpoints2(points)))=var_fdi
                ENDIF
                qc_flags(qc_flag_columns(nlive_qc_flags),tpoints(tpoints2(points)))=1

            ENDIF ELSE BEGIN
                ;**************************************************************
                ; What proportion have a QC flag value that is non-zero?
                ;**************************************************************
                qctot=WHERE(tempqc_array GT 0.,qccount)

                proportion=FLOAT(qccount)/FLOAT(varcnt+qccount)

                IF proportion GE 0.4 THEN BEGIN 
                    ;**************************************************************
                    ; increased from 0.2 to 0.4 22/09/09 KATE - too much removal or good data
                    ;**************************************************************
                    ; Too many flags for this specific variable in this month so apply all as missing.
                    ;**************************************************************
                    IF KEYWORD_SET(mask) THEN BEGIN
                       flagged_values[flagged_column,tpoints(tpoints2(points))]=var(tpoints(tpoints2(points)))
                       var(tpoints(tpoints2(points)))=var_fdi
                    ENDIF
                    qc_flags(qc_flag_columns(nlive_qc_flags),tpoints(tpoints2(points)))=1

                    ;**************************************************************
                    ; also want to note these data for future eyeballing
                    ;**************************************************************
                    OPENW,99,pwd+'/spurious_stations.txt',/append 
                    year=((seg+1)/12+1973)
                    month=(seg+1)-12*((seg+1)/12)
                    PRINTF,99,'Clean Up Months',station_id,year,month,label,'    Large proportion of flags - month cleaned up',FORMAT='(A20,A20,I10,I4,A20,A)'
                    CLOSE,99

                    

                ENDIF
            ENDELSE
        ENDIF
    ENDIF
ENDFOR
new_flags=WHERE(qc_flags(qc_flag_columns(nlive_qc_flags)) GT 0,bad_count)
printstring="   Clean Up Months Flags"
PRINT,printstring, bad_count, FORMAT='(A-38,I8)'

RETURN

END

;************************************************************************
;                                 END
;************************************************************************
