;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;British Crown Copyright (c) 2012, the Met Office All rights reserved.
;
;Redistribution and use in source and binary forms, with or without 
;modification, are permitted provided that the following conditions 
;are met:
;
;Redistributions of source code must retain the above copyright notice, 
;this list of conditions and the following disclaimer. 
;
;Redistributions in binary form must reproduce the above copyright 
;notice, this list of conditions and the following disclaimer in the 
;documentation and/or other materials provided with the distribution. 
;
;Neither the name of the Met Office nor the names of its contributors 
;may be used to endorse or promote products derived from this software 
;without specific prior written permission. 
;
;THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
;"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
;LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
;A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
;HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
;SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
;LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
;DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
;THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
;(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
;OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

PRO CLIMATOLOGICAL_CHECK, last_year, times, full_times, h_times, full_h_times, values, full_values, $
                          mdi, fdi, columns, qc_flags, filter_flags, $
                          diagnostics, station_id, flagged_values, mask=mask, oldflags=oldflags
;
; Program undertakes a climatological check that removes individual obs greater
; than 10 inter-quartile range pseudo-sigma values from the median value for that hour and
; month 

; NAME:
;     CLIMATOLOGICAL_CHECK
;
; PURPOSE:
;     To check for observations a large distance from the
;     climatological mean.
;
; CALLING SEQUENCE:
;     streak_check, last_year, times, values, mdi, type, qc_flags, full_times,
;     full_values, diagnostics, station_id,
;     mask=mask, oldflags=oldflags 
;
; INPUTS:
;     last_year - keyword to set the last year of the observations
;
;     times - array containing the times of the observations in
;             fractions of a day
;
;     full_times - array containing the uncompressed times of the
;                  observations in fractions of a day
;
;     h_times - array containing the times of the observations in hours
;
;     full_h_times - array containing the uncompressed times of the
;                    observations in hours
;
;     values - array containing the values of the observations
;
;     full_values - array containing the uncompressed values of the observations
;
;     mdi - missing data indicator
;
;     fdi - flagged data indicator
;
;     columns - integer describing which column of qc_flags should be used.
;
;     qc_flags - array containing the values of the flags for the station
;
;     filter_flags - array containing all flags previously set on values
;
;     diagnostics - keyword for running diagnostic tests
;
;     station_id - station identifier for the diagnostic plot names
;
;     flagged_values - array containing observation values which have
;                      been flagged
;
;     mask - keyword to set whether masking is applied
;
;     oldflags - keyword to calculate the number of streak check flags
;                already set
;
; METHOD:
;     Finds average climatology for each hour from the filtered data.  Then winsorises
;     (truncate distribution at 5 and 95% and then set values which
;     exceed this to the 5 or 95% value).  Finds mean, anomalises and
;     finds the interQ range to normalise the unfiltered data.  Uses a low-pass filter to
;     average out any yearly affect.  
;
;     Two tests are run.  Firstly, gaps are searched for in the
;     distribution which are further from the median than where the
;     fitted Gaussian crosses the y=0.1 line.  Any points
;     further from the median than the gap are flagged.  Secondly, any
;     points between where the Gaussian crosses the line and any gap
;     (if found) are tentatively flagged, and these flags can be
;     removed using the neighbour check. 
;
;     FIRM FLAG - 1
;     TENTATIVE FLAG - 2
;
; OUTPUTS:
;
;     None - arrays returned to control script
;
; RESTRICTIONS:
;
;
;   
; HISTORY:
;      #21 - Initial version as received in October 2010 RJHD
;      #24 - Commented, adjusted to run once per variable.  Also uses
;            the EXECUTE function to set up repetetive names.  Takes a
;            copy of full_values and re-inserts that at the end.
;            8 November 2010 RJHD
;      #33 - updated reporting_accuracy call
;            9 November 2010 RJHD
;      #40 - added diagnostic plotting capability
;            10 November 2010 RJHD
;      #50 - updated diagnostic plots
;            15 November 2010 RJHD
;      #61 - updated the low pass filter to avoid division by Zero
;            17 November 2010 RJHD
;      #81 - Added mask keyword
;            18 November 2010 RJHD
;      #92 - Overplot of gaussian fit to histogram
;            19 November 2010 RJHD
;      #95 - Adjusted winsorising so it works properly and a monthly
;            histogram output.
;            25 November 2010 RJHD
;      #105 - Changed flagging routine - now will only flag if there
;             is a gap in the distribution > 10 IQR out, like
;             gap_check.pro.  Old code kept in at end - set with hard
;                                                       wired switch.
;            29 November 2010 RJHD
;      #148 - Updated flagging routine, during which inconsistency
;             found in #105 version.  That version was to find a gap,
;             and flag anything  >10IQR.  However, if the gap was
;             >10IQR from the main dist. and the main dist. spanned
;             10IQR, it still flagged from 10.  This version looks for
;             the gap (now need only be one bin wide), if this occurs
;             more than 7IQR from the median, then flag. Each tail
;             done separately  Plotting updated.
;             10 December 2010 RJHD
;      #164 - Method section of the header updated.
;             7 January 2011 RJHD
;      #171 - Added filter flags to pass through the total flags set
;             for each value
;             14 January 2011 RJHD
;      #173 - Used filtered values to calculate anomalies, but
;             then flagged on unfiltered ones.
;             17 January 2011 RJHD
;      #174 - Added oldflags keyword to allow the calculation of the
;             previously set flags.
;             18 January 2011 RJHD
;      #200 - changed last_year from optional keyword to required input
;             to remove the hardwiring of the end date
;             11 March 2011 RJHD
;      #229 - Improved labelling of diagnostic histogram
;             28 September 2011 RJHD
;      #237 - SVN information added to header
;             Added fdi and flagged_values array functionality
;             16 November 2011 RJHD
;      #243 - Swapped numerical values of firm and tentative flags
;             21 November 2011 RJHD
;      #295 - Tested for a dividing by zero error in low-pass filter
;             section 
;             19 March 2012 RJHD
;      #299 - Added hourly times to do matching and changed arrays
;             from floats to doubles
;             19 March 2012 RJHD
;
;
;************************************************************************
;                    SVN Info
;$Rev:: 308                                           $:  Revision of last commit
;$Author:: rdunn                                      $:  Author of last commit
;$Date:: 2012-07-04 14:47:15 +0100 (Wed, 04 Jul 2012) $:  Date of last commit
;************************************************************************
;                                 START
;************************************************************************
CASE columns OF
    24: BEGIN
        type=0
        label="temperatures"
        flagged_col=0
    END

    25: BEGIN
        type=1
        label="dewpoints"
        flagged_col=1
    END
ENDCASE

IF KEYWORD_SET(oldflags) THEN BEGIN
    prev_flags=WHERE(qc_flags(columns,*) GT 0, pflcnt)
ENDIF



;**************************************************************
; Obtain the reporting accuracy of the instrument
;**************************************************************

;*************************************************************
; Firstly ascertain the *typical* recording precision based upon the prevalence of given 
; decimal points within the legal values of the series for the recorded parameters. This then 
; has an implication for the critical value derivation later on
;*************************************************************

REPORTING_ACCURACY, last_year, values, mdi, fdi, vres, remhist, times, vfreq


;**************************************************************
; Set up the monthly values
;**************************************************************


MONTH_SETUP,last_year,nmonths,nyears,times,full_times,monthly_segment_starts,$
  monthly_segment_ends, monthly_segment_lengths,/full


monthly_segment_lengths(nmonths-1)=744
monthly_segment_ends(nmonths-1)=monthly_segment_ends(nmonths-1)+1


monthly_segment_starts=REFORM(monthly_segment_starts,12,nyears)
monthly_segment_ends=REFORM(monthly_segment_ends,12,nyears)
monthly_segment_lengths=REFORM(monthly_segment_lengths,12,nyears)

;**************************************************************
; Outer month loop - minimise space by considering months in turn for creating a normalised
; index set.
;**************************************************************

;**************************************************************
; Set up a duplicate of full_values - one needed for script and is
;                                     amended during the run
;**************************************************************
untouched_full_values=full_values
filtered_full_values=full_values

;**************************************************************
; Mask values with past flags
;**************************************************************
MATCH, h_times, full_h_times, suba, subb

past_flags=WHERE(filter_flags GT 0, pfcnt)

IF pfcnt GE 1 THEN BEGIN
    filtered_full_values(subb(past_flags))=mdi
ENDIF

;*****************************************
; initialise a new flagging array
;*****************************************
qc_flags_climate=INTARR(n_elements(full_values))
qc_flags_climate(*)=0

FOR m=0,11 DO BEGIN
   ;**************************************************************
   ; Create the set of climatologies
   ;**************************************************************

    vclims=DBLARR(24)

    FOR hr=0,23 DO BEGIN
        ;**************************************************************
        ; Use EXECUTE to run through all hours and set up climatologies
      ;**************************************************************
          clim_mdi=STRING(hr,FORMAT='("vclm",I2.2,"=mdi")')
        Exec_run=EXECUTE(clim_mdi)
    ENDFOR
    
    nobs=FLTARR(24)
    
    FOR yr=0,nyears-1 DO BEGIN
        ;**************************************************************
        ; Use filtered values to create hourly climatology
        ;**************************************************************
        ndays=monthly_segment_lengths(m,yr)/24
        temparr=filtered_full_values(monthly_segment_starts(m,yr):monthly_segment_ends(m,yr))
        
        temparr=REFORM(temparr,24,ndays)
        
        FOR hr=0,23 DO BEGIN
            ;**************************************************************
            ; Use read in hourly values into appropriate array.
            ;**************************************************************
            FMT1="points=WHERE(((temparr("
            FMT2=",*) NE mdi) AND (temparr("
            FMT3=",*) NE fdi)), vpc)"
            
            clim_mdi=STRING(FMT1,hr,FMT2,hr,FMT3,FORMAT='(A23,I2,A26,I2,A20)')
            IF KEYWORD_SET(diagnostics) and 1 eq 0  THEN print,clim_mdi
            Exec_run=EXECUTE(clim_mdi)

            FMT1="IF vpc GE 1 THEN vclm"
            FMT2="=[vclm"
            FMT3=",REFORM(temparr("
            FMT4=",points),vpc)]"

            clim_mdi=STRING(FMT1,hr,FMT2,hr,FMT3,hr,FMT4,FORMAT='(A21,I2.2,A6,I2.2,A16,I2,A14)')
            IF KEYWORD_SET(diagnostics) and 1 eq 0  THEN print,clim_mdi
            Exec_run=EXECUTE(clim_mdi)



        ENDFOR
        
    ENDFOR
    FOR hr=0,23 DO BEGIN
        
        FMT1="IF(hr EQ "
        FMT2=") THEN subarr=vclm"
        FMT3="(SORT(vclm"
        FMT4="))"

        clim_mdi=STRING(FMT1,hr,FMT2,hr,FMT3,hr,FMT4,FORMAT='(A9,I2,A18,I2.2,A10,I2.2,A2)')
        IF KEYWORD_SET(diagnostics) and 1 eq 0 THEN print,clim_mdi
        Exec_run=EXECUTE(clim_mdi)

        IF (n_elements(subarr) GT 100) THEN BEGIN
            ;**************************************************************
            ; calculate the 5% and 95% percentile limits
            ; Winsorise (set values <5% and >95% to the 
            ; 5% and 95% values and add to total climatologies
            ; array
            ;**************************************************************
            pct5=FIX(FLOOR(((n_elements(subarr)-1)/100.)*5.))
            pct95=FIX(FLOOR(((n_elements(subarr)-1)/100.)*95.))
            ;**************************************************************
            ; old version, pre winsorising
            ;vclims(hr)=TOTAL(subarr(pct5+1:pct95+1))/n_elements(subarr(pct5+1:pct95+1))  ; a "mean"-like value
            ;**************************************************************
            ; Proper Winsorising code 25/11/10 RJHD
            ;**************************************************************
            subarr(0:pct5+1)=subarr(pct5+1)
            subarr(pct95+1:n_elements(subarr)-1)=subarr(pct95+1)
            vclims(hr)=TOTAL(subarr)/(n_elements(subarr)-1)

        ENDIF ELSE vclims(hr)=mdi
    ENDFOR

   ;**************************************************************
   ; Anomalise
   ;**************************************************************

    allvs=[0] ; Arrays to contain full set of anomalies in each parameter. Saves a further loop!
  
    FOR yr=0,nyears-1 DO BEGIN
        ndays=monthly_segment_lengths(m,yr)/24
        ;**************************************************************
        ; subtract the filtered mean from the non-filtered values
        ;**************************************************************
        temparr_F=filtered_full_values(monthly_segment_starts(m,yr):monthly_segment_ends(m,yr))
        temparr  =         full_values(monthly_segment_starts(m,yr):monthly_segment_ends(m,yr))

        temparr_F=REFORM(temparr,24,ndays)
        temparr=REFORM(temparr,24,ndays)
        FOR hr=0,23 DO BEGIN
            points=WHERE(((temparr_F(hr,*) NE mdi) AND (temparr_F(hr,*) NE fdi)), vpc)
            IF (vpc GE 1) AND (vclims(hr) NE mdi) THEN BEGIN ; check there are no dodgey things due to sub hourly reporting frequency
                temparr(hr,points)=temparr(hr,points)-vclims(hr)
                temparr_F(hr,points)=temparr_F(hr,points)-vclims(hr)
                vt=temparr(hr,points)
                vt=REFORM(vt,vpc)
                allvs=[allvs,vt]
            ENDIF ELSE BEGIN
               temparr(hr,*)=mdi
               temparr_F(hr,*)=mdi
            ENDELSE
        ENDFOR
        
        filtered_full_values(monthly_segment_starts(m,yr):monthly_segment_ends(m,yr))=temparr_F
        full_values(monthly_segment_starts(m,yr):monthly_segment_ends(m,yr))=temparr

    ENDFOR


   ;**************************************************************
   ; Ascertain the inter-quartile ranges in anomaly space - these are robust to outliers anyway so no need to winsorise
   ;**************************************************************
 
    count=N_ELEMENTS(allvs)
    IF count GE 10 THEN BEGIN 
        rnk_tmp=SORT(allvs)
        allvs=allvs(rnk_tmp)
        vinterq=(allvs(ROUND(3.*FLOAT(count)/4.))-allvs(ROUND(FLOAT(count)/4.)))/2.
        IF vinterq LE 1.5 THEN vinterq=1.5 ; Added as 1 deg to avoid problems in very stable sites 03/09/09, changed to 2deg 22/09/09
    ENDIF ELSE vinterq=-999.


   ;**************************************************************
   ; Normalise by inter-q range, and save the monthly averages for developing a simple filter
   ;**************************************************************
    monthly_vqvs=DBLARR(nyears)
    
    FOR yr=0,nyears-1 DO BEGIN
        ;**************************************************************
        ; divide the non-filtered values by the filtered IQR
        ;**************************************************************
        ndays=monthly_segment_lengths(m,yr)/24
        temparr_F=filtered_full_values(monthly_segment_starts(m,yr):monthly_segment_ends(m,yr))
        temparr  =         full_values(monthly_segment_starts(m,yr):monthly_segment_ends(m,yr))

        temparr_F=REFORM(temparr,24,ndays)
        temparr=REFORM(temparr,24,ndays)
        FOR hr=0,23 DO BEGIN
            points=WHERE(((temparr_F(hr,*) NE mdi) AND (temparr_F(hr,*) NE fdi)), vpc)
            IF (vpc GE 1) AND (vinterq NE -999.) THEN BEGIN
                temparr_F(hr,points)=temparr_F(hr,points)/vinterq
                temparr(hr,points)=temparr(hr,points)/vinterq
            ENDIF
        ENDFOR
        filtered_full_values(monthly_segment_starts(m,yr):monthly_segment_ends(m,yr))=temparr_F
        full_values(monthly_segment_starts(m,yr):monthly_segment_ends(m,yr))=temparr

        pts=WHERE(((temparr NE mdi) AND (temparr NE fdi)), betty)
        IF betty GE 1 THEN monthly_vqvs(yr)=MEDIAN(temparr_F(pts))
        
     ENDFOR

    ;**************************************************************
    ; Now have an array of values anomalised by the median and IQR of the
    ;  filtered values.  The rest of the analysis is now done on these 
    ;  unfiltered but anomalised-by-the-filtered-median-&-IQR values
    ;**************************************************************

    ;**************************************************************
    ; Derive and apply a low-pass filter to take into account trends across the series
    ;**************************************************************
    filterweights=[1.,2.,3.,2.,1.]
    FOR yr=0,nyears-1 DO BEGIN
        CASE yr OF
            0: BEGIN
                if TOTAL(ABS(monthly_vqvs(2:4))) NE 0 THEN BEGIN
                    IF TOTAL(filterweights(2:4)*CEIL(monthly_vqvs(0:2)-FLOOR(monthly_vqvs(0:2)))) EQ 0 THEN BEGIN
                       temp=0
                    ENDIF ELSE BEGIn
                       temp=TOTAL(filterweights(2:4)*monthly_vqvs(0:2))/TOTAL(filterweights(2:4)*CEIL(monthly_vqvs(0:2)-FLOOR(monthly_vqvs(0:2)))) 
                    ENDELSE
                    ; should this divisor be made appropriate to howmany present data points there are?	 
                    temparr=full_values(monthly_segment_starts(m,yr):monthly_segment_ends(m,yr))
                    points=WHERE(((temparr NE mdi) AND (temparr NE fdi)), cnt)
                    IF cnt GE 1 THEN temparr(points)=temparr(points)-temp
                    full_values(monthly_segment_starts(m,yr):monthly_segment_ends(m,yr))=temparr
                ENDIF
            END


            1: BEGIN
                if TOTAL(ABS(monthly_vqvs(0:3))) NE 0 THEN BEGIN
                    IF TOTAL(filterweights(1:4)*CEIL(monthly_vqvs(0:3)-FLOOR(monthly_vqvs(0:3)))) EQ 0 THEN BEGIN
                       temp=0
                    ENDIF ELSE BEGIN
                       temp=TOTAL(filterweights(1:4)*monthly_vqvs(0:3))/TOTAL(filterweights(1:4)*CEIL(monthly_vqvs(0:3)-FLOOR(monthly_vqvs(0:3))))
                    ENDELSE
                    ; should this divisor be made appropriate to howmany present data points there are?	 
                    temparr=full_values(monthly_segment_starts(m,yr):monthly_segment_ends(m,yr))
                    points=WHERE(((temparr NE mdi) AND (temparr NE fdi)), cnt)
                    IF cnt GE 1 THEN temparr(points)=temparr(points)-temp
                    full_values(monthly_segment_starts(m,yr):monthly_segment_ends(m,yr))=temparr
                ENDIF
            END
            
            
            nyears-2: BEGIN
                if TOTAL(ABS(monthly_vqvs(nyears-4:nyears-1))) NE 0 THEN BEGIN
                    IF TOTAL(filterweights(0:3)*CEIL(monthly_vqvs(nyears-4:nyears-1)-FLOOR(monthly_vqvs(nyears-4:nyears-1)))) EQ 0 THEN BEGIN
                       temp=0
                    ENDIF ELSE BEGIN 
                       temp=TOTAL(filterweights(0:3)*monthly_vqvs(nyears-4:nyears-1))/TOTAL(filterweights(0:3)*CEIL(monthly_vqvs(nyears-4:nyears-1)-FLOOR(monthly_vqvs(nyears-4:nyears-1))))
                    ENDELSE
                    ; should this divisor be made appropriate to howmany present data points there are?	 
                    temparr=full_values(monthly_segment_starts(m,yr):monthly_segment_ends(m,yr))
                    points=WHERE(((temparr NE mdi) AND (temparr NE fdi)), cnt)
                    IF cnt GE 1 THEN temparr(points)=temparr(points)-temp
                    full_values(monthly_segment_starts(m,yr):monthly_segment_ends(m,yr))=temparr
                ENDIF
            END
            
            
            nyears-1: BEGIN
                if TOTAL(ABS(monthly_vqvs(nyears-3:nyears-1))) NE 0 THEN BEGIN
                    IF TOTAL(filterweights(0:2)*CEIL(monthly_vqvs(nyears-3:nyears-1)-FLOOR(monthly_vqvs(nyears-3:nyears-1)))) EQ 0 THEN BEGIN
                       temp=0
                    ENDIF ELSE BEGIN
                       temp=TOTAL(filterweights(0:2)*monthly_vqvs(nyears-3:nyears-1))/TOTAL(filterweights(0:2)*CEIL(monthly_vqvs(nyears-3:nyears-1)-FLOOR(monthly_vqvs(nyears-3:nyears-1))))
                    ENDELSE
                    ; should this divisor be made appropriate to howmany present data points there are?	 
                    temparr=full_values(monthly_segment_starts(m,yr):monthly_segment_ends(m,yr))
                    points=WHERE(((temparr NE mdi) AND (temparr NE fdi)), cnt)
                    IF cnt GE 1 THEN temparr(points)=temparr(points)-temp
                    full_values(monthly_segment_starts(m,yr):monthly_segment_ends(m,yr))=temparr
                ENDIF
            END
            
            
            ELSE: BEGIN
                if TOTAL(ABS(monthly_vqvs(yr-2:yr+2))) NE 0 THEN BEGIN
                    IF TOTAL(filterweights*CEIL(monthly_vqvs(yr-2:yr+2)-FLOOR(monthly_vqvs(yr-2:yr+2)))) EQ 0 THEN BEGIN
                       temp=0
                    ENDIF ELSE BEGIN
                       temp=TOTAL(filterweights*monthly_vqvs(yr-2:yr+2))/TOTAL(filterweights*CEIL(monthly_vqvs(yr-2:yr+2)-FLOOR(monthly_vqvs(yr-2:yr+2))))
                    ENDELSE
                    ; should this divisor be made appropriate to howmany present data points there are?	 
                    temparr=full_values(monthly_segment_starts(m,yr):monthly_segment_ends(m,yr))
                    points=WHERE(((temparr NE mdi) AND (temparr NE fdi)), cnt)
                    IF cnt GE 1 THEN temparr(points)=temparr(points)-temp
                    full_values(monthly_segment_starts(m,yr):monthly_segment_ends(m,yr))=temparr
                ENDIF

            END
            
            
         ENDCASE
 
    ENDFOR 

    ;*****************************************
    ; New section to check for gaps in the distribution
    ; as well as separation from the median
    ;*****************************************
    gap_check=1
    IF gap_check EQ 1 THEN BEGIN

        monthly_values=DBLARR(744,nyears)
        monthly_values(*)=mdi

        ;*****************************************
        ; Read in the values for a given month
        ;  for all years observed
        ;*****************************************
        FOR yr=0,nyears-1 DO BEGIN
            temp_values=full_values(monthly_segment_starts(m,yr):monthly_segment_ends(m,yr))
            monthly_values(0:N_ELEMENTS(temp_values)-1,yr)=temp_values
        ENDFOR

        month_mdi_test=WHERE(((monthly_values NE mdi) AND (monthly_values NE fdi)), mdicnt)

        IF mdicnt GE 1 THEN BEGIN

            monthly_values=monthly_values(month_mdi_test)

            ;*****************************************
            ; outside of diagnostics as used to initiate counting for gap check
            ;*****************************************
            min_val=FLOOR(MIN(monthly_values))-1 ; just in case it is negative and doesn't work
            max_val=CEIL(MAX(monthly_values))

            binsize=1.0
            hist = HISTOGRAM(monthly_values, BINSIZE=binsize,LOCATION=loc, MAX=max_val, MIN=min_val)
            hist=[0,0,0,temporary(hist),0,0,0] 
                
            ;******************************************
            ; adjust for padding in histogram
            ;******************************************
            min_bin=MIN(loc)
            max_bin=MAX(loc)
            binns=[min_bin-(3.*binsize),min_bin-(2.*binsize),min_bin-binsize,loc,max_bin+binsize,max_bin+(2.*binsize),max_bin+(3.*binsize)]+(binsize/2.)

            ;******************************************
            ; fit gaussian to histogram
            ;******************************************
            yfit = GAUSSFIT(binns, hist, coeff, NTERMS=3, ESTIMATES=[max(hist),0.,1.5],chisq=chi_sq)

            ;*****************************************
            ; Plot the histogram of the distribution of 
            ; all observations (anomalised) over all years
            ; for this one month
            ;*****************************************
            IF KEYWORD_SET(diagnostics) THEN BEGIN
                mdi_loc=WHERE(((monthly_values NE mdi) AND (monthly_values NE fdi)), mc)
                IF mc GT 1 THEN BEGIN
                    
                    set_plot,'PS'
                    cd, "./", current=outdirplot
                    device,filename=outdirplot+'/images/'+station_id+'_Climatological_'+label+'_'+strcompress(m+1,/remove_all)+'.eps'$
                      ,/color,/encapsulate,/helvetica,/bold,/landscape,ysize=20,xsize=26
                    !P.Font=0
                    !P.Multi=[0,1,1]
                    !P.Thick=4
                    plotsym,3,1.2
                    PLOT,binns,hist,min_value=-100,xstyle=1,ystyle=1,ytitle="Frequency",xtitle=label+' offset (IQR)',$
                      title="Climatological Check - Diagnostic Histogram - "+label+' - month '+strcompress(m+1,/remove_all),charsize=1,ythick=4,xthick=4,psym=10,YLog=1,YRange=[0.1,max(hist)*5], POSITION=[0.10, 0.10, 0.90, 0.90]
                                ;******************************************
                                ; overplot fitted gaussian
                                ;******************************************
                    OPLOT, binns, yfit, COLOR=40, THICK=2
                ENDIF
            ENDIF
            
            frequency_threshold=0.1
        
               
            ;**************************************************************
            ; Find any points further from the median than where the gaussian 
            ;   crosses the y=0.1 line, rounded up to next whole integer
            ;**************************************************************
            minimum_threshold=ROUND(1.+coeff[1]+(coeff[2]*sqrt(-2.* ALOG(frequency_threshold/coeff[0]))))
            
            upper_loc=WHERE((monthly_values) GT minimum_threshold, uppercount)
            lower_loc=WHERE((monthly_values) LT -minimum_threshold, lowercount)
            largecount=uppercount+lowercount
            
            clean_lim=-10
        
            ;**************************************************************
            ; size of the gap to notice
            ;**************************************************************
            gap_limit=1
            
            ;*****************************************
            ; Initially we wanted to flag all months where such an outlier
            ; has occurred, but currrently just flagging the outliers.
            ;  The initial idea would be very harsh
            ;*****************************************
            IF (largecount GE 1) THEN BEGIN

                ;*****************************************
                ; Check if there is a gap in the distribution between the
                ; median and any potential outliers
                ; Use any gap that is found to determine the threshold value
                ; Run for each tail separately
                ;*****************************************
                max_value=MAX(hist,start_point)

                goodupdist='yes'
                n=0
                upper_gap_cnt=0
                upper_first_gap=0
        
                WHILE goodupdist EQ 'yes' DO BEGIN
                    
                    n=n+1
                    IF start_point+n LE n_elements(hist)-1 THEN BEGIN
                        IF (hist[start_point+n] EQ 0) THEN BEGIN
                            upper_gap_cnt=upper_gap_cnt+1
                            
                            IF  ((upper_first_gap EQ 0) AND (binns[start_point+n]+(binsize/2.) GE minimum_threshold)) THEN BEGIN
                                upper_first_gap=binns[start_point+n]+(binsize/2.)
                            ENDIF
                        ENDIF
                        ;*****************************************
                        ; wipe the gap if it is too small
                        ;*****************************************
                        IF ((hist[start_point+n] NE 0) AND (upper_gap_cnt LT gap_limit))THEN upper_gap_cnt=0
                    ENDIF

                    ;*****************************************
                    ; test if the gap is large enough and there's data
                    ; beyond it and we haven't gone past that data 
                    ; by wiping a small gap
                    ;*****************************************
                    IF start_point+n LE n_elements(hist)-1 THEN BEGIN
                        IF ((hist(start_point+n) NE 0) AND (upper_first_gap NE 0) AND (upper_gap_cnt GE gap_limit) AND (uppercount GE 1)) THEN goodupdist='no'
                    ENDIF
                
                    ;*****************************************
                    ; Escape clause
                    ;*****************************************
                    IF start_point+n EQ N_ELEMENTS(hist)-1 THEN BREAK
                ENDWHILE
            
                goodlodist='yes'
                n=0
                lower_gap_cnt=0
                lower_first_gap=0
                WHILE goodlodist EQ 'yes' DO BEGIN
                    
                    n=n+1
                    IF start_point-n GE 0 THEN BEGIN
                        IF (hist[start_point-n] EQ 0) THEN BEGIN
                            lower_gap_cnt=lower_gap_cnt+1
                            IF ((lower_first_gap EQ 0) AND (binns[start_point-n]-(binsize/2.) LE -minimum_threshold)) THEN BEGIN
                                lower_first_gap=binns[start_point-n]-(binsize/2.)
                            ENDIF
                        ENDIF
                        ;*****************************************
                        ; wipe the gap if it is too small
                        ;*****************************************
                        IF ((hist[start_point-n] NE 0) AND (lower_gap_cnt LT gap_limit)) THEN lower_gap_cnt=0
                    ENDIF
                
                    ;*****************************************
                    ; test if the gap is large enough and there's data
                    ; beyond it and we haven't gone past that data
                    ; by wiping a small gap
                    ;*****************************************
                
                    IF start_point-n GE 0 THEN BEGIN
                        IF ((hist(start_point-n) NE 0) AND (lower_first_gap NE 0) AND (lower_gap_cnt GE gap_limit) AND (lowercount GE 1)) THEN goodlodist='no'
                    ENDIF
                    ;*****************************************
                    ; Escape clause
                    ;*****************************************
                    IF start_point EQ n THEN BREAK
                ENDWHILE
            

           
                ;*****************************************
                ; If the distribution has no gaps, currently just highlight
                ;  where the IQR limit is 10 DEC 2010 RJHD
                ;*****************************************
                ;IF (goodupdist EQ 'yes' AND KEYWORD_SET(diagnostics)) THEN BEGIN
                ;    IF (uppercount GE 1) THEN oplot,[minimum_threshold,minimum_threshold],[0.1,max(hist)*5],psym=0,color=140,YLog=1, linestyle=1    
                ;ENDIF 
                ;IF (goodlodist EQ 'yes' AND KEYWORD_SET(diagnostics)) THEN BEGIN
                ;    IF (lowercount GE 1) THEN oplot,[-minimum_threshold,-minimum_threshold],[0.1,max(hist)*5],psym=0,color=140,YLog=1, linestyle=1
                ;ENDIF 
                
                ;*****************************************
                ; Loop through both tails to check, flag and plot
                ;*****************************************

                FOR portion=1,2 DO BEGIN
                    
                    CASE portion OF
                        1: BEGIN
                            dist=goodupdist
                            threshold=minimum_threshold
                            ;*****************************************
                            ; If a gap has been found in upper tail
                            ;*****************************************
                            IF (dist EQ 'no')  THEN BEGIN
                                gap_cnt=upper_gap_cnt
                                portion_cnt=uppercount
                                first_gap=upper_first_gap
                                gap_plot_values=DBLARR(744,nyears)
                                gap_plot_values(*)=mdi
                            ENDIF
                        END
                        2: BEGIN
                            dist=goodlodist
                            threshold=-minimum_threshold
                            ;*****************************************
                            ; If a gap has been found in lower tail
                            ;*****************************************
                            IF (dist EQ 'no') THEN BEGIN
                                gap_cnt=lower_gap_cnt
                                portion_cnt=lowercount
                                first_gap=lower_first_gap
                                gap_plot_values=DBLARR(744,nyears)
                                gap_plot_values(*)=mdi
                            ENDIF
                        END
                    ENDCASE
                    
                    plot_values=DBLARR(744,nyears)
                    plot_values(*)=mdi
                    
                    FOR yr=0,nyears-1 DO BEGIN

                        ;*****************************************
                        ; Need to convert between the full and normal
                        ; set of values for the plotting and flagging
                        ; run for each wing separately
                        ;*****************************************
                        year_values=full_values(monthly_segment_starts(m,yr):monthly_segment_ends(m,yr))
                        year_flags=qc_flags_climate(monthly_segment_starts(m,yr):monthly_segment_ends(m,yr))
                        
                        mdi_test=WHERE(((year_values ne mdi) AND (year_values ne fdi)), mdi_cnt)
                        
                        IF mdi_cnt GE 1 THEN BEGIN
                            cleaned_values=year_values(WHERE(((year_values ne mdi) AND (year_values ne fdi)), mdi_cnt))
                            cleaned_flags=year_flags(WHERE((year_values ne mdi) AND (year_values ne fdi)))
                            
                            ;*****************************************
                            ; 1) Flag all which are > threshold value away
                            ;    from the median [TENTATIVE] flag=2
                            ; 2) If a gap has been found, then flag for
                            ;    values beyond the gap. [FIRM] flag=1
                            ; Previous version was flagging >10IQR if it
                            ; found a gap, even if the gap was located 
                            ; beyond 10IQR, and there were bins part of 
                            ; the main distribution which were located at 10.
                            ; 10-13 DEC 2010 RJHD
                            ;*****************************************

                            ;IF portion EQ 1 THEN cleaned_locations=WHERE(cleaned_values GT clean_lim, clean_cnt) $
                            ;  ELSE IF portion EQ 2 THEN  cleaned_locations=WHERE(cleaned_values LT clean_lim, clean_cnt)
                            ;********* ^^ Old version^^ ************

                            IF portion EQ 1 THEN cleaned_locations=WHERE(cleaned_values GT threshold, clean_cnt) $
                            ELSE IF portion EQ 2 THEN  cleaned_locations=WHERE(cleaned_values LT threshold, clean_cnt)
                            
                            IF dist EQ "no" THEN BEGIN
                                IF portion EQ 1 THEN gap_cleaned_locations=WHERE(cleaned_values GT first_gap, gap_clean_cnt) $
                                ELSE IF portion EQ 2 THEN  gap_cleaned_locations=WHERE(cleaned_values LT first_gap, gap_clean_cnt)
                            ENDIF
                            
                            IF (clean_cnt GE 1) THEN BEGIN ; TENTATIVE FLAGS to be set
                                ;*****************************************
                                ; testing if flagging (and hence plotting)
                                ; should be performed.
                                ;*****************************************
                                plot_values(0:N_ELEMENTS(cleaned_values(cleaned_locations))-1,yr)=cleaned_values(cleaned_locations) 
                                cleaned_flags(cleaned_locations)=2
                            ENDIF
                            
                            IF dist EQ "no" THEN BEGIN ; FIRM FLAGS to be set
                                IF (gap_cnt GE gap_limit) AND (portion_cnt GE 1) AND (gap_clean_cnt GE 1) THEN BEGIN
                                    ;*****************************************
                                    ; testing if flagging (and hence plotting)
                                    ; should be performed.
                                    ;*****************************************
                                    gap_plot_values(0:N_ELEMENTS(cleaned_values(gap_cleaned_locations))-1,yr)=cleaned_values(gap_cleaned_locations) 
                                    cleaned_flags(gap_cleaned_locations)=1
                                ENDIF
                            ENDIF
                            year_flags(WHERE(((year_values ne mdi) AND (year_values ne fdi))))=cleaned_flags
                            qc_flags_climate(monthly_segment_starts(m,yr):monthly_segment_ends(m,yr))=year_flags
                        ENDIF
                        
                    ENDFOR
                    
                    plot_test=WHERE(((plot_values NE mdi) AND (plot_values NE fdi)), pltcnt)
                    
                    IF dist EQ "no" THEN gap_plot_test=WHERE(((gap_plot_values NE mdi) AND (gap_plot_values NE fdi)), gap_pltcnt)

                    ;*****************************************
                    ; Highlight which portions of the histogram are flagged
                    ;*****************************************
                    IF KEYWORD_SET(diagnostics) THEN BEGIN
                        
                        IF pltcnt GE 1 THEN BEGIN 
                            ;*****************************************
                            ; the testing if anything should be flagged or plotted was done above.
                            ; here just checking if there is anything to plot
                            ;*****************************************
                        
                            oplot,[threshold,threshold],[0.1,max(hist)*5],psym=0,color=200,YLog=1 ;, linestyle=2
                            plot_values=plot_values(plot_test)
                            histp = HISTOGRAM(plot_values, BINSIZE=binsize,LOCATION=loc, MAX=max_val, MIN=min_val)
                            histp=[0,0,0,temporary(histp),0,0,0] 
                            binns=[min_bin-(3.*binsize),min_bin-(2.*binsize),min_bin-binsize,loc,max_bin+binsize,max_bin+(2.*binsize),max_bin+(3.*binsize)]+(binsize/2.)
                            OPLOT,binns,histp,min_value=-100,xstyle=1,ystyle=1,ytitle="Frequency",xtitle=label+' offset (IQR)',$
                              title="Climatological Check - Diagnostic Histogram - "+label+' - month '+strcompress(m+1,/remove_all),charsize=1,ythick=4,xthick=4,psym=10,$
                              YLog=1,YRange=[0.1,max(hist)*5], POSITION=[0.10, 0.10, 0.90, 0.90],color=200
                            
                        ENDIF
                        IF dist EQ "no" THEN BEGIN
                            IF gap_pltcnt GE 1 THEN BEGIN 
                                ;*****************************************
                                ; testing if anything should be flagged or plotted was done above.
                                ; here just checking if there is anything to plot
                                ;*****************************************
                                
                                oplot,[first_gap,first_gap],[0.1,max(hist)*5],psym=0,color=239,YLog=1 ;, linestyle=2
                                gap_plot_values=gap_plot_values(gap_plot_test)
                                histp = HISTOGRAM(gap_plot_values, BINSIZE=binsize,LOCATION=loc, MAX=max_val, MIN=min_val)
                                histp=[0,0,0,temporary(histp),0,0,0] 
                                binns=[min_bin-(3.*binsize),min_bin-(2.*binsize),min_bin-binsize,loc,max_bin+binsize,max_bin+(2.*binsize),max_bin+(3.*binsize)]+(binsize/2.)
                                OPLOT,binns,histp,min_value=-100,xstyle=1,ystyle=1,ytitle="Frequency",xtitle=label+' offset (IQR)',$
                                  title="Climatological Check - Diagnostic Histogram - "+label+' - month '+strcompress(m+1,/remove_all),charsize=1,ythick=4,xthick=4,psym=10,$
                                  YLog=1,YRange=[0.1,max(hist)*5], POSITION=[0.10, 0.10, 0.90, 0.90],color=239
                            ENDIF
                        ENDIF
                    ENDIF                 
                                    
                ENDFOR                    
            ENDIF
            IF largecount GT 1000 THEN BEGIN
                cd, "./", current=pwd
                OPENW,99,pwd+'/spurious_stations.txt',/append
                PRINTF,99,'Climatological',station_id,'1973-'+strcompress(last_year,/remove_all),strcompress(m+1,/remove_all),label,'    Large number of observations offset from main distribution',FORMAT='(A20,A20,A10,I4,A20,A)'
                CLOSE,99
            ENDIF
            IF KEYWORD_SET(diagnostics) THEN device,/close
        ENDIF
    ENDIF
    
ENDFOR


IF gap_check EQ 1 THEN BEGIN

    MATCH, h_times,full_h_times,lc1,lc2,count=count
    
    qc_flags_matched=qc_flags_climate(lc2)

   ;**************************************************************
   ; only copy across non_zero flags so that the old flags are not wiped
   ;**************************************************************

    newflags=WHERE(qc_flags_matched GT 0, nfcnt)

    IF nfcnt GE 1 THEN BEGIN
        qc_flags(columns,newflags)=qc_flags_matched(newflags)
    ENDIF
    
    points=WHERE(qc_flags(columns,*) GT 0, bad_count)

    If bad_count GE 1 THEN filter_flags(points)=1

    IF KEYWORD_SET(mask) THEN BEGIN
       flagged_values[flagged_col,points]=values[points]
       values[points]=fdi
       MATCH, h_times(points), full_h_times, suba, subb
       full_values[subb]=fdi
    ENDIF

    printstring="Climatologic Check Flags  "
    IF KEYWORD_SET(oldflags) THEN BEGIN
        PRINT,printstring,type,':',bad_count-pflcnt,'  Old Flags:',pflcnt, FORMAT='(A-38,I1,A1,I8,A-15,I8)'
    ENDIF ELSE BEGIN
        PRINT,printstring,type,':',bad_count, FORMAT='(A-38,I1,A1,I8)'
    ENDELSE

ENDIF ELSE IF gap_check EQ 0 THEN BEGIN
;*****************************************
; Old version of flagging all points > 10 
; pseudo sigma away
;
; THIS IS NOT RUN BUT KEPT AS ARCHIVE
;*****************************************

;**************************************************************
; Remove any points > 10 pseudo-sigma away
;**************************************************************

    non_missing=WHERE(((full_values NE mdi) AND (full_values NE fdi)), dc)
    IF dc GE 1 THEN BEGIN
        missing=WHERE(((full_values EQ mdi) AND (full_values EQ fdi)), ec)
        IF ec GE 1 THEN full_values(missing)=0 ;;; doesn't matter, simply makes less nested loops!


; ***KATE JUL 2010 changed 8. to 10. - this is quite large so may not actually kick out anything anymore
        bad_v_points=WHERE(ABS(full_values) GE 10., bc) ; was 8 changed to 6 23/09/09. Reverted 9/10
;  bc2=0
;  print,full_values(WHERE(ABS(full_values) GE 10.)),5.*vres
  ; ***KATE JUL 2010 THIS NEXT LINE IS TOTALLY POINTLESS!
; **************RJHD 29 NOV 2010 - removed the pointless line********
;  IF bc GE 1 THEN really_bad_v_points=WHERE(ABS(full_values(bad_v_points)) GE 5.*vres, bc2)  ; added 10/8/09
;  IF bc2 GE 1 THEN BEGIN
;    PRINT,'Temperatures',bc2
        IF bc GE 1 THEN BEGIN
            FOR b=0L,bc-1 DO BEGIN
      ; Find the time loc of the bad temperature observation.
                this_time=full_times(bad_v_points(b))
                vmatch=WHERE(times EQ this_time, checkval)
                IF checkval EQ 1 THEN BEGIN
                    bads=WHERE(((values(vmatch) NE mdi) AND (values(vmatch) NE fdi)),count) ; kate added to only flag present values
                    IF (count GT 0) THEN BEGIN
                        qc_flags(columns,vmatch(bads))=1
                        IF KEYWORD_SET(mask) THEN BEGIN
                           flagged_values[flagged_col, vmatch]=values[vmatch]
                           values(vmatch)=fdi
                        ENDIF
                    ENDIF
                ENDIF ELSE STOP
            ENDFOR
        ENDIF
    ENDIF
    
    IF ec GE 1 THEN full_values(missing)=mdi



    IF KEYWORD_SET(diagnostics) THEN BEGIN
        min_val=FIX(MIN(full_values(non_missing)))-1 ; just in case it is negative and doesn't work
        max_val=CEIL(MAX(full_values(non_missing)))
        
        binsize=1.0
        hist = HISTOGRAM(full_values(non_missing), BINSIZE=binsize,LOCATION=loc, MAX=max_val, MIN=min_val)
        hist=[0,0,0,temporary(hist),0,0,0] 

      ;******************************************
      ; adjust for padding in histogram
      ;******************************************
        min_bin=MIN(loc)
        max_bin=MAX(loc)
        binns=[min_bin-(3.*binsize),min_bin-(2.*binsize),min_bin-binsize,loc,max_bin+binsize,max_bin+(2.*binsize),max_bin+(3.*binsize)]+(binsize/2)
        
        set_plot,'PS'
        cd, "./", current=outdirplot
        device,filename=outdirplot+'/images/'+station_id+'_Climatological_'+label+'.eps'$
          ,/color,/encapsulate,/helvetica,/bold,/landscape,ysize=20,xsize=26
        !P.Font=0
        !P.Multi=[0,1,1]
        !P.Thick=4
        plotsym,3,1.2
        plot,binns,hist,min_value=-100,xstyle=1,ystyle=1,ytitle="Frequency",xtitle=label+' offset',$
          title="Climatological_Check - Diagnostic Histogram - "+label,charsize=1,ythick=4,xthick=4,psym=10,YLog=1,YRange=[0.1,max(hist)*5], POSITION=[0.10, 0.10, 0.90, 0.90]
        oplot,[-10.,-10.],[0.1,max(hist)*5],psym=0,color=240,YLog=1
        oplot,[10.,10.],[0.1,max(hist)*5],psym=0,color=240,YLog=1
    ;******************************************
    ;overplot fitted gaussian
    ;******************************************
        ; 1.5IQR ~ 1 sigma
        yfit = GaussFit(binns, hist, coeff, NTERMS=3, ESTIMATES=[max(hist),0.0,1.5])
        oplot, binns, yfit, COLOR=40, THICK=2


    ENDIF



    points=WHERE(qc_flags(columns,*) EQ 1, bad_count)
    printstring="Climatologic Check Flags  "
    PRINT,printstring,type,':',bad_count, FORMAT='(A-38,I1,A1,I8)'

    IF KEYWORD_SET(diagnostics) and bad_count GE 1 THEN BEGIN
        binsize=1.0
        hist = HISTOGRAM(full_values(bad_v_points), BINSIZE=binsize,LOCATION=loc, MAX=max_val, MIN=min_val)
        hist=[0,0,0,temporary(hist),0,0,0] 
        oplot,binns,hist,min_value=-100,xstyle=1,ystyle=1,ytitle="Frequency",xtitle=label+' offset',$
          title="Climatological_Check - Diagnostic Histogram - "+label,charsize=1,ythick=4,xthick=4,psym=10,YLog=1,YRange=[0.1,max(hist)*5], POSITION=[0.10, 0.10, 0.90, 0.90],color=240
        
        device,/close

    ENDIF ELSE IF KEYWORD_SET(diagnostics) THEN device,/close
    

ENDIF

full_values=untouched_full_values

RETURN

END

;************************************************************************
;                                 END
;************************************************************************
