;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;British Crown Copyright (c) 2012, the Met Office All rights reserved.
;
;Redistribution and use in source and binary forms, with or without 
;modification, are permitted provided that the following conditions 
;are met:
;
;Redistributions of source code must retain the above copyright notice, 
;this list of conditions and the following disclaimer. 
;
;Redistributions in binary form must reproduce the above copyright 
;notice, this list of conditions and the following disclaimer in the 
;documentation and/or other materials provided with the distribution. 
;
;Neither the name of the Met Office nor the names of its contributors 
;may be used to endorse or promote products derived from this software 
;without specific prior written permission. 
;
;THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
;"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
;LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
;A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
;HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
;SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
;LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
;DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
;THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
;(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
;OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

PRO CREATE_FULLTIMES, last_year, temperatures, temperatures_mdi, dewpoints, slp, windspeeds, winddirs, times,$  
                      full_times_reset, h_times, full_h_times_reset, full_temperatures, full_dewpoints, full_slp, full_windspeeds,full_winddirs,$ 
                      duplicate_qc_flags, dewpoints_mdi, slp_mdi, windspeeds_mdi, winddirs_mdi, $
                      diagnostics, station_id, flagged_values, full_flagged_values, $
                      total_cloud_cover_mdi, low_cloud_cover_mdi, mid_cloud_cover_mdi, high_cloud_cover_mdi
;*************************************************************

; NAME:
;     CREATE_FULLTIMES
;
; PURPOSE:
;     
;
; CALLING SEQUENCE:
;     CREATE_FULLTIMES last_year, temperatures, temperatures_mdi,
;     dewpoints, slp, windspeeds, winddirs, times,full_times_reset,
;     full_temperatures, full_dewpoints, full_slp,
;     full_windspeeds,full_winddirs,duplicate_qc_flags, dewpoints_mdi,
;     slp_mdi, windspeeds_mdi, winddirs_mdi,diagnostics, station_id,
;     flagged_values, full_flagged_values,total_cloud_cover_mdi,
;     low_cloud_cover_mdi, mid_cloud_cover_mdi, high_cloud_cover_mdi 
;
; INPUTS:
;     last_year - the last year of observations
;     
;     temperatures - array containing the temperature data
;     
;     temperatures_mdi - missing data indicator for the temperatures
;
;     dewpoints - array containing the dewpoint data
;
;     slp - array containing the Sea Level Pressure data
;
;     windspeeds - array containing the windspeed data
;
;     winddirs - array containing the wind direction data
;
;     times - array containing the times of the observations in
;             fractions of a day
;
;     full_times_reset - array containing the times of the observations
;
;     h_times - array containing the times of the observations in hours
;
;     full_h_times_reset - array containing the times of the
;                          observations in hours
;
;     full_temperatures - array containing the uncompressed temperature data 
;
;     full_dewpoints - array containing the uncompressed dewpoint data 
;
;     full_slp  - array containing the uncompressed SLP data 
;
;     full_windpeeds - array containing the uncompressed windspeed data 
;
;     full_winddirs - array containing the uncompressed winddirs data 
;
;     duplicate_qc_flags - duplicate of QC flag array
;
;     dewpoints_mdi - missing data indicator for the dewpoints
;
;     slp_mdi - missing data indicator for the SLP
;
;     windspeeds_mdi - missing data indicator for the windspeeds
;
;     winddirs_mdi, - missing data indicator for the wind directions
;
;     diagnostics - keyword for turning on diagnostic features (plots)
;
;     station_id - station ID for plot names
;
;     flagged_values - array to store flagged values on masking
;
;     full_flagged_values - uncompressed array to store flagged values
;
;     total_cloud_cover_mdi - mdi for total cloud
;
;     low_cloud_cover_mdi - mdi for low cloud
;
;     mid_cloud_cover_mdi - mdi for mid cloud
;
;     high_cloud_cover_mdi - mdi for high cloud
;
; OUTPUTS:
;     Returns filled arrays back to control script
;
; RESTRICTIONS:
;
;     None known
;
; COMMENTS
;
;
; HISTORY:
;     #237 - Created
;            16 November 2011 RJHD
;     #299 - Changed matching of timestamps into hours and passed
;            these array back to qc_all_vars_internal
;            6 June 2012 RJHD
;
;************************************************************************
;                    SVN Info
;$Rev:: 317                                           $:  Revision of last commit
;$Author:: rdunn                                      $:  Author of last commit
;$Date:: 2012-08-29 17:45:37 +0100 (Wed, 29 Aug 2012) $:  Date of last commit
;************************************************************************
;                                 START
;************************************************************************

;************************************************************************
; Use the last_year value to define how many hours and months we are
; to consider.
;************************************************************************
start_date = JULDAY(1, 1, 1973, 0)
end_date = JULDAY(1, 1, last_year+1, 0)
nhours=LONG((end_date-start_date)*24.)
nmonths=(last_year-1973+1)*12
nyears=last_year-1973+1

;************************************************************************
; Next we need to unpack the compressed time axis that the netcdf
; data file was held in and map the temperatures onto hourly times.
;************************************************************************

;************************************************************************
; Create the hourly index
;************************************************************************

full_times = TIMEGEN(nhours, UNIT = 'Hours',  START = JULDAY(1, 1, 1973, 0, 0 ,0))
current_time=JULDAY(1, 1, 1973, 0, 0, 0) ; outputs a double
full_times_reset=full_times-current_time ; this is output as full_times back to the control code

;************************************************************************
; Work in integers for the match - ie in hours since, rather than days since
;************************************************************************


full_h_times=ROUND(full_times*24.)
full_h_times_reset=ROUND(full_h_times-(current_time*24.))

h_times=ROUND(times*24.)


;************************************************************************
; Find the locs and fill them into full_temperatures array.
;************************************************************************
full_temperatures=DBLARR(nhours)
full_temperatures(*)=temperatures_mdi
full_dewpoints=DBLARR(nhours)
full_dewpoints(*)=dewpoints_mdi
full_slp=DBLARR(nhours)
full_slp(*)=slp_mdi
full_windspeeds=DBLARR(nhours)
full_windspeeds(*)=windspeeds_mdi
full_winddirs=DBLARR(nhours)
full_winddirs(*)=winddirs_mdi

full_flagged_values=FLTARR(9,nhours)
full_flagged_values[0,*]=temperatures_mdi
full_flagged_values[1,*]=dewpoints_mdi
full_flagged_values[2,*]=slp_mdi
full_flagged_values[3,*]=total_cloud_cover_mdi
full_flagged_values[4,*]=low_cloud_cover_mdi
full_flagged_values[5,*]=mid_cloud_cover_mdi
full_flagged_values[6,*]=high_cloud_cover_mdi
full_flagged_values[7,*]=windspeeds_mdi
full_flagged_values[8,*]=winddirs_mdi


loc=0L
MATCH,h_times,full_h_times_reset, suba, subb, count=cnt
full_temperatures(subb)=temperatures(suba)
full_dewpoints(subb)=dewpoints(suba)
full_slp(subb)=slp(suba)
full_windspeeds(subb)=windspeeds(suba)
full_winddirs(subb)=winddirs(suba)

; set up the full_flagged values array
full_flagged_values[*,subb]=flagged_values[*,suba]

RETURN

END
;************************************************************************
;                                 END
;************************************************************************
