;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;British Crown Copyright (c) 2012, the Met Office All rights reserved.
;
;Redistribution and use in source and binary forms, with or without 
;modification, are permitted provided that the following conditions 
;are met:
;
;Redistributions of source code must retain the above copyright notice, 
;this list of conditions and the following disclaimer. 
;
;Redistributions in binary form must reproduce the above copyright 
;notice, this list of conditions and the following disclaimer in the 
;documentation and/or other materials provided with the distribution. 
;
;Neither the name of the Met Office nor the names of its contributors 
;may be used to endorse or promote products derived from this software 
;without specific prior written permission. 
;
;THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
;"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
;LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
;A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
;HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
;SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
;LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
;DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
;THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
;(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
;OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

PRO cross_check_clouds, times, total_cloud_cover, total_cloud_cover_mdi, total_cloud_cover_fdi, low_cloud_cover, low_cloud_cover_mdi, $
                       low_cloud_cover_fdi,  mid_cloud_cover, mid_cloud_cover_mdi, mid_cloud_cover_fdi, high_cloud_cover, high_cloud_cover_mdi,   $
			high_cloud_cover_fdi, cloud_base, cloud_base_mdi, qc_flags, flagged_values, mask=mask, diagnostics

; 9 sky obscured, 10 partial obstruction therefore set to missing qcflag = 'cloud_unobservable'

; 1: if total, low, med or high = 9 or 10 = fail which ever to (missing) - 'cloud_unobservable'
; 2: if total_cloud < low+med+high = fail - 'cloud_smalltot'
; 3: if total_cloud > low+med+high = fail - 'cloud_bigtot'
; 4: if low_cloud = 8 and med or high has a value = fail (med or high) - 'cloud_fulllow'
; 5: if med_cloud = 8 and high has a value = fail (high) - 'cloud_fullmid'
; 6: if cloud_base = 22000 (means cloud base unobservable - sky clear) = set to -10 - no qc alteration

;************************************************************************
; NAME:
;     CROSS_CHECK_CLOUDS
;
; PURPOSE:
;     Cross check the cloud data so that it makes sense
;
; CALLING SEQUENCE:
;     cross_check_clouds, times, total_cloud_cover,
;     total_cloud_cover_mdi, low_cloud_cover, low_cloud_cover_mdi,
;     mid_cloud_cover, mid_cloud_cover_mdi, high_cloud_cover,
;     high_cloud_cover_mdi, cloud_base, cloud_base_mdi, qc_flags, mask, diagnostics
;
; INPUTS:
;     times - array containing the times of the observations
;
;     total_cloud_cover - array containing the total cloud cover data
;
;     total_cloud_cover_mdi - total cloud cover missing data indicator
;
;     total_cloud_cover_fdi - total cloud cover flagged data indicator
;
;     low_cloud_cover - array containing the low cloud cover data
;
;     low_cloud_cover_mdi - low cloud cover missing data indicator 
;
;     low_cloud_cover_fdi - low cloud cover flagged data indicator
;
;     mid_cloud_cover - array containing the mid cloud cover data
;
;     mid_cloud_cover_mdi -  mid cloud cover missing data indicator
;
;     mid_cloud_cover_fdi - mid cloud cover flagged data indicator
;
;     high_cloud_cover - array containing the high cloud cover data
;
;     high_cloud_cover_mdi - high cloud cover missing data indicator 
;
;     high_cloud_cover_fdi - high cloud cover flagged data indicator 
;
;     cloud_base - array containing the cloud base data
;
;     cloud_base_mdi -  cloud base missing data indicator
;
;     qc_flags - array containing the flags
;
;     flagged_values - array containing the observation values which
;                      have been flagged
;
;     mask - keyword to allow masking of the data if desired.
;
;     diagnostics - keyword to allow diagnostic information to be printed to screen (very verbose)
;
; OUTPUTS:
;
;     None - arrays returned to control script
;
; RESTRICTIONS:
;
;     
;   
; HISTORY:
;     # 1 - Initial version of the program as written by Kate Willett
;           9 November 2010 RJHD
;     # 2 - Commented out - and added mask keyword
;           9 November 2010 RJHD
;     #40 - Added diagnostic information switch
;           10 November 2010 RJHD
;     #50 - Reduced verbosity of diagnostic information with extra
;           built-in switch
;           15 November 2010 RJHD
;     #237 - SVN information added to header
;            Added fdi and flagged_values functionality
;            16 November 2011 RJHD
;     #308 - All cloud values flagged if the low=8 and mid or high
;            have a value, and mid=8 and high has a value, rather than
;            just the upper layers
;            2 July 2012 RJHD
;************************************************************************
;                    SVN Info
;$Rev:: 308                                           $:  Revision of last commit
;$Author:: rdunn                                      $:  Author of last commit
;$Date:: 2012-07-04 14:47:15 +0100 (Wed, 04 Jul 2012) $:  Date of last commit
;************************************************************************
;                                 START
;************************************************************************

;********************************
; all flags being set so that they do not wipe any preexisting ones.
;********************************

ntimes=n_elements(times)

printall=0

FOR i=0L,ntimes-1 DO BEGIN
;************************************************************************
;1: T,L,M,H = 9 or 10
;************************************************************************
    IF ((total_cloud_cover(i) EQ 9) OR (total_cloud_cover(i) EQ 10)) THEN BEGIN
        IF KEYWORD_SET(mask) THEN BEGIN
            flagged_values[3,i]=total_cloud_cover[i]
            total_cloud_cover(i)=total_cloud_cover_fdi
        ENDIF
        qc_flags(33,i)=1
        IF KEYWORD_SET(diagnostics) AND (printall EQ 1) THEN BEGIN
            print,"Total Cloud Unobservable"
        ENDIF
    ENDIF
    IF ((low_cloud_cover(i) EQ 9) OR (low_cloud_cover(i) EQ 10)) THEN BEGIN
        IF KEYWORD_SET(mask) THEN BEGIN
            flagged_values[4,i]=low_cloud_cover[i]
            low_cloud_cover(i)=low_cloud_cover_fdi
        ENDIF
        qc_flags(34,i)=1
        IF KEYWORD_SET(diagnostics) AND (printall EQ 1) THEN BEGIN
            print,"Low Cloud Unobservable"
        ENDIF
    ENDIF
    IF ((mid_cloud_cover(i) EQ 9) OR (mid_cloud_cover(i) EQ 10)) THEN BEGIN
        IF KEYWORD_SET(mask) THEN BEGIN
            flagged_values[5,i]=mid_cloud_cover[i]
            mid_cloud_cover(i)=mid_cloud_cover_fdi
        ENDIF
        qc_flags(35,i)=1
        IF KEYWORD_SET(diagnostics) AND (printall EQ 1) THEN BEGIN
            print,"Mid Cloud Unobservable"
        ENDIF
    ENDIF
    IF ((high_cloud_cover(i) EQ 9) OR (high_cloud_cover(i) EQ 10)) THEN BEGIN
        IF KEYWORD_SET(mask) THEN BEGIN
            flagged_values[6,i]=high_cloud_cover[i]
            high_cloud_cover(i)=high_cloud_cover_fdi
        ENDIF
        qc_flags(36,i)=1
        IF KEYWORD_SET(diagnostics) AND (printall EQ 1) THEN BEGIN
            print,"High Cloud Unobservable"
        ENDIF
    ENDIF
  
;************************************************************************
; 2: TOTAL< MAX(L,M,H)
;************************************************************************
    IF (total_cloud_cover(i) NE total_cloud_cover_mdi) AND (total_cloud_cover(i) NE total_cloud_cover_fdi)THEN BEGIN
        tot_cloud=0
        IF (low_cloud_cover(i) NE low_cloud_cover_mdi) AND (low_cloud_cover(i) NE low_cloud_cover_fdi)  THEN tot_cloud=low_cloud_cover(i)
        IF ((mid_cloud_cover(i) NE mid_cloud_cover_mdi) AND (mid_cloud_cover(i) NE mid_cloud_cover_fdi) $
            AND (mid_cloud_cover(i) GT tot_cloud)) THEN tot_cloud=mid_cloud_cover(i)
        IF ((high_cloud_cover(i) NE high_cloud_cover_mdi) AND (high_cloud_cover(i) NE high_cloud_cover_fdi) $
            AND (high_cloud_cover(i) GT tot_cloud)) THEN tot_cloud=high_cloud_cover(i)
        
        IF (total_cloud_cover(i) LT tot_cloud) THEN BEGIN
            IF KEYWORD_SET(mask) THEN BEGIN
                flagged_values[3,i]=total_cloud_cover[i]
                total_cloud_cover(i)=total_cloud_cover_fdi
                flagged_values[4,i]=low_cloud_cover[i]
                low_cloud_cover(i)=low_cloud_cover_fdi
                flagged_values[5,i]=mid_cloud_cover[i]
                mid_cloud_cover(i)=mid_cloud_cover_fdi
                flagged_values[6,i]=high_cloud_cover[i]
                high_cloud_cover(i)=high_cloud_cover_fdi
            ENDIF
            qc_flags(37,i)=1 ; NOT REMOVING CLOUD BASE - MAY STILL BE OF SOME USE
            IF KEYWORD_SET(diagnostics) AND (printall EQ 1) THEN BEGIN
                print,"Total Cloud < Max of Low, Mid & High"
            ENDIF
        ENDIF 
    
;************************************************************************
; 3: TOTAL>L+M+H
;     - no - as can have full High, but <full medium which you can see
;       'cos it moves.
;************************************************************************
;    tot_cloud=0
;    IF (low_cloud_cover(i) NE low_cloud_cover_mdi) THEN tot_cloud=low_cloud_cover(i)
;    IF (mid_cloud_cover(i) NE mid_cloud_cover_mdi) THEN tot_cloud=tot_cloud+mid_cloud_cover(i)
;    IF (high_cloud_cover(i) NE high_cloud_cover_mdi) THEN tot_cloud=tot_cloud+high_cloud_cover(i)
;
;    IF (total_cloud_cover(i) GT tot_cloud) THEN BEGIN
;      total_cloud_cover(i)=total_cloud_cover_mdi
;      low_cloud_cover(i)=low_cloud_cover_mdi
;      mid_cloud_cover(i)=mid_cloud_cover_mdi
;      high_cloud_cover(i)=high_cloud_cover_mdi
;      ;IF (qc_flags(i) EQ 'pass') THEN qc_flags(i)='cloud_bigtot' 	; NOT REMOVING CLOUD BASE - MAY STILL BE OF SOME USE
;        IF KEYWORD_SET(diagnostics) THEN BEGIN
;            print,"Total Cloud > Sum of Low, Mid & High"
;        ENDIF
;    ENDIF   
    ENDIF
  
;************************************************************************
; 4: LOW CLOUD FULL - OTHERS OBSCURED - FLAG ALL LEVELS
;************************************************************************
    IF (low_cloud_cover(i) EQ 8) THEN BEGIN
        IF ((mid_cloud_cover(i) NE mid_cloud_cover_mdi) AND (mid_cloud_cover(i) NE mid_cloud_cover_fdi)) THEN BEGIN
            IF KEYWORD_SET(mask) THEN BEGIN
                flagged_values[5,i]=mid_cloud_cover[i]
                mid_cloud_cover(i)=mid_cloud_cover_fdi

                flagged_values[4,i]=low_cloud_cover[i]
                low_cloud_cover(i)=low_cloud_cover_fdi
            ENDIF
            IF KEYWORD_SET(diagnostics) AND (printall EQ 1) THEN BEGIN
                print,"Low Cloud Full - Mid had values, both flagged"
            ENDIF
            qc_flags(38,i)=1
        ENDIF
        IF (high_cloud_cover(i) NE high_cloud_cover_mdi) AND (high_cloud_cover(i) NE high_cloud_cover_fdi) THEN BEGIN
            IF KEYWORD_SET(mask) THEN BEGIN
                flagged_values[6,i]=high_cloud_cover[i]
                high_cloud_cover(i)=high_cloud_cover_fdi

                ; test if flag already set
                IF low_cloud_cover(i) NE low_cloud_cover_fdi THEN BEGIN
                   flagged_values[4,i]=low_cloud_cover[i]
                   low_cloud_cover(i)=low_cloud_cover_fdi
                ENDIF
            ENDIF
            IF KEYWORD_SET(diagnostics) AND (printall EQ 1) THEN BEGIN
                print,"Low Cloud Full - High had values, both flagged"
            ENDIF
            qc_flags(38,i)=1
        ENDIF
    ENDIF
    
;************************************************************************
; 5: MID CLOUD FULL - OTHERS OBSCURED - FLAG M & H LEVELS
;************************************************************************
    IF (mid_cloud_cover(i) EQ 8) THEN BEGIN
        IF (high_cloud_cover(i) NE high_cloud_cover_mdi) AND (high_cloud_cover(i) NE high_cloud_cover_fdi) THEN BEGIN
            IF KEYWORD_SET(mask) THEN BEGIN
                flagged_values[6,i]=high_cloud_cover[i]
                high_cloud_cover(i)=high_cloud_cover_fdi

                flagged_values[5,i]=mid_cloud_cover[i]
                mid_cloud_cover(i)=mid_cloud_cover_fdi
            ENDIF
            IF KEYWORD_SET(diagnostics) AND (printall EQ 1) THEN BEGIN
                print,"Mid Cloud Full - High had value. Both flagged"
            ENDIF
            qc_flags(39,i)=1
        ENDIF
    ENDIF
    
;************************************************************************
; 6: CLOUD BASE AT 22000
;************************************************************************
    IF (cloud_base(i) EQ 22000) THEN BEGIN
    ;***************************************
    ; Purposefully not placing a QC flag
    ;***************************************
        cloud_base(i)=cloud_base_mdi
        ;IF KEYWORD_SET(diagnostics) THEN print,"Cloud base unobservable - but no flag set"
    ENDIF
ENDFOR

points=WHERE(qc_flags(33,*) GT 0, bad_count)
printstring="Cross Check Clouds Flags"
PRINT,printstring,1,':', bad_count, FORMAT='(A-38,I1,A1,I8)'
points=WHERE(qc_flags(34,*) GT 0, bad_count)
printstring="Cross Check Clouds Flags"
PRINT,printstring,2,':', bad_count, FORMAT='(A-38,I1,A1,I8)'
points=WHERE(qc_flags(35,*) GT 0, bad_count)
printstring="Cross Check Clouds Flags"
PRINT,printstring,3,':', bad_count, FORMAT='(A-38,I1,A1,I8)'
points=WHERE(qc_flags(36,*) GT 0, bad_count)
printstring="Cross Check Clouds Flags"
PRINT,printstring,4,':', bad_count, FORMAT='(A-38,I1,A1,I8)'
points=WHERE(qc_flags(37,*) GT 0, bad_count)
printstring="Cross Check Clouds Flags"
PRINT,printstring,5,':', bad_count, FORMAT='(A-38,I1,A1,I8)'
points=WHERE(qc_flags(38,*) GT 0, bad_count)
printstring="Cross Check Clouds Flags"
PRINT,printstring,6,':', bad_count, FORMAT='(A-38,I1,A1,I8)'
points=WHERE(qc_flags(39,*) GT 0, bad_count)
printstring="Cross Check Clouds Flags"
PRINT,printstring,7,':', bad_count, FORMAT='(A-38,I1,A1,I8)'

;************************************************************************
; 7: Do the clean up cloud value check by removing values less than zero that can't be right ...
; and update the qc flag value accordingly.
;************************************************************************

vals=WHERE(((total_cloud_cover NE total_cloud_cover_mdi) AND (total_cloud_cover NE total_cloud_cover_fdi)), cnt)
IF cnt GE 1 THEN BEGIN
    badvals=WHERE(total_cloud_cover(vals) LT 0, badcnt)
    IF badcnt GE 1 THEN BEGIN
        IF KEYWORD_SET(mask) THEN BEGIN
            flagged_values[3,i]=total_cloud_cover[i]
            total_cloud_cover(vals(badvals))=total_cloud_cover_fdi
        ENDIF
        IF KEYWORD_SET(diagnostics) AND (printall EQ 1) THEN BEGIN
            print,"Negative Cloud value"
        ENDIF
        qc_flags(40,vals(badvals))=1
    ENDIF
ENDIF
vals=WHERE(((low_cloud_cover NE low_cloud_cover_mdi) AND (low_cloud_cover NE low_cloud_cover_fdi)), cnt)
IF cnt GE 1 THEN BEGIN
    badvals=WHERE(low_cloud_cover(vals) LT 0, badcnt)
    IF badcnt GE 1 THEN BEGIN
        IF KEYWORD_SET(mask) THEN BEGIN
            flagged_values[4,i]=low_cloud_cover[i]
            low_cloud_cover(vals(badvals))=low_cloud_cover_fdi
        ENDIF
        IF KEYWORD_SET(diagnostics) AND (printall EQ 1) THEN BEGIN
            print,"Negative Cloud value"
        ENDIF
        qc_flags(40,vals(badvals))=1
    ENDIF
ENDIF
vals=WHERE(((mid_cloud_cover NE mid_cloud_cover_mdi) AND (mid_cloud_cover NE mid_cloud_cover_fdi)), cnt)
IF cnt GE 1 THEN BEGIN
    badvals=WHERE(mid_cloud_cover(vals) LT 0, badcnt)
    IF badcnt GE 1 THEN BEGIN
        IF KEYWORD_SET(mask) THEN BEGIN
            flagged_values[5,i]=mid_cloud_cover[i]
            mid_cloud_cover(vals(badvals))=mid_cloud_cover_fdi
        ENDIF
        IF KEYWORD_SET(diagnostics) AND (printall EQ 1) THEN BEGIN
            print,"Negative Cloud value"
        ENDIF
        qc_flags(40,vals(badvals))=1
    ENDIF
ENDIF
vals=WHERE(((high_cloud_cover NE high_cloud_cover_mdi) AND (high_cloud_cover NE high_cloud_cover_fdi)), cnt)
IF cnt GE 1 THEN BEGIN
    badvals=WHERE(high_cloud_cover(vals) LT 0, badcnt)
    IF badcnt GE 1 THEN BEGIN
        IF KEYWORD_SET(mask) THEN BEGIN
            flagged_values[5,i]=high_cloud_cover[i]
            high_cloud_cover(vals(badvals))=high_cloud_cover_fdi
        ENDIF
        IF KEYWORD_SET(diagnostics) AND (printall EQ 1) THEN BEGIN
            print,"Negative Cloud value"
        ENDIF
        qc_flags(40,vals(badvals))=1
    ENDIF
ENDIF   

points=WHERE(qc_flags(40,*) EQ 1, bad_count)
printstring="Cross Check Clouds Flags"
PRINT,printstring,8,':', bad_count, FORMAT='(A-38,I1,A1,I8)'

RETURN

END

  

    
    
;************************************************************************
;                                 END
;************************************************************************
