;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;British Crown Copyright (c) 2012, the Met Office All rights reserved.
;
;Redistribution and use in source and binary forms, with or without 
;modification, are permitted provided that the following conditions 
;are met:
;
;Redistributions of source code must retain the above copyright notice, 
;this list of conditions and the following disclaimer. 
;
;Redistributions in binary form must reproduce the above copyright 
;notice, this list of conditions and the following disclaimer in the 
;documentation and/or other materials provided with the distribution. 
;
;Neither the name of the Met Office nor the names of its contributors 
;may be used to endorse or promote products derived from this software 
;without specific prior written permission. 
;
;THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
;"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
;LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
;A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
;HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
;SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
;LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
;DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
;THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
;(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
;OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

PRO CROSS_CHECK_T_TW, last_year, times,temperatures,temperatures_mdi,temperatures_fdi,dewpoints,$
                      dewpoints_mdi,dewpoints_fdi, qc_flags,t_qc_flags, dp_qc_flags, past_sigwx1,$
                      precip1, cloud_base, flagged_values ,mask=mask,$
                      diagnostics, station_id, oldflags=oldflags

; program to run QC humidity specific QC checks
;	1: Tdew greater than T - shouldn't really be any of these as its an NCDC QC
;	2: Zero DPD removal - strings of greater than 12 hours Zero DPD (wetbulb reservoir drying or freezing
;	3: Cutoffs - data with >50% per year T with no simulataneous Tdew
;
; created by Kate Willett: Kate.Willett@metoffice.gov.uk
; last modified 29.10.08
;
; run from qc_all_vars_internal.pro
; READS IN: times, temperatures, temperatures_mdi, dewpoints, dewpoints_mdi, qc_flags
; FROM: '483270-99999.nc'
;
; OUTPUT: 'pass' or 'ttw_sat' (for supersaturated) or 'ttw_dpd' (for dewpoint depressions) or 'ttw_cut' (for cutoffs) in qc_flags
;
;*************************************************************
; NAME:
;     CROSS_CHECK_T_TW
;
; PURPOSE:
;     To check that the temperatures and wet-bulb temperatures are
;     consistent with each other and have no artefacts.
;
; CALLING SEQUENCE:
;     cross_check_t_tw, last_year, times,temperatures,temperatures_mdi,dewpoints,
;               dewpoints_mdi,qc_flags,past_sigwx1,precip1, mask,
;               diagnostics, station_id, oldflags=oldflags
;
; INPUTS:
;     last_year - the final year of the data sequence
;
;     times - array containing the times
;
;     temperatures - array containing the temperature data
;
;     temperatures_mdi - missing data indicator for the temperature
;                        data
;
;     temperatures_fdi - flagged data indicator for the temperature
;                        data
;
;     dewpoints - array containing the dewpoint data
;
;     dewpoints_mdi - missing data indicator for the dewpoint
;                        data
;
;     dewpoints_fdi - flagged data indicator for the dewpoint
;                        data
;
;     qc_flags - array containing the flags for the station dataset
;
;     t_qc_flags - array containing the filtering flags for the temperature
;
;     dp_qc_flags - array containing the filtering flags for the dewpoint
;
;     past_sigwx1 - array containing information about the weather
;                   (significant weather) at the time of the observation.
;
;     precip1 - array containing information about the precipitation
;               at the time of the observation
;
;     cloud_base - cloud base level
;
;     flagged_values - array containing observation values which have
;                      been flagged
;
;     mask - keyword to do the masking inside the subroutine
;
;     diagnostics - keyword to run diagnostic plots
;
;     station_id - station identifier for the plots
;
;     oldflags - keyword to calculate the number of streak check flags
;                already set
;
; METHOD:
;     Check for:
;	1: Tdew greater than T - shouldn't really be any of these as its an NCDC QC
;	2: Zero DPD removal - strings of greater than 12 hours Zero DPD (wetbulb reservoir drying or freezing
;	3: Cutoffs - data with >50% per year T with no simulataneous Tdew
;
; OUTPUTS:
;
;     None - files written to specified directory
;
; RESTRICTIONS:
;
;     Assumes netcdf file inputs.
;   
; HISTORY:
;     #28 - initial upload - 9 November 2010 RJHD
;     #29 - commented fully.
;           9 November 2010 RJHD
;     #40 - Added diagnostic plotting capability
;           10 November 2010 RJHD
;     #52 - Updated diagnostic plotting capability
;           15 November 2010 RJHD
;     #53 - Corrected diagnostic plotting capability
;           15 November 2010 RJHD
;     #98 - Added cloudbase check to DPD code (0<cb<1000, then could
;           be foggy)
;           26 November 2010 RJHD
;     #171 - Added filter flags to pass through the total flags set
;            for each value
;            14 January 2011 RJHD
;     #174 - Added oldflags keyword to allow the calculation of the
;            previously set flags.
;            18 January 2011 RJHD
;     #177 - Temperatures no longer flagged by this test.
;            27 January 2011 RJHD
;     #200 - changed last_year from optional keyword to required input
;            to remove the hardwiring of the end date
;            11 March 2011 RJHD
;     #237 - SVN information added to header
;            Added fdi and flagged_values functionality
;            16 November 2011 RJHD
;     #299 - Changed float to double arrays
;            6 June 2012 RJHD
;
;
;************************************************************************
;                    SVN Info
;$Rev:: 308                                           $:  Revision of last commit
;$Author:: rdunn                                      $:  Author of last commit
;$Date:: 2012-07-04 14:47:15 +0100 (Wed, 04 Jul 2012) $:  Date of last commit
;************************************************************************
;                                 START
;************************************************************************


MONTH_SETUP,last_year,nmonths,nyears,times,times,monthly_segment_starts,$
  monthly_segment_ends, monthly_segment_lengths

ntimes=N_ELEMENTS(times)

;************************************************************************
;1: simple Tdew
; remove both T and dewpoint when simultaneous dewpoint is greater than T
; if Tdew>T for more than 20% of month then remove the whole month - it must be all rubbish!
;************************************************************************
IF KEYWORD_SET(oldflags) THEN BEGIN
    prev_flags=WHERE(qc_flags(30,*) GT 0, pfcnt)
ENDIF

start_time=times(0)
end_time=times(ntimes-1)

;********************************
; loop through all observations in monthly batches
;********************************
FigCounter=0
FigMCounter=0
FOR t=0L,nmonths-1 DO BEGIN
    count_data=0.
    count_SSS=0.
    FOR m=monthly_segment_starts(t),monthly_segment_ends(t) DO BEGIN
        IF ((temperatures(m) NE temperatures_mdi) AND (dewpoints(m) NE dewpoints_mdi)) AND $
           ((temperatures(m) NE temperatures_fdi) AND (dewpoints(m) NE dewpoints_fdi))THEN BEGIN
            count_data=count_data+1
            IF (dewpoints(m) GT temperatures(m)) THEN BEGIN
            ;********************************
            ; loop through all observations in monthly batches
            ;********************************

                IF KEYWORD_SET(mask) THEN BEGIN
                    ;********************************
                    ; if set to mask, then do
                    ;********************************
                    ;********************************
                    ; Temperatures no longer masked by this test
                    ;  27/01/2011 RJHD & KW
                    ;********************************
                    ; temperatures(m)=temperatures_mdi

                    flagged_values[1,m]=dewpoints[m]
                    dewpoints(m)=dewpoints_fdi
                ENDIF
                ;********************************
                ; only bad points flagged - old ones not wiped
                ;********************************
                qc_flags(30,m)=1
                ;t_qc_flags(m)=1
                dp_qc_flags(m)=1

                count_SSS=count_SSS+1
                IF KEYWORD_SET(diagnostics) THEN BEGIN
            
                    RANGE=30

                    IF m<RANGE THEN MIN_X=0 ELSE MIN_X=m-RANGE
                    IF ntimes-1-m<RANGE THEN MAX_X=ntimes-1 ELSE MAX_X=m+RANGE           

                    set_plot,'PS'
                    tvlct,0,0,0,0 ; black for kept datapoints
                    cd, "./", current=outdirplot
                    device,filename=outdirplot+'/images/'+station_id+'_T_TW_'+string(FigCounter,FORMAT='(I2.2)')+'SSS_indiv.eps'$
                      ,/color,/encapsulate,/helvetica,/bold,/landscape,ysize=20,xsize=26
                    !P.Font=0
                    !P.Thick=4
                    !P.Multi=[0,1,1]
                    plotsym,3,1.2

                    temp_temps=temperatures(MIN_X:MAX_X)
                    temp_dps=dewpoints(MIN_X:MAX_X)
                    useful_temps=WHERE((temp_temps NE temperatures_mdi) AND (temp_temps NE temperatures_fdi))
                    useful_dps=WHERE((temp_dps NE dewpoints_mdi) AND (temp_dps NE dewpoints_fdi))
                    min_y=min(temp_temps(useful_temps),temp_dps(useful_dps))
                    max_y=max(temp_temps(useful_temps),temp_dps(useful_dps))

                    IF min_y LT 0 THEN min_y=min_y*1.1 ELSE min_y=min_y*0.9
                    IF max_y LT 0 THEN max_y=max_y*0.9 ELSE max_y=max_y*1.1

    
                    dummy=LABEL_DATE(DATE_FORMAT=['%D/%N/%Y'])
                    start_date=JULDAY(1, 1, 1973, 0)


                    plot,times(MIN_X:MAX_X)+start_date,temperatures(MIN_X:MAX_X),$
                      min_value=-100,xstyle=1,ystyle=1,ytitle="temperature",xtitle="time",xtickformat='LABEL_DATE',$
                      title="T-TW_Check - Diagnostic Timeseries",charsize=1,ythick=4,xthick=4,psym=2,yrange=[min_y,max_y], POSITION=[0.10, 0.10, 0.90, 0.90]
                    oplot,times(MIN_X:MAX_X)+start_date,temperatures(MIN_X:MAX_X),$
                      min_value=-100,xstyle=1,ystyle=1,ytitle="temperature",xtitle="time",xtickformat='LABEL_DATE',$
                      title="T-TW_Check - Diagnostic Timeseries",charsize=1,ythick=4,xthick=4,psym=0, POSITION=[0.10, 0.10, 0.90, 0.90]
                    
                    oplot,times(MIN_X:MAX_X)+start_date,dewpoints(MIN_X:MAX_X),$
                      min_value=-100,xstyle=1,ystyle=1,ytitle="temperature",xtitle="time",xtickformat='LABEL_DATE',$
                      title="T-TW_Check - Diagnostic Timeseries",charsize=1,ythick=4,xthick=4,psym=2,yrange=[min_y,max_y],color=79, POSITION=[0.10, 0.10, 0.90, 0.90]
                    oplot,times(MIN_X:MAX_X)+start_date,dewpoints(MIN_X:MAX_X),$
                      min_value=-100,xstyle=1,ystyle=1,ytitle="temperature",xtitle="time",xtickformat='LABEL_DATE',$
                      title="T-TW_Check - Diagnostic Timeseries",charsize=1,ythick=4,xthick=4,psym=0,color=79, POSITION=[0.10, 0.10, 0.90, 0.90]
                    
                    oplot,[times(m)+start_date],[temperatures(m)],min_value=-100,xtickformat='LABEL_DATE',$
                      xstyle=1,ystyle=1,ytitle="temperature",xtitle="time", title="T-TW_Check - Diagnostic Timeseries", $
                      charsize=1,ythick=4,xthick=4,psym=4,color=239, POSITION=[0.10, 0.10, 0.90, 0.90]
                    oplot,[times(m)+start_date],[dewpoints(m)],min_value=-100,xtickformat='LABEL_DATE',$
                      xstyle=1,ystyle=1,ytitle="temperature",xtitle="time", title="T-TW_Check - Diagnostic Timeseries", $
                      charsize=1,ythick=4,xthick=4,psym=4,color=239, POSITION=[0.10, 0.10, 0.90, 0.90]
                    
                    device,/close
                    
                    FigCounter=FigCounter+1
                ENDIF

                ;PRINT,'Supersaturation'
            ENDIF
        ENDIF
    ENDFOR
    IF count_data GT 0 THEN BEGIN
        IF ((count_SSS/count_data) GE 0.2) THEN BEGIN
        ;********************************
        ; sufficient bad data (20%) within the month to scrap whole month
        ;********************************
            bads=WHERE(((dewpoints(monthly_segment_starts(t):monthly_segment_ends(t)) NE dewpoints_mdi) AND $
                        (dewpoints(monthly_segment_starts(t):monthly_segment_ends(t)) NE dewpoints_fdi)),cbads)
        ;********************************
        ; only data present flagged as missing KATE
        ;********************************
            IF KEYWORD_SET(mask) THEN BEGIN
            ;********************************
            ; if set to mask, then do
            ;********************************
                ;********************************
                ; Temperatures no longer masked by this test
                ;  27/01/2011 RJHD & KW
                ;********************************
                ;temperatures(bads+monthly_segment_starts(t))=temperatures_mdi
                flagged_values[1,bads+monthly_segment_starts(t)]=dewpoints(bads+monthly_segment_starts(t))
                dewpoints(bads+monthly_segment_starts(t))=dewpoints_fdi
            ENDIF
            ;********************************
            ; only bad points flagged - old ones not wiped
            ;********************************
            qc_flags(30,bads+monthly_segment_starts(t))=1

            ;t_qc_flags(bads+monthly_segment_starts(t))=1
            dp_qc_flags(bads+monthly_segment_starts(t))=1

            IF KEYWORD_SET(diagnostics) THEN BEGIN
                
                
                set_plot,'PS'
                tvlct,0,0,0,0   ; black for kept datapoints
                cd, "./", current=outdirplot
                device,filename=outdirplot+'/images/'+station_id+'_T_TW_'+'_'+string(FigMCounter,FORMAT='(I2.2)')+'SSS_month.eps'$
                  ,/color,/encapsulate,/helvetica,/bold,/landscape,ysize=20,xsize=26
                !P.Font=0
                !P.Thick=4
                !P.Multi=[0,1,1]
                plotsym,3,1.2
                
                temp_temps=temperatures(monthly_segment_starts(t):monthly_segment_ends(t))
                temp_dps=dewpoints(monthly_segment_starts(t):monthly_segment_ends(t))
                useful_temps=WHERE((temp_temps NE temperatures_mdi) AND (temp_temps NE temperatures_fdi))
                useful_dps=WHERE((temp_dps NE dewpoints_mdi) AND (temp_dps NE dewpoints_fdi))
                min_y=min(temp_temps(useful_temps),temp_dps(useful_dps))*0.9
                max_y=max(temp_temps(useful_temps),temp_dps(useful_dps))*1.1
                
                plot,times(monthly_segment_starts(t):monthly_segment_ends(t))+start_date,$
                  temperatures(monthly_segment_starts(t):monthly_segment_ends(t)),$
                  min_value=-100,xstyle=1,ystyle=1,ytitle="temperature",xtitle="time",xtickformat='LABEL_DATE',$
                  title="T-TW_Check - Diagnostic Timeseries - SSS",charsize=1,ythick=4,xthick=4,psym=2,yrange=[min_y,max_y], POSITION=[0.10, 0.10, 0.90, 0.90]
                oplot,times(monthly_segment_starts(t):monthly_segment_ends(t))+start_date,$
                  temperatures(monthly_segment_starts(t):monthly_segment_ends(t)),$
                  min_value=-100,xstyle=1,ystyle=1,ytitle="temperature",xtitle="time",xtickformat='LABEL_DATE',$
                  title="T-TW_Check - Diagnostic Timeseries - SSS",charsize=1,ythick=4,xthick=4,psym=0, POSITION=[0.10, 0.10, 0.90, 0.90]
                
                plot,times(monthly_segment_starts(t):monthly_segment_ends(t))+start_date,$
                  dewpoints(monthly_segment_starts(t):monthly_segment_ends(t)),$
                  min_value=-100,xstyle=1,ystyle=1,ytitle="temperature",xtitle="time",xtickformat='LABEL_DATE',$
                  title="T-TW_Check - Diagnostic Timeseries - SSS",charsize=1,ythick=4,xthick=4,psym=2,yrange=[min_y,max_y],color=79, POSITION=[0.10, 0.10, 0.90, 0.90]
                oplot,times(monthly_segment_starts(t):monthly_segment_ends(t))+start_date,$
                  dewpoints(monthly_segment_starts(t):monthly_segment_ends(t)),$
                  min_value=-100,xstyle=1,ystyle=1,ytitle="temperature",xtitle="time",xtickformat='LABEL_DATE',$
                  title="T-TW_Check - Diagnostic Timeseries - SSS",charsize=1,ythick=4,xthick=4,psym=0,color=79, POSITION=[0.10, 0.10, 0.90, 0.90]
                
                oplot,times(bads+monthly_segment_starts(t))+start_date,temperatures(bads+monthly_segment_starts(t)),min_value=-100,xtickformat='LABEL_DATE',$
                  xstyle=1,ystyle=1,ytitle="temperature",xtitle="time", title="T-TW_Check - Diagnostic Timeseries - SSS", $
                  charsize=1,ythick=4,xthick=4,psym=4,color=239, POSITION=[0.10, 0.10, 0.90, 0.90]
                oplot,times(bads+monthly_segment_starts(t))+start_date,dewpoints(bads+monthly_segment_starts(t)),min_value=-100,xtickformat='LABEL_DATE',$
                  xstyle=1,ystyle=1,ytitle="temperature",xtitle="time", title="T-TW_Check - Diagnostic Timeseries - SSS", $
                  charsize=1,ythick=4,xthick=4,psym=4,color=239, POSITION=[0.10, 0.10, 0.90, 0.90]
                
                device,/close
            
                FigMCounter=FigMCounter+1
            ENDIF
        ENDIF  
    ENDIF
ENDFOR
FigCounter=0
FigMCounter=0

points=WHERE(qc_flags(30,*) GT 0, bad_count)
printstring="Cross Check T & TW Flags"
IF KEYWORD_SET(oldflags) THEN BEGIN
    PRINT,printstring,1,':', bad_count-pfcnt,'  Old Flags:',pfcnt, FORMAT='(A-38,I1,A1,I8,A-15,I8)'
ENDIF ELSE BEGIN
    PRINT,printstring,1,':', bad_count, FORMAT='(A-38,I1,A1,I8)'
ENDELSE


;************************************************************************
;2: Zero DPD removal  (Dew Point Depression)
; count continuous strings of zero DPD
; if strings exceed 24 hours, and this must include at least 4 obs then remove string
; if string exceeds any 6 hour period without an ob 
; (for example: when there is a lot of missing data) then the flag
; will be cancelled, unless the first 12+ hours have 4+ obs
;************************************************************************
IF KEYWORD_SET(oldflags) THEN BEGIN
    prev_flags=WHERE(qc_flags(31,*) GT 0, pfcnt)
ENDIF


;************************************************************************
; make DPD timeseries - only where temp and tdew are both not missing
;************************************************************************
dpds=DBLARR(ntimes)
dpds(*)=temperatures_mdi
valid_ts=WHERE((temperatures NE temperatures_mdi) AND (temperatures NE temperatures_fdi))
valid_both=WHERE((dewpoints(valid_ts) NE dewpoints_mdi) AND (dewpoints(valid_ts) NE dewpoints_fdi))
dpds(valid_ts(valid_both))=temperatures(valid_ts(valid_both))-dewpoints(valid_ts(valid_both))

last_dpds=0.
strstarttime=times(0)
startloc=0
FOR t=1L,ntimes-1 DO BEGIN
    IF (dpds(t) NE temperatures_mdi) AND (dpds(t) NE temperatures_fdi) THEN BEGIN
        IF dpds(t) NE last_dpds THEN BEGIN 
            ;********************************
            ; No longer constant
            ;********************************
            ; Is the difference between the string start and now >24 hours?
            ;********************************
            IF times(t-1)-strstarttime GE 1. THEN BEGIN
                ;PRINT,times(t-1),strstarttime,times(t-1)-strstarttime,last_dpds
                temparr=dpds(startloc:t-1)
                pts=WHERE(((temparr NE temperatures_mdi) AND (temparr NE temperatures_fdi)), ptscnt)
                ;********************************
                ; modified to turn only data present to mdi/flag rather than all within the string KATE
                ;********************************

                IF temperatures(t) GE 0 THEN absdiff=0.25 ELSE absdiff=1.
                IF (ptscnt GE 4) AND (ABS(last_dpds) LE absdiff) THEN BEGIN
                    ;********************************
                    ; Use the significant weather, precipitation and 
                    ; base keywords to find periods of precipitation/fog
                    ;********************************
                    sigw_subarr=past_sigwx1(startloc:t-1)
                    precip_subarr=precip1(startloc:t-1)
                    cloudbase_subarr=cloud_base(startloc:t-1)
                    fogs=WHERE((sigw_subarr(pts) GE 4) OR (precip_subarr(pts) GT 0.0) $
                               OR ((cloudbase_subarr LT 1000.) AND (cloudbase_subarr GT 0.)),cfog) 
                    ;********************************
                    ; find fog / precip events where there are dpds
                    ;********************************
                    IF cfog GE 1 THEN BEGIN
                        IF ((FLOAT(cfog)/FLOAT(ptscnt)) LT 0.333) THEN BEGIN 
                            ;********************************
                            ; if less than 1/3rd of values correspond with fog/precip treat as wick drying
                            ;********************************
                            IF KEYWORD_SET(mask) THEN BEGIN
                                ;********************************
                                ; if set to mask, then do
                                ;********************************
                                ;********************************
                                ; Temperatures no longer masked by this test
                                ;  27/01/2011 RJHD & KW
                                ;********************************
                                ; temperatures(pts+startloc)=temperatures_mdi
                                flagged_values[1,pts+startloc]=dewpoints(pts+startloc)
                                dewpoints(pts+startloc)=dewpoints_fdi
                             ENDIF
                            ;********************************
                            ; only bad points flagged - old ones not wiped
                            ;********************************
                            qc_flags(31,pts+startloc)=1

                            ;t_qc_flags(pts+startloc)=1
                            dp_qc_flags(pts+startloc)=1


	                    ;PRINT,'Dew point depression'
                        ENDIF
                    ENDIF ELSE BEGIN
                        IF KEYWORD_SET(mask) THEN BEGIN
                            ;********************************
                            ; if set to mask, then do
                            ;********************************
                            ;********************************
                            ; Temperatures no longer masked by this test
                            ;  27/01/2011 RJHD & KW
                            ;********************************
                            ; temperatures(pts+startloc)=temperatures_mdi
                            flagged_values[1,pts+startloc]=dewpoints(pts+startloc)
                            dewpoints(pts+startloc)=dewpoints_mdi
                        ENDIF
                        ;********************************
                        ; only bad points flagged - old ones not wiped
                        ;********************************
                        qc_flags(31,pts+startloc)=1

                        ;t_qc_flags(pts+startloc)=1
                        dp_qc_flags(pts+startloc)=1

                        IF KEYWORD_SET(diagnostics) THEN BEGIN
            
                            RANGE=30
                            
                            IF startloc-RANGE<0 THEN MIN_X=0 ELSE MIN_X=startloc-RANGE
                            IF ntimes-1-t<RANGE THEN MAX_X=ntimes-1 ELSE MAX_X=t+RANGE           

                            set_plot,'PS'
                            tvlct,0,0,0,0 ; black for kept datapoints
                            cd, "./", current=outdirplot
                            device,filename=outdirplot+'/images/'+station_id+'_T_TW_'+string(FigCounter,FORMAT='(I2.2)')+'DPD_indiv.eps'$
                              ,/color,/encapsulate,/helvetica,/bold,/landscape,ysize=20,xsize=26
                            !P.Font=0
                            !P.Thick=4
                            !P.Multi=[0,1,1]
                            plotsym,3,1.2
                                                        
                            start_date=JULDAY(1, 1, 1973, 0)

                            temp_temps=temperatures(MIN_X:MAX_X)
                            temp_dps=dewpoints(MIN_X:MAX_X)
                            useful_temps=WHERE((temp_temps NE temperatures_mdi) AND (temp_temps NE temperatures_fdi))
                            useful_dps=WHERE((temp_dps NE dewpoints_mdi) AND (temp_dps NE dewpoints_fdi))
                            min_y=min(temp_temps(useful_temps),temp_dps(useful_dps))*0.9
                            max_y=max(temp_temps(useful_temps),temp_dps(useful_dps))*1.1

                            plot,times(MIN_X:MAX_X)+start_date,temperatures(MIN_X:MAX_X),$
                              min_value=-100,xstyle=1,ystyle=1,ytitle="temperature",xtitle="time",xtickformat='LABEL_DATE',$
                              title="T-TW_Check - Diagnostic Timeseries - DPD",charsize=1,ythick=4,xthick=4,psym=2,yrange=[min_y,max_y], POSITION=[0.10, 0.10, 0.90, 0.90]
                            oplot,times(MIN_X:MAX_X)+start_date,temperatures(MIN_X:MAX_X),$
                              min_value=-100,xstyle=1,ystyle=1,ytitle="temperature",xtitle="time",xtickformat='LABEL_DATE',$
                              title="T-TW_Check - Diagnostic Timeseries - DPD",charsize=1,ythick=4,xthick=4,psym=0, POSITION=[0.10, 0.10, 0.90, 0.90]
                            
                            oplot,times(MIN_X:MAX_X)+start_date,dewpoints(MIN_X:MAX_X),$
                              min_value=-100,xstyle=1,ystyle=1,ytitle="temperature",xtitle="time",xtickformat='LABEL_DATE',$
                              title="T-TW_Check - Diagnostic Timeseries - DPD",charsize=1,ythick=4,xthick=4,psym=2,yrange=[min_y,max_y],color=79, POSITION=[0.10, 0.10, 0.90, 0.90]
                            oplot,times(MIN_X:MAX_X)+start_date,dewpoints(MIN_X:MAX_X),$
                              min_value=-100,xstyle=1,ystyle=1,ytitle="temperature",xtitle="time",xtickformat='LABEL_DATE',$
                              title="T-TW_Check - Diagnostic Timeseries - DPD",charsize=1,ythick=4,xthick=4,psym=0,color=79, POSITION=[0.10, 0.10, 0.90, 0.90]
                            
                            oplot,[times(pts+startloc)+start_date],[temperatures(pts+startloc)],min_value=-100,xtickformat='LABEL_DATE',$
                              xstyle=1,ystyle=1,ytitle="temperature",xtitle="time", title="T-TW_Check - Diagnostic Timeseries - DPD", $
                              charsize=1,ythick=4,xthick=4,psym=4,color=239, POSITION=[0.10, 0.10, 0.90, 0.90]
                            oplot,[times(pts+startloc)+start_date],[dewpoints(pts+startloc)],min_value=-100,xtickformat='LABEL_DATE',$
                              xstyle=1,ystyle=1,ytitle="temperature",xtitle="time", title="T-TW_Check - Diagnostic Timeseries - DPD", $
                              charsize=1,ythick=4,xthick=4,psym=4,color=239, POSITION=[0.10, 0.10, 0.90, 0.90]
                            
                            device,/close
                            
                            FigCounter=FigCounter+1
                        ENDIF
                    ENDELSE
                ENDIF
            ENDIF
            strstarttime=times(t)
            startloc=t
            last_dpds=dpds(t)
        ENDIF
    ENDIF
ENDFOR

points=WHERE(qc_flags(31,*) GT 0, bad_count)
printstring="Cross Check T & TW Flags"
IF KEYWORD_SET(oldflags) THEN BEGIN
    PRINT,printstring,2,':', bad_count-pfcnt,'  Old Flags:',pfcnt, FORMAT='(A-38,I1,A1,I8,A-15,I8)'
ENDIF ELSE BEGIN
    PRINT,printstring,2,':', bad_count, FORMAT='(A-38,I1,A1,I8)'
ENDELSE
  
;************************************************************************
;3: Cutoffs
; read in data for each month
; bin into 10 degree bins -80 to -70, -70 to -60 etc
; calculate % of data for which T is present but Tdew is not
; if this exceeds 50% for anyone month, remove it.
;

;***
; need to loop through all months
; get month of t and tdew data
; filter data into T bins of 10 degrees from -80 to 60 = 14 bins
; calc % of missing data for each bin
; if  >50% then remove month
;************************************************************************

IF KEYWORD_SET(oldflags) THEN BEGIN
    prev_flags=WHERE(qc_flags(32,*) GT 0, pfcnt)
ENDIF

binval=[-90.,-80.,-70.,-60.,-50.,-40.,-30.,-20.,-10.,0.,10.,20.,30.,40.,50.,60.,70.]
FOR i=0,nmonths-1 DO BEGIN

    ; put a check in to ensure that reporting frequency of T and Tdew are vaguely similar, if not then don't test
    IF monthly_segment_lengths(i) GE 112 THEN BEGIN 
        ;********************************
        ; Assumes at least 4 /day for shortest month (10/8/2009)
        ;********************************
        ; Set up monthly arrays of data
        ;********************************
        tmon_arr=0              ; vals reused for each month
        tdmon_arr=0
        tbins=0                 ; vals reused for each bin
        tdewbins=0
        tdewtheres=0
        subtims=times(monthly_segment_starts(i):monthly_segment_ends(i))
        tmon_arr=temperatures(monthly_segment_starts(i):monthly_segment_ends(i))
        tdmon_arr=dewpoints(monthly_segment_starts(i):monthly_segment_ends(i))
        qc_flags_this_mon=qc_flags(monthly_segment_starts(i):monthly_segment_ends(i))
        
        j=1
        contloop='yes'
        WHILE contloop EQ 'yes' DO BEGIN
            ;********************************
            ; Find number of observations within the 10C bin
            ;********************************
            vals=WHERE((tmon_arr GE binval(j-1)) AND (tmon_arr LT binval(j)),numcount)
            IF (numcount GT 20) THEN BEGIN
                tbins=tmon_arr(vals)
                tdewbins=tdmon_arr(vals)
                pres=WHERE(((tbins NE temperatures_mdi) AND (tbins NE temperatures_fdi)),Tcount)
                IF (Tcount GE 1) THEN BEGIN ; Changed 0 to 1 10/8/2009
                    tdewtheres=tdewbins(pres)
                    pres=WHERE(((tdewtheres NE dewpoints_mdi) AND (tdewtheres NE dewpoints_fdi)),Dcount)
                    rats=0.     ; Changed 10/8/2009
                    IF (Dcount NE 0) THEN rats=100.-((FLOAT(Dcount)/FLOAT(Tcount))*100.) ; get % of missing Tdew data
                    IF (rats GE 50.) THEN BEGIN
                        ;********************************
                        ; checking that monthly values are at similar temporal resolution
                        ;********************************
                        present_t=WHERE(((tmon_arr NE temperatures_mdi) AND (tmon_arr NE temperatures_fdi)),tpcnt)
                        t_resids=subtims(present_t)-FIX(subtims(present_t))
                        t_hours=n_elements(UNIQ(t_resids(SORT(t_resids))))
                        present_d=WHERE(((tdmon_arr NE dewpoints_mdi) AND (tdmon_arr NE dewpoints_fdi)),tdpcnt)
                        td_resids=subtims(present_d)-FIX(subtims(present_d))
                        td_hours=n_elements(UNIQ(td_resids(SORT(td_resids))))
                        ;********************************
                        ; only continue if reporting hours and total number of obs are sufficiently similar between t and tdew
                        ;********************************
                        IF (t_hours NE td_hours) AND (FLOAT(tdpcnt)/FLOAT(tpcnt) LT 0.666) THEN BEGIN
                            BREAK
                        ENDIF ELSE BEGIN
                            ;********************************
                            ; only bad points flagged - old ones not wiped
                            ;********************************
                            qc_flags(32,present_d+monthly_segment_starts(i))=1

                            ;t_qc_flags(present_d+monthly_segment_starts(i))=1
                            dp_qc_flags(present_d+monthly_segment_starts(i))=1


                            IF KEYWORD_SET(mask) THEN BEGIN
                                 ;********************************
                                 ; if set to mask, then do
                                 ;********************************
                                 ;********************************
                                 ; Temperatures no longer masked by this test
                                 ;  27/01/2011 RJHD & KW
                                 ;********************************
                                 ;temperatures(present_d+monthly_segment_starts(i))=temperatures_mdi ; only flagging present values KATE
                                 flagged_values[1,present_d+monthly_segment_starts(i)]=dewpoints(present_d+monthly_segment_starts(i))
                                 dewpoints(present_d+monthly_segment_starts(i))=dewpoints_mdi ; only flagging present values KATE
                            ENDIF
                        ENDELSE
	    ;PRINT,'Cutoffs',i,j
                        contloop='no'
	    
                    ENDIF
                ENDIF
            ENDIF
            j=j+1
            IF j GT 16 THEN contloop='no'
        ENDWHILE
    ENDIF
ENDFOR
points=WHERE(qc_flags(32,*) GT 0, bad_count)
printstring="Cross Check T & TW Flags"
IF KEYWORD_SET(oldflags) THEN BEGIN
    PRINT,printstring,3,':', bad_count-pfcnt,'  Old Flags:',pfcnt, FORMAT='(A-38,I1,A1,I8,A-15,I8)'
ENDIF ELSE BEGIN
    PRINT,printstring,3,':', bad_count, FORMAT='(A-38,I1,A1,I8)'
ENDELSE


RETURN

END
;************************************************************************
;                                 END
;************************************************************************
