;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;British Crown Copyright (c) 2012, the Met Office All rights reserved.
;
;Redistribution and use in source and binary forms, with or without 
;modification, are permitted provided that the following conditions 
;are met:
;
;Redistributions of source code must retain the above copyright notice, 
;this list of conditions and the following disclaimer. 
;
;Redistributions in binary form must reproduce the above copyright 
;notice, this list of conditions and the following disclaimer in the 
;documentation and/or other materials provided with the distribution. 
;
;Neither the name of the Met Office nor the names of its contributors 
;may be used to endorse or promote products derived from this software 
;without specific prior written permission. 
;
;THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
;"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
;LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
;A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
;HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
;SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
;LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
;DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
;THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
;(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
;OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

PRO detect_outliers, first_times, second_times, first_series, second_series, $
                     dubious_count, present_count, variable_mdi, variable_fdi, diagnostics, station_id,neighbour_id,label,$
                     neigh_number,all_series,neigh_flags,neighbour_dist=neighbour_dist
;
; Subroutine written to detect outliers in the distribution of differences between a candidate station and a
; neighbour series. Has to do a bunch of subsetting and incrementing data availability counters before
; trying to ascertain where the potentially dubious data are and incrementing this counter. Will only apply to 
; differences which we expect to be normally distributed (so not the wind variables).
;
;
;*************************************************************
; NAME:
;     DETECT_OUTLIERS
;
; PURPOSE:
;     Find outliers in the distribution of differences between a
;     candidate station and a neighbouring one.
;
; CALLING SEQUENCE:
;     detect_outliers, first_times, second_times, first_series,
;     second_series,dubious_count, present_count, variable_mdi,
;     diagnostics, station_id,label,neigh_number, all_series, neighbour_dist=neighbour_dist
; 
; INPUTS:
;     first_times - target time series
;
;     second_times - neighbouring time series
;
;     first_series - target data series
;
;     second_series - neighbouring time series
;
;     dubious_count - array containing the flags for the dubious observations
;
;     present_count - array containing flags showing where data
;                     matched between the series
;
;     variable_mdi - missing data indicator
;
;     variable_fdi - flagged data indicator
;
;     diagnostics - keyword to turn on diagnostic information
;
;     station_id - station ID for the plots
;
;     label - plot label - the data type
;
;     neigh_number - which neighbour are we dealing with - for filling
;                    the all_series array
;
;     all_series - array to contain the timeseries of all the data
;                  which matches the series being compared.  For
;                  diagnostic plotting in external script
;
;     neigh_flags - array containing the summed flag values for each
;                   observation for the neighbour.  Only values
;                   equalling Zero are definitely clean.
;     
;     neighbour_dist - distance from the target to the neighbouring
;                      station - for storm exclusion.
;
; METHOD:
;     If there are sufficient clean neighbour values, find the
;     distribution of the differences between the sequences and use
;     that to find outliers which are noted.  These are then all
;     compared and combined from all neighbours so that only if
;     sufficient of the neighbours say a value is bad, is the flag set.
;     
;
; OUTPUTS:
;
;     None - files written to specified directory
;
; RESTRICTIONS:
;
;     Assumes netcdf file inputs.
;   
; HISTORY:
;     #56 - initial upload - 16 November 2010 RJHD
;     #57 - Commented out and added diagnostic histogram
;           16 November 2010 RJHD 
;     #92 - Commented out and added diagnostic histogram
;           19 November 2010 RJHD 
;     #115 - Added section to disregard values from neighbours which
;            are flagged.  If non-cleaned neighbours are used, all
;            flags are set to zero in parent script.
;           30 November 2010 RJHD 
;     #237 - SVN information added to header
;            16 November 2011 RJHD
;     #238 - Added fdi tests and functionality
;            16 November 2011 RJHD
;
;************************************************************************
;                    SVN Info
;$Rev:: 303                                           $:  Revision of last commit
;$Author:: rdunn                                      $:  Author of last commit
;$Date:: 2012-06-14 14:02:44 +0100 (Thu, 14 Jun 2012) $:  Date of last commit
;************************************************************************
;                                 START
;************************************************************************

;**********************************************************
; First match the time arrays, after excluding the flags and making a
; copy to return to the parent script at the end
;**********************************************************

untouched_second_series=second_series
untouched_second_times=second_times

;**********************************************************
; Remove the flags from the neighbour series.
;**********************************************************
second_series=second_series(WHERE(neigh_flags EQ 0))
second_times=second_times(WHERE(neigh_flags EQ 0))


MATCH,first_times,second_times,timelocs1,timelocs2,count=ntimematches


;**********************************************************
; Fill the all_series array for diagnostic plotting
;   Flags have already been set and removed
;**********************************************************
all_series(neigh_number,timelocs1)=second_series(timelocs2)


;**********************************************************
; If sufficient matching times between the series'
;**********************************************************
IF ntimematches GE 100 THEN BEGIN
    ;**********************************************************
    ; Then form a subset
    ;**********************************************************

    matched_series=FLTARR(ntimematches,2)
    matched_series(*,0)=first_series(timelocs1)
    matched_series(*,1)=second_series(timelocs2)
  
    ;**********************************************************
    ; Calculate a difference series
    ;**********************************************************
  
    differences=FLTARR(ntimematches)
    differences(*)=variable_mdi
                                ;
    FOR t=0L,ntimematches-1 DO BEGIN
        IF (matched_series(t,0) NE variable_mdi) AND (matched_series(t,1) NE variable_mdi) AND $
           (matched_series(t,0) NE variable_fdi) AND (matched_series(t,1) NE variable_fdi) THEN $
          differences(t)=matched_series(t,0)-matched_series(t,1)
    ENDFOR
    
    ;**********************************************************
    ; Where these are legal values increment present_count array
    ;**********************************************************
  
    validpnts=WHERE(((differences NE variable_mdi) AND (differences NE variable_fdi)), datamatches)

    ;**********************************************************
    ; Plot diagnostic histogram
    ;**********************************************************
    IF KEYWORD_SET(diagnostics) THEN BEGIN
        min_val=FIX(MIN(differences(validpnts)))-1 ; just in case it is negative and doesn't work
        max_val=CEIL(MAX(differences(validpnts)))

        binsize=1.0
        hist = HISTOGRAM(differences(validpnts), BINSIZE=binsize,LOCATION=loc, MAX=max_val, MIN=min_val)
        hist=[0,0,0,temporary(hist),0,0,0] 
        
        
        ;******************************************
        ; adjust for padding in histogram
        ;******************************************
        min_bin=MIN(loc)
        max_bin=MAX(loc)
        binns=[min_bin-(3.*binsize),min_bin-(2.*binsize),min_bin-binsize,loc,max_bin+binsize,max_bin+(2.*binsize),max_bin+(3.*binsize)]+(binsize/2)
        set_plot,'PS'
        cd, "./", current=outdirplot
        device,filename=outdirplot+'/images/'+station_id+'_'+neighbour_id+'_Outlier_'+label+'.eps'$
          ,/color,/encapsulate,/helvetica,/bold,/landscape,ysize=20,xsize=26
        !P.Font=0
        !P.Thick=4
        !P.Multi=[0,1,1]
        plotsym,3,1.2
        plot,binns,hist,min_value=-100,xstyle=1,ystyle=1,ytitle="Frequency",xtitle='differences',$
          title="Outlier_Check - Diagnostic Histogram - "+label,charsize=1,ythick=4,xthick=4,psym=10,YLog=1,YRange=[0.1,max(hist)*5], POSITION=[0.10, 0.10, 0.90, 0.90]

    ENDIF

    IF datamatches GE 100 THEN BEGIN
        present_count(timelocs1(validpnts))=present_count(timelocs1(validpnts))+1
    
        ;**********************************************************
        ; Find the inter-quartile range of the differences.
        ;**********************************************************
    
        subset_differences=differences(validpnts)
        subset_differences=subset_differences[SORT(subset_differences)]
        interq=ABS(subset_differences(LONG(datamatches/4.))-subset_differences(LONG((3.*datamatches)/4.)))
        IF interq LE 2. THEN interq=2.

        IF KEYWORD_SET(diagnostics) THEN BEGIN
            ;******************************************
            ;overplot fitted gaussian
            ;******************************************
            ; 1.5IQR ~ 1 sigma
            nonzero=WHERE(hist NE 0)
            yfit = GaussFit(binns(nonzero), hist(nonzero), coeff, NTERMS=3, ESTIMATES=[max(hist),0.0,interq])
            oplot, binns(nonzero), yfit, COLOR=40, THICK=2
            oplot,[5.*interq,5.*interq],[0.1,max(hist)*5],psym=0,color=240,YLog=1
            oplot,[-5.*interq,-5.*interq],[0.1,max(hist)*5],psym=0,color=240,YLog=1
        ENDIF
    
        ;**********************************************************
        ; Find the locations where the difference is greater than 5 inter-quartile range values
        ;**********************************************************
    
        dubious_points=WHERE(ABS(differences(validpnts)/interq) GE 5., dubcnt) ;***KATE Jul 2010 changed 3. to 5.

        IF (dubcnt GE 1) THEN BEGIN
            
            ;**********************************************************
            ; SLP only
            ;**********************************************************
            IF KEYWORD_SET(neighbour_dist) THEN BEGIN ; we should have a neighbour distance if SLP
              ;**********************************************************
              ; find clusters of outliers where station is negative 
              ; relative to the neighbour - if neighbour is >100km away then ignore
              ;**********************************************************
                positive_dubs=WHERE((differences(validpnts(dubious_points))/interq) GE 5., poscnt)
                negative_dubs=WHERE((differences(validpnts(dubious_points))/interq) LE -5., negcnt)
                
                IF (poscnt/FLOAT(dubcnt) LT 0.333) AND (neighbour_dist GT 100.) THEN BEGIN	
                    ;**********************************************************
                    ; no point looking if there really aren't any hurricanes!
                    ;**********************************************************
                    ; IF 2/3 majority are negative and station is > 100km then 
                    ; assume these are storms so ignore all negatives but flag +ves
                    ;**********************************************************
                    IF (poscnt GT 0) THEN dubious_count(timelocs1(validpnts(dubious_points(positive_dubs))))=dubious_count(timelocs1(validpnts(dubious_points(positive_dubs))))+1  

                    dubcnt=0 ; reset so that dubious_count isn't over written  
                    
                    IF negcnt GE 1 THEN $
                    ;**********************************************************
                    ;  Adjust the number of flags identified.
                    ;**********************************************************
                      present_count(timelocs1(validpnts(dubious_points(negative_dubs))))=present_count(timelocs1(validpnts(dubious_points(negative_dubs))))-1    
            
                ENDIF ELSE BEGIN
                     
                    ;**********************************************************
                    ; Otherwise just flag as usual
                    ;**********************************************************
                    dubious_count(timelocs1(validpnts(dubious_points)))=dubious_count(timelocs1(validpnts(dubious_points)))+1  

                ENDELSE

           
            ;**********************************************************
            ; Temperature & Dewpoints
            ;**********************************************************
            ENDIF ELSE IF dubcnt GE 1 THEN BEGIN
                ;**********************************************************
                ; Set the dubious points in the temporary array
                ;**********************************************************
                dubious_count(timelocs1(validpnts(dubious_points)))=dubious_count(timelocs1(validpnts(dubious_points)))+1
            ENDIF
            
            
        ENDIF

    ENDIF
    IF KEYWORD_SET(diagnostics) THEN BEGIN
        device,/close
    ENDIF
ENDIF
second_series=untouched_second_series
second_times=untouched_second_times

RETURN

END
;************************************************************************
;                                 END
;************************************************************************
;
;
;NOTES for a possible extension
;
;;TOO COMPLEX!!!; look for 3 day windows of negative values - if there are more than 10 -ve values and the station is greater than 100km then ignore these
;         problem_neg_times=first_times(timelocs1(validpoints(dubious_points(negative_dubs))))
;	  dayone=FLOOR(problem_neg_times(0))
;	  dayend=FLOOR(problem_neg_times(negcnt-1))
;	  n_neg_days=(dayend-dayone)+1
;	  FOR dloop=0,n_neg_days-4 DO BEGIN	; end with three days left
