;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;British Crown Copyright (c) 2012, the Met Office All rights reserved.
;
;Redistribution and use in source and binary forms, with or without 
;modification, are permitted provided that the following conditions 
;are met:
;
;Redistributions of source code must retain the above copyright notice, 
;this list of conditions and the following disclaimer. 
;
;Redistributions in binary form must reproduce the above copyright 
;notice, this list of conditions and the following disclaimer in the 
;documentation and/or other materials provided with the distribution. 
;
;Neither the name of the Met Office nor the names of its contributors 
;may be used to endorse or promote products derived from this software 
;without specific prior written permission. 
;
;THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
;"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
;LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
;A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
;HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
;SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
;LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
;DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
;THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
;(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
;OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

PRO frequent_value_check_seasonal, last_year, value, times, mdi, fdi, qc_flags, filter_flags, type, diagnostics, station_id

; Program checks for anomalously high occurence of a given value in the context of the underlying distribution. 
; Applied to those variables for which we are reasonably confident that the underlying data is at least approximately
; normally distributed. Creates a histogram and finds uncharacteristically high spikes in a local context and removes 
; all values within the spike. Histogram approach minimises the computational cost but at expense of chucking out a
; (small) data range rather than specific values.
;
; NAME:
;     Frequent_Value_Check_Seasonal
;
; PURPOSE:
;     Checks for anomalously high occurrance of a range of values
;     (binned data) assuming an underlying normal distribution.
;     Should only be applied where a normal distribution is expected.
;     Works through all seasons, and then over the whole year too.
;     DJF is for the calendar year - ie JF+D 
;
; CALLING SEQUENCE:
;     frequent_value_check_seasonal, last_year, value, mdi, fdi, qc_flags,
;     filter_flags, type, diagnostics, station_id
;
; INPUTS:
;     last_year - the final year of the data sequence
;
;     value - array containing the data of interest
;
;     times - array containing the timeseries for the data of interest
;
;     mdi - missing data indicator
;
;     fdi - flagged data indicator
;
;     qc_flags - array containing all the quality control flags for
;                the complete script
;
;     filter_flags - array containing all flags previously set on values
;
;     type - the sort of data - used to select columns in the qc_flags
;
;     diagnostics - keyword so that the histograms used are plotted for
;                   visual inspection
;
;     station_id - the name of the station - for the plot files
;
;
; OUTPUTS:
;
;     None - arrays are returned to control script
;
; METHOD:
;
;     Creates a histogram and finds uncharacteristically high spikes
;     in a local context.  All values within the flag are spiked.
;     Histogramatical approach makes this comparatively quick, but
;     does chuck out a (small) range of data, rather than just the
;     values.  The bin in question is compared to the 3 on each side.
;     For a bin to be flagged it must:
;        1) contain more than 50% of the points of the total in the 7
;        bins
;        2) contain more than 30 points.
;
;     ADDITION - flagging now done on year by year basis using the bad
;                values highlighted in the method above to restrict
;                the search.  Reduces the amount of good data removed.
;
; RESTRICTIONS:
;
;     Assumes normal distribution for underlying population
;   
; HISTORY:
;     NB - not all revisions are noted here.
;      #239 - Copied from version 237 of frequent_value_check.  
;             18 November 2011 RJHD
;      #247 - Bug Fix - monthly extraction now working properly.
;             24 November 2011 RJHD
;      #248 - Flag setting now working properly.  Also adjusted the thresholds
;             24 November 2011 RJHD
;
;************************************************************************
;                    SVN Info
;$Rev:: 308                                           $:  Revision of last commit
;$Author:: rdunn                                      $:  Author of last commit
;$Date:: 2012-07-04 14:47:15 +0100 (Wed, 04 Jul 2012) $:  Date of last commit
;************************************************************************
;                                 START
;************************************************************************

MONTH_SETUP,last_year,nmonths,nyears,times,times,monthly_segment_starts,$
            monthly_segment_ends, monthly_segment_lengths
  
monthly_segment_lengths(nmonths-1)=744
monthly_segment_ends(nmonths-1)=monthly_segment_ends(nmonths-1)+1

monthly_segment_starts=REFORM(monthly_segment_starts,12,nyears)
monthly_segment_ends=REFORM(monthly_segment_ends,12,nyears)
monthly_segment_lengths=REFORM(monthly_segment_lengths,12,nyears)

;******************************************
; Months count from 1/Jan/1973 and end on 1/1/last_year+1
;
;******************************************

;******************************************
; Mask values with past flags
;******************************************
untouched_value=value
past_flags=WHERE(filter_flags GT 0, pfcnt)
   
IF pfcnt GE 1 THEN BEGIN
   value(past_flags)=mdi
ENDIF

;******************************************
; Obtain reporting accuracy for binsizes
;******************************************
REPORTING_ACCURACY, last_year, value,mdi,fdi,vres,remhist,times, vfreq


;******************************************
; Assign labels for plots - if required.
;******************************************
CASE type OF
   1:BEGIN
      label="temperature"
   END
   2:BEGIN
      label="dewpoints"
   END
   3:BEGIN
      label="slp"
   END
ENDCASE
   

FOR season=0,4 DO BEGIN
   CASE season OF
      0: BEGIN
         SeasonName='YEAR'
         m1=0
         m2=6
         m3=11
         threshold=30
         threshold2=20
         threshold3=10
      END
      1: BEGIN
         SeasonName='MAM'
         m1=2
         m2=3
         m3=4
         threshold=20
         threshold2=15
         threshold3=10
      END
      2: BEGIN
         SeasonName='JJA'
         m1=5
         m2=6
         m3=7
         threshold=20
         threshold2=15
         threshold3=10
      END
      3: BEGIN
         SeasonName='SON'
         m1=8
         m2=9
         m3=10
         threshold=20
         threshold2=15
         threshold3=10
      END
      4: BEGIN
         SeasonName='DJF'
         m1=11
         m2=0
         m3=1
         threshold=20
         threshold2=15
         threshold3=10
      END
   ENDCASE
   
   IF season NE 0 THEN BEGIN
      FOR yr=0,nyears-1 DO BEGIN
      ; if not for whole year, extract each month
         month1=[mdi]
         month2=[mdi]
         month3=[mdi]
         IF monthly_segment_starts(m1,yr) LT monthly_segment_ends(m1,yr) THEN month1=value(monthly_segment_starts(m1,yr):monthly_segment_ends(m1,yr))
         IF monthly_segment_starts(m2,yr) LT monthly_segment_ends(m2,yr) THEN month2=value(monthly_segment_starts(m2,yr):monthly_segment_ends(m2,yr))
         IF monthly_segment_starts(m3,yr) LT monthly_segment_ends(m3,yr) THEN month3=value(monthly_segment_starts(m3,yr):monthly_segment_ends(m3,yr))
         
         ; initialised to mdi - no effect as these removed later.


         ; ordering doesn't matter for identification
         YearValue=[month1,month2,month3]
         IF yr EQ 0 THEN BEGIN
            SeasonValue=YearValue
         ENDIF ELSE BEGIN
            SeasonValue=[SeasonValue,month1,month2,month3]
         ENDELSE
      ENDFOR
      SeasonValue=REFORM(SeasonValue)

   ENDIF ELSE BEGIN
      SeasonValue=value[monthly_segment_starts(m1,0):monthly_segment_ends(m3,nyears-1)]
   ENDELSE
      
   
;******************************************
; Only create histogram if enough observations
;   otherwise grossly undersampled
;******************************************
   non_missing=WHERE(((SeasonValue NE mdi) AND (SeasonValue NE fdi)), cnt)


   IF cnt GE 500 THEN BEGIN  
      
      subsampled_value=SeasonValue(non_missing)
      
  ;******************************************
  ; Find limits
  ; extra 0.05 added for razor-edge problems with Histogram
  ;******************************************

      min_val=FIX(MIN(subsampled_value))-1.05 ; just in case it is negative and doesn't work
      max_val=CEIL(MAX(subsampled_value))

  ;******************************************
  ; Create histogram
  ;******************************************

      IF vres LE 0.5 THEN binsize=0.5 ELSE binsize=1.0
      
      hist = HISTOGRAM(subsampled_value, BINSIZE=binsize, MAX=max_val, MIN=min_val,LOCATION=loc)

  ;******************************************
  ; Pad out the endpoints - so can scan through.
  ;******************************************

      hist=[0,0,0,temporary(hist),0,0,0] 
      min_bin=MIN(loc)
      max_bin=MAX(loc)
      binns=[min_bin-(3.*binsize),min_bin-(2.*binsize),min_bin-binsize,loc,max_bin+binsize,max_bin+(2.*binsize),max_bin+(3.*binsize)]
      
  ;******************************************
  ; Plot histogram if diagnostics are turned on
  ;******************************************

      IF KEYWORD_SET(diagnostics) THEN BEGIN

      ;******************************************
      ; adjust for padding in histogram
      ;******************************************
      
         set_plot,'PS'
         cd, "./", current=outdirplot
         device,filename=outdirplot+'/images/'+station_id+'_FrequentValue_'+label+'_'+SeasonName+'.eps'$
                ,/color,/encapsulate,/helvetica,/bold,/landscape,ysize=20,xsize=26
         !P.Font=0
         !P.Thick=4
         !P.Multi=[0,1,1]
         plotsym,3,1.2
         plot,binns+(binsize/2)+0.05,hist,min_value=-100,xstyle=1,ystyle=1,ytitle="Frequency",xtitle=label,$
              title="Frequent Value Check - Diagnostic Histogram - "+label+" "+SeasonName,charsize=1,ythick=4,$
              xthick=4,psym=10, POSITION=[0.10, 0.10, 0.90, 0.90],ylog=1,YRange=[0.1,max(hist)*5]
         
      ;******************************************
      ;overplot fitted gaussian
      ;******************************************
      ; 1.5IQR ~ 1 sigma

         yfit = GaussFit(binns+(binsize/2)+0.05, hist, coeff, NTERMS=3, ESTIMATES=[max(hist),MEDIAN(subsampled_value),MEANABSDEV(subsampled_value,/median)])
         oplot, binns+(binsize/2)+0.05, yfit, COLOR=40, THICK=2
         
         oplot, [MIN(binns+(binsize/2)+0.05),MAX(binns+(binsize/2)+0.05)],[threshold,threshold], COLOR=239, THICK=2
         
      ENDIF

      badbins=[0.]
      
      nelements=((max_val+(binsize/2.))-(min_val-(binsize/2.)))/binsize
      
      FOR e=3,nelements+2 DO BEGIN

    ;******************************************
    ;  We look just at the seven most local bins.
    ;******************************************
         temp=hist[e-3:e+3]  

    ;******************************************
    ; We only need to even look any further if this bin is the maximum
    ;   of the temporary array otherwise it isn't going to be a problem.
    ;   if the array is all zeros, then also ignore
    ;   Temporary values to avoid evaluation in loops multiple times
    ;******************************************
         TestValue=temp(3)
         TotalTestValue=TOTAL(temp)
         MaxTestValue=MAX(temp)

         IF (TestValue EQ MaxTestValue) AND (TestValue NE 0) THEN BEGIN
        ;******************************************
        ; 1) Is it the maximum of the local temporary array?
        ; 2) Is not equal to Zero?
        ;******************************************
            IF (TestValue/TotalTestValue GE 0.5) AND (TestValue GE threshold) THEN BEGIN
 
            ;******************************************
            ; 3) Does it contain more than 50% of all the data in temp?
            ; 4) a) If whole year - Are there more than thirty occurences in the bin? This reduces the chances of deleting true tails.
            ;    b) If a season   - Are there more than twenty occurences in the bin? This reduces the chances of deleting true tails.
            ;
            ; If YES to all then this bin is anomalous -- start flagging process.
            ;******************************************

            ;******************************************
            ; Ascertain the data range that we are going to kick out.
            ;******************************************
            ;range_min=min_val-0.25+(e-3)*0.5
            ;range_max=min_val+0.25+(e-3)*0.5
               range_min=binns[e]
               range_max=binns[e+1]
               
               badbins=[badbins,range_min]
               
               points_to_delete=WHERE((SeasonValue GE range_min) AND (SeasonValue LE range_max),badcnt)
            
            ;******************************************
            ; Highlight the bars in red so that they can be identified
            ;******************************************
               IF KEYWORD_SET(diagnostics) THEN BEGIN
                  plot_points=SeasonValue(points_to_delete)
                  plot_hist=HISTOGRAM(plot_points, BINSIZE=binsize, MAX=max_val, MIN=min_val)
                  plot_hist=[0,0,0,temporary(plot_hist),0,0,0] 
                  oplot,binns+(binsize/2)+0.05,plot_hist,min_value=-100,xstyle=1,ystyle=1,ytitle="Frequency",xtitle=label,$
                        title="Frequent Value Check - Diagnostic Histogram - "+label,charsize=1,ythick=4,$
                        xthick=4,psym=10, POSITION=[0.10, 0.10, 0.90, 0.90],ylog=1,YRange=[0.1,max(hist)*5],color=239
               ENDIF
               
               
               IF badcnt NE temp(3) THEN STOP, 'mismatch between histogram and range'
               
            ;qc_flags(type,points_to_delete)=1
            ENDIF
         ENDIF
         
      ENDFOR
   ENDIF
   
   IF KEYWORD_SET(diagnostics) THEN BEGIN
      device,/close
   ENDIF
   
;******************************************
; Flag on un_filtered values so revert
;******************************************
   value=untouched_value


;**************************************
; only continue if there are values highlighted
;**************************************
   IF N_ELEMENTS(badbins) GE 2 THEN BEGIN 

      badbins=badbins[1:N_ELEMENTS(badbins)-1]
    

    ;************************************************************************
    ; Having identified the bins which have particularly frequent values
    ; do the same on a yearly basis to do the flagging
    ;************************************************************************
    
      FOR yr=0,nyears-1 DO BEGIN
         plotstart=0
          ;******************************************
          ; Extract single years' values for this season
          ;******************************************
         
         ; check that there is valid data in the section to be analysed.
         ;   if DJF, as for single calendar year, rather than contiguous
         ;   winter, need to check for each bit separately
         IF ((monthly_segment_starts(m1,yr) LT monthly_segment_ends(m3,yr)) AND season NE 0) OR $
            (((monthly_segment_starts(m1,yr) LT monthly_segment_ends(m1,yr)) OR $
             (monthly_segment_starts(m2,yr) LT monthly_segment_ends(m3,yr))) AND season EQ 0) THEN BEGIN
            
            IF season NE 4 THEN BEGIN ; not DJF

               year_values=value(monthly_segment_starts(m1,yr):monthly_segment_ends(m3,yr))
            
            ;******************************************
            ; Copies the old flags across - they are not wiped
            ;******************************************
               year_flags=qc_flags(type,monthly_segment_starts(m1,yr):monthly_segment_ends(m3,yr))

            ENDIF ELSE IF ((season EQ 4) AND (monthly_segment_starts(m2,yr) GE monthly_segment_ends(m3,yr))) THEN BEGIN ; test that JF bad, but therefore D OK
               year_values=[value(monthly_segment_starts(m1,yr):monthly_segment_ends(m1,yr))]

               year_flags=qc_flags[type,monthly_segment_starts(m1,yr):monthly_segment_ends(m1,yr)]
               
            ENDIF ELSE IF ((season EQ 4) AND (monthly_segment_starts(m1,yr) GE monthly_segment_ends(m1,yr))) THEN BEGIN ; test that D bad, therefore JF OK
               year_values=[value(monthly_segment_starts(m2,yr):monthly_segment_ends(m3,yr))]

               year_flags=qc_flags[type,monthly_segment_starts(m2,yr):monthly_segment_ends(m3,yr)]
               
            ENDIF ELSE IF season EQ 4 THEN BEGIN ; DJF
               ; treat as D+JF
               year_values=[value(monthly_segment_starts(m1,yr):monthly_segment_ends(m1,yr)),$
                            value(monthly_segment_starts(m2,yr):monthly_segment_ends(m3,yr))]

               flags1=REFORM(qc_flags[type,monthly_segment_starts(m1,yr):monthly_segment_ends(m1,yr)])
               flags2=REFORM(qc_flags[type,monthly_segment_starts(m2,yr):monthly_segment_ends(m3,yr)])
               year_flags=[flags1,flags2]
               
               split_point=N_ELEMENTS(flags1)-1

            ENDIF

            good_data=WHERE(((year_values NE mdi) AND (year_values NE fdi)), gd_dta_cnt)
            
            IF gd_dta_cnt GE 100 THEN BEGIN
            
                ;******************************************
                ; Create histogram
                ;******************************************
               hist = HISTOGRAM(year_values, BINSIZE=binsize, MAX=max_val, MIN=min_val,LOCATION=loc)
            
                ;******************************************
                ; Pad out the endpoints - so can scan through.
                ;******************************************

               hist=[0,0,0,temporary(hist),0,0,0] 
               min_bin=MIN(loc)
               max_bin=MAX(loc)
               binns=[min_bin-(3.*binsize),min_bin-(2.*binsize),min_bin-binsize,loc,max_bin+binsize,max_bin+(2.*binsize),max_bin+(3.*binsize)]
               
               badbin_cnt=0
               
               FOR e=3,nelements+2 DO BEGIN
                  
                  IF badbin_cnt LE N_ELEMENTS(badbins)-1 THEN BEGIN
                     IF binns[e] EQ badbins[badbin_cnt] THEN BEGIN
                        
                            ;****************************************
                            ; Re-test the highlighted bins if they
                            ;   are still larger than expected
                            ;****************************************
                            
                        temp=hist[e-3:e+3]  
                        TestValue=temp(3)
                        TotalTestValue=TOTAL(temp)
                        MaxTestValue=MAX(temp)
                        
                        IF (TestValue EQ MaxTestValue) AND (TestValue NE 0) THEN BEGIN
                           
                           IF ((TestValue/TotalTestValue GE 0.9) AND (TestValue GE threshold3)) OR $
                              ((TestValue/TotalTestValue GE 0.5) AND (TestValue GE threshold2))  THEN BEGIN
                              range_min=binns[e]
                              range_max=binns[e+1]
                              points_to_delete=$
                                 WHERE((year_values GE range_min) AND $
                                       (year_values LE range_max),badcnt)
                              
                              IF (KEYWORD_SET(diagnostics) AND (badcnt GT 0)) THEN BEGIN
                                 
                                 
                                        ;******************************************
                                        ; adjust for padding in histogram
                                        ;******************************************
                                 IF plotstart EQ 0 THEN BEGIN
                                    set_plot,'PS'
                                    cd, "./", current=outdirplot
                                    device,filename=outdirplot+'/images/'+station_id+'_FrequentValue_'+label+'_'+string(yr,FORMAT='(I2.2)')+'_'+SeasonName+'.eps'$
                                           ,/color,/encapsulate,/helvetica,/bold,/landscape,ysize=20,xsize=26
                                    !P.Font=0
                                    !P.Thick=4
                                    !P.Multi=[0,1,1]
                                    plotsym,3,1.2
                                    plot,binns+(binsize/2)+0.05,hist,min_value=-100,xstyle=1,ystyle=1,ytitle="Frequency",xtitle=label,$
                                         title="Frequent Value Check - Diagnostic Histogram - "+label+' '+string(yr+1973,FORMAT='(I4.4)')+" "+SeasonName,$
                                         charsize=1,ythick=4,xthick=4,psym=10, POSITION=[0.10, 0.10, 0.90, 0.90],ylog=1,YRange=[0.1,max(hist)*5]
                                    
                                        ;******************************************
                                        ;overplot fitted gaussian
                                        ;******************************************
                                        ; 1.5IQR ~ 1 sigma

                                    yfit = GaussFit(binns+(binsize/2)+0.05, hist, coeff, NTERMS=3, ESTIMATES=[max(hist),MEDIAN(subsampled_value),MEANABSDEV(subsampled_value,/median)])
                                    oplot, binns+(binsize/2)+0.05, yfit, COLOR=40, THICK=2
                                    plotstart=1
                                 ENDIF
                                 
                                 plot_points=year_values(points_to_delete)
                                 plot_hist=HISTOGRAM(plot_points, BINSIZE=binsize, MAX=max_val, MIN=min_val)
                                 plot_hist=[0,0,0,temporary(plot_hist),0,0,0] 
                                 
                                 oplot,binns+(binsize/2)+0.05,plot_hist,min_value=-100,xstyle=1,ystyle=1,ytitle="Frequency",xtitle=label,$
                                       title="Frequent Value Check - Diagnostic Histogram - "+label+' '+string(yr+1973,FORMAT='(I4.4)')+" "+SeasonName,$
                                       charsize=1,ythick=4,xthick=4,psym=10, POSITION=[0.10, 0.10, 0.90, 0.90],ylog=1,YRange=[0.1,max(hist)*5],color=239
                                 
                                 oplot, [MIN(binns+(binsize/2)+0.05),MAX(binns+(binsize/2)+0.05)],[threshold2,threshold2], COLOR=239, THICK=2
                                 oplot, [MIN(binns+(binsize/2)+0.05),MAX(binns+(binsize/2)+0.05)],[threshold3,threshold3], COLOR=239, THICK=2
                                 xyouts,0.01,0.01,'Remember, will only search for bins which have been highlighted over the whole record'+$
                                        ' even if they look wierd here.',/normal
                                        
                              ENDIF
                              IF badcnt NE temp(3) THEN STOP, 'mismatch between histogram and range'
                                    ;******************************************
                                    ; Only sets flagged points to 1 -
                                    ;    old flags are not wiped
                                    ;******************************************
                              year_flags(points_to_delete)=1
                              
                           ENDIF
                        ENDIF
                        
                        badbin_cnt=badbin_cnt+1
                     ENDIF
                  ENDIF
               ENDFOR
               
               IF season NE 4 THEN BEGIN ; not DJF
                ;**********************************
                ; flags - new and old - copied back
                ;**********************************
                  qc_flags(type,monthly_segment_starts(m1,yr):monthly_segment_ends(m3,yr))=year_flags

                ;**********************************
                ; Set the filtering flags
                ;**********************************
                  yf_points=WHERE(year_flags EQ 1, yfcnt)
                  IF (yfcnt GE 1) THEN BEGIN
                     
                     temp_flags=filter_flags(monthly_segment_starts(m1,yr):monthly_segment_ends(m3,yr))
                     temp_flags(yf_points)=1
                     filter_flags(monthly_segment_starts(m1,yr):monthly_segment_ends(m3,yr))=temp_flags
                  ENDIF
               ENDIF ELSE IF ((season EQ 4) AND (monthly_segment_starts(m2,yr) GE monthly_segment_ends(m3,yr))) THEN BEGIN ; testing that JF not OK - see above
                  qc_flags(type,monthly_segment_starts(m1,yr):monthly_segment_ends(m1,yr))=year_flags

                   yf_points=WHERE(year_flags EQ 1, yfcnt)
                  IF (yfcnt GE 1) THEN BEGIN
                     temp_flags=filter_flags(monthly_segment_starts(m1,yr):monthly_segment_ends(m1,yr))
                     temp_flags(yf_points)=1
                     filter_flags(monthly_segment_starts(m1,yr):monthly_segment_ends(m1,yr))=temp_flags
                  ENDIF

               ENDIF ELSE IF ((season EQ 4) AND (monthly_segment_starts(m1,yr) GE monthly_segment_ends(m1,yr))) THEN BEGIN ; testing that D not OK - see above
                  qc_flags(type,monthly_segment_starts(m2,yr):monthly_segment_ends(m3,yr))=year_flags

                  yf_points=WHERE(year_flags EQ 1, yfcnt)
                  IF (yfcnt GE 1) THEN BEGIN
                     temp_flags=filter_flags(monthly_segment_starts(m2,yr):monthly_segment_ends(m3,yr))
                     temp_flags(yf_points)=1
                     filter_flags(monthly_segment_starts(m2,yr):monthly_segment_ends(m3,yr))=temp_flags
                  ENDIF

               ENDIF ELSE IF season EQ 4 THEN BEGIN ; DJF
                  ; treat as D+JF
                  qc_flags(type,monthly_segment_starts(m1,yr):monthly_segment_ends(m1,yr))=year_flags[0:split_point]
                  qc_flags(type,monthly_segment_starts(m2,yr):monthly_segment_ends(m3,yr))=year_flags[split_point+1:N_ELEMENTS(year_flags)-1]

                  yf_points=WHERE(year_flags EQ 1, yfcnt)
                  IF (yfcnt GE 1) THEN BEGIN
                     ; do for D and JF separately
                     temp_flags=filter_flags(monthly_segment_starts(m1,yr):monthly_segment_ends(m1,yr))
                     temp_flags(yf_points)=1
                     filter_flags(monthly_segment_starts(m1,yr):monthly_segment_ends(m1,yr))=temp_flags

                     temp_flags=filter_flags(monthly_segment_starts(m2,yr):monthly_segment_ends(m3,yr))
                     temp_flags(yf_points)=1
                     filter_flags(monthly_segment_starts(m2,yr):monthly_segment_ends(m3,yr))=temp_flags
                  ENDIF

               ENDIF
               
            ENDIF
            IF KEYWORD_SET(diagnostics) THEN BEGIN
               bad_locs=WHERE(year_flags GT 0, bad_cnt)
               PRINT, string(yr+1973, FORMAT='(I4.4)'), bad_cnt
               device,/close
            ENDIF
         ENDIF
      ENDFOR
   ENDIF
   
ENDFOR ; season

RETURN

END
;************************************************************************
;                                 END
;************************************************************************
