;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;British Crown Copyright (c) 2012, the Met Office All rights reserved.
;
;Redistribution and use in source and binary forms, with or without 
;modification, are permitted provided that the following conditions 
;are met:
;
;Redistributions of source code must retain the above copyright notice, 
;this list of conditions and the following disclaimer. 
;
;Redistributions in binary form must reproduce the above copyright 
;notice, this list of conditions and the following disclaimer in the 
;documentation and/or other materials provided with the distribution. 
;
;Neither the name of the Met Office nor the names of its contributors 
;may be used to endorse or promote products derived from this software 
;without specific prior written permission. 
;
;THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
;"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
;LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
;A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
;HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
;SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
;LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
;DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
;THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
;(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
;OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

PRO gap_check, last_year, times, h_times, values, full_times, full_h_times, full_values, mdi, fdi, column, qc_flags_gap=qc_flags_gap, $
               filter_flags, diagnostics, station_id, windsp=windsp, full_windspeeds=full_windspeeds

; Program written to identify data that clearly has a wrong station assignment. 
; Uses the distribution of values of monthly mean anomalies to assign rank bad
; months that sit way outside the distribution and removes all data in any month
; assigned as an outlier.
;
; Limitations: Assumption on first year is hardwired
;
;
; NAME:
;     GAP_CHECK
;
; PURPOSE:
;     To check for gaps in the distribution of the data.
;
; CALLING SEQUENCE:
;     gap_check, last_year, times, values, full_times, full_values, mdi, column,
;     qc_flags_gap=qc_flags_gap, diagnostics, station_id
;
; INPUTS:
;     last_year - keyword to set the last year of the observations
;
;     times - array containing the times of the observations
;
;     full_times - array containing the uncompressed times of the observations
;
;     values - array containing the values of the observations
;
;     full_values - array containing the all uncompressed values of the
;                   observations 
;
;     columns - integer describing which type of data is being
;               analysed and which column in the flags array to use
;
;     mdi - missing data indicator
;
;     fdi - flagged data indicator
;
;     qc_flags_gap - array containing the values of the flags for the
;                    test
;
;     filter_flags - array containing all flags previously set on values
;
;     diagnostics - keyword for running diagnostic plot histograms of
;                   the values to compare with the flagging. 
;
;     station_id - station name for diagnostic files
;
;     windsp - array containing the windspeed data (SLP only)
;
;     full_windspeeds - array containing the uncompressed windspeed
;                       data (SLP only)
;
; METHOD:
;     Finds the median values for all months individually.  Then
;     finds the mean and IQR of these median values for each month (all
;     Januarys, all Februarys etc) to get the climatological
;     result, and scales all the medians by these 12 means and IQRs.
;     These offsets are then sorted and, counting out from the middle,
;     pairwise comparisons are made.  Any pair which is greater than
;     1.5IQR from the mean where the difference is >2x is assumed to
;     be bad, and all further months in that tail are discarded.
;
;     NOTE - if there are bad months in the other tail, they will be
;            retained. 
;
;     ADDITION - distribution of values for all calendar months.
;                Looks for gaps outside of gaussian distribution and
;                flag values beyond.
;
;     FIRM FLAG - 1
;     TENTATIVE FLAG - 2
;
; OUTPUTS:
;
;     None - arrays returned to control script
;
; RESTRICTIONS:
;
;     Can only work on one tail per value.
;   
; HISTORY:
;      #15 - Initial version as received in October 2010 RJHD
;      #16 - Set to work on single variable, and looped by control
;            script.  Added comments.  Used Month_Setup script
;            5 November 2010 RJHD
;   by #40 - diagnostic histogram has been added.
;            10 November 2010 RJHD
;      #93 - Bug found in setting of flags in correct column of
;            array. Set so all offsets > 5IQR flagged.  Adjusted
;            diagnostic histogram to show what things are flagged by
;            which bit of the routine.
;            24 November 2010 RJHD
;      #97 - New part of the routine to flag individual observations
;            which are separated from the median by >5IQR and also
;            have a gap in the distribution of the anomalised values -
;            a bi-modal or outlier-y distribution.
;            26 November 2010 RJHD
;      #98 - Adjusted so that there has to be a gap between good and
;            bad data in the histogram, and has to be at least two
;            binwidths.  Else it was reaching the end of the
;            distribution on one side, and flagging erroneously.
;            26 November 2010 RJHD
;      #105 - Added out put of very flagged months to text file to
;             allow for further investigation.  Also adjusted
;             histogram plotting format.
;             29 November 2010 RJHD
;      #119 - Adjusted first plotting call - there has to be valid
;             data to plot
;             03 December 2010 RJHD
;      #139 - Adjusted file output of spurious stations
;             08 December 2010 RJHD
;      #155 - Updated Gap check routine to match that in the
;             climatological check.  The undoable flagging currently
;             hard coded to be off (non_gap_flag)
;             15 December 2010 RJHD
;      #171 - Added filter flags to pass through the total flags set
;             for each value
;             14 January 2011 RJHD
;      #173 - Used filtered values to find median and IQR to anomalise
;             data, but flag unfiltered values.
;             17 January 2011 RJHD
;      #179 - Added escape if all values within one bin.  They
;             shouldn't be, but just in case they are.
;             02 February 2011 RJHD
;      #181 - BUG - RJHD monthly Gap addition removes hurricanes.
;             Using SLP and windspeed find storms, extend time
;             coverage by 6hours and remove the flags.  Tested on
;             912180 and 915920
;             07 February 2011 RJHD
;      #191 - Added two level flagging system.  To match with
;             Climatological Check, Firm flags = 2, tentative flags =
;             1.  Allows for lack of windspeed information, and can
;             unset the flag later on.
;             08 March 2011 RJHD
;      #194 - Adjusted two level flagging system.  All low SLPs are
;             tentatively flagged.  Flag removed here if there is
;             suitable windspeed information.  Else the neighbours
;             have to bring it back in.
;             09 March 2011 RJHD
;      #199 - Changed MAD offset for storm threshold to 4.5 (from 4.0)
;             to match a) definition of 3sigma for gaussian
;             distribution and b) neighbour unflagging
;             11 March 2011 RJHD
;      #200 - changed last_year from optional keyword to required input
;             to remove the hardwiring of the end date
;             11 March 2011 RJHD
;      #229 - Improved labelling of diagnostic histogram
;             28 September 2011 RJHD
;      #231 - Improved labelling of diagnostic histogram (again)
;             30 September 2011 RJHD
;      #237 - SVN information added to header
;             16 November 2011 RJHD
;      #243 - Swapped numerical values of firm and tentative flags.
;             Tentative Flagging turned on.  Was never activated since
;             inclusion at Version 155.  Technically a Bug as it was
;             documented as on in all write-ups so far.
;             21 November 2011 RJHD
;      #249 - Tentative flagging turned off.  Was flagging too many
;             actual records & although part of the method was
;             documented, the tentative flagging was not.  Working as
;             before now.
;             25 November 2011 RJHD
;
;************************************************************************
;                    SVN Info
;$Rev:: 308                                           $:  Revision of last commit
;$Author:: rdunn                                      $:  Author of last commit
;$Date:: 2012-07-04 14:47:15 +0100 (Wed, 04 Jul 2012) $:  Date of last commit
;************************************************************************
;                                 START
;************************************************************************
cd, "./", current=pwd
;******************************************
; Assign labels for plots - if required.
;******************************************
CASE column OF
    0:BEGIN
        label="temperature"
    END
    1:BEGIN
        label="dewpoints"
    END
    2:BEGIN
        label="slp"
    END
ENDCASE

;**************************************************************
; Set up the monthly values
;**************************************************************

MONTH_SETUP,last_year,nmonths,nyears,times,times,monthly_segment_starts,$
  monthly_segment_ends, monthly_segment_lengths


qc_flags_gap=INTARR(N_ELEMENTS(times))


;******************************************
; Mask values with past flags
;******************************************
untouched_values=values

past_flags=WHERE(filter_flags GT 0, pfcnt)

IF pfcnt GE 1 THEN BEGIN
    values(past_flags)=mdi
ENDIF


;*********************************************************************************
; Form monthly average series by median of all available obs (if the obs hour changes
; substantially this will cause a problem but we're looking for such big fish this shouldn't
; be a huge issue here and doing it more formally by averaging hours
; first is computationally far too expensive)
;*********************************************************************************



monthly_mean_values=DBLARR(nmonths)
monthly_mean_values(*)=mdi

monthly_mean_values_filtered=DBLARR(nmonths)
monthly_mean_values_filtered(*)=mdi

FOR mon=0,nmonths-1 DO BEGIN
  IF monthly_segment_lengths(mon) GE 112 THEN BEGIN   
    ;********************************************************
    ; Assumes at least 4 a day for February length month as a minimum 10/8/09
    ;   Calculates median for all months in the data series
    ;********************************************************

    TempArr=values(monthly_segment_starts(mon):monthly_segment_ends(mon))
    real_data=WHERE(((TempArr NE mdi) AND (TempArr NE fdi)), cnt)
    IF cnt GE 50 THEN monthly_mean_values_filtered(mon)=MEDIAN(TempArr(real_data))  ; Changed 5 to 50 10/8/09

    ;********************************************************
    ; Second concurrent version using the unfiltered data
    ;********************************************************
    TempArr=untouched_values(monthly_segment_starts(mon):monthly_segment_ends(mon))
    real_data=WHERE(((TempArr NE mdi) AND (TempArr NE fdi)), cnt)
    IF cnt GE 50 THEN monthly_mean_values(mon)=MEDIAN(TempArr(real_data))  

  ENDIF
ENDFOR


;********************************************************
; Derive monthly climatologies as mean value and "pseudo-SD" as inter-1/4 range, create
; a standardised set of values.
;********************************************************

monthly_mean_values=REFORM(TEMPORARY(monthly_mean_values),12,nyears)
monthly_mean_values_filtered=REFORM(TEMPORARY(monthly_mean_values_filtered),12,nyears)

monthly_standardised_filtered=monthly_mean_values_filtered
monthly_standardised=monthly_mean_values

FOR m=0,11 DO BEGIN
  ;********************************************************
  ; Slice the array to extract all the same months (all Januarys etc)
  ;********************************************************
  TempArrF=monthly_mean_values_filtered(m,*)
  TempArr=monthly_mean_values(m,*)

  valid_monsF=WHERE(((TempArrF NE mdi) AND (TempArrF NE mdi)), vF, complement=invalid_monsF, ncomplement=wF)
  valid_mons=WHERE(((TempArr NE mdi) AND (TempArr NE fdi)), v, complement=invalid_mons, ncomplement=w)
  IF vF GE 5 THEN BEGIN
    ;********************************************************
    ; If more than 5 months in the series find the median and InterQ
    ;********************************************************

    clim=MEDIAN(TempArrF(valid_monsF))
    smaller_array=TempArrF(valid_monsF)
    smaller_array=smaller_array[SORT(smaller_array)]
    interq=ABS(smaller_array(FIX(vF/4.))-smaller_array(FIX((3.*vF)/4.)))

    IF interq LE 2. THEN interq=2.

    ;********************************************************
    ; Save medians as offset from the mean of the medians, scaled by
    ; the InterQ range
    ; Apply to the unfiltered data, but was calculated from the filtered
    ;   data.  Use unfiltered TempArr but filtered valid_mons, clim+interq (17/1/11)
    ;********************************************************
    FOR t=0,vF-1 DO monthly_standardised(m,valid_monsF(t))=(TempArr(valid_monsF(t))-clim)/interq
    IF wF GE 1 THEN monthly_standardised(m,invalid_monsF)=mdi

  ENDIF ELSE IF vF GE 1 THEN  monthly_standardised(m,valid_monsF)=mdi
ENDFOR

;********************************************************
;  Now the monthly_standardised array contains the Anomalised data
;  using the clim and interq from the filtered data.
;  This is now used to find the asymmetries. (17/1/11)
;********************************************************

monthly_standardised=REFORM(monthly_standardised,12*nyears)


valid_months=WHERE(monthly_standardised NE mdi, cnt)
IF KEYWORD_SET(diagnostics) and (cnt GE 1) THEN BEGIN
    binsize=0.25
    min_val=FIX(MIN(monthly_standardised(valid_months)))-1 ; just in case it is negative and FIX doesn't work
    max_val=CEIL(MAX(monthly_standardised(valid_months)))
    hist = HISTOGRAM(monthly_standardised(valid_months), BINSIZE=binsize, MAX=max_val, MIN=min_val,LOCATION=loc)
    hist=[0,0,0,temporary(hist),0,0,0] 
    
    ;******************************************
    ; adjust for padding in histogram
    ;******************************************

    min_bin=MIN(loc)
    max_bin=MAX(loc)
    binns=[min_bin-(3.*binsize),min_bin-(2.*binsize),min_bin-binsize,loc,max_bin+binsize,max_bin+(2.*binsize),max_bin+(3.*binsize)]+(binsize/2)
    set_plot,'PS'
    cd, "./", current=outdirplot
    device,filename=outdirplot+'/images/'+station_id+'_Gap_'+label+'.eps'$
      ,/color,/encapsulate,/helvetica,/bold,/landscape,ysize=20,xsize=26
    !P.Font=0
    !P.Thick=4
    !P.Multi=[0,1,1]
    plotsym,3,1.2
    plot,binns,hist,min_value=-100,xstyle=1,ystyle=1,ytitle="Frequency (months)",xtitle=label+" offset (IQR)",$
      title="Gap_Check - Diagnostic Histogram - "+label,charsize=1,ythick=4,xthick=4,psym=10, POSITION=[0.10, 0.10, 0.90, 0.90],ylog=1,yrange=[0.1,max(hist)*5.], thick=4
    oplot,[1.5,1.5],[0.1,max(hist)*5.],min_value=-100,xstyle=1,ystyle=1,ytitle="Frequency (months)",xtitle=label+" offset (IQR)",$
      title="Gap_Check - Diagnostic Histogram - "+label,charsize=1,ythick=4,xthick=4,psym=0,color=239, POSITION=[0.10, 0.10, 0.90, 0.90],ylog=1,yrange=[0.1,max(hist)*5.]
    oplot,[-1.5,-1.5],[0.1,max(hist)*5.],min_value=-100,xstyle=1,ystyle=1,ytitle="Frequency (months)",xtitle=label+" offset (IQR)",$
      title="Gap_Check - Diagnostic Histogram - "+label,charsize=1,ythick=4,xthick=4,psym=0,color=239, POSITION=[0.10, 0.10, 0.90, 0.90],ylog=1,yrange=[0.1,max(hist)*5.]
    ;******************************************
    ;overplot fitted gaussian
    ;******************************************

    yfit = GaussFit(binns, hist, coeff, NTERMS=3,ESTIMATES=[max(hist),0.0,1.5])
    oplot, binns, yfit, COLOR=40, THICK=2

ENDIF



;********************************************************
;  Now step outwards from the median and compare the values.
;********************************************************
IF cnt GE 60 THEN BEGIN     ; Changed 10 to 60 10/8/09

  subset_monthly_standardised=monthly_standardised(valid_months)
  month_rankings=SORT(subset_monthly_standardised)
  valid_sorted=subset_monthly_standardised(month_rankings)

  ;********************************************************
  ;  First, Trim the data
  ;    Flag anything which is more than 5 IQR (approx 7.5sigma)
  ;    away from the median 24/11/10 RJHD
  ;********************************************************

  LargeOffsets=WHERE(ABS(valid_sorted) GT 5, lo_cnt)
  IF lo_cnt GE 1 THEN BEGIN
      bad_months=valid_months(month_rankings(LargeOffsets))
      nmonths_bad=N_ELEMENTS(bad_months)


      IF lo_cnt GE 1 THEN BEGIN
          FOR b=0,nmonths_bad-1 DO BEGIN
              qc_flags_gap(monthly_segment_starts(bad_months(b)):$
                           monthly_segment_ends(bad_months(b)))=1
          ENDFOR
          
          IF KEYWORD_SET(diagnostics) THEN BEGIN
              
              tempvar=WHERE(qc_flags_gap GT 0,flag_cnt)
              printstring=" Gap Check Flags - >5IQR"
              PRINT,printstring,column,':', flag_cnt, FORMAT='(A-38,I1,A1,I8)'
              
              LargeOffsets=WHERE(ABS(subset_monthly_standardised) GT 5, lo_cnt)
              forplotting=subset_monthly_standardised(LargeOffsets)
              
              hist = HISTOGRAM(forplotting, BINSIZE=binsize, MAX=max_val, MIN=min_val,LOCATION=loc)
              hist=[0,0,0,temporary(hist),0,0,0] 
              oplot,[5,5],[0.1,max(hist)*50.],min_value=-100,xstyle=1,ystyle=1,ytitle="Frequency (months)",xtitle=label+" offset (IQR)",$
                title="Gap_Check - Diagnostic Histogram - "+label,charsize=1,ythick=4,xthick=4,psym=0,color=130, POSITION=[0.10, 0.10, 0.90, 0.90],ylog=1,yrange=[0.1,max(hist)*50.], linestyle=1
              oplot,[-5,-5],[0.1,max(hist)*50.],min_value=-100,xstyle=1,ystyle=1,ytitle="Frequency (months)",xtitle=label+" offset (IQR)",$
                title="Gap_Check - Diagnostic Histogram - "+label,charsize=1,ythick=4,xthick=4,psym=0,color=130, POSITION=[0.10, 0.10, 0.90, 0.90],ylog=1,yrange=[0.1,max(hist)*50.], linestyle=1
              oplot,binns,hist,min_value=-100,xstyle=1,ystyle=1,ytitle="Frequency (months)",xtitle=label+" offset (IQR)",$
                title="Gap_Check - Diagnostic Histogram - "+label,charsize=1,ythick=4,xthick=4,psym=10, POSITION=[0.10, 0.10, 0.90, 0.90],ylog=1,yrange=[0.1,max(hist)*5.],color=130, thick=4
          ENDIF
          
      ENDIF
  ENDIF


  ;******************************************
  ; start in the centre and assume a sensible distribution
  ;******************************************
  mid_point=FIX(cnt/2.)
  gooddist='yes'
  n=1

  WHILE gooddist EQ 'yes' DO BEGIN
    ;******************************************
    ; the moment a gap is found, exit the loop and flag anything
    ;   further away from the median
    ;******************************************
      
    IF valid_sorted(mid_point-n) NE valid_sorted(mid_point+n) THEN BEGIN
      tempvals=[ABS(valid_sorted(mid_point-n)),ABS(valid_sorted(mid_point+n))]
      tempvals_sorted=SORT(tempvals)
      tempvals=tempvals(tempvals_sorted)

      
      IF tempvals(0) NE 0. THEN BEGIN
        IF (tempvals(1)/tempvals(0) GE 2.) AND tempvals(0) GE 1.5 THEN BEGIN   ; 0.75 -> 1.5 10/8/09
          ;********************************************************
          ; We've found a substantial break in the distribution. 
          ;    at least 1.5 InterQ from the mean, and difference of 2x
          ; Delete everything in the tail beyond this point in the tail 
	      ; that contains the value with the high estimate.
          ;********************************************************
           IF ABS(valid_sorted(mid_point-n)) EQ tempvals(1) THEN BEGIN
            ;********************************************************
	        ; Low tail value months (LHS of distribution)
            ;    Flag anything further away from the mean
            ;********************************************************
              bad_months=valid_months(month_rankings(0:mid_point-n))
              forplotting=valid_sorted(0:mid_point-n)
           ENDIF ELSE BEGIN
            ;********************************************************
	        ; High tail value months (RHS of distribution)
            ;    Flag anything further away from the mean
            ;********************************************************
              bad_months=valid_months(month_rankings(mid_point+n:cnt-1))
              forplotting=valid_sorted(mid_point+n:cnt-1)
           ENDELSE
            ;********************************************************
  	        ; Flag the data from these months
            ;********************************************************
           nmonths_bad=N_ELEMENTS(bad_months)
           FOR b=0,nmonths_bad-1 DO BEGIN
              qc_flags_gap(monthly_segment_starts(bad_months(b)):$
                           monthly_segment_ends(bad_months(b)))=1
           ENDFOR

           IF KEYWORD_SET(diagnostics) THEN BEGIN
              whatplot=WHERE(forplotting LE 5., plt_cnt)
              IF plt_cnt GE 1. THEN BEGIN
                  forplotting=forplotting(whatplot)
                  hist = HISTOGRAM(forplotting, BINSIZE=binsize, MAX=max_val, MIN=min_val,LOCATION=loc)
                  hist=[0,0,0,temporary(hist),0,0,0] 

                  gtfive=WHERE(binns GE 5., gtcnt)
                  IF gtcnt GE 1 THEN hist[gtfive]=0
                  ltfive=WHERE(binns LE -5., ltcnt)
                  IF ltcnt GE 1 THEN hist[ltfive]=0
                  
                  oplot,binns,hist,min_value=-100,xstyle=1,ystyle=1,ytitle="Frequency (months)",xtitle=label+" offset (IQR)",$
                    title="Gap_Check - Diagnostic Histogram - "+label,charsize=1,ythick=4,xthick=4,psym=10, POSITION=[0.10, 0.10, 0.90, 0.90],ylog=1,yrange=[0.1,max(hist)*5.],color=239, thick=4
              ENDIF
           ENDIF
           gooddist='no'
        ENDIF
      ENDIF
    ENDIF
    n=n+1
    IF n EQ mid_point THEN gooddist='no' ; end of the distribution, so exit
  ENDWHILE
ENDIF

IF KEYWORD_SET(diagnostics) THEN device,/close


;************************************************************************
;  New section (25/11/10) to account for strange reporting well away
;     from most of the points
;  Works by using all data for each month, and looking at the distribution
;************************************************************************

;******************************************
; Mask values with past flags (17/1/11)
;******************************************
untouched_full_values=full_values

MATCH, h_times, full_h_times, suba, subb

past_flags=WHERE(filter_flags GT 0, pfcnt)

IF pfcnt GE 1 THEN BEGIN
    full_values(subb(past_flags))=mdi
ENDIF


;*****************************************
; initialise a new flagging array
;*****************************************
qc_flags_gap2=INTARR(n_elements(full_values))
qc_flags_gap2(*)=0

;*****************************************
; new monthly setup as using full times
;*****************************************
MONTH_SETUP,last_year,nmonths,nyears,times,full_times,monthly_segment_starts,$
  monthly_segment_ends, monthly_segment_lengths,/full


monthly_segment_ends(nmonths-1)=monthly_segment_ends(nmonths-1)+1
monthly_segment_starts=REFORM(monthly_segment_starts,12,nyears)
monthly_segment_ends=REFORM(monthly_segment_ends,12,nyears)

FOR m=0,11 DO BEGIN
    ;**************************************************************
    ; If SLP then calculate the median windspeed for the month in question
    ;**************************************************************
    IF column EQ 2 THEN BEGIN

        temparr=[0.d]
        FOR yr=0,nyears-1 DO BEGIN
            temparr=[temparr,full_windspeeds(monthly_segment_starts(m,yr):monthly_segment_ends(m,yr))]
        ENDFOR
        
        temparr=temparr[1:N_ELEMENTS(temparr)-1]
        good_test=WHERE(((temparr NE mdi) AND (temparr NE fdi)), wdcnt)
        
        IF wdcnt GE 1 THEN BEGIN
            temparr=temparr(good_test)
            monthly_median_windspeeds=MEDIAN(temparr)
            monthly_mad_windspeeds=MEANABSDEV(temparr,/median)
        ENDIF ELSE BEGIN
            monthly_median_windspeeds=0.
            monthly_mad_windspeeds=0.
        ENDELSE

        temparr=[0.d]
        FOR yr=0,nyears-1 DO BEGIN
            temparr=[temparr,full_values(monthly_segment_starts(m,yr):monthly_segment_ends(m,yr))]
        ENDFOR
        
        temparr=temparr[1:N_ELEMENTS(temparr)-1]
        good_test=WHERE(((temparr NE mdi) AND (temparr NE fdi)), scnt)

        IF scnt GE 1 THEN BEGIN
            temparr=temparr(good_test)
            monthly_median_slp=MEDIAN(temparr)
            monthly_mad_slp=MEANABSDEV(temparr,/median)
        ENDIF ELSE BEGIN
            monthly_median_slp=0.
            monthly_mad_slp=0.
        ENDELSE


    ENDIF

    monthly_values=DBLARR(744,nyears)
    monthly_values(*)=mdi

    monthly_values_filtered=DBLARR(744,nyears)
    monthly_values_filtered(*)=mdi

    monthly_flags=INTARR(744,nyears)
    monthly_flags(*)=0

    ;*****************************************
    ; Read in the values for a given month
    ;  for all years observed
    ;*****************************************
    FOR yr=0,nyears-1 DO BEGIN
        temp_values          =untouched_full_values(monthly_segment_starts(m,yr):monthly_segment_ends(m,yr))
        temp_values_filtered =full_values(monthly_segment_starts(m,yr):monthly_segment_ends(m,yr))
        temp_flags           =qc_flags_gap2(monthly_segment_starts(m,yr):monthly_segment_ends(m,yr))
 
        monthly_values(0:N_ELEMENTS(temp_values)-1,yr)=temp_values
        monthly_values_filtered(0:N_ELEMENTS(temp_values_filtered)-1,yr)=temp_values_filtered
        monthly_flags(0:N_ELEMENTS(temp_values)-1,yr)=temp_flags
    ENDFOR
    
    ;*****************************************
    ; Obtain Median, IQR and anomalise.
    ;*****************************************
    mdi_test_F=WHERE(((monthly_values_filtered NE mdi) AND (monthly_values_filtered NE fdi)), mdicnt_F)
    mdi_test=WHERE(((monthly_values NE mdi) AND (monthly_values NE fdi)), mdicnt)

    IF mdicnt_F GE 50 THEN BEGIN 
        ;*****************************************
        ; Need enough to get a distribution. 
        ; Changed from >=1 to >=10.  20/12/10 RJHD
        ; Changed from >=10 to >=50.  19/01/11 RJHD
        ;*****************************************
        monthly_values_filtered=monthly_values_filtered(mdi_test_F)
        monthly_values=monthly_values(mdi_test)
        monthly_median=MEDIAN(monthly_values_filtered)
        
        count=N_ELEMENTS(monthly_values_filtered)
        sorted_monthly_values=monthly_values_filtered(SORT(monthly_values_filtered))
        
        monthly_iqr=(sorted_monthly_values(ROUND(3.*FLOAT(count)/4.))-sorted_monthly_values(ROUND(FLOAT(count)/4.)))
        
        IF monthly_iqr EQ 0.0 THEN BEGIN
            ;*********************************************
            ; Added 19/1/11 - failure on monthly_iqr=0 - so increase
            ;    the range to try and get *some* idea of what is 
            ;    going on.  Increased the minimum number needed
            ;    to create a distribution at the same time.
            ;*********************************************
            monthly_iqr=(sorted_monthly_values(ROUND(95.*FLOAT(count)/100.))-sorted_monthly_values(ROUND(5.*FLOAT(count)/100.)))
            OPENW,99,pwd+'/spurious_stations.txt',/append
            PRINT,station_id,label,strcompress(m+1,/remove_all),monthly_iqr, "Scaling IQR changed to 5-95%"
            PRINTF,99,'Gap Check',station_id,'1973-'+strcompress(last_year,/remove_all),strcompress(m+1,/remove_all),label,'    Scaling IQR changed to 5-95%le',FORMAT='(A20,A20,A10,I4,A20,A)'
            CLOSE,99
        ENDIF
        IF monthly_iqr NE 0.0 THEN BEGIN
            ;*********************************************
            ; Added 02/02/11 - failure on monthly_iqr=0 - even after
            ; increasing the range, so just leave it.  Found on 766546-99999
            ;  which had repeating values it was trying to fit
            ;*********************************************

            ;*****************************************
            ; Unfiltered monthly values - filtered median & IQR (17/1/11)
            ;*****************************************
            monthly_values=(monthly_values-monthly_median)/monthly_iqr

            ;*****************************************
            ; outside of diagnostics as used to initiate counting for gap check
            ;*****************************************
            min_val=FLOOR(MIN(monthly_values))-1 ; just in case it is negative and doesn't work
            max_val=CEIL(MAX(monthly_values))
            binsize=0.25
            hist = HISTOGRAM(monthly_values, BINSIZE=binsize,LOCATION=loc, MAX=max_val, MIN=min_val)
            hist=[0,0,0,temporary(hist),0,0,0]
            min_bin=MIN(loc)
            max_bin=MAX(loc)
            binns=[min_bin-(3.*binsize),min_bin-(2.*binsize),min_bin-binsize,loc,max_bin+binsize,max_bin+(2.*binsize),max_bin+(3.*binsize)]+(binsize/2.)
            yfit = GAUSSFIT(binns, hist, coeff, NTERMS=3, ESTIMATES=[max(hist),0.,1.5])
            ;*****************************************
            ; Plot the histogram of the distribution of
            ; all observations (anomalised) over all years
            ; for this one month
            ;*****************************************
            IF KEYWORD_SET(diagnostics) THEN BEGIN
                mdi_loc=WHERE(((monthly_values NE mdi) AND (monthly_values NE fdi)), mc)
                IF mc GT 1 THEN BEGIN
                    temparr=temparr(mdi_loc)
                    minimum=monthly_values

                    ;******************************************
                    ; adjust for padding in histogram
                    ;******************************************
                    set_plot,'PS'
                    cd, "./", current=outdirplot
                    device,filename=outdirplot+'/images/'+station_id+'_Gap_'+label+'_'+strcompress(m+1,/remove_all)+'.eps'$
                      ,/color,/encapsulate,/helvetica,/bold,/landscape,ysize=20,xsize=26
                    !P.Font=0
                    !P.Multi=[0,1,1]
                    !P.Thick=4
                    plotsym,3,1.2
                    PLOT,binns,hist,min_value=-100,xstyle=1,ystyle=1,ytitle="Frequency",xtitle=label+' offset (IQR)',$
                      title="Gap_Check - Diagnostic Histogram - "+label+" - month "+string(m+1,FORMAT='(I2.2)'),charsize=1,$
                      ythick=4,xthick=4,psym=10,YLog=1,YRange=[0.1,max(hist)*5], POSITION=[0.10, 0.10, 0.90, 0.90]
                    ;******************************************
                    ;overplot fitted gaussian
                    ;******************************************
                    ; 1 IQR ~ 1.5 sigma
                    yfit = GAUSSFIT(binns, hist, coeff, NTERMS=3, ESTIMATES=[max(hist),0.,1.5])
                    OPLOT, binns, yfit, COLOR=40, THICK=2        

                ENDIF
            ENDIF

            ;*****************************************
            ; Switch whether to flag with a removeable flag
            ; all |values|>threshold, like climatological check.
            ;*****************************************
            non_gap_flag=0 ; 0 - off, 1 - on

            ;*****************************************
            ; Find any points further from the mean than where the gaussian
            ;   crosses the y=0.1 line, rounded up to next whole integer
            ;*****************************************
            frequency_threshold=0.1
            minimum_threshold=ROUND(1.+coeff[1]+(coeff[2]*sqrt(-2.* ALOG(frequency_threshold/coeff[0]))))
            upper_loc=WHERE((monthly_values) GT minimum_threshold, uppercount)
            lower_loc=WHERE((monthly_values) LT -minimum_threshold, lowercount)
            largecount=uppercount+lowercount

            ;**************************************************************
            ; size of the gap to notice
            ;**************************************************************
            gap_limit=2

            ;*****************************************
            ; Initially we wanted to flag all months where such an outlier
            ; has occurred, but currrently just flagging the outliers.
            ;*****************************************
            IF (largecount GE 1) THEN BEGIN

                ;*****************************************
                ; check that there is a gap in the distribution between the
                ; median and the potential outliers
                ;*****************************************
                max_value=MAX(hist,start_point)
                goodupdist='yes'
                n=0
                upper_gap_cnt=0
                upper_first_gap=0

                WHILE goodupdist EQ 'yes' DO BEGIN
                    n=n+1
                    IF start_point+n LE n_elements(hist)-1 THEN BEGIN
                        IF (hist[start_point+n] EQ 0) THEN BEGIN
                            upper_gap_cnt=upper_gap_cnt+1

                            IF  ((upper_first_gap EQ 0) AND (binns[start_point+n]+(binsize/2.) GE minimum_threshold)) THEN BEGIN
                                upper_first_gap=binns[start_point+n]+(binsize/2.)
                            ENDIF
                        ENDIF
                        ;*****************************************
                        ; wipe the gap if it is too small
                        ;*****************************************
                        IF ((hist[start_point+n] NE 0) AND (upper_gap_cnt LT gap_limit))THEN upper_gap_cnt=0
                    ENDIF

                    ;*****************************************
                    ; test if the gap is large enough and there's data
                    ; beyond it and we haven't gone past that data
                    ; by wiping a small gap
                    ;*****************************************
                    IF start_point+n LE n_elements(hist)-1 THEN BEGIN
                        IF ((hist(start_point+n) NE 0) AND (upper_first_gap NE 0) AND (upper_gap_cnt GE gap_limit) AND (uppercount GE 1)) THEN goodupdist='no'
                    ENDIF

                    ;*****************************************
                    ; Escape clause
                    ;*****************************************
                    IF start_point+n EQ N_ELEMENTS(hist)-1 THEN BREAK

                ENDWHILE

                goodlodist='yes'
                n=0
                lower_gap_cnt=0
                lower_first_gap=0

                WHILE goodlodist EQ 'yes' DO BEGIN

                    n=n+1
                    IF start_point-n GE 0 THEN BEGIN
                        IF (hist[start_point-n] EQ 0) THEN BEGIN
                            lower_gap_cnt=lower_gap_cnt+1
                            IF ((lower_first_gap EQ 0) AND (binns[start_point-n]-(binsize/2.) LE -minimum_threshold)) THEN BEGIN
                                lower_first_gap=binns[start_point-n]-(binsize/2.)
                            ENDIF
                        ENDIF
                    ;*****************************************
                    ; wipe the gap if it is too small
                    ;*****************************************
                        IF ((hist[start_point-n] NE 0) AND (lower_gap_cnt LT gap_limit)) THEN lower_gap_cnt=0
                    ENDIF

                    ;*****************************************
                    ; test if the gap is large enough and there's data
                    ; beyond it and we haven't gone past that data
                    ; by wiping a small gap
                    ;*****************************************

                    IF start_point-n GE 0 THEN BEGIN
                        IF ((hist(start_point-n) NE 0) AND (lower_first_gap NE 0) AND (lower_gap_cnt GE gap_limit) AND (lowercount GE 1)) THEN goodlodist='no'
                    ENDIF
                    ;*****************************************
                    ; Escape clause
                    ;*****************************************
                    IF start_point EQ n THEN BREAK
                ENDWHILE

                ;*****************************************
                ; Loop through both tails to check, flag and plot
                ;*****************************************
                FOR portion=1,2 DO BEGIN

                    CASE portion OF
                        1: BEGIN
                            dist=goodupdist
                            threshold=minimum_threshold
                            ;*****************************************
                            ; If a gap has been found in upper tail
                            ;*****************************************
                            IF (dist EQ 'no')  THEN BEGIN
                                gap_cnt=upper_gap_cnt
                                portion_cnt=uppercount
                                first_gap=upper_first_gap
                                gap_plot_values=DBLARR(744,nyears)
                                gap_plot_values(*)=mdi
                            ENDIF
                        END
                        2: BEGIN
                            dist=goodlodist
                            threshold=-minimum_threshold
                            ;*****************************************
                            ; If a gap has been found in lower tail
                            ;*****************************************
                            IF (dist EQ 'no') THEN BEGIN
                                gap_cnt=lower_gap_cnt
                                portion_cnt=lowercount
                                first_gap=lower_first_gap
                                gap_plot_values=DBLARR(744,nyears)
                                gap_plot_values(*)=mdi
                            ENDIF
                        END
                    ENDCASE

                    plot_values=DBLARR(744,nyears)
                    plot_values(*)=mdi

                    FOR yr=0,nyears-1 DO BEGIN

                        ;*****************************************
                        ; Need to convert between the full and normal
                        ; set of values for the plotting and flagging
                        ; run for each wing separately
                        ;*****************************************
                        year_values=full_values(monthly_segment_starts(m,yr):monthly_segment_ends(m,yr))
                        year_flags=qc_flags_gap2(monthly_segment_starts(m,yr):monthly_segment_ends(m,yr))

                        IF column EQ 2 THEN BEGIN
                            year_slp=full_values(monthly_segment_starts(m,yr):monthly_segment_ends(m,yr))
                            year_winds=full_windspeeds(monthly_segment_starts(m,yr):monthly_segment_ends(m,yr))
                            year_times=full_times(monthly_segment_starts(m,yr):monthly_segment_ends(m,yr))
                        ENDIF

                        mdi_test=WHERE(((year_values ne mdi) AND (year_values ne fdi)), mdi_cnt)

                        IF mdi_cnt GE 1 THEN BEGIN
                            cleaned_values=(year_values(mdi_test)-monthly_median)/monthly_iqr
                            cleaned_flags=year_flags(mdi_test)

                            IF column EQ 2 THEN BEGIN
                                cleaned_slp=year_slp(mdi_test)
                                cleaned_winds=year_winds(mdi_test)
                                cleaned_times=year_times(mdi_test)
                            ENDIF


                            ;*****************************************
                            ; inactive code to flag all values GT
                            ; threshold tentatively, cf Climatological
                            ;*****************************************
                            IF (non_gap_flag EQ 1) THEN BEGIN
                                IF portion EQ 1 THEN cleaned_locations=WHERE(cleaned_values GT threshold, clean_cnt) $
                                ELSE IF portion EQ 2 THEN  cleaned_locations=WHERE(cleaned_values LT threshold, clean_cnt)

                                IF (clean_cnt GE 1) THEN BEGIN
                                    ;*****************************************
                                    ; testing if flagging (and hence plotting)
                                    ; should be performed.
                                    ;*****************************************
                                    plot_values(0:N_ELEMENTS(cleaned_values(cleaned_locations))-1,yr)=cleaned_values(cleaned_locations)
                                    cleaned_flags(cleaned_locations)=2  ; tentative flag (but not implemented)
                                ENDIF
                            ENDIF

                            IF dist EQ "no" THEN BEGIN
                                ;*****************************************
                                ; Gaps in the distribution - check for 
                                ; storm signals - lowcount,portion=2
                                ;*****************************************
                                IF portion EQ 1 THEN gap_cleaned_locations=WHERE(cleaned_values GT first_gap, gap_clean_cnt) $
                                ELSE IF portion EQ 2 THEN  gap_cleaned_locations=WHERE(cleaned_values LT first_gap, gap_clean_cnt)

                                IF (gap_cnt GE gap_limit) AND (portion_cnt GE 1) AND (gap_clean_cnt GE 1) THEN BEGIN 
                                    ; there is data after a large enough gap which is far enough from the centre.
                                    ;*****************************************
                                    ; testing if flagging (and hence plotting)
                                    ; should be performed.
                                    ;****************************************
                                    gap_plot_values(0:N_ELEMENTS(cleaned_values(gap_cleaned_locations))-1,yr)=cleaned_values(gap_cleaned_locations)
                                    cleaned_flags(gap_cleaned_locations)=1 ; 2 level system - 1=FIRM flag, 2=TENTATIVE flag.



                                    IF (column EQ 2) AND (portion EQ 2) THEN BEGIN ; low side of SLP distribution

                                        ;*****************************
                                        ; 7-MAR-2011 initiate 2 level flagging system
                                        ;  Flag these tentatively, and
                                        ; and see if any can be removed
                                        ; 2 level system - 1=FIRM flag, 2=TENTATIVE flag.
                                        ;*****************************
                                        cleaned_flags(gap_cleaned_locations)=2

                                        ;*****************************
                                        ; storm signal check - ONLY for
                                        ; SLP and low-value gaps
                                        ;*****************************

                                        wind_present=WHERE(((cleaned_winds NE mdi) AND (cleaned_winds NE fdi)), wpcnt)
                                        ;*****************************
                                        ; storms where SLP low and 
                                        ; wind high.  If no wind
                                        ; just use SLP
                                        ;*****************************
                                        IF wpcnt GE 1 THEN BEGIN
                                            storms=WHERE(((monthly_median_slp-cleaned_slp)/monthly_mad_slp GE 4.5) AND ((cleaned_winds-monthly_median_windspeeds)/monthly_mad_windspeeds GE 4.5), stmcnt)
                                        ENDIF ELSE BEGIN
                                            ;*****************************
                                            ;9 MARCH 2011 line below commented out as it basically undoes the test itself
                                            ;  This way, all low SLPs are tentatively flagged and can be brought back
                                            ;  in by neighbours.  If there is wind speed information, then can undo the flag
                                            ;  here.  Note the effect though.
                                            ;storms=WHERE((monthly_median_slp-cleaned_slp)/monthly_mad_slp GE 4., stmcnt)
                                            ;*****************************
                                            OPENW,99,pwd+'/spurious_stations.txt',/append
                                            PRINTF,99,'Gap Check',station_id,'1973-2007',strcompress(m+1,/remove_all),label,'    No windspeed to check for storms',FORMAT='(A20,A20,A10,I4,A20,A)'
                                            CLOSE,99
                                            stmcnt=0

                                        ENDELSE

                                        IF stmcnt GE 2 THEN BEGIN
                                        ;*****************************
                                        ; use the first difference series to
                                        ; find where there are gaps in
                                        ; contiguous sequences of storm
                                        ; observations - want to split 
                                        ; separate storm events
                                        ;*****************************
                                            storm1diff=storms(1:stmcnt-1)-storms(0:stmcnt-2)

                                            separations=WHERE(storm1diff NE 1, sepcnt)
                                            ;*********************************
                                            ; expand around storm signal
                                            ; so that all low SLP values
                                            ; covered and unflagged
                                            ;*********************************
                                            
                                            IF sepcnt GE 1 THEN BEGIN
                                                ;*****************************
                                                ; if more than one storm signal that
                                                ; month then use intervals in the
                                                ; first difference series to expand
                                                ; around the first interval done alone
                                                ;*****************************
                                                start=0
                                                finish=separations[0]
                                                tempstorms=storms[start:finish]

                                                FOR I=1,6 DO BEGIN
                                                    tempstorms=[tempstorms[0]-1,tempstorms]                                                    
                                                    tempstorms=[tempstorms,tempstorms[N_ELEMENTS(tempstorms)-1]+1]
                                                ENDFOR
                                                
                                                newstorms=tempstorms
                                                start=finish+1

                                                ;*****************************
                                                ; the remainder done in the for loop
                                                ;*****************************
                                                FOR J=1,sepcnt DO BEGIN

                                                    IF J GE sepcnt THEN finish=N_ELEMENTS(storms)-1 ELSE finish=separations[J]
                                                    tempstorms=storms[start:finish]

                                                    
                                                    FOR I=1,6 DO BEGIN
                                                        tempstorms=[tempstorms[0]-1,tempstorms]                                                    
                                                        tempstorms=[tempstorms,tempstorms[N_ELEMENTS(tempstorms)-1]+1]
                                                    ENDFOR
                                                    
                                                    newstorms=[newstorms,tempstorms]

                                                    start=finish+1
                                                ENDFOR
                                                newstorms=newstorms(UNIQ(newstorms))
                                                storms=newstorms
                                            ENDIF ELSE BEGIN
                                                ;*****************************
                                                ; else just expand around the signal
                                                ; by 6 hours either way
                                                ;*****************************
                                            
                                                FOR I=1,6 DO BEGIN
                                                    storms=[storms[0]-1,storms]                                                    
                                                    storms=[storms,storms[N_ELEMENTS(storms)-1]+1]
                                                ENDFOR
    
                                            ENDELSE

                                        ENDIF
                                        
                                        IF stmcnt GE 1 THEN BEGIN
                                            ; undo the flag                                           
                                            PRINT,"Tropical Storm signal in " ,m+1,yr+1973.
                                            cleaned_flags(storms)=0
                                        
                                        ENDIF
                                    ENDIF
                                ENDIF
                            ENDIF

                            year_flags(mdi_test)=cleaned_flags
                            qc_flags_gap2(monthly_segment_starts(m,yr):monthly_segment_ends(m,yr))=year_flags
                        ENDIF

                    ENDFOR
                    pltcnt=0
                    gap_pltcnt=0
                    IF non_gap_flag EQ 1 THEN plot_test=WHERE(((plot_values NE mdi) AND (plot_values NE fdi)), pltcnt)

                    IF dist EQ "no" THEN gap_plot_test=WHERE(((gap_plot_values NE mdi) AND (gap_plot_values NE fdi)), gap_pltcnt)

                    ;*****************************************
                    ; Highlight which portions of the histogram are flagged
                    ;*****************************************
                    IF KEYWORD_SET(diagnostics) THEN BEGIN

                        IF pltcnt GE 1 THEN BEGIN 
                            ;*****************************************
                            ; testing if anything should be flagged or plotted was done above.
                            ; here just checking if there is anything to plot
                            ;*****************************************

                            oplot,[threshold,threshold],[0.1,max(hist)*5],psym=0,color=200,YLog=1 ;, linestyle=2
                            plot_values=plot_values(plot_test)
                            histp = HISTOGRAM(plot_values, BINSIZE=binsize,LOCATION=loc, MAX=max_val, MIN=min_val)
                            histp=[0,0,0,temporary(histp),0,0,0]
                            binns=[min_bin-(3.*binsize),min_bin-(2.*binsize),min_bin-binsize,loc,max_bin+binsize,max_bin+(2.*binsize),max_bin+(3.*binsize)]+(binsize/2.)
                            OPLOT,binns,histp,min_value=-100,xstyle=1,ystyle=1,ytitle="Frequency",xtitle=label+' offset',$
                              title="Gap_Check - Diagnostic Histogram - "+label,charsize=1,ythick=4,xthick=4,psym=10,$
                              YLog=1,YRange=[0.1,max(hist)*5], POSITION=[0.10, 0.10, 0.90, 0.90],color=200

                        ENDIF
                        IF dist EQ "no" THEN BEGIN
                            IF gap_pltcnt GE 1 THEN BEGIN
                                ;*****************************************
                                ; testing if anything should be flagged or plotted was done above.
                                ; here just checking if there is anything to plot
                                ;*****************************************

                                oplot,[first_gap,first_gap],[0.1,max(hist)*5],psym=0,color=239,YLog=1 ;, linestyle=2
                                gap_plot_values=gap_plot_values(gap_plot_test)
                                histp = HISTOGRAM(gap_plot_values, BINSIZE=binsize,LOCATION=loc, MAX=max_val, MIN=min_val)
                                histp=[0,0,0,temporary(histp),0,0,0]
                                binns=[min_bin-(3.*binsize),min_bin-(2.*binsize),min_bin-binsize,loc,max_bin+binsize,max_bin+(2.*binsize),max_bin+(3.*binsize)]+(binsize/2.)
                                OPLOT,binns,histp,min_value=-100,xstyle=1,ystyle=1,ytitle="Frequency",xtitle=label+' offset',$
                                  title="Gap_Check - Diagnostic Histogram - "+label,charsize=1,ythick=4,xthick=4,psym=10,$
                                  YLog=1,YRange=[0.1,max(hist)*5], POSITION=[0.10, 0.10, 0.90, 0.90],color=239
                            ENDIF
                        ENDIF ELSE oplot,[threshold,threshold],[0.1,max(hist)*5],psym=0,color=200,YLog=1 ;, linestyle=2

                    ENDIF
                ENDFOR                    


                ;*************************************************
                ; Want to write out to file if a large number have been flagged.
                ;   Could be a Fahrenheit/Celsius problem - for eyeballing later
                ;*************************************************
                IF largecount GT 1000 THEN BEGIN
                    OPENW,99,pwd+'/spurious_stations.txt',/append
                    PRINTF,99,'Gap Check',station_id,'1973-2007',strcompress(m+1,/remove_all),label,'    Large number of observations offset from main distribution',FORMAT='(A20,A20,A10,I4,A20,A)'
                    CLOSE,99
                ENDIF


            ENDIF
            IF KEYWORD_SET(diagnostics) THEN device,/close
        ENDIF ELSE BEGIN
            PRINT,station_id,label,strcompress(m+1,/remove_all), "   No IQR calculable.  Extremely narrow distribution"
            OPENW,99,pwd+'/spurious_stations.txt',/append
            PRINTF,99,'Gap Check',station_id,'1973-2007',strcompress(m+1,/remove_all),label,'    No IQR calculable.  Extremely narrow distribution',FORMAT='(A20,A20,A10,I4,A20,A)'
            CLOSE,99
        ENDELSE
    ENDIF
ENDFOR
IF KEYWORD_SET(diagnostics) THEN device,/close

;*****************************************
; Copy the flags across
;*****************************************
MATCH, h_times,full_h_times,lc1,lc2,count=count

qc_flags_matched=qc_flags_gap2(lc2)

newflags=WHERE(qc_flags_gap2(lc2) GT 0, flgcnt)

oldflags=WHERE(qc_flags_gap EQ 0)

qc_flags_gap(oldflags)=qc_flags_matched(oldflags)

;******************************************
; Revert to un filtered data
;******************************************
values=untouched_values
full_values=untouched_full_values

RETURN

END
;************************************************************************
;                                 END
;************************************************************************
