#!/bin/bash
####################################################################
#British Crown Copyright (c) 2012, the Met Office All rights reserved.
#
#Redistribution and use in source and binary forms, with or without 
#modification, are permitted provided that the following conditions 
#are met:
#
#Redistributions of source code must retain the above copyright notice, 
#this list of conditions and the following disclaimer. 
#
#Redistributions in binary form must reproduce the above copyright 
#notice, this list of conditions and the following disclaimer in the 
#documentation and/or other materials provided with the distribution. 
#
#Neither the name of the Met Office nor the names of its contributors 
#may be used to endorse or promote products derived from this software 
#without specific prior written permission. 
#
#THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
#"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
#LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
#A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
#HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
#SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
#LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
#DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
#THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
#(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
#OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
####################################################################

set -x
set -v
#************************************************************************
# NAME: HADISD_QC_CONTROL
#     
#
# PURPOSE: To control the quality control checks during the creation
# of the HadISD data set.  Designed so that the internal checks could 
# if possible.  If parallel option chosen, requires startpoints.txt and endpoints.txt
# to be passed as restart_id and end_id's to the internal qc script.
#     
#
# CALLING SEQUENCE:
#     hadisd_qc_control.bash, last_year, parallel, plots, diagnostics, pflags
#
#     RJHD usual call will be:   hadisd_qc_control.bash 2011 1 0 0 0 1 1 1 1 1 1
#
# INPUTS:
#     last_year - the final year of the data sequence
#
#     parallel = 1   do parallel processing
#              = 0   do not run concurrent IDL sessions
#
#     plots = 1   do the plots
#           = 0   do not the plots
#
#     diagnostics = 1  turn on diagnostic plots
#                 = 0  turn on diagnostic plots
#
#     pflags = 1    plot the flags on the timeseries'
#            = 0    plot the flags on the timeseries'
#
#     internal = 1    do first internal run
#              = 0    or not
#
#     external = 1    do first external run
#              = 0    or not
#
#     mask     = 1    do first masking run
#              = 0    or not
#
#     internal2 = 1    do second internal run
#               = 0    or not
#
#     external2 = 1    do second external run
#               = 0    or not
#
#     mask2     = 1    do second masking run
#               = 0    or not
#
#
# OUTPUTS:
#     No direct outputs (16/11/2011, RJHD)
#     
#
# RESTRICTIONS:
#
#     
#   
# HISTORY:
#     NB - not all revisions are noted here.
#     # 237 - creation, copied from hadisd_qc_contro.pro
#     # 241 - Added parallelisation capability and enforced keyword setting at programme call
#             21 November 2011 RJHD
#     # 244 - Added call to qc_summary_create in parallel section
#             22 November 2011 RJHD
#     # 247 - Added switches to run each section of QC separately
#             24 November 2011 RJHD
#     # 252 - Added option of using IDL vs TIDL with $idlcommand.  Added checking and testing
#             for successful completion and restarting on failure of NC_ERROR=-31.  Removed
#             error status check on IDL command as not useful in this case.
#             9 December 2011 RJHD
#     # 264 - Removed UK version of station_locations_and_distances.nc
#             15 December 2011 RJHD
#     # 295 - Added tests for required files for run
#             19 March 2012 RJHD
#
#************************************************************************
#                    SVN Info
# $Rev:: 326                                           $:  Revision of last commit
# $Author:: rdunn                                      $:  Author of last commit
# $Date:: 2011-12-09 16:15:47 +0000 (Fri, 09 Dec 2011) $:  Date of last commit
#************************************************************************
#                                 START
#************************************************************************

##################
# Assess if correct number of parameters passed
##################

if [ $# -ne 11 ]; then 
    echo "Usage is "
    echo "      hadisd_qc_control.bash, last_year, parallel, plots, diagnostics, pflags"
    echo ""
    echo "where"
    echo "     last year   -   last year of processing"
    echo ""
    echo "     parallel    -   1 parallel      0 not"
    echo "     plots       -   1 plot          0 not"
    echo "     diagnostics -   1 do            0 not"
    echo "     pflags      -   1 plot flags    0 not"
    echo "     internal    -   1 1st internal  0 not"
    echo "     external    -   1 1st external  0 not"
    echo "     mask        -   1 1st mask      0 not"
    echo "     internal2   -   1 2nd internal  0 not"
    echo "     external2   -   1 2nd external  0 not"
    echo "     mask2       -   1 2nd mask      0 not"
    exit 1
fi

##################
# set input keywords
##################
last_year=$1
parallel=$2
plots=$3
diagnostics=$4
pflags=$5
internal=$6
external=$7
mask=$8
internal2=$9
external2=${10}
mask2=${11}

date +'%d-%b-%Y %R'

##################
# Check that required files are present
##################
if [ ! -e attributes.dat ]; then
    echo "File attributes.dat missing"
    exit
fi
if [ ! -e duplicates_ordered.txt ]; then
    echo "File duplicates_ordered.txt missing"
    exit
fi
if [ ! -e kept_station_info.txt ]; then
    echo "File kept_station_info.txt missing"
    exit
fi
if [ $parallel -eq 1 ]; then
    if [ ! -e startpoints.dat ]
    then
	    echo "File startpoints.dat missing"
	    exit
    fi
    if [ ! -e endpoints.dat ]
    then
	    echo "File endpoints.dat missing"
	    exit
    fi
fi

##################
# allow choice of IDL vs TIDL
##################
# select IDL 7.1 for Jan 2013 run
idlcommand=/usr/local/bin/idl71

cwd=`pwd`

sleeptime=300 # 300 = 5 min, 3000 = 50 min

##################
# location of netcdf datafiles
##################
datadir=../netcdffiles_2013

##################
# set meta-data for input to .pro files
##################
processing_start=`date`
svn_version=`svnversion`


##################
# make the images directory
##################

if [ ! -d images ]; then
    mkdir images
fi
ImageDir=${cwd}/images

##################
# Unzip the raw data (only the raw files are unzipped
#   automatically - for single process runs unzipping
#   needs to be done manually)
##################
echo "CHECK FILE CREATION DATES BEFORE ANSWERING"
if [ $internal -eq 1 ]; then
    ZippedFiles=`ls ${datadir}/??????-?????.nc.gz`
    if [ "$ZippedFiles" != "" ]; then 
        echo "Do you want to unzip old raw files? y/n"
        read -e ziptest
        if [ "$ziptest" == "y" ]; then
            echo 'Unzipping'
            gunzip  -f $ZippedFiles
        else
            echo "not unzipping"
        fi
    fi
elif [ $external -eq 1 ]; then
    ZippedFiles=`ls ${datadir}/??????-?????_internal.nc.gz`
    if [ "$ZippedFiles" != "" ]; then 
        echo "Do you want to unzip old internal files? y/n"
        read -e ziptest
        if [ "$ziptest" == "y" ]; then
            echo 'Unzipping'
            gunzip  -f $ZippedFiles
        else
            echo "not unzipping"
        fi
    fi
elif [ $mask -eq 1 ]; then
    ZippedFiles=`ls ${datadir}/??????-?????_external.nc.gz`
    if [ "$ZippedFiles" != "" ]; then 
        echo "Do you want to unzip old external files? y/n"
        read -e ziptest
        if [ "$ziptest" == "y" ]; then
            echo 'Unzipping'
            gunzip  -f $ZippedFiles
        else
            echo "not unzipping"
        fi
    fi
elif [ $internal2 -eq 1 ]; then
    ZippedFiles=`ls ${datadir}/??????-?????_mask.nc.gz`
    if [ "$ZippedFiles" != "" ]; then 
        echo "Do you want to unzip old mask files? y/n"
        read -e ziptest
        if [ "$ziptest" == "y" ]; then
            echo 'Unzipping'
            gunzip  -f $ZippedFiles
        else
            echo "not unzipping"
        fi
    fi
elif [ $external2 -eq 1 ]; then
    ZippedFiles=`ls ${datadir}/??????-?????_internal2.nc.gz`
    if [ "$ZippedFiles" != "" ]; then 
        echo "Do you want to unzip old internal2 files? y/n"
        read -e ziptest
        if [ "$ziptest" == "y" ]; then
            echo 'Unzipping'
            gunzip  -f $ZippedFiles
        else
            echo "not unzipping"
        fi
    fi
elif [ $mask2 -eq 1 ]; then
    ZippedFiles=`ls ${datadir}/??????-?????_external2.nc.gz`
    if [ "$ZippedFiles" != "" ]; then 
        echo "Do you want to unzip old external2 files? y/n"
        read -e ziptest
        if [ "$ziptest" == "y" ]; then
            echo 'Unzipping'
            gunzip  -f $ZippedFiles
        else
            echo "not unzipping"
        fi
    fi
fi

##################
# Check that distances and locations set up
##################
LocFileExist=`ls station_locations_and_distances.nc*`
if [ "$LocFileExist" = "" ]; then
    DistLogFile=Calculate_Distances.log
    $idlcommand << EOF >> $DistLogFile
turbo
CALCULATE_DISTANCES
EOF
else
    gunzip *station_locations_and_distances.nc.*
fi



if [ $internal -eq 1 ]; then
    "rm" composites.dat
    "rm" spurious_stations.txt
    ##################
    # check if set to do parallelisation
    ##################
    if [ $parallel -eq 1 ]; then 
	    date +'%d-%b-%Y %R'
        ##################
        # Remove the "completion" files
        ##################
	    "rm" CompletedParallel*txt
    
        ##################
        # find number of consecutive loops to run and set them up.
        ##################

	    ParallelSessionNumber=`wc startpoints.dat | awk '{print $1}'`
	
	    StartIDs=`more startpoints.dat`
	    EndIDs=`more endpoints.dat`
	
	    SessionNumber=0
	
	    for restart_id in $StartIDs
	    do
	    
	        let SessionNumber=$SessionNumber+1
	    
	        echo "Setting Session Number $SessionNumber/$ParallelSessionNumber going"
	        
	        end_id=`echo $EndIDs | awk "{print \$ $SessionNumber}"`
	        
	        echo "Restart ID is $restart_id, End ID is $end_id"
	    
            ##################
            # Internal Checks 1
            ##################
	        echo "qc_all_vars_internal,$last_year, '$processing_start','$svn_version',/all_tests,suffix='internal',plots=$plots,diagnostics=$diagnostics, /dynamic_diurnal, datadir='$datadir', restart_id='$restart_id', end_id='$end_id', parallel=$SessionNumber"
            
	        InternalLog=InternalLog$SessionNumber.txt
	        $idlcommand << EOF &> $InternalLog &
turbo
!EXCEPT=0
qc_all_vars_internal,$last_year, '$processing_start','$svn_version',/all_tests,suffix='internal',plots=$plots,diagnostics=$diagnostics, /dynamic_diurnal, datadir='$datadir', restart_id='$restart_id', end_id='$end_id', parallel=$SessionNumber
EOF
	    done
	
        ##############
        # Set all the processes going
        # now need to monitor that they've finished successfully
        ##############
	    CompletionNumber=1
	    while [ 1 = 1 ]
	    do
	        date +'%d-%b-%Y %R'
	        if [ -e "CompletedParallel$CompletionNumber.txt" ]; then
		        echo "Completion Test Number $CompletionNumber"
		        let CompletionNumber=$CompletionNumber+1
	        else
                ##############
		        # test for finished in all parallel sessions.
                ##############

		        SessionNumber=0
		        for end_id in $EndIDs
		        do
		            let SessionNumber=$SessionNumber+1

                    ##############
		            # test if finished.
                    ##############
		            InternalLog=InternalLog$SessionNumber.txt
                    
		            CompletionTest=`tail -n1 $InternalLog`
                    
		            if [ "$CompletionTest" = "FINISHED" ]; then
                        ##############
			            # move on to next station in loop
                        ##############
			            continue
                        
		            elif [ "$CompletionTest" = "% Execution halted at: \$MAIN\$          " ]; then
                        ##############
	    		        # failure in IDL - likely a -31 error
			            #    find the last station processed, and restart from that point
                        ##############

			            LastStation=`grep "Station Identifier" $InternalLog | tail -n1 | awk '{print $4}'`
			            echo "qc_all_vars_internal,$last_year, '$processing_start','$svn_version',/all_tests,suffix='internal',plots=$plots,diagnostics=$diagnostics, /dynamic_diurnal, datadir='$datadir', restart_id='$LastStation', end_id='$end_id', parallel=$SessionNumber"
                        
			            $idlcommand << EOF &>> $InternalLog &
turbo
!EXCEPT=0
qc_all_vars_internal,$last_year, '$processing_start','$svn_version',/all_tests,suffix='internal',plots=$plots,diagnostics=$diagnostics, /dynamic_diurnal, datadir='$datadir', restart_id='$LastStation', end_id='$end_id', parallel=$SessionNumber
EOF

		            fi
	    
	  	        done
                ##############
		        # checked all - and now sleep before testing again
                ##############
		        sleep $sleeptime
	        fi
	        
	        if [ "$CompletionNumber" -gt "$ParallelSessionNumber" ]; then
		        break
	        fi
	        
	    done
	
        ##############
        # Create the summary files
        ##############
	    SummaryLog=SummaryLog_Internal.txt
	    $idlcommand << EOF &> $SummaryLog 
turbo
!EXCEPT=0
qc_summary_create,suffix='internal', datadir='$datadir', outputdir='./'
EOF

    else
        ##################
        # Internal Checks 1
        ##################

	    LastStation=`head -1 kept_station_info.txt  | awk '{print $1}'`
	    InternalLog=InternalLog.txt
	    rm $InternalLog
        
	    while [ 1 = 1 ]
	    do
	        date +'%d-%b-%Y %R'
	        echo "qc_all_vars_internal,$last_year, '$processing_start', '$svn_version',/all_tests,suffix='internal',plots=$plots,diagnostics=$diagnostics, /dynamic_diurnal, datadir='$datadir', restart_id='$LastStation'"

	    
	        $idlcommand << EOF &>> $InternalLog 
turbo
!EXCEPT=0
qc_all_vars_internal,$last_year, '$processing_start','$svn_version',/all_tests,suffix='internal',plots=$plots,diagnostics=$diagnostics, /dynamic_diurnal, datadir='$datadir', restart_id='$LastStation
EOF
	        CompletionTest=`tail -n1 $InternalLog`
	        if [ "$CompletionTest" = "FINISHED" ]; then
		        break
	        elif [ "$CompletionTest" = "% Execution halted at: \$MAIN\$          " ]; then
                ##############
	            # failure in IDL - likely a -31 error
		        #    find the last station processed
		        #    restart from this point on next iteration of while-loop
                ##############
		        LastStation=`grep "Station Identifier" $InternalLog | tail -n1 | awk '{print $4}'`
	        fi
	    done
        ##############
        # Create the summary files
        ##############
	    SummaryLog=SummaryLog_Internal.txt
	    $idlcommand << EOF &> $SummaryLog 
turbo
!EXCEPT=0
qc_summary_create,suffix='internal', datadir='$datadir', outputdir='./'
EOF

    fi
    
    ZippedFiles=`ls ${datadir}/??????-?????.nc`
    if [ "$ZippedFiles" != "" ]; then 
	    find $datadir -name "??????-?????.nc" | xargs gzip -f &
    fi
    
fi


if [ $external -eq 1 ]; then
    ##################
    # External Checks 1
    ##################

    LastStation=`head -1 kept_station_info.txt  | awk '{print $1}'`
    ExternalLog=ExternalLog.txt
    rm $ExternalLog

    while [ 1 = 1 ]
    do
	    date +'%d-%b-%Y %R'
	    echo "qc_all_vars_external,$last_year, '$processing_start', '$svn_version',suffix='external',/clean,/unflag,plots=$plots,diagnostics=$diagnostics, datadir='$datadir',  restart_id='$LastStation'"
	    
	    $idlcommand << EOF &>> $ExternalLog
turbo
!EXCEPT=0
qc_all_vars_external,$last_year, '$processing_start', '$svn_version',suffix='external',/clean,/unflag,plots=$plots,diagnostics=$diagnostics, datadir='$datadir',  restart_id='$LastStation'
EOF
	
	    CompletionTest=`tail -n1 $ExternalLog`
	    if [ "$CompletionTest" = "FINISHED" ]; then
	        break
	    elif [ "$CompletionTest" = "% Execution halted at: \$MAIN\$          " ]; then
            ##############
	        # failure in IDL - likely a -31 error
	        # find the last station processed
	        #    restart from this point on next iteration of while-loop
            ##############
	        LastStation=`grep "Station Identifier" $ExternalLog | tail -n1 | awk '{print $4}'`
	    elif [ "$CompletionTest" = "%                      $MAIN$          " ]; then
            ##############
	        # failure in IDL - likely a -31 error
	        # find the last station processed
	        #    restart from this point on next iteration of while-loop
            ##############
	        LastStation=`grep "Station Identifier" $ExternalLog | tail -n1 | awk '{print $4}'`
	    fi
	
    done
    ZippedFiles=`ls ${datadir}/*_internal.nc`
    if [ "$ZippedFiles" != "" ]; then 
	    find $datadir -name "*_internal.nc" | xargs gzip -f &
    fi
fi

if [ $mask -eq 1 ]; then
    ##################
    # Masking Checks 1
    ##################
    LastStation=`head -1 kept_station_info.txt  | awk '{print $1}'`
    MaskingLog=MaskingLog.txt
    rm $MaskingLog

    while [ 1 = 1 ]
    do
	    date +'%d-%b-%Y %R'
	    echo "qc_masking,$last_year, '$processing_start','$svn_version',suffix='mask',plots=plots, datadir='$datadir', restart_id='$LastStation'"
        
	    $idlcommand << EOF &>> $MaskingLog
turbo
!EXCEPT=0
qc_masking,$last_year, '$processing_start', '$svn_version',suffix='mask',plots=plots, datadir='$datadir', restart_id='$LastStation'
EOF
    
	    CompletionTest=`tail -n1 $MaskingLog`
	    if [ "$CompletionTest" = "FINISHED" ]; then
	        break
	    elif [ "$CompletionTest" = "% Execution halted at: \$MAIN\$          " ]; then
            ##############
	        # failure in IDL - likely a -31 error
	        # find the last station processed
	        #    restart from this point on next iteration of while-loop
            ##############
	        LastStation=`grep "Station Identifier" $MaskingLog | tail -n1 | awk '{print $4}'`
	    fi
	    
    done
    ZippedFiles=`ls ${datadir}/*_external.nc`
    if [ "$ZippedFiles" != "" ]; then 
	    find $datadir -name "*_external.nc" | xargs gzip -f &
    fi
fi


if [ $internal2 -eq 1 ]; then
    ##################
    # check if set to do parallelisation
    ##################
    if [ $parallel -eq 1 ]; then 
	    date +'%d-%b-%Y %R'
        ##################
        # Remove the "completion" files
        ##################
	    "rm" CompletedParallel*txt

        ##################
        # find number of consecutive loops to run and set them up.
        ##################

	    ParallelSessionNumber=`wc startpoints.dat | awk '{print $1}'`
	    
	    StartIDs=`more startpoints.dat`
	    EndIDs=`more endpoints.dat`
	    
	    SessionNumber=0
	    
	    for restart_id in $StartIDs
	    do
	        
	        let SessionNumber=$SessionNumber+1
	        
	        echo "Setting Session Number $SessionNumber/$ParallelSessionNumber going"
	        
	        end_id=`echo $EndIDs | awk "{print \$ $SessionNumber}"`
	        
	        echo "Restart ID is $restart_id, End ID is $end_id"
	        
	        ##################
	        # Internal Checks 2
	        ##################
	        echo "qc_all_vars_internal,$last_year, '$processing_start','$svn_version',/odd,/spike,suffix='internal2',plots=$plots,diagnostics=$diagnostics, /flags, datadir='$datadir', restart_id='$restart_id', end_id='$end_id', parallel=$SessionNumber"
	    
	        InternalLog=InternalLog${SessionNumber}_2.txt
	        $idlcommand << EOF &> $InternalLog &
turbo
!EXCEPT=0
qc_all_vars_internal,$last_year, '$processing_start','$svn_version',/odd,/spike,suffix='internal2',plots=$plots,diagnostics=$diagnostics, /flags, datadir='$datadir', restart_id='$restart_id', end_id='$end_id', parallel=$SessionNumber
EOF
	    
	    done

        ##############
        # Set all the processes going
        # now need to monitor that they've finished successfully
        ##############
	    CompletionNumber=1
	    while [ 1 = 1 ]
	    do
	        date +'%d-%b-%Y %R'
	        if [ -e "CompletedParallel$CompletionNumber.txt" ]; then
		        echo "Completion Test Number $CompletionNumber"
		        let CompletionNumber=$CompletionNumber+1
	        else
                ##############
		        # test for finished in all parallel sessions.
                ##############

		        SessionNumber=0
		        for end_id in $EndIDs
		        do
		            let SessionNumber=$SessionNumber+1
                    
                    ##############
		            # test if finished.
                    ##############
		            InternalLog=InternalLog${SessionNumber}_2.txt
                    
		            CompletionTest=`tail -n1 $InternalLog`
                    
		            if [ "$CompletionTest" = "FINISHED" ]; then
                        ##############
			            # move on to next station in loop
                        ##############
			            continue
                        
		            elif [ "$CompletionTest" = "% Execution halted at: \$MAIN\$          " ]; then
                        ##############
	    		        # failure in IDL - likely a -31 error
			            #    find the last station processed, and restart from that point
                        ##############

			            LastStation=`grep "Station Identifier" $InternalLog | tail -n1 | awk '{print $4}'`
			            echo "qc_all_vars_internal,$last_year, '$processing_start','$svn_version',/odd,/spike,suffix='internal2',plots=$plots,diagnostics=$diagnostics, /flags, datadir='$datadir', restart_id='$LastStation', end_id='$end_id', parallel=$SessionNumber"
                        
			            $idlcommand << EOF &>> $InternalLog &
turbo
!EXCEPT=0
qc_all_vars_internal,$last_year, '$processing_start','$svn_version',/odd,/spike,suffix='internal2',plots=$plots,diagnostics=$diagnostics, /flags, datadir='$datadir', restart_id='$LastStation', end_id='$end_id', parallel=$SessionNumber
EOF

		            fi
	    
	  	        done
                ##############
		        # checked all - and now sleep before testing again
                ##############
		        sleep $sleeptime
	        fi
	        
	        if [ "$CompletionNumber" -gt "$ParallelSessionNumber" ]; then
		        break
	        fi
	        
	    done
	    
        
        ##############
        # Create the summary files
        ##############
	    SummaryLog=SummaryLog_Internal2.txt
	    $idlcommand << EOF &> $SummaryLog 
turbo
!EXCEPT=0
qc_summary_create,suffix='internal2', datadir='$datadir', outputdir='./'
EOF


    else
        ##################
        # Internal Checks 2
        ##################

	    LastStation=`head -1 kept_station_info.txt  | awk '{print $1}'`
	    InternalLog=InternalLog_2.txt
	    rm $InternalLog
        
	    while [ 1 = 1 ]
	    do
	        date +'%d-%b-%Y %R'
	        echo "qc_all_vars_internal,$last_year, '$processing_start','$svn_version',/odd,/spike,suffix='internal2',plots=$plots,diagnostics=$diagnostics, /flags, datadir='$datadir', restart_id='$LastStation'"
            
	        $idlcommand << EOF &>> $InternalLog
turbo
!EXCEPT=0
qc_all_vars_internal,$last_year, '$processing_start','$svn_version',/odd,/spike,suffix='internal2',plots=$plots,diagnostics=$diagnostics, /flags, datadir='$datadir', restart_id='$LastStation
EOF
	
	        CompletionTest=`tail -n1 $InternalLog`
	        if [ "$CompletionTest" = "FINISHED" ]; then
		        break
	        elif [ "$CompletionTest" = "% Execution halted at: \$MAIN\$          " ]; then
                ##############
	            # failure in IDL - likely a -31 error
		        #    find the last station processed
		        #    restart from this point on next iteration of while-loop
                ##############
		        LastStation=`grep "Station Identifier" $InternalLog | tail -n1 | awk '{print $4}'`
	        fi
	    done
    fi
    ZippedFiles=`ls ${datadir}/*_mask.nc`
    if [ "$ZippedFiles" != "" ]; then 
	    find $datadir -name "*_mask.nc" | xargs gzip -f &
    fi
fi

if [ $external2 -eq 1 ]; then
    ##################
    # External Checks 2
    ##################

    LastStation=`head -1 kept_station_info.txt  | awk '{print $1}'`
    ExternalLog=ExternalLog_2.txt
    rm $ExternalLog

    while [ 1 = 1 ]
    do
	    date +'%d-%b-%Y %R'
	    echo "qc_all_vars_external,$last_year, '$processing_start', '$svn_version',suffix='external2',/clean,/upclean,/unflag,plots=plots,diagnostics=diagnostics,datadir='$datadir', restart_id='$LastStation'"
        
	    $idlcommand << EOF &>> $ExternalLog
turbo
!EXCEPT=0
qc_all_vars_external,$last_year, '$processing_start', '$svn_version',suffix='external2',/clean,/upclean,/unflag,plots=plots,diagnostics=diagnostics,datadir='$datadir', restart_id='$LastStation'
EOF

	    CompletionTest=`tail -n1 $ExternalLog`
	    if [ "$CompletionTest" = "FINISHED" ]; then
	        break
	    elif [ "$CompletionTest" = "% Execution halted at: \$MAIN\$          " ]; then
            ##############
	        # failure in IDL - likely a -31 error
	        # find the last station processed
	        #    restart from this point on next iteration of while-loop
            ##############
	        LastStation=`grep "Station Identifier" $ExternalLog | tail -n1 | awk '{print $4}'`
	    elif [ "$CompletionTest" = "%                      $MAIN$          " ]; then
            ##############
	        # failure in IDL - likely a -31 error
	        # find the last station processed
	        #    restart from this point on next iteration of while-loop
            ##############
	        LastStation=`grep "Station Identifier" $ExternalLog | tail -n1 | awk '{print $4}'`
	    fi
	    
    done
    ZippedFiles=`ls ${datadir}/*_internal2.nc`
    if [ "$ZippedFiles" != "" ]; then 
	    find $datadir -name "*_internal2.nc" | xargs gzip -f &
    fi
fi

if [ $mask2 -eq 1 ]; then
    ##################
    # Masking Checks 2
    ##################
    LastStation=`head -1 kept_station_info.txt  | awk '{print $1}'`
    MaskingLog=MaskingLog_2.txt
    rm $MaskingLog

    while [ 1 = 1 ]
    do
	    date +'%d-%b-%Y %R'
	    echo "qc_masking,$last_year, '$processing_start', '$svn_version',suffix='mask2',plots=plots, datadir='$datadir', restart_id='$LastStation'"
        
	    $idlcommand << EOF &>> $MaskingLog
turbo
!EXCEPT=0
qc_masking,$last_year, '$processing_start', '$svn_version',suffix='mask2',plots=plots, datadir='$datadir', restart_id='$LastStation'
EOF
	    CompletionTest=`tail -n1 $MaskingLog`
	    if [ "$CompletionTest" = "FINISHED" ]; then
	        break
	    elif [ "$CompletionTest" = "% Execution halted at: \$MAIN\$          " ]; then
            ##############
	        # failure in IDL - likely a -31 error
	        # find the last station processed
	        #    restart from this point on next iteration of while-loop
            ##############
	        LastStation=`grep "Station Identifier" $MaskingLog | tail -n1 | awk '{print $4}'`
	    fi
    done


    ##############
    # Create the summary files
    ##############
    SummaryLog=SummaryLog_mask2.txt
    $idlcommand << EOF &> $SummaryLog 
turbo
!EXCEPT=0
qc_summary_create,suffix='mask2', datadir='$datadir', outputdir='./'
EOF


    ZippedFiles=`ls ${datadir}/*_external2.nc`
    if [ "$ZippedFiles" != "" ]; then 
	    find $datadir. -name "*_external2.nc" | xargs gzip -f
    fi
    ZippedFiles=`ls ${datadir}/*_mask2.nc`
    if [ "$ZippedFiles" != "" ]; then 
	    find $datadir -name "*_mask2.nc" | xargs gzip -f
    fi
fi

ZippedFiles=`ls ${datadir}/*.nc`
if [ "$ZippedFiles" != "" ]; then 
    find $datadir -name "*.nc" | xargs gzip -f
fi
ZippedFiles=`ls ${ImageDir}/*.*ps`
if [ "$ZippedFiles" != "" ]; then 
    find $ImageDir -name "*.*ps" | xargs gzip -f
fi

echo "Processing Complete"
date +'%d-%b-%Y %R'
#************************************************************************
#                                 END
#************************************************************************



