;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;British Crown Copyright (c) 2012, the Met Office All rights reserved.
;
;Redistribution and use in source and binary forms, with or without 
;modification, are permitted provided that the following conditions 
;are met:
;
;Redistributions of source code must retain the above copyright notice, 
;this list of conditions and the following disclaimer. 
;
;Redistributions in binary form must reproduce the above copyright 
;notice, this list of conditions and the following disclaimer in the 
;documentation and/or other materials provided with the distribution. 
;
;Neither the name of the Met Office nor the names of its contributors 
;may be used to endorse or promote products derived from this software 
;without specific prior written permission. 
;
;THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
;"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
;LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
;A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
;HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
;SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
;LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
;DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
;THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
;(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
;OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

pro map_selection, criteria_array, outfile
;------------------------------------------------------------------------
; NAME:
;     MAP_SELECTION
;
; PURPOSE:
;     To plot locations of stations which match the given selection criteria
;
; CALLING SEQUENCE:
;     map_selection,criteria_array,outfile
;
; INPUTS:
;     criteria_array - array of 13 1s and 0s to indicate which of the
;                      following criteria are met (in this order)
;
; 1. 81-00 climatology possible
; 2. 76-05 climatology possible
; 3. 86-05 climatology possible
; 4. 91-00 climatology possible
; 5. Humidity exists
; 6. Temperature exists
; 7. wind exists
; 8. Cloud cover total exists
; 9. Hourly resolution data
; 10. Hourly resolution for some times
; 11. Three-hourly resolution data
; 12. six-hourly resolution data
; 13. twelve-hourly resolution
;
;     outfile - name of output .ps file and also .txt file.  Must
;               include the .ps
;
;
; METHOD:
;     Reads ish-history files and criteria_match.txt file and outputs a ps file
;
; OUTPUTS:
;     One .ps and one .txt file
;
; RESTRICTIONS:
;     Requires criteria_match.txt file
;     
;   
; HISTORY:
;      #271 - copied from station_selection on KMW.
;             22 December 2011 RJHD
;      #272 - Adjusted to RJHD file system, added header
;             information.  Converted to Robinson projection and
;             removed elevation information.
;             22 December 2011 RJHD
;************************************************************************
;                    SVN Info
;$Rev:: 317                                           $:  Revision of last commit
;$Author:: rdunn                                      $:  Author of last commit
;$Date:: 2012-08-29 17:45:37 +0100 (Wed, 29 Aug 2012) $:  Date of last commit
;
;************************************************************************
;                                 START
;************************************************************************


; map_selection,[0,1,0,0,1,1,1,1,0,0,1,0,0],"allvars_7605_3hourly.ps"
; map_selection,[0,1,0,0,1,1,1,1,1,0,0,0,0],"allvars_7605_1hourly.ps"
; map_selection,[0,1,0,0,1,1,1,1,0,0,0,0,1],"allvars_7605_12hourly.ps"

ON_ERROR,1

; Program plots as a map those points which meet or exceed the given
; criteria given in criteria array on pass and outputs to a postscript
; plot file for subsequent consideration.

; Check that the criteria array is valid and that the outfile is a string 
; variable of at least four characters and contains '.ps' extension.
; Is the criteria array an integer array of 13?
check=SIZE(criteria_array)
IF check(1) NE 13 THEN MESSAGE,'Passed array is of wrong size'
; Does it contain at least one value that is 1?
points=WHERE(criteria_array EQ 1, pointcount)
IF pointcount LE 0 THEN MESSAGE,'Passed array contains no criteria against which to assess.'
; Is the outfile name at least four characters long?
length_of_string=STRLEN(outfile)
IF length_of_string LE 3 THEN MESSAGE,'outfile is too short'
; Does it contain the .ps extension?
ps_match=STRPOS(outfile,'.ps')
IF ps_match LE 0 THEN MESSAGE,'No .ps extension in outfile'
; Make a .txt filename for the station list output.
outfile_trunc=STRMID(outfile,0,ps_match)
outfile_trunc='./'+outfile_trunc+'.txt'
PRINT,outfile_trunc

; Read in the full list of location info first so we have something to match against.
station_ids=STRARR(28986)
lats=FLTARR(28986)
lats(*)=-999.
lons=FLTARR(28986)
lons(*)=-999.
elevs=FLTARR(28986)
elevs(*)=-999.

OPENR,1,'./ish-history.txt'
line=''
FOR i=0,16 DO READF,1,line
FOR st=0,28162 DO BEGIN
  READF,1,line
  station_ids(st)=STRCOMPRESS(STRMID(line,0,6)+'-'+STRMID(line,7,5),/remove_all)
  temp=STRCOMPRESS(STRMID(line,58,6),/remove_all)
  IF STRLEN(temp) GE 2 THEN lats(st)=FLOAT(temp)/1000.
  temp=STRCOMPRESS(STRMID(line,65,7),/remove_all)
  IF STRLEN(temp) GE 2 THEN lons(st)=FLOAT(temp)/1000.
  temp=STRCOMPRESS(STRMID(line,74,6),/remove_all)
  IF STRLEN(temp) GE 2 THEN elevs(st)=FLOAT(temp)/10.
ENDFOR
CLOSE,1
 
OPENR,1,'./isd-history-additional-data.txt'
FOR i=0,20 DO READF,1,line
FOR st=28163,28985 DO BEGIN
  READF,1,line
  station_ids(st)=STRCOMPRESS(STRMID(line,0,6)+'-'+STRMID(line,8,5),/remove_all)
  temp=STRCOMPRESS(STRMID(line,80,6),/remove_all)
  IF STRLEN(temp) GE 2 THEN lats(st)=FLOAT(temp)/1000.
  temp=STRCOMPRESS(STRMID(line,88,7),/remove_all)
  IF STRLEN(temp) GE 2 THEN lons(st)=FLOAT(temp)/1000.
  temp=STRCOMPRESS(STRMID(line,96,6),/remove_all)
  IF STRLEN(temp) GE 2 THEN elevs(st)=FLOAT(temp)/10.
ENDFOR
CLOSE,1
;
; Missing fix on elevations
points=WHERE(elevs GE 6000.)
elevs(points)=-999.

SET_PLOT,'ps'
Device,file='./images/'+outfile,/color,/helvetica
!P.font=0
!P.Position=[0.08,0.08,0.96,0.9]
plotsym,0,0.55,/fill
RESTORE_COLOURS,'shortbow'
Map_set,/continents,/noborder,color=0,/noerase,/robinson,/grid, glinestyle=2
; Read in the criteria_match.txt file and for each station that matches
; plot a cross on the map and output to the text file.
OPENR,1,'criteria_match.txt'
OPENW,2,outfile_trunc
lat=0.
lon=0.
station_id=''
this_array=INTARR(13)
points=WHERE(criteria_array EQ 1, pointcount)
total_stns=0
WHILE NOT EOF(1) DO BEGIN
  ; Read in the current line
  READF,1,station_id,this_array,FORMAT='(A12,1x,13(I1,2x))'
  ; Work out if a match to criteria
  match_points=WHERE(this_array(points) EQ 1, match_count)
  ; If a match then plot on the map and add to the output file.
  IF match_count EQ pointcount THEN BEGIN
     match=WHERE(station_ids EQ station_id, c)
     this_color=7
    IF c LE 0 THEN STOP,'non-match found'
    IF elevs(match(0)) LE 1000. THEN BEGIN
      this_color=7
    ENDIF ELSE IF elevs(match(0)) LE 2000. THEN BEGIN
      this_color=7
    ENDIF ELSE this_color=7
    PLOTS,lons(match(0)),lats(match(0)),psym=8,color=this_color 
    total_stns=total_stns+1  
    ;
    PRINTF,2,station_id,lats(match(0)),lons(match(0)),elevs(match(0)),FORMAT='(A12,1x,3(F9.4,1x))'
  ENDIF
ENDWHILE
CLOSE,1
CLOSE,2

PLOTS,[-179.9,0],[89.9,89.9],color=0
PLOTS,[0,179.9],[89.9,89.9],color=0
PLOTS,[-179.9,0],[-89.9,-89.9],color=0
PLOTS,[0,179.9],[-89.9,-89.9],color=0

XYOUTS,-180,85,'90N ',alignment=1,charsize=0.9,color=0
XYOUTS,-180,43,'45N ',alignment=1,charsize=0.9,color=0
XYOUTS,-180,-2,'0 ',alignment=1,charsize=0.9,color=0
XYOUTS,-180,-47,'45S  ',alignment=1,charsize=0.9,color=0
XYOUTS,-180,-89,'90S      ',alignment=1,charsize=0.9,color=0

XYOUTS,0.26,0.05,'180W',/normal,charsize=0.9,color=0
XYOUTS,0.38,0.05,'90W',/normal,charsize=0.9,color=0
XYOUTS,0.512,0.05,'0',/normal,charsize=0.9,color=0
XYOUTS,0.64,0.05,'90E',/normal,alignment=1,charsize=0.9,color=0
XYOUTS,0.78,0.05,'180E',/normal,alignment=1,charsize=0.9,color=0


XYOUTS,0.02,0.02,'Total stations: '+STRCOMPRESS(STRING(total_stns),/remove_all),/normal

IF criteria_array[8] EQ 1 THEN titlestring='Locations of Stations - 1 hourly'
IF criteria_array[10] EQ 1 THEN titlestring='Locations of Stations - 3 hourly'
IF criteria_array[12] EQ 1 THEN titlestring='Locations of Stations - 12 hourly'
XYOUTS,0.5,0.95,titlestring,/normal,alignment=0.5

DEVICE,/close_file

SET_PLOT,'X'
 
RETURN
 
END
