;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;British Crown Copyright (c) 2012, the Met Office All rights reserved.
;
;Redistribution and use in source and binary forms, with or without 
;modification, are permitted provided that the following conditions 
;are met:
;
;Redistributions of source code must retain the above copyright notice, 
;this list of conditions and the following disclaimer. 
;
;Redistributions in binary form must reproduce the above copyright 
;notice, this list of conditions and the following disclaimer in the 
;documentation and/or other materials provided with the distribution. 
;
;Neither the name of the Met Office nor the names of its contributors 
;may be used to endorse or promote products derived from this software 
;without specific prior written permission. 
;
;THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
;"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
;LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
;A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
;HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
;SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
;LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
;DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
;THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
;(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
;OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

PRO MONTH_SETUP, last_year, nmonths, nyears, times, full_times, full=full, $
                 monthly_segment_starts, monthly_segment_ends, monthly_segment_lengths
;*************************************************************

; NAME:
;     MONTH_SETUP
;
; PURPOSE:
;     Sets up the monthly arrays - month starts, ends and lengths
;
; CALLING SEQUENCE:
;     month_setup,last_year, nmonths, nyears, full_times, full,
;     monthly_segment_starts, monthly_segment_ends,
;     monthly_segment_lengths 
;
; INPUTS:
;     last_year - the final year of the data sequence
;
;     nmonths - variable containing the number of months of data
;
;     nyear   - variable containing the number of years of data
;
;     times   - variable containing the times when data was obtained.
; 
;     full_times - array containing all time when data could have been
;                  taken over the life of the station
;
;     full - keyword whether to use full_times or just the ones where
;            there is data
;
; OUTPUTS:
;     monthly_segment_starts - array containing the starts of all the months
; 
;     monthly_segment_ends   - array containing the ends of all the months
;
;     monthly_segment_lengths - array containing all the lengths of
;                               the months
;
;            ALL IN DATA SPACE.
;
;
; RESTRICTIONS:
;
;     None known
;
; COMMENTS
;
;     NOT used in check_duplicates.pro (031110) as this is the first
;     time many of the arrays are defined.
;
; HISTORY:
;     #1 - Created from the duplicate proceedures in the QC
;          subroutines.  Tried to make as universal as possible; so
;          that it encompasses all of the functionality of the 6
;          separate versions by Peter Thorne & Kate Willett
;          3 November 2010 RJHD
;     #24 - Adjusted the end_time value on the "full" run.  Reordered
;           the end loops
;           7 November 2010 RJHD
;     #200 - changed last_year from optional keyword to required input
;            to remove the hardwiring of the end date
;            11 March 2011 RJHD
;     #237 - SVN information added to header
;            16 November 2011 RJHD
;     #299 - Added option of two equally good matches
;            6 June 2012 RJHD
;
;************************************************************************
;                    SVN Info
;$Rev:: 303                                           $:  Revision of last commit
;$Author:: rdunn                                      $:  Author of last commit
;$Date:: 2012-06-14 14:02:44 +0100 (Thu, 14 Jun 2012) $:  Date of last commit
;************************************************************************
;                                 START
;************************************************************************
;
;**************************************************************
; Use the keyword last_year to define how many hours and months we are
; to consider.
;**************************************************************
start_date = JULDAY(1, 1, 1973, 0)
end_date = JULDAY(1, 1, last_year+1, 0)
nhours=LONG((end_date-start_date)*24.)
nmonths=(last_year-1973+1)*12
nyears=last_year-1973+1


;**************************************************************
; Define segment breaks as being 00UTC on first of each month and fill
; the monthly segments arrays.
;**************************************************************
first_point=0
years=INTARR(nmonths)
months=INTARR(nmonths)
year_ints=INDGEN(nyears+1)+1973
month_ints=INDGEN(12)+1
c=0
FOR y=0,nyears DO BEGIN
    IF y EQ 0 THEN BEGIN
        FOR m=1,11 DO BEGIN
            years(c)=year_ints(y)
            months(c)=month_ints(m)
            c=c+1
        ENDFOR
    ENDIF ELSE IF y EQ nyears THEN BEGIN
        years(c)=year_ints(y)
        months(c)=month_ints(0)
    ENDIF ELSE BEGIN
        FOR m=0,11 DO BEGIN
            years(c)=year_ints(y)
            months(c)=month_ints(m)
            c=c+1
        ENDFOR
    ENDELSE
ENDFOR

monthly_segment_starts=LONARR(nmonths)
monthly_segment_starts(0)=0L
monthly_segment_ends=LONARR(nmonths)
monthly_segment_lengths=INTARR(nmonths)

 
;**************************************************************
; We want to know the last point (we already know the first) of
; the current month. Actually do this by going forwards to 1st of
; next month and count back because this is going to be easier to cope
; with varying month lengths and leap years.  
;**************************************************************
IF KEYWORD_SET(full) THEN BEGIN
    FOR seg=0,nmonths-1 DO BEGIN
        end_time=JULDAY(months(seg), 1, years(seg), 0, 0)-JULDAY(1,1,1973,0,0)
        time_diffs=ABS(full_times-end_time)
        minimum_value=MIN(time_diffs)
        last_point=WHERE(time_diffs EQ minimum_value, matched)
        IF matched EQ 1 THEN BEGIN
            monthly_segment_ends(seg)=last_point(0)-1
            monthly_segment_lengths(seg)=monthly_segment_ends(seg)-monthly_segment_starts(seg)+1
            IF seg NE nmonths-1 THEN monthly_segment_starts(seg+1)=last_point ; resets
        ENDIF ELSE IF matched EQ 2 THEN BEGIN
            ; added 6 Jun 2012 RJHD
            monthly_segment_ends(seg)=last_point(0)
            monthly_segment_lengths(seg)=monthly_segment_ends(seg)-monthly_segment_starts(seg)+1
            IF seg NE nmonths-1 THEN monthly_segment_starts(seg+1)=last_point[1] ; resets
        ENDIF ELSE BEGIN
           PRINT, matched
           STOP, 'Times do not match exactly. Better exception handling required.'
           
        ENDELSE
    ENDFOR
ENDIF ELSE BEGIN
    new_times=times+start_date
    FOR seg=0,nmonths-1 DO BEGIN
        end_time=JULDAY(months(seg), 1, years(seg), 0, 0)
        time_diffs=ABS(new_times-end_time)
        minimum_value=MIN(time_diffs)
        last_point=WHERE(time_diffs EQ minimum_value, matched)
        monthly_segment_ends(seg)=last_point(0)-1
        monthly_segment_lengths(seg)=monthly_segment_ends(seg)-monthly_segment_starts(seg)
        IF seg ne nmonths-1 THEN monthly_segment_starts(seg+1)=last_point(0)
    ENDFOR
ENDELSE
 

RETURN

END
;************************************************************************
;                                 END
;************************************************************************
