;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;British Crown Copyright (c) 2012, the Met Office All rights reserved.
;
;Redistribution and use in source and binary forms, with or without 
;modification, are permitted provided that the following conditions 
;are met:
;
;Redistributions of source code must retain the above copyright notice, 
;this list of conditions and the following disclaimer. 
;
;Redistributions in binary form must reproduce the above copyright 
;notice, this list of conditions and the following disclaimer in the 
;documentation and/or other materials provided with the distribution. 
;
;Neither the name of the Met Office nor the names of its contributors 
;may be used to endorse or promote products derived from this software 
;without specific prior written permission. 
;
;THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
;"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
;LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
;A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
;HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
;SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
;LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
;DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
;THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
;(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
;OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

PRO NEIGHBOUR_ASSIGN,station_id=station_id,locfile=locfile,diagnostics=diagnostics,Nbours=neighbours,Ndists=neighdists,$
                     Nelevs=neighelevs,Nquadrants=neighquadrants,num_Nbours=num_neighbours

; NAME:
;     NEIGHBOUR_ASSIGN
;
; PURPOSE:
;     To assign neighbours to the station passed to the subroutine.
;     Optionally information on the neighbours is returned.
;
; CALLING SEQUENCE:
;     neighbour_assign,station_id=station_id,locfile=locfile,
;     diagnostics=diagnostics,Nbours=neighbours,
;     Ndists=neighdists,Nelevs=neighelevs,Nquadrants=neighquadrants,num_Nbours=num_neighbours 
;
; INPUTS:
;     station_id - the WMO ID of the station in question
;
;     locfile - the file containing the locations and angles of all
;               the stations from all other stations.
;
;     diagnostics - keyword as to whether diagnostic information
;                   should be printed to screen
;
;
; METHOD:
;     Looks for neighbours within 300km and 500m height difference
;     with at least two from each quadrant
;
; OUTPUTS:
;     Nbours - array returned containing the names of the neighbours
;
;     Ndists - array containing the distances of these neighbours
;
;     Nelevs - array containing the elevations of the neighbours
;
;     Nquadrants - array containing the quadrants in which the
;                  neighbours sit
;
;     num_Nbours - the number of neighbours found given the limits of
;                  the search
;    
;
; RESTRICTIONS:
;
;     Needs file station_locations_and_distances.nc to work from
;   
; HISTORY:
;      #1 - Initial version created from the code in
;            qc_all_vars_external.pro
;            16 November 2010 RJHD
;      #2 - BUG found in the setting of the elevations - on comparison
;           with original code, it had been introduced when creating
;           this programme.
;            18 November 2010 RJHD
;      #237 - SVN information added to header
;             16 November 2011 RJHD
;
;************************************************************************
;                    SVN Info
;$Rev:: 308                                           $:  Revision of last commit
;$Author:: rdunn                                      $:  Author of last commit
;$Date:: 2012-07-04 14:47:15 +0100 (Wed, 04 Jul 2012) $:  Date of last commit
;************************************************************************
;                                 START
;************************************************************************





  ;************************************************************************
  ; Read in the info on stations to be considered and their great arc distances
  ; from the netcdf file created by calculate_distances.pro in this directory.
  ;************************************************************************
cd, "./", current=pwd

locfile=pwd+'/station_locations_and_distances.nc'

;****************************************
; "fred" is the station location file
; "betty" contains information about
;  the structure of the netcdf file, fred
;****************************************

fred=NCDF_OPEN(locfile,/nowrite)

NCDF_CONTROL,fred,/verbose
betty=NCDF_INQUIRE(fred)
stationdimid=NCDF_DIMID(fred,'station')
chardimid=NCDF_DIMID(fred,'character_length')  
NCDF_DIMINQ,fred,stationdimid,stationdim,nstation
NCDF_DIMINQ,fred,chardimid,chardim,nchar
stationinfo=NCDF_VARINQ(fred,'station')
latinfo=NCDF_VARINQ(fred,'latitude')
loninfo=NCDF_VARINQ(fred,'longitude')
elevinfo=NCDF_VARINQ(fred,'elevation')
distinfo=NCDF_VARINQ(fred,'distance')
angleinfo=NCDF_VARINQ(fred,'angle')

NCDF_VARGET,fred,'station',stations
stations=STRING(stations)
stations=REFORM(stations,nstation)
NCDF_VARGET,fred,'longitude',longitudes
NCDF_VARGET,fred,'latitude',latitudes
NCDF_VARGET,fred,'elevation',elevations
NCDF_VARGET,fred,'distance',distances
NCDF_VARGET,fred,'angle',angles
    
NCDF_CLOSE,fred

;************************************************************************
; Resolve angles to quadrants. The numbers assigned are solely for computational expediency!
;************************************************************************
quadrants=angles
points=WHERE(angles LE -90.)
quadrants(points)=1000.
points=WHERE(angles GE 90.)
quadrants(points)=4000.
points=WHERE(quadrants LE 0.)
quadrants(points)=2000.
points=WHERE(quadrants Le 100.)
quadrants(points)=3000.

;************************************************************************
; Free up memory. Angles now changed to quadrants.
;************************************************************************
angles=0.  



;************************************************************************
; If a station name is given, look it up.  Else do all stations.
;************************************************************************

IF KEYWORD_SET(station_id) THEN BEGIN
    
    these_stations=[WHERE(stations EQ station_id)]
    print,'Station ',station_id,', Number ',strcompress(these_stations,/remove_all)

ENDIF ELSE BEGIN

    these_stations=INDGEN(n_elements(stations))
ENDELSE

nstations=n_elements(these_stations)-1

FOR station_number=0,nstations DO BEGIN

    st=these_stations[station_number]
;************************************************************************
; Which are the ten nearest neighbours to the current station?
;************************************************************************
    st_distances=distances(st,*)
    st_quads=quadrants(st,*)
    st_elevs=elevations  ; was elevations(st,*) which set all elevations to that for this station 18/11/10 RJHD
    
    sortorder=SORT(st_distances)
    candidates=stations[sortorder]
    canddists=st_distances[sortorder]
    candquads=st_quads[sortorder]
    candelevs=st_elevs[sortorder]
    no_neighbours=1
    counter=1                   ; avoids the same station issue
    num_neighbours=0
    cont=1
    WHILE cont NE 0 DO BEGIN
    ;************************************************************************
    ; is it within 300 km? - THIS IS QUITE LARGE
    ;************************************************************************
        IF canddists(counter) LE 300. THEN BEGIN ;***KATE JUL 2010 changed from 500 to 300
        ;************************************************************************
        ; Is it within 500m height?
        ;************************************************************************
            IF ABS(elevations(st)-candelevs(counter)) LE 500. THEN BEGIN
                IF no_neighbours EQ 1 THEN BEGIN
                    neighbours=[candidates(counter)]
                    neighdists=[canddists(counter)]
                    neighelevs=[candelevs(counter)]
                    neighquadrants=[candquads(counter)]
                    no_neighbours=0
                ENDIF ELSE BEGIN
                    neighbours=[neighbours,candidates(counter)]
                    neighdists=[neighdists,canddists(counter)]
                    neighelevs=[neighelevs,candelevs(counter)]
                    neighquadrants=[neighquadrants,candquads(counter)]
                ENDELSE
                num_neighbours=num_neighbours+1
                                
                IF num_neighbours GE 10 THEN BEGIN
           ;************************************************************************
	       ; Check whether we have at least two from each quadrant. If we do not then we go round until we do.
           ;************************************************************************
                    points=WHERE(neighquadrants EQ 1000., cnt1)
                    points=WHERE(neighquadrants EQ 2000., cnt2)
                    points=WHERE(neighquadrants EQ 3000., cnt3)
                    points=WHERE(neighquadrants EQ 4000., cnt4)
                    IF cnt1 GE 2 AND cnt2 GE 2 AND cnt3 GE 2 AND cnt4 GE 2 THEN cont=0
                ENDIF
            ENDIF 
        ENDIF ELSE cont=0       ; Not within 500Km
        counter=counter+1
    ENDWHILE

;************************************************************************
; We now thin the number down if they are greater than ten by
; providing the up to two nearest in each quadrant plus the next
; N nearest stations.
;************************************************************************

    IF N_ELEMENTS(neighbours) GE 11 THEN BEGIN
        oldneighbours=neighbours
        oldneighdists=neighdists
        oldneighelevs=neighelevs
        oldneighquadrants=neighquadrants
        
        started=0
        points=WHERE(oldneighquadrants EQ 1000., cnt1)
        IF cnt1 GE 1 THEN BEGIN
            IF cnt1 EQ 1 THEN BEGIN
                neighbours=[oldneighbours(points(0))]
                neighdists=[oldneighdists(points(0))]
                neighelevs=[oldneighelevs(points(0))]
                neighquadrants=[oldneighquadrants(points(0))]
            ENDIF ELSE BEGIN 
                neighbours= [oldneighbours(points(0:1))]
                neighdists=[oldneighdists(points(0:1))]
                neighelevs=[oldneighelevs(points(0:1))]
                neighquadrants=[oldneighquadrants(points(0:1))]
            ENDELSE
            started=1
        ENDIF
        points=WHERE(oldneighquadrants EQ 2000., cnt2)
        IF cnt2 GE 1 THEN BEGIN
            IF started EQ 0 THEN BEGIN
                IF cnt2 EQ 1 THEN BEGIN
                    neighbours=[oldneighbours(points(0))]
                    neighdists=[oldneighdists(points(0))]
                    neighelevs=[oldneighelevs(points(0))]
                    neighquadrants=[oldneighquadrants(points(0))]
                ENDIF ELSE BEGIN 
                    neighbours= [oldneighbours(points(0:1))]
                    neighdists=[oldneighdists(points(0:1))]
                    neighelevs=[oldneighelevs(points(0:1))]
                    neighquadrants=[oldneighquadrants(points(0:1))]
                ENDELSE
                started=1
            ENDIF ELSE BEGIN
                IF cnt2 EQ 1 THEN BEGIN
                    neighbours=[neighbours,oldneighbours(points(0))]
                    neighdists=[neighdists,oldneighdists(points(0))]
                    neighelevs=[neighelevs,oldneighelevs(points(0))]
                    neighquadrants=[neighquadrants,oldneighquadrants(points(0))]
                ENDIF ELSE BEGIN 
                    neighbours= [neighbours,oldneighbours(points(0:1))]
                    neighdists=[neighdists,oldneighdists(points(0:1))]
                    neighelevs=[neighelevs,oldneighelevs(points(0:1))]
                    neighquadrants=[neighquadrants,oldneighquadrants(points(0:1))]
                ENDELSE
            ENDELSE
        ENDIF
        points=WHERE(oldneighquadrants EQ 3000., cnt3)
        IF cnt3 GE 1 THEN BEGIN
            IF started EQ 0 THEN BEGIN
                IF cnt3 EQ 1 THEN BEGIN
                    neighbours=[oldneighbours(points(0))]
                    neighdists=[oldneighdists(points(0))]
                    neighelevs=[oldneighelevs(points(0))]
                    neighquadrants=[oldneighquadrants(points(0))]
                ENDIF ELSE BEGIN 
                    neighbours= [oldneighbours(points(0:1))]
                    neighdists=[oldneighdists(points(0:1))]
                    neighelevs=[oldneighelevs(points(0:1))]
                    neighquadrants=[oldneighquadrants(points(0:1))]
                ENDELSE
                started=1
            ENDIF ELSE BEGIN
                IF cnt3 EQ 1 THEN BEGIN
                    neighbours=[neighbours,oldneighbours(points(0))]
                    neighdists=[neighdists,oldneighdists(points(0))]
                    neighelevs=[neighelevs,oldneighelevs(points(0))]
                    neighquadrants=[neighquadrants,oldneighquadrants(points(0))]
                ENDIF ELSE BEGIN 
                    neighbours= [neighbours,oldneighbours(points(0:1))]
                    neighdists=[neighdists,oldneighdists(points(0:1))]
                    neighelevs=[neighelevs,oldneighelevs(points(0:1))]
                    neighquadrants=[neighquadrants,oldneighquadrants(points(0:1))]
                ENDELSE
            ENDELSE
        ENDIF
        points=WHERE(oldneighquadrants EQ 4000., cnt4)
        IF cnt4 GE 1 THEN BEGIN
            IF started EQ 0 THEN BEGIN
                IF cnt4 EQ 1 THEN BEGIN
                    neighbours=[oldneighbours(points(0))]
                    neighdists=[oldneighdists(points(0))]
                    neighelevs=[oldneighelevs(points(0))]
                    neighquadrants=[oldneighquadrants(points(0))]
                ENDIF ELSE BEGIN 
                    neighbours= [oldneighbours(points(0:1))]
                    neighdists=[oldneighdists(points(0:1))]
                    neighelevs=[oldneighelevs(points(0:1))]
                    neighquadrants=[oldneighquadrants(points(0:1))]
                ENDELSE
                started=1
            ENDIF ELSE BEGIN
                IF cnt4 EQ 1 THEN BEGIN
                    neighbours=[neighbours,oldneighbours(points(0))]
                    neighdists=[neighdists,oldneighdists(points(0))]
                    neighelevs=[neighelevs,oldneighelevs(points(0))]
                    neighquadrants=[neighquadrants,oldneighquadrants(points(0))]
                ENDIF ELSE BEGIN 
                    neighbours= [neighbours,oldneighbours(points(0:1))]
                    neighdists=[neighdists,oldneighdists(points(0:1))]
                    neighelevs=[neighelevs,oldneighelevs(points(0:1))]
                    neighquadrants=[neighquadrants,oldneighquadrants(points(0:1))]
                ENDELSE
            ENDELSE
        ENDIF
        
;************************************************************************
; We've taken the up to the first two from each quadrant. Now fill with remaining stations with no preference.
;************************************************************************
    
        num_to_fill=10-N_ELEMENTS(neighbours)
        filled=0
        ctr=0
        WHILE filled NE num_to_fill DO BEGIN
            matchedval=WHERE(neighbours EQ oldneighbours(ctr), cnt5)
            IF cnt5 NE 1 THEN BEGIN
        ; Non-matched value so append
                neighbours=[neighbours,oldneighbours(ctr)]
                neighdists=[neighdists,oldneighdists(ctr)]
                neighelevs=[neighelevs,oldneighelevs(ctr)]
                neighquadrants=[neighquadrants,oldneighquadrants(ctr)]
                filled=filled+1
            ENDIF
            ctr=ctr+1
        ENDWHILE 
        num_neighbours=10       ; Fixes this counter.
    ENDIF
ENDFOR

IF KEYWORD_SET(diagnostics) AND KEYWORD_SET(station_id) THEN BEGIN
    neigh_count=N_ELEMENTS(neighbours)
   
    PRINT,'Neighbour ','Dist','Elev',FORMAT='(A14,A8,A8)'
    FOR nc=0,neigh_count-1 DO BEGIN
        PRINT,neighbours[nc],neighdists[nc],neighelevs[nc],FORMAT='(A14,F8.2,F8.1)'
        
    ENDFOR
ENDIF


RETURN

END
