;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;British Crown Copyright (c) 2012, the Met Office All rights reserved.
;
;Redistribution and use in source and binary forms, with or without 
;modification, are permitted provided that the following conditions 
;are met:
;
;Redistributions of source code must retain the above copyright notice, 
;this list of conditions and the following disclaimer. 
;
;Redistributions in binary form must reproduce the above copyright 
;notice, this list of conditions and the following disclaimer in the 
;documentation and/or other materials provided with the distribution. 
;
;Neither the name of the Met Office nor the names of its contributors 
;may be used to endorse or promote products derived from this software 
;without specific prior written permission. 
;
;THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
;"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
;LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
;A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
;HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
;SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
;LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
;DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
;THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
;(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
;OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

PRO odd_cluster_check, times,full_times, h_times,full_h_times,full_values, mdi, fdi, qc_flags, flagcol, filter_flags, $
                       diagnostics, station_id

; NAME:
;     ODD_CLUSTER CHECK
;
; PURPOSE:
;     to check for small groups of isolated data - up to 6 hours in a
;     day and separated by up to 2 days on either side.
;
; CALLING SEQUENCE:
;     odd_cluster_check, times, values, mdi, qc_flags, flagcol, diagnostics, station_id
;
; INPUTS:
;     times - array containing the times the data was obtained in
;             fractions of a day
;
;     full_times - array containing the uncompressed times the data
;                  was obtained in fractions of a day
;
;     h_times - array containing the times the data was obtained - in hours
;
;     full_h_times - array containing the uncompressed times the data
;                    was obtained in hours
;
;     full_values - array containing the uncompressed data matched to the times.
;
;     mdi -  missing data indicator - appropriate for the type
;
;     fdi -  flagged data indicator - appropriate for the type
;
;     qc_flags - array containing flags for the data
;
;     flagcol -  column identifier for qc_flags
;
;     filter_flags - array containing all flags previously set on values
;
;     diagnostics - keyword to allow plots and other diagnostic
;                   information to be produced
;
;     station_id - station identifier for the plots
;
;
; OUTPUTS:
;
;     None - returns amended arrays to the control script
;
; RESTRICTIONS:
;
;
;
; METHOD:
;     Loops through the values counting missing and present data
;     periods.  Flagging triggered if criteria are met.
;     
; HISTORY:
;     NB - not all revisions are noted here.
;     # 1 - contains the form of the program as obtained
;           October 2010 RJHD 
;     # 9 - adjusted the script to run for any data, and data type set
;           by control script.  Now much shorter & a bit clearer
;           03 November 2010 RJHD
;     #37 - included diagnostic plots of when an odd cluster is found
;           10 November 2010 RJHD
;     #42 - added colour to highlight the cluster in question
;           10 November 2010 RJHD
;     #50 - EndOfNormalData given starting value.  Diagnostic plots updated
;           15 November 2010 RJHD
;     #92 - Changed how option 3 was dealt with - assume beginning of
;           next cluster rather than normal data.
;           24 November 2010 RJHD
;     #127 - Changed to run with full_times to more fully find odd
;            clusters of data.
;            6 December 2010 RJHD
;     #171 - Added filter flags to pass through the total flags set
;            for each value
;            14 January 2011 RJHD
;     #173 - Used filtered values to find odd clusters in timeseries
;            The only previous test is check_diurnal.
;            17 January 2011 RJHD
;     #174 - Adjusted how the flags are set into the array to prevent
;            the wiping of previously set flags.
;            18 January 2011 RJHD
;     #237 - SVN information added to header
;            Added fdi checks to mdi checks
;            16 November 2011 RJHD
;     #299 - Added time-checks using hours rather than fractions of a day
;            6 June 2012 RJHD
;
;************************************************************************
;                    SVN Info
;$Rev:: 308                                           $:  Revision of last commit
;$Author:: rdunn                                      $:  Author of last commit
;$Date:: 2012-07-04 14:47:15 +0100 (Wed, 04 Jul 2012) $:  Date of last commit
;************************************************************************
;                                 START
;************************************************************************

;**************************************************************
; Set up filtered values.
;**************************************************************
untouched_full_values=full_values

; match using hours to avoid rounding issues

MATCH, h_times, full_h_times, suba, subb, count=cnt


past_flags=WHERE(filter_flags GT 0, pfcnt)

IF pfcnt GE 1 THEN BEGIN
    full_values(subb(past_flags))=mdi
ENDIF


ntimes=N_ELEMENTS(full_times)

qc_flags_odd=INTARR(ntimes)
qc_flags_odd(*)=0

;*************************************************************

pre_last_data_time=full_h_times(0)-double(1.0)	; set to hour before first data point
begin_odd_cluster_time=double(mdi)
cluster_length=0
cluster_locs=double(0.0)
end_odd_cluster_time=double(mdi)

;*************************************************************
; switch to say that we are within a potential cluster
;     0=normal, 1=between present data pre cluster, 2=within potential cluster, 3=between present data post cluster		
;*************************************************************
potential_cluster=1	
EndOfNormalData=0
FigCounter=0 ; for diagnostic plots
FOR i=0L,ntimes-1 DO BEGIN
  IF (((full_values(i) NE mdi) AND (full_values(i) NE fdi))) THEN BEGIN
    ;0=normal, 1=between present data pre cluster, 
    ;2=within potential cluster, 3=between present data post cluster		
    CASE potential_cluster OF
        ; note, tested in order below - hence non-numeric order
        ; should be fast this way
        ; if there is data in the record, then enter CASE, else below.
      0: BEGIN
          pre_last_data_time=full_h_times(i)
          EndOfNormalData=i
      END
      2: BEGIN   
        IF (cluster_length EQ 6) OR (full_h_times(i)-begin_odd_cluster_time GT 1.0*24.) THEN BEGIN 
            ;*************************************************************
            ; we're in a potential cluster
            ; and length is 6 or outside of a single day (1.0 * 24hrs)
            ; then actually not a cluster (as defined here)
            ; but the start of a larger set of observations
            ;*************************************************************
          potential_cluster=0
          pre_last_data_time=full_h_times(i) 
          begin_odd_cluster_time=double(mdi)
          cluster_length=0
          cluster_locs=double(0.0)
          end_odd_cluster_time=double(mdi)
        ENDIF ELSE BEGIN
            ;*************************************************************
            ; we're in a potential cluster
            ; length is less than 6 hours and
            ; and haven't scanned through more than 24hr
            ;*************************************************************
          cluster_length=cluster_length+1
          cluster_locs=[cluster_locs,full_h_times(i)]
          end_odd_cluster_time=full_h_times(i)
        ENDELSE
      END
      1: BEGIN
        IF (full_h_times(i)-pre_last_data_time GE 2.0*24.) THEN BEGIN 
          ;*************************************************************
          ; we're in a potential cluster
          ; increment cluster length by one
          ; record start time
          ; record end time (to be amended)
          ; note location
          ; at least one missing observation
          ;*************************************************************
          potential_cluster=2 
          cluster_length=cluster_length+1 
          begin_odd_cluster_time=full_h_times(i) 
          end_odd_cluster_time=full_h_times(i) 
          cluster_locs=full_h_times(i) 
        ENDIF ELSE BEGIN 
          ;*************************************************************
          ; else no missing observation - set to normal
          ;*************************************************************
          potential_cluster=0
          pre_last_data_time=full_h_times(i) 
          begin_odd_cluster_time=double(mdi)
          cluster_length=0
          cluster_locs=double(0.0)
          end_odd_cluster_time=double(mdi)
        ENDELSE
      END
      3: BEGIN
        IF (full_h_times(i)-end_odd_cluster_time GE 2.0*24.) THEN BEGIN	
          ;*************************************************************
          ; here is an isolated cluster within a 48hr (24*2.0) period, so initiate flagging
          ;*************************************************************
          MATCH,full_h_times,cluster_locs,lc1,lc2,count=count

          qc_flags_odd(lc1)=1

          ;*************************************************************
          ; Match these flags into the complete flagging array.
          ;   Do not wipe any pre-existing flags so only copy across
          ;   where flags=1
          ;*************************************************************

          MATCH,h_times,full_h_times,tft1,tft2,count=count
          new_flags=WHERE(qc_flags_odd(tft2) EQ 1, nfcnt)

          IF nfcnt GE 1 THEN BEGIN
              qc_flags(flagcol,new_flags)=qc_flags_odd(tft2(new_flags))
          ENDIF

          ;**********************************
          ; Set the filtering flags
          ;**********************************
          oc_flags=WHERE(qc_flags_odd(tft2) GT 0, occnt)

          IF occnt GE 1 THEN BEGIN
              filter_flags(oc_flags)=1
          ENDIF
          
          IF KEYWORD_SET(diagnostics) THEN BEGIN

              CASE flagcol OF
                  54:BEGIN
                      label="temperature"
                  END
                  55:BEGIN
                      label="dewpoints"
                  END
                  56:BEGIN
                      label="wind"
                  END
                  57:BEGIN
                      label="slp"
                  END
              ENDCASE
              set_plot,'PS'
              tvlct,0,0,0,0     ; black for kept datapoints
              cd, "./", current=outdirplot
              device,filename=outdirplot+'/images/'+station_id+'_OddCluster_'+label+'_'+string(FigCounter,FORMAT='(I2.2)')+'.eps'$
                ,/color,/encapsulate,/helvetica,/bold,/landscape,ysize=20,xsize=26
              !P.Font=0
              !P.Thick=4
              !P.Multi=[0,1,1]
              plotsym,3,1.2

              dummy=LABEL_DATE(DATE_FORMAT=['%D/%N/%Y'])
              start_date=JULDAY(1, 1, 1973, 0)


              temp_values=full_values(WHERE(((full_values NE mdi) AND (full_values NE fdi)),mdicnt))
              temp_times=full_h_times(WHERE(((full_values NE mdi) AND (full_values NE fdi)),mdicnt))
              
              MATCH, full_h_times(i),temp_times, fttt1,fttt2
              ntemptimes=n_elements(temp_times)

              RANGE=30         ; show 100 points either side.
              IF (fttt2-RANGE) LT 0 THEN MIN_X=0 ELSE MIN_X=fttt2-RANGE
              IF (ntemptimes-1-fttt2) LT RANGE THEN MAX_X=ntemptimes-1 ELSE MAX_X=fttt2+RANGE
              
              useful_values=temp_values(MIN_X:MAX_X)
              useful_times=temp_times(MIN_X:MAX_X)
              MINY=MIN(useful_values)
              MAXY=MAX(useful_values)
              
              plot,useful_times+start_date,useful_values,min_value=-100,xstyle=1,ystyle=1,ytitle=label,xtitle="time",xtickformat='LABEL_DATE',$
                title="Odd Cluster - Diagnostic Timeseries - "+label,charsize=1,ythick=4,xthick=4,psym=2,yrange=[0.9*MINY,1.1*MAXY], POSITION=[0.10, 0.10, 0.90, 0.90]
              oplot,useful_times+start_date,useful_values,min_value=-100,xstyle=1,ystyle=1,ytitle=label,xtitle="time",xtickformat='LABEL_DATE',$
                title="Odd Cluster - Diagnostic Timeseries - "+label,charsize=1,ythick=4,xthick=4,psym=0,yrange=[0.9*MINY,1.1*MAXY], POSITION=[0.10, 0.10, 0.90, 0.90]

              oplot,[full_times(lc1)+start_date],[full_values(lc1)],min_value=-100,xstyle=1,ystyle=1,ytitle=label,xtitle="time",xtickformat='LABEL_DATE',$
                title="Odd Cluster - Diagnostic Timeseries - "+label,charsize=1,ythick=4,xthick=4,psym=4, color=239, POSITION=[0.10, 0.10, 0.90, 0.90]

              device,/close
              
              FigCounter=FigCounter+1
              EndOfNormalData=i

          ENDIF
          ;*************************************************************
          ; Changed - the last data was the end
          ; of the cluster and we do not know
          ; how long the current stretch of data will be
          ; 24 November 2010 RJHD
          ;*************************************************************
          ; potential_cluster=0
          ; pre_last_data_time=times(i) 
          ; cluster_length=0
          ; cluster_locs=double(0.0)
          ; end_odd_cluster_time=double(mdi)
          ; begin_odd_cluster_time=double(mdi)

          pre_last_data_time=end_odd_cluster_time
          potential_cluster=2
          begin_odd_cluster_time=full_h_times(i) 
          end_odd_cluster_time=full_h_times(i) 
          cluster_locs=full_h_times(i) 
          cluster_length=1

        ENDIF ELSE IF (full_h_times(i)-begin_odd_cluster_time LE 1.0*24.) AND (cluster_length LT 6) THEN BEGIN
          ;*************************************************************
          ; have data, but cluster length <6 and span <24hr
          ; so increment cluster
          ;*************************************************************
          potential_cluster=2
          cluster_length=cluster_length+1
          cluster_locs=[cluster_locs,full_h_times(i)]
          end_odd_cluster_time=full_h_times(i)
        ENDIF ELSE BEGIN	
          ;*************************************************************
          ; actually no longer a cluster, but gappy run of data
          ; set to normal
          ;*************************************************************
          potential_cluster=0
          pre_last_data_time=full_h_times(i) 
          begin_odd_cluster_time=double(mdi)
          cluster_length=0
          cluster_locs=double(0.0)
          end_odd_cluster_time=double(mdi)
        ENDELSE
      END
    ENDCASE  
  ENDIF ELSE BEGIN
     ;*************************************************************
     ; value == missing data indicator, therefore
     ;*************************************************************
    IF (potential_cluster EQ 2) THEN potential_cluster=3 ; now after a cluster
    IF (potential_cluster EQ 0) THEN BEGIN
        potential_cluster=1     ; now before a potential cluster
        EndOfNormalData=i
    ENDIF

  ENDELSE

ENDFOR

;*********************************
; Revert to unfiltered values
;*********************************
full_values=untouched_full_values

RETURN

END   
	
;************************************************************************
;                                 END
;************************************************************************
