;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;British Crown Copyright (c) 2012, the Met Office All rights reserved.
;
;Redistribution and use in source and binary forms, with or without 
;modification, are permitted provided that the following conditions 
;are met:
;
;Redistributions of source code must retain the above copyright notice, 
;this list of conditions and the following disclaimer. 
;
;Redistributions in binary form must reproduce the above copyright 
;notice, this list of conditions and the following disclaimer in the 
;documentation and/or other materials provided with the distribution. 
;
;Neither the name of the Met Office nor the names of its contributors 
;may be used to endorse or promote products derived from this software 
;without specific prior written permission. 
;
;THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
;"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
;LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
;A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
;HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
;SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
;LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
;DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
;THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
;(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
;OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

PRO OUTLIER_PLOT,values,mdi,fdi,times,qc_flags,flag_column,all_values,type,label,stationid, num_neighbours, sum_neighbours

;
; Subroutine written to detect outliers in the distribution of differences between a candidate station and a
; neighbour series. Has to do a bunch of subsetting and incrementing data availability counters before
; trying to ascertain where the potentially dubious data are and incrementing this counter. Will only apply to 
; differences which we expect to be normally distributed (so not the wind variables).
;
;
;*************************************************************
; NAME:
;     OUTLIER_PLOT
;
; PURPOSE:
;     Plot the flagged outliers with all the other neighbouring
;     station datasets below
;
; CALLING SEQUENCE:
;     OUTLIER_PLOT, values, mdi, fdi, times, qc_flags, flag_column, all_values, label,
;     stationid, num_neighbours
; 
; INPUTS:
;     values - array of the data values
;
;     mdi - missing data indicator for the data
;
;     fdi - flagged data indicator for the data
;
;     times - array containing the time data
;
;     qc_flags - array containing the QC flags
;
;     flag_column - the column of QC flags to read for this data
;
;     all_values - array containing all the data value from the
;                  neighbours
;
;     type - label for the plots (the test type)
;
;     label - label for the plots (the data type)
;
;     stationid - the station name for the plots
;
;     num_neighbours - the number of neighbours
;
;     sum_neighbours - the sum of valid neighbour values
;
; METHOD:
;     plots the values of the series in question, all the neighbouring
;     series' and then the result of the flagging or unflagging as appropriate.
;     
;
; OUTPUTS:
;
;     .ps  files in a subdirectory "images" of the current one.
;
; RESTRICTIONS:
;
;   
; HISTORY:
;     #1 - Created.  18 November 2010
;   #168 - Added type input for file names.  13 January 2011 RJHD
;   #191 - Now three levels for highlighted flags.  Also print number
;          of neighbours on the plots
;          8 March 2011 RJHD
;   #237 - SVN information added to header
;          16 November 2011 RJHD
;   #238 - Added fdi functionality
;          18 November 2011 RJHD
;   #249 - Plotting only run if data present
;          25 November 2011 RJHD
;
;************************************************************************
;                    SVN Info
;$Rev:: 303                                           $:  Revision of last commit
;$Author:: rdunn                                      $:  Author of last commit
;$Date:: 2012-06-14 14:02:44 +0100 (Thu, 14 Jun 2012) $:  Date of last commit
;************************************************************************
;                                 START
;************************************************************************
            
;**************************************
; Determine how many are flagged
;**************************************
flagged=WHERE(qc_flags(flag_column,*) NE 0,flgcnt)

set_plot,'PS'
tvlct,0,0,0,0                   ; black for kept datapoints
cd, "./", current=outdirplot
Figcounter=0

flag_counter=0

PRINT,'  RED - flags retained, insufficient neighbours'
PRINT,' BLUE - sufficient neighbours, but flags retained'
PRINT,'GREEN - flag unset - falls within range of neigbours'

WHILE flag_counter LT flgcnt DO BEGIN


    location=flagged(flag_counter)
    
    device,filename=outdirplot+'/images/'+stationid+'_'+type+'_'+label+'_'+string(FigCounter,FORMAT='(I2.2)')+'.eps'$
      ,/color,/encapsulate,/helvetica,/bold,/landscape,ysize=20,xsize=26
    !P.Font=0
    !P.Thick=4
    !P.Multi=[0,1,1]
    
    plotsym,3,1.2
    

    ;**************************************
    ; Select those which are not mdi
    ;**************************************
    IF location-24 LT 0 THEN minloc=0 ELSE minloc=location-24
    IF location+24 GT N_ELEMENTS(values)-1 THEN maxloc=N_ELEMENTS(values)-1 ELSE maxloc=location+24
    temp_values=values(minloc:maxloc)
    temp_times=times(minloc:maxloc)
    temp_neigh=sum_neighbours(minloc:maxloc)
    temp_neigh_values=all_values(0:num_neighbours-1,minloc:maxloc)

    useful_neigh_values=WHERE(((temp_neigh_values NE mdi) AND (temp_neigh_values NE fdi)), unvcnt)
    useful_temp_values=WHERE(((temp_values NE mdi) AND (temp_values NE fdi)),utvcnt)

    IF (unvcnt GE 1) AND (utvcnt GE 1) THEN BEGIN
       total_values=[temp_neigh_values(useful_neigh_values),temp_values(useful_temp_values)]

    
    ;**************************************
    ; Select those which are not mdi from 
    ; all values for the max and min
    ;**************************************

       MINY=MIN(total_values)
       IF label EQ 'SLP' THEN BEGIN
          MAXY=MAX(total_values)*1.01
          MINY=MINY*0.99
       ENDIF ELSE BEGIN
          MAXY=MAX(total_values)*1.1
          IF ABS(MINY) LE 1 THEN MINY=-1 ELSE IF MINY LT -1 THEN MINY=MINY*1.1 ELSE MINY=MINY*0.9
       ENDELSE
       
       dummy=LABEL_DATE(DATE_FORMAT=['%D/%N/%Y'])
       start_date=JULDAY(1, 1, 1973, 0)
       
       plot,times(minloc:maxloc)+start_date,values(minloc:maxloc),$
            min_value=-100,xstyle=1,ystyle=1,ytitle=label,xtitle="time",$
            title="Outlier Check - Diagnostic Timeseries",charsize=1,xtickformat='LABEL_DATE',$
            ythick=4,xthick=4,psym=2,color=0,yrange=[MINY,MAXY], POSITION=[0.10, 0.10, 0.90, 0.90],/nodata
    
    ;**************************************
    ; For all the neighbours, plot those which are not mdi
    ;**************************************
       FOR n=0,num_neighbours-1 DO BEGIN
          
          temp_neigh_values=all_values(n,minloc:maxloc)
          temp_neigh_times=times(minloc:maxloc)
          
          oplot,temp_neigh_times+start_date,temp_neigh_values,$
                min_value=-100,xstyle=1,ystyle=1,ytitle=label,xtitle="time",$
                title="Outlier Check - Diagnostic Timeseries",charsize=1,xtickformat='LABEL_DATE',$
                ythick=4,xthick=4,psym=2,color=200,yrange=[MINY,MAXY], POSITION=[0.10, 0.10, 0.90, 0.90]
                   
       ENDFOR
    
    ;**************************************
    ; Overplot the data of interest
    ;**************************************
       oplot,times(minloc:maxloc)+start_date,values(minloc:maxloc),$
             min_value=-100,xstyle=1,ystyle=1,ytitle=label,xtitle="time",$
             title="Outlier Check - Diagnostic Timeseries",charsize=1,xtickformat='LABEL_DATE',$
             ythick=4,xthick=4,psym=2,color=0,yrange=[MINY,MAXY], POSITION=[0.10, 0.10, 0.90, 0.90]
       
       oplot,times(minloc:maxloc)+start_date,values(minloc:maxloc),$
             min_value=-100,xstyle=1,ystyle=1,ytitle=label,xtitle="time",$
             title="Outlier Check - Diagnostic Timeseries",charsize=1,xtickformat='LABEL_DATE',$
             ythick=4,xthick=4,psym=0,color=0,yrange=[MINY,MAXY], POSITION=[0.10, 0.10, 0.90, 0.90]
    
    ;**************************************
    ; Highlight the point of interest
    ; and also any subsequent flags which fall
    ; in the range plotted
    ; Red points - flags retained
    ; Blue points - sufficient neighbours,but flag retained
    ; Green points - flags removed
    ;**************************************


       flagged_points=WHERE(qc_flags(flag_column,minloc:maxloc) EQ 1,flgpts) ;red
       
       IF flgpts GE 1 THEN BEGIN
          oplot,[temp_times(flagged_points)+start_date],[temp_values(flagged_points)],min_value=-100,$
                xstyle=1,ystyle=1,ytitle=label,xtitle="time",$
                title="Outlier Check- Diagnostic Timeseries",symsize=2,charsize=1,xtickformat='LABEL_DATE',$
                ythick=4,xthick=4,psym=4,color=239,yrange=[MINY,MAXY], POSITION=[0.10, 0.10, 0.90, 0.90]
          XYOUTS,[temp_times(flagged_points)+start_date], miny*1.01,[temp_neigh(flagged_points)], charsize=0.7, alignment=0.7
       ENDIF
       
       flagged_points=WHERE(qc_flags(flag_column,minloc:maxloc) EQ 2,flgpts) ;blue
       
       IF flgpts GE 1 THEN BEGIN
          oplot,[temp_times(flagged_points)+start_date],[temp_values(flagged_points)],min_value=-100,$
                xstyle=1,ystyle=1,ytitle=label,xtitle="time",$
                title="Outlier Check- Diagnostic Timeseries",symsize=2,charsize=1,xtickformat='LABEL_DATE',$
                ythick=4,xthick=4,psym=4,color=25,yrange=[MINY,MAXY], POSITION=[0.10, 0.10, 0.90, 0.90]
          XYOUTS,[temp_times(flagged_points)+start_date], miny*1.01,[temp_neigh(flagged_points)], charsize=0.7, alignment=0.7
       ENDIF
       
       flagged_points=WHERE(qc_flags(flag_column,minloc:maxloc) EQ 3,flgpts) ;green
       
       IF flgpts GE 1 THEN BEGIN
          oplot,[temp_times(flagged_points)+start_date],[temp_values(flagged_points)],min_value=-100,$
                xstyle=1,ystyle=1,ytitle=label,xtitle="time",$
                title="Outlier Check- Diagnostic Timeseries",symsize=2,charsize=1,xtickformat='LABEL_DATE',$
                ythick=4,xthick=4,psym=4,color=120,yrange=[MINY,MAXY], POSITION=[0.10, 0.10, 0.90, 0.90]
          XYOUTS,[temp_times(flagged_points)+start_date], miny*1.01,[temp_neigh(flagged_points)], charsize=0.7, alignment=0.7
       ENDIF
       
       
       device,/close
       FigCounter=FigCounter+1
       
    ENDIF
    flagged_points=WHERE(qc_flags(flag_column,minloc:maxloc) NE 0,flgpts)
    flag_counter=flag_counter+flgpts
ENDWHILE

            
RETURN

END
;************************************************************************
;                                 END
;************************************************************************
