;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;British Crown Copyright (c) 2012, the Met Office All rights reserved.
;
;Redistribution and use in source and binary forms, with or without 
;modification, are permitted provided that the following conditions 
;are met:
;
;Redistributions of source code must retain the above copyright notice, 
;this list of conditions and the following disclaimer. 
;
;Redistributions in binary form must reproduce the above copyright 
;notice, this list of conditions and the following disclaimer in the 
;documentation and/or other materials provided with the distribution. 
;
;Neither the name of the Met Office nor the names of its contributors 
;may be used to endorse or promote products derived from this software 
;without specific prior written permission. 
;
;THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
;"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
;LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
;A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
;HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
;SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
;LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
;DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
;THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
;(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
;OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

PRO qc_all_vars_external,last_year, processing_start, $
                         svn_version, restart_id=restart_id,datadir=datadir, $
                         outputdir=outputdir, diagnostics=diagnostics, mask=mask, test=test, clean=clean, $
                         plots=plots, unflag=unflag, upclean=upclean, suffix=suffix, pflags=pflags

; Program runs through a suite of external buddy checks on the data. Also sorts out some
; issues with cloud cover that weren't picked up by internal qc code rather than running
; that code again which is computationally expensive. Code undertakes pairwise comparisons 
; between neighbouring stations matched by altitude and lat / lon. We are looking for single 
; outlier values in the difference series for most variables. 
; RJHD notes - cloud routine mentioned above no longer in this version.
;
; NAME:
;     QC_ALL_VARS_EXTERNAL
;
; PURPOSE:
;     Using the nearest neighbour stations, the values are checked for
;     validity.  A number of other tests are undone if the neighbour
;     checks allow them to be retained.  Finally, months with few
;     remaining datapoints or excessive flagging are flagged in their entirety.
;
;
; CALLING SEQUENCE:
;     qc_all_vars_external,last_year,processing_start, 
;     svn_version,restart_id,datadir,outputdir,
;        diagnostics, mask, test, clean, plots, unflag, upclean, suffix
;
; INPUTS:
;     last_year - final year in the data sequence (to be passed to the
;                  check scripts)
;
;     processing_start - date at which the run of processing began
;
;     svn_version - version number of this QC code
;
;     restart_id - starting station number (allows continuation of
;                  terminated run)
;
;     input_dir - directory from which the input files are read
;
;     output_dir - directory into which the output files are written
;
;     diagnostics - should diagnostics be run (produces intermediate
;                   plots, and more verbose output)
;
;     mask - keyword to set whether bad data should be masked rather
;            than just flagged
;
;     test - keyword to run a single station as a test run
;
;     clean - keyword to used cleaned neighbours in the neighbour check
;
;     plots - keyword to run the summary plots at the end of the script
;
;     unflag - keyword to run the unflagging section
;
;     upclean - keyword to run the clean-up-months routine
;
;     suffix - label for the files to be read in and out of the script
;              depending on at which stage it is run in the QC proceedure
;
;     pflags - keyword for the plotting script - plot the flags on the
;              timeseries outputs or not.
;
; OUTPUTS:
;
;     None - files written to specified directory
;
; RESTRICTIONS:
;
;     Assumes netcdf file inputs.  Assumes variable names and formats
;     are as in the code
;   
; HISTORY:
;     NB - not all revisions are noted here.
;     # 55 - contains the form of the program as obtained
;            November 2010 RJHD 
;     # 57 - moved neighbour assignment into separate programme.  Also
;            put the outlier detection into a case structure, and put
;            the plotting into a separate programme
;     # 81 - added a minimum neighbour number before flagging
;            18 November 2010 RJHD 
;     # 112 - added an "unflagging" section where flags from the
;             odd_cluster test are reversed if there are sufficient
;             neighbour values.  No other flags are unchecked, so if
;             these observations are bad for other reasons they will
;             still be removed.
;             30 November 2010 RJHD 
;     # 114 - Flags required in neighbour check - so that cleaned
;             neighbours could be used. Keyword "clean" added to
;             control this.
;             Also added the reporting accuracy information -
;             transferred from internal to external file.
;             30 November 2010 RJHD
;     # 135 - Updated program to run in a wrapper
;             (hadisd_qc_control.pro).  Keywords added (plot & suffix) which
;             allow the selection of the correct filenames for the
;             input and output files.  Also updated flag selection for
;             detect_outliers.pro.  Added in sigwx1 and precip1 to
;             the output .nc file
;             7 December 2010 RJHD
;     # 162 - Sum across all flags when passing to the detect outliers
;             added else only using the frequent value flag.
;             4 January 2011 RJHD
;     # 165 - Unflagging now done in separate routine
;             7 January 2011 RJHD
;     # 168 - Diagnostic plotting now added to unflagging routine
;             13 January 2011 RJHD
;     # 172 - Diagnostic plotting now added to unflagging routine
;             14 January 2011 RJHD
;     # 191 - Unflagging now done on negative SLP gaps - to counter
;             the lack of windspeed information.  Slight adjustments
;             to outlier_plot call.
;             8 March 2011 RJHD
;     # 193 - Adjusted the columns which appear in Clean_up_months call
;             8 March 2011 RJHD
;     # 200 - changed last_year from optional keyword to required input
;             to remove the hardwiring of the end date
;             11 March 2011 RJHD
;     #206/7- BUGFIX - added subarray limit to sum_neighbours in the
;             outlier determination part.  Want the number of
;             neighbours with valid data (ie sufficient over the
;             course of the series and this number can be amended by
;             the SLP specific part of detect_outliers for storms.
;             16 March 2011 RJHD
;     # 233 - qc_summary file output corrected. Files updated using
;             qc_summary_create code.
;             03 October 2011 RJHD
;     # 237 - SVN information added to header
;             16 November 2011 RJHD
;     # 238 - Added fdi functionality.
;             18 November 2011 RJHD
;      #243 - Changed numerical order of FIRM and TENTATIVE FLAGS
;             21 November 2011 RJHD
;     # 245 - Changed reporting accuracy format in netcdf files
;             23 November 2011 RJHD
;     # 249 - Added reporting accuracy information to unflagging section
;             25 November 2011 RJHD
;     # 250 - Adjusted type of MetaData and the way it is processed
;             29 November 2011 RJHD
;     # 252 - Added finish statement to log file for completion
;             monitoring.  Adjusted netcdf variable names after further
;             metadata additions
;             9 December 2011 RJHD
;     # 256 - Moved the writing of the summary file after the creation
;             of the netcdf file section to prevent duplications on crashes
;             12 December 2011 RJHD
;
;
;************************************************************************
;                    SVN Info
;$Rev:: 318                                           $:  Revision of last commit
;$Author:: rdunn                                      $:  Author of last commit
;$Date:: 2012-09-03 08:30:06 +0100 (Mon, 03 Sep 2012) $:  Date of last commit
;************************************************************************
;                                 START
;************************************************************************

IF KEYWORD_SET(unflag) AND KEYWORD_SET(mask) THEN $
   STOP, 'UNFLAG and MASK both set.  If masking has been done on processing no unflagging can be done.'

total_starttime=systime(1)
print,systime()
;*******************
; Check input and output directories have been set, if not, use
; default setting
;*******************
cd, "./", current=pwd
IF NOT KEYWORD_SET(datadir) THEN fulldatadir=pwd+'/' ELSE fulldatadir=pwd+'/'+datadir+'/'
IF NOT KEYWORD_SET(outputdir) THEN outputdir=fulldatadir
IF NOT KEYWORD_SET(suffix) THEN suffix='external'


;*******************
; Check the diagnostics keyword
;*******************
IF NOT KEYWORD_SET(diagnostics) THEN diagnostics=0
IF KEYWORD_SET(diagnostics) THEN BEGIN
    RESTORE_COLOURS,'shortbow'
ENDIF

locfile=pwd+'/station_locations_and_distances.nc'

fred=NCDF_OPEN(locfile,/nowrite)

NCDF_CONTROL,0,/verbose
betty=NCDF_INQUIRE(fred)
stationdimid=NCDF_DIMID(fred,'station')
NCDF_DIMINQ,fred,stationdimid,stationdim,nstation
stationinfo=NCDF_VARINQ(fred,'station')
latinfo=NCDF_VARINQ(fred,'latitude')
loninfo=NCDF_VARINQ(fred,'longitude')
elevinfo=NCDF_VARINQ(fred,'elevation')
NCDF_VARGET,fred,'station',stations
NCDF_VARGET,fred,'longitude',longitudes
NCDF_VARGET,fred,'latitude',latitudes
NCDF_VARGET,fred,'elevation',elevations
stations=STRING(stations)
stations=REFORM(stations,nstation)
NCDF_CLOSE,fred

  
IF KEYWORD_SET(restart_id) THEN BEGIN
  match=WHERE(stations EQ restart_id, c)
  IF c NE 1 THEN STOP, 'restart_id passed on call does not match valid id'
  start_loc=match(0)
ENDIF ELSE BEGIN
  ; use the kept_station_info.txt file to find the start and end points
  Station_info_file=pwd+'/kept_station_info.txt'
  nstation=nlines(Station_info_file)
  station_ids=STRARR(nstation)
  id=''
  st=0
  OPENR,1,Station_info_file
  WHILE NOT EOF(1) DO BEGIN
     READF,1,id,FORMAT='(A)'
     station_ids(st)=STRMID(id,0,12)
     st=st+1
  ENDWHILE
  CLOSE,1

  restart_id=station_ids[0]

  match=WHERE(stations EQ restart_id, c)
  IF c NE 1 THEN STOP, 'restart_id from kept_station_info.txt does not match valid id'
  start_loc=match(0)
ENDELSE 

IF KEYWORD_SET(end_id) THEN BEGIN
  match=WHERE(stations EQ end_id, c)
  IF c NE 1 THEN STOP, 'end_id passed on call does not match valid id'
  end_loc=match(0)
ENDIF ELSE BEGIN
  ; use the kept_station_info.txt file to find the start and end points
  Station_info_file=pwd+'/kept_station_info.txt'
  nstation=nlines(Station_info_file)
  station_ids=STRARR(nstation)
  id=''
  st=0
  OPENR,1,Station_info_file
  WHILE NOT EOF(1) DO BEGIN
     READF,1,id,FORMAT='(A)'
     station_ids(st)=STRMID(id,0,12)
     st=st+1
  ENDWHILE
  CLOSE,1

  end_id=station_ids[N_ELEMENTS(station_ids)-1]

  match=WHERE(stations EQ end_id, c)
  IF c NE 1 THEN STOP, 'end_id from kept_station_info.txt does not match valid id'
  end_loc=match(0)
ENDELSE



IF KEYWORD_SET(test) THEN BEGIN
  ; so that only a single station is run and all subroutines are run too
  nstation=start_loc+1
ENDIF

IF NOT KEYWORD_SET(test) THEN BEGIN
    ; echo the current station being processed
    OPENW,1,pwd+'/qc_summary_'+suffix+'.txt',/append 
ENDIF ELSE BEGIN                ; test is set
    FILE_DELETE,pwd+'/qc_summary_'+suffix+'_only_'+stations(0)+'.txt', /ALLOW_NONEXISTENT
    ; delete the individual station qc storage file
    OPENW,1,pwd+'/qc_summary_'+suffix+'_only_'+stations(0)+'.txt',/append 
ENDELSE


FOR st=start_loc,end_loc DO BEGIN
 
  ;***************************************************************************
  ;    START THE DATA READING
  ;***************************************************************************


    IF NOT KEYWORD_SET(test) THEN BEGIN
        PRINT,'Station Number :',st+1,"/",nstation, FORMAT='(A-40,I4,A1,I4)'
        printstring='Station Identifier :'
        PRINT,printstring,stations(st), FORMAT='(A-40,A15)'
    ENDIF

    ;************************************************************************
    ; Read in the station file - fred again.
    ;************************************************************************
    IF suffix EQ 'external' THEN file=fulldatadir+stations(st)+'_internal.nc' $
      ELSE IF suffix EQ 'external2' THEN  file=fulldatadir+stations(st)+'_internal2.nc' 

    fred=NCDF_OPEN(file,/nowrite)
    NCDF_CONTROL,0,/verbose
    betty=NCDF_INQUIRE(fred)
    tid=NCDF_DIMID(fred,'time')
    NCDF_DIMINQ,fred,tid,timedim,nt
    chptid=NCDF_DIMID(fred,'changepoints')
    NCDF_DIMINQ,fred,chptid,chptdim,nchpts
    chgsid=NCDF_DIMID(fred,'changes')
    NCDF_DIMINQ,fred,chgsid,chgsdim,nchgs

    timesinfo=NCDF_VARINQ(fred,'time')
    tempsinfo=NCDF_VARINQ(fred,'temperatures')
    idsinfo=NCDF_VARINQ(fred, 'input_station_id')
    dewsinfo=NCDF_VARINQ(fred, 'dewpoints')
    totclinfo=NCDF_VARINQ(fred, 'total_cloud_cover')
    lowclinfo=NCDF_VARINQ(fred, 'low_cloud_cover')
    midclinfo=NCDF_VARINQ(fred, 'mid_cloud_cover')
    hiclinfo=NCDF_VARINQ(fred, 'high_cloud_cover')
    clbaseinfo=NCDF_VARINQ(fred, 'cloud_base')
    windspinfo=NCDF_VARINQ(fred, 'windspeeds')
    winddirinfo=NCDF_VARINQ(fred, 'winddirs')
    slpinfo=NCDF_VARINQ(fred, 'slp')
    pastsigwx1info=NCDF_VARINQ(fred,'past_sigwx1') 
    precipvar=NCDF_VARINQ(fred,'precip1_depth') 
    flaginfo=NCDF_VARINQ(fred, 'qc_flags')
    fvinfo=NCDF_VARINQ(fred, 'flagged_values')  
    chngpntinfo=NCDF_VARINQ(fred, 'change_points')  
    chngntinfo=NCDF_VARINQ(fred, 'change_notes')  

    NCDF_VARGET,fred,'time',times
    NCDF_VARGET,fred,'temperatures',temperatures
    NCDF_VARGET,fred,'input_station_id',input_station_id
    NCDF_VARGET,fred,'dewpoints',dewpoints
    NCDF_VARGET,fred,'total_cloud_cover',total_cloud_cover
    NCDF_VARGET,fred,'low_cloud_cover',low_cloud_cover
    NCDF_VARGET,fred,'mid_cloud_cover',mid_cloud_cover
    NCDF_VARGET,fred,'high_cloud_cover',high_cloud_cover
    NCDF_VARGET,fred,'cloud_base',cloud_base
    NCDF_VARGET,fred,'windspeeds',windspeeds
    NCDF_VARGET,fred,'winddirs',winddirs
    NCDF_VARGET,fred,'slp',slp
    NCDF_VARGET,fred,'past_sigwx1',past_sigwx1
    NCDF_VARGET,fred,'precip1_depth',precip1
    NCDF_VARGET,fred,'qc_flags',qc_flags
    NCDF_VARGET,fred,'flagged_values',flagged_values
    NCDF_VARGET,fred,'change_points',changepoints
    NCDF_VARGET,fred,'change_notes',changenotes

    ; get the mdi values
    NCDF_ATTGET,fred,'temperatures','missing_value',temperatures_mdi
    NCDF_ATTGET,fred,'dewpoints','missing_value',dewpoints_mdi
    NCDF_ATTGET,fred,'total_cloud_cover','missing_value',total_cloud_cover_mdi
    NCDF_ATTGET,fred,'low_cloud_cover','missing_value',low_cloud_cover_mdi
    NCDF_ATTGET,fred,'mid_cloud_cover','missing_value',mid_cloud_cover_mdi
    NCDF_ATTGET,fred,'high_cloud_cover','missing_value',high_cloud_cover_mdi
    NCDF_ATTGET,fred,'cloud_base','missing_value',cloud_base_mdi
    NCDF_ATTGET,fred,'windspeeds','missing_value',windspeeds_mdi
    NCDF_ATTGET,fred,'winddirs','missing_value',winddirs_mdi
    NCDF_ATTGET,fred,'slp','missing_value',slp_mdi
    ; get the fdi values
    NCDF_ATTGET,fred, 'temperatures','flagged_value',temperatures_fdi
    NCDF_ATTGET,fred, 'dewpoints','flagged_value',dewpoints_fdi
    NCDF_ATTGET,fred, 'slp','flagged_value',slp_fdi
    NCDF_ATTGET,fred, 'total_cloud_cover','flagged_value',total_cloud_cover_fdi
    NCDF_ATTGET,fred, 'low_cloud_cover','flagged_value',low_cloud_cover_fdi
    NCDF_ATTGET,fred, 'mid_cloud_cover','flagged_value',mid_cloud_cover_fdi
    NCDF_ATTGET,fred, 'high_cloud_cover','flagged_value',high_cloud_cover_fdi
    NCDF_ATTGET,fred, 'windspeeds','flagged_value',windspeeds_fdi
    NCDF_ATTGET,fred, 'winddirs','flagged_value',winddirs_fdi
    ; get metadata


    NCDF_ATTGET,fred,'title',title,/GLOBAL
    NCDF_ATTGET,fred,'institution',institution,/GLOBAL
    NCDF_ATTGET,fred,'source',source,/GLOBAL
    NCDF_ATTGET,fred,'references',references,/GLOBAL
    NCDF_ATTGET,fred,'creator_name',creator_name,/GLOBAL
    NCDF_ATTGET,fred,'creator_url',creator_url,/GLOBAL
    NCDF_ATTGET,fred,'creator_email',creator_email,/GLOBAL
    NCDF_ATTGET,fred,'time_coverage_start',time_coverage_start,/GLOBAL
    NCDF_ATTGET,fred,'time_coverage_end',time_coverage_end,/GLOBAL
    NCDF_ATTGET,fred,'cdm_data_type',cdm_data_type,/GLOBAL
    NCDF_ATTGET,fred,'keywords',keywords,/GLOBAL
    NCDF_ATTGET,fred,'summary',summary,/GLOBAL
    NCDF_ATTGET,fred,'hadisd_version',hadisd_version,/GLOBAL
    NCDF_ATTGET,fred,'isd_version',isd_version,/GLOBAL
    NCDF_ATTGET,fred,'isd_date',isd_date,/GLOBAL
    NCDF_ATTGET,fred,'composite_notes',composite_notes,/GLOBAL
    NCDF_ATTGET,fred,'flag_notes',flag_notes,/GLOBAL
    NCDF_ATTGET,fred,'history',bhistory,/GLOBAL ; byte array - convert to string
    oldhistory=STRING(bhistory)
    history=STRING(SYSTIME())+'*** EXTERNAL TESTS ***\n'
    ;
    NCDF_CLOSE,fred
    
    ;*******************
    ; Find the neighbours.
    ;*******************
    neighbours=0
    neighdists=0
    neighelevs=0
    neighquadrants=0
    num_neighbours=0

    NEIGHBOUR_ASSIGN,station_id=stations(st),/diagnostics,Nbours=neighbours,Ndists=neighdists,$
                     Nelevs=neighelevs,Nquadrants=neighquadrants,num_Nbours=num_neighbours

    
    history=history+STRING(SYSTIME())+' Number of Neighbours is '+STRCOMPRESS(num_neighbours,/remove_all)+'\n '      
    
    print,''
  ;************************************************************************
  ; If there are sufficient neighbours then continue (does not check
  ; that all have data at all the times)
  ;************************************************************************
  IF num_neighbours GE 3 THEN BEGIN

    ;************************************************************************
    ; Perform neighbour based checks iterating through the candidate neighbour series one by one.
    ; Firstly requires setting up arrays for the analysis step. Not considering cloud as not normally distributed.
    ; May need to consider those variables without a large enough spatial scale. 
    ;************************************************************************
      temperature_dubious_count=INTARR(nt)
      temperature_present_count=INTARR(nt)
      dewpoint_dubious_count=INTARR(nt)
      dewpoint_present_count=INTARR(nt)
      slp_dubious_count=INTARR(nt)
      slp_present_count=INTARR(nt)
      
      all_temperatures=FLTARR(10,nt)
      all_slps=FLTARR(10,nt)
      all_dewpoints=FLTARR(10,nt)
      all_dewpoint_flags=FLTARR(nt)
      dpd_flags=FLTARR(nt)
      
      all_temperatures(*)=temperatures_mdi
      all_slps(*)=slp_mdi
      all_dewpoints(*)=dewpoints_mdi
      all_dewpoint_flags(*)=0

      
      FOR n=0,num_neighbours-1 DO BEGIN
      ;************************************************************************
      ; Read in values commands. Only necessary to read in the variables that will be actively considered in the
      ; neighbour cross-check.  barney is the neighbour file
      ;************************************************************************
          IF suffix EQ 'external' THEN BEGIN
              ;************************************************************************
              ; allow the option of comparing to a cleaned neighbour.
              ;************************************************************************
              IF KEYWORD_SET(clean) THEN file=fulldatadir+string(neighbours(n))+'_internal.nc' $
              ELSE file=fulldatadir+string(neighbours(n))+'.nc'
          ENDIF ELSE IF suffix EQ 'external2' THEN BEGIN
              IF KEYWORD_SET(clean) THEN file=fulldatadir+string(neighbours(n))+'_internal2.nc' $
              ELSE file=fulldatadir+string(neighbours(n))+'.nc'
          ENDIF       
          print,string(neighbours(n))

          barney=NCDF_OPEN(file,/nowrite)
          NCDF_CONTROL,0,/verbose
          wilma=NCDF_INQUIRE(barney)
          
          tid=NCDF_DIMID(barney,'time')
          NCDF_DIMINQ,barney,tid,timedim,nt2
          timesinfo=NCDF_VARINQ(barney,'time')
          tempsinfo=NCDF_VARINQ(barney,'temperatures')
          dewsinfo=NCDF_VARINQ(barney, 'dewpoints')
          slpinfo=NCDF_VARINQ(barney, 'slp')
          IF KEYWORD_SET(clean) THEN qcinfo=NCDF_VARINQ(barney, 'qc_flags')
          
      ;************************************************************************
      ; get latitude and longitude info
      ;************************************************************************
  
          NCDF_VARGET,barney,'time',times2
          NCDF_VARGET,barney,'temperatures',temperatures2
          NCDF_VARGET,barney,'dewpoints',dewpoints2
          NCDF_VARGET,barney,'slp',slp2
          IF KEYWORD_SET(clean) THEN NCDF_VARGET,barney,'qc_flags',qc_flags2
          
          NCDF_CLOSE,barney


          IF KEYWORD_SET(clean) THEN BEGIN

          ;************************************************************************
          ; Combine all the flags relevant to each of the data types.
          ; Climatological flags (24-26) and Odd cluster flags (54, 55 & 57)
          ;   are not excluded in the clean neighbour test as these can
          ;   be used to mutually unset each other if the values are passed
          ;   by the neighbour check                    10 DEC 2010 RJHD
          ;************************************************************************
          ; TOTAL sum across rows added else only using the
          ;    frequent values check.  4 JAN 2011 RJHD
          ; Outlier check has -1 if no neighbours.  However, if 
          ;    there are no neighbours, can't detect if there is an
          ;    outlier, hence can stick to using TOTAL (29-AUG-12 RHD)
          ; Include temperature record exceedence in dewpoint flag
          ;    19 January 2011 RJHD
          ; Do not include the T and DP cross check flags in the temperature
          ;    Cols 30,31,32 - only the dewpoint is flagged by these tests.
          ;    27 January 2011 RJHD & KW
          ;************************************************************************
              temperature_flags=TOTAL(qc_flags2([0,1,4,5,8,12,16,20,27,41,44,58],*),1)
              
              dewpoint_flags=TOTAL(qc_flags2([0,2,4,6,8,9,13,17,21,28,30,31,32,42,45,59],*),1)
              
              slp_flags=TOTAL(qc_flags2([0,3,4,7,11,15,19,23,29,43,46,60],*),1)
              
          ;******************************
          ; To undo DPD flags later
          ;******************************
              MATCH,times, times2, timelocs1,timelocs2,count=ntimematches
              dpd_flags(*)=0
              dpd_flags(timelocs1)=qc_flags2(31,timelocs2)
              all_dewpoint_flags(timelocs1)=all_dewpoint_flags(timelocs1)+dpd_flags

          ENDIF ELSE BEGIN
              length=N_ELEMENTS(times2)
              temperature_flags=INTARR(length)
              dewpoint_flags=INTARR(length)
              slp_flags=INTARR(length)
              temperature_flags(*)=0
              dewpoint_flags(*)=0
              slp_flags(*)=0
              
          ENDELSE

          
          ;************************************************************************
          ; Detect outliers in those variables we expect to be continuosly sampled and to have large enough 
          ; correlation decay distances. Do this by calling a routine that performs exactly the same operation
          ; for each variable that we expect to be normally distributed in its difference series in turn otherwise 
          ; this program becomes the monster that ate your homework.
          ;************************************************************************

          DETECT_OUTLIERS, times, times2, temperatures, temperatures2, temperature_dubious_count,$
            temperature_present_count, temperatures_mdi,temperatures_fdi,diagnostics,stations(st),$
                           string(neighbours(n)),'Temperature',n,all_temperatures,temperature_flags
      
          DETECT_OUTLIERS, times, times2, dewpoints, dewpoints2, dewpoint_dubious_count,$
            dewpoint_present_count, dewpoints_mdi,dewpoints_fdi,diagnostics,stations(st),$
                           string(neighbours(n)),'Dewpoint',n,all_dewpoints,dewpoint_flags
      
          DETECT_OUTLIERS, times, times2, slp, slp2, slp_dubious_count,$
            slp_present_count, slp_mdi,slp_fdi,diagnostics,stations(st),$
                           string(neighbours(n)), 'SLP',n,all_slps,slp_flags,neighbour_dist=neighdists(n) 
          ; added in neighbour distance to check for hurricanes 
      
          ;************************************************************************
          ;  all_values have the flags applied - and those values flagged
          ; have been removed.
          ;************************************************************************
      ENDFOR
    
    ;************************************************************************
    ; Start to determine the outliers from the values of
    ; the neighbouring stations.
    ;************************************************************************
    
      FOR type=0,2 DO BEGIN
          
          CASE type OF
              0: BEGIN
                  value_dubious_count=temperature_dubious_count
                  value_present_count=temperature_present_count
                  flag_column=41
                  values=temperatures
                  mdi=temperatures_mdi
                  fdi=temperatures_fdi
                  all_values=all_temperatures
                  label='Temperature'
                  odd_column=54
                  spike_column=27
                  climate_column=24
                  flagged_column=0
              END
              1: BEGIN
                  value_dubious_count=dewpoint_dubious_count
                  value_present_count=dewpoint_present_count
                  flag_column=42
                  values=dewpoints
                  mdi=dewpoints_mdi
                  fdi=dewpoints_fdi
                  all_values=all_dewpoints
                  label='Dewpoints'
                  odd_column=55
                  spike_column=28
                  climate_column=25
                  dpd_column=31                
                  flagged_column=1
              END
              2: BEGIN
                  value_dubious_count=slp_dubious_count
                  value_present_count=slp_present_count
                  flag_column=43
                  values=slp
                  mdi=slp_mdi
                  fdi=slp_fdi
                  all_values=all_slps
                  label='SLP'
                  odd_column=57
                  spike_column=29
                  climate_column=26
                  gap_column=7
                  flagged_column=2
              END
          ENDCASE
          PRINT,label
            ;************************************************************************
            ; Decide where the outliers in the station rather than the neighbours are and append these
            ; with a flag. Decision is more than 2/3 of neighbours differ by more than 5 inter-q ranges
            ; of their respective differences 
            ;************************************************************************
          potentially_dubious=WHERE(value_dubious_count GE 1, tcnt)
          
          ;************************************************************************
          ; Collapse down all_values to get a count of how
          ; many neighbours had valid values at the times
          ;************************************************************************
          
          sum_neighbours=INTARR(nt)
          sum_neighbours(*)=0

          neighbour_exist =WHERE((value_present_count NE mdi) AND (value_present_count NE fdi))

          sum_neighbours(neighbour_exist)=value_present_count(neighbour_exist)
 
          
          IF tcnt GE 1 THEN BEGIN
              ratios=float(value_dubious_count(potentially_dubious))/float(value_present_count(potentially_dubious))
              badpnts=WHERE((ratios GT 2./3.) AND (sum_neighbours(potentially_dubious) GE 3), bcnt) 
              ;more 3 neighbours have to have data (and clean data at that if /clean has been set)
              IF bcnt GE 1 THEN BEGIN

                  IF KEYWORD_SET(mask) THEN BEGIN
                     ; copy across the values into the flagged array
                      flagged_values[flagged_column, potentially_dubious(badpnts)]=values(potentially_dubious(badpnts))
                      values(potentially_dubious(badpnts))=fdi
                      IF type EQ 0 THEN BEGIN
                         temperatures=values
                      ENDIF ELSE IF type EQ 1 THEN BEGIN
                         dewpoints=values
                      ENDIF ELSE IF type EQ 2 THEN BEGIN
                         slp=values
                      ENDIF
                  ENDIF

                  qc_flags(flag_column,potentially_dubious(badpnts))=1                  
                  
                  IF KEYWORD_SET(diagnostics) THEN BEGIN
                      
                      OUTLIER_PLOT,values, mdi,fdi,times,$
                                   qc_flags,flag_column,all_values,label,$
                                   'Outlier',stations(st),num_neighbours, sum_neighbours
                  ENDIF
                  printstring=label+" Outlier Check Flags"
                  PRINT,printstring, bcnt, FORMAT='(A-50,I8)'
                  history=history+STRING(SYSTIME())+' Neighbour Check - '+label+' \n '      
              ENDIF


              ; for the locations where there are insufficient neighbours
              ; with valid data at that time stamp to determine
              ; if an observation is dubious or not.
              untestable=WHERE(sum_neighbours(potentially_dubious) LT 3, bcnt) 

              IF bcnt GE 1 THEN BEGIN
                 qc_flags(flag_column,untestable)=-1
              ENDIF
    
          ENDIF

          ; Trying to ensure that poor reporting accuracy doesn't
          ; create artificially small distribution.
          ; find the worst reporting accuracy of all the neighbours.
          NeighRes=FLTARR(num_neighbours)
          FOR neigh=0,num_neighbours-1 DO BEGIN

             REPORTING_ACCURACY, last_year, all_values[neigh,*], mdi, fdi, vres, remhist, times, vfreq
             NeighRes[neigh]=vres
          ENDFOR

          max_vres=MAX(NeighRes)
          
          ;************************************************************************
          ; Create arrays of median and standard deviation for neighbours
          ;   at each hour
          ;************************************************************************


          mean_values=FLTARR(nt)
          std_values=FLTARR(nt)

          mean_values(*)=mdi
          std_values(*)=mdi

          FOR h=0L,nt-1 DO BEGIN

              hourly_values=all_values(*,h)

              good_values=WHERE(((hourly_values NE mdi) AND (hourly_values NE fdi)), gvc)

              IF gvc GE 1 THEN BEGIN
                  
                  mean_values(h)=MEDIAN(hourly_values(good_values))
                  std_values(h)=MEANABSDEV(hourly_values(good_values),/median)

              ENDIF ELSE BEGIN

                  mean_values(h)=mdi
                  std_values(h)=mdi

              ENDELSE

           ENDFOR

          ;*****************************************************************
          ; Adjust the MAD (std_values) to be at least the lowest resolution
          ;   of the stations involved in the test (neighbours + current)
          ;   Using 0.5*vres - so have 2.25C spread for 1C resolution
          ;   Using 1*vres would give 4.5C spread for 1C resolution
          ;     which seemed a little broad, even if the distribution
          ;     from which this is all calculated was narrow because
          ;     all the points happened to be the same value.
          ;          25 November 2011 RJHD
          good_values=WHERE(((std_values NE mdi) AND (std_values NE fdi)), gvc)

          IF gvc GE 1 THEN BEGIN
             narrow_distribution=WHERE(std_values[good_values] LT 0.5*max_vres, ndcnt)
             IF ndcnt GE 1 THEN BEGIN
                std_values[good_values[narrow_distribution]]=0.5*max_vres
             ENDIF
          ENDIF
          ;*****************************************************************

          
          difference_values=FLTARR(nt)
          difference_values(*)=mdi

          value_exist=WHERE(((values NE mdi) AND (values NE fdi)), vec)
          IF vec GE 1 THEN BEGIN

              mean_exist=WHERE(((mean_values(value_exist) NE mdi) AND $
                                (mean_values(value_exist) NE fdi)), mec)

              IF mec GE 1 THEN BEGIN
                  
                  difference_values(value_exist(mean_exist))=ABS(values(value_exist(mean_exist))- $
                             mean_values(value_exist(mean_exist)))/std_values(value_exist(mean_exist))
              ENDIF

           ENDIF


          IF KEYWORD_SET(unflag) THEN BEGIN

              ; 2 level flagging system: 1 - FIRM, 2 - TENTATIVE

              ;************************************************************************
              ; Undo Odd Cluster check flags if no other flags have been set
              ;    and if there are sufficient (>3) the neighbours
              ;  If the neighbour check has set a flag, then although the odd
              ;  cluster flag is removed, the neighbour flag remains, and the
              ;  observation is still masked.
              ;************************************************************************
          
              plotting_flags=qc_flags(0,*)
              plotting_flags(*)=0

              UNFLAGGING, qc_flags,plotting_flags, difference_values, odd_column, sum_neighbours, label, "Odd Cluster"
              IF KEYWORD_SET(diagnostics) THEN BEGIN
                  OUTLIER_PLOT,values,mdi,fdi,times,plotting_flags,0,all_values,label,'Unflag_Odd',stations(st),num_neighbours, sum_neighbours
              ENDIF


              ;************************************************************************
              ; Undo Climatological check flags if no other flags have been set
              ;************************************************************************
          
              plotting_flags(*)=0
              UNFLAGGING, qc_flags,plotting_flags, difference_values, climate_column, sum_neighbours, label, "Climatological", /no_neighbours
              IF KEYWORD_SET(diagnostics) THEN BEGIN
                  OUTLIER_PLOT,values,mdi,fdi,times,plotting_flags,0,all_values,label,'Unflag_Climate', stations(st),num_neighbours, sum_neighbours
              ENDIF

              IF (label EQ "Temperature") AND (KEYWORD_SET(diagnostics)) THEN BEGIN
                 dummy=WHERE(qc_flags(climate_column,*) GT 0, cccnt)
                 IF cccnt GE 1 THEN PRINT,times(dummy)
              ENDIF

              ;************************************************************************
              ; Undo Spike check flags if no other flags have been set
              ;************************************************************************
              plotting_flags(*)=0
              UNFLAGGING, qc_flags,plotting_flags, difference_values, spike_column, sum_neighbours, label, "Spike"
              IF KEYWORD_SET(diagnostics) THEN BEGIN
                  OUTLIER_PLOT,values,mdi,fdi,times,plotting_flags,0,all_values,label,'Unflag_Spike',stations(st),num_neighbours, sum_neighbours
              ENDIF



              ;************************************************************************
              ; Undo Dewpoint Depression flag if neighbours also 
              ;    have this flag set.
              ;************************************************************************
              IF type EQ 1 THEN BEGIN
                  plotting_flags(*)=0
                  UNFLAGGING, qc_flags,plotting_flags, difference_values, dpd_column, sum_neighbours, label, "DPD", /dpd, all_dewpoint_flags=all_dewpoint_flags
                  IF KEYWORD_SET(diagnostics) THEN BEGIN
                      OUTLIER_PLOT,values,mdi,fdi,times,plotting_flags,0,all_values,label,'Unflag_DPD',stations(st),num_neighbours, sum_neighbours
                  ENDIF
                  
              ENDIF

              ;************************************************************************
              ; Undo GAP check flags if no other flags have been set
              ; ONLY for SLP, ONLY if neighbour flag is not set.
              ; ADDITION 7 MARCH 2011, RJHD
              ;************************************************************************
              IF type EQ 2 THEN BEGIN
                  plotting_flags(*)=0
                  UNFLAGGING, qc_flags,plotting_flags, difference_values, gap_column, sum_neighbours, label, "Gap"

                  IF KEYWORD_SET(diagnostics) THEN BEGIN
                      OUTLIER_PLOT,values,mdi,fdi,times,plotting_flags,0,all_values,label,'Unflag_Gap_SLP',stations(st),num_neighbours, sum_neighbours
                  ENDIF

              ENDIF
              ;************************************************************************
              ; #END ADDITION 7 MARCH 2011, RJHD
              ;************************************************************************

              
           ENDIF
          history=history+STRING(SYSTIME())+' Unflagging - '+label+'\n '      

      ENDFOR
      
      
  ENDIF ELSE BEGIN

    ;************************************************************************
    ; Write out location info to a file to know that insufficient neighbour buddy checks were possible
    ;************************************************************************
      OPENW,99,'no_neighbour_checks.txt',/append
      PRINTF,99,stations(st),FORMAT='(A)'
      CLOSE,99 


      
  ENDELSE


  IF KEYWORD_SET(upclean) THEN BEGIN
      ;***********************************************************************************************
      ; Add an extra set of QC that removes months with either a high proportion of flagged values or
      ; too low a total data count to be much use to anyone.
      ;  The last subset_cols sent has to be the column to be filled by
      ;  this test
      ;***********************************************************************************************
      ;
      ; Temperature related elements
      subset_cols=[0,1,4,5,8,12,16,20,24,27,41,54,58,44]             
      CLEAN_UP_MONTHS,last_year,times,temperatures,temperatures_mdi,temperatures_fdi,subset_cols,qc_flags, flagged_values,0,mask=mask, stations(st) ,'Temperature'
      ;
      ; Dewpoint related elements
      subset_cols=[0,2,4,6,8,9,13,17,21,25,28,30,31,32,42,55,59,45]
      CLEAN_UP_MONTHS,last_year,times,dewpoints,dewpoints_mdi,dewpoints_fdi,subset_cols,qc_flags, flagged_values,1,mask=mask, stations(st) ,'Dewpoint'
      ;
      ; SLP related elements
      subset_cols=[0,3,4,7,11,15,19,23,26,29,43,57,60,46]
      CLEAN_UP_MONTHS,last_year,times,slp,slp_mdi,slp_fdi,subset_cols,qc_flags, flagged_values,2,mask=mask, stations(st) ,'SLP'
      ;
      ; Windspeeds related elements
      subset_cols=[0,4,10,14,18,22,56,47]
      CLEAN_UP_MONTHS,last_year,times,windspeeds,windspeeds_mdi,windspeeds_fdi,subset_cols,qc_flags, flagged_values,7,mask=mask,stations(st) ,'Windspeed'
      ;
      ; Winddirs related elements
      subset_cols=[0,4,10,14,18,22,56,48]
      CLEAN_UP_MONTHS,last_year,times,winddirs,winddirs_mdi,winddirs_fdi,subset_cols,qc_flags, flagged_values,8,mask=mask, stations(st) ,'Wind Dir.'
      ;
      ;subset_cols=[1,2,3,4,5,6,7,33,37,40,49]
      ;CLEAN_UP_MONTHS,times,total_cloud_cover,total_cloud_cover_mdi,subset_cols,qc_flags, last_year=last_year
      ;
      ;subset_cols=[1,2,3,4,5,6,7,34,37,40,50]
      ;CLEAN_UP_MONTHS,times,low_cloud_cover,low_cloud_cover_mdi,subset_cols,qc_flags, last_year=last_year
      ;
      ;subset_cols=[1,2,3,4,5,6,7,35,37,38,40,51]
      ;CLEAN_UP_MONTHS,times,mid_cloud_cover,mid_cloud_cover_mdi,subset_cols,qc_flags, last_year=last_year
      ;
      ;subset_cols=[1,2,3,4,5,6,7,36,37,38,39,40,52]
      ;CLEAN_UP_MONTHS,times,high_cloud_cover,high_cloud_cover_mdi,subset_cols,qc_flags, last_year=last_year
      ;
      ;subset_cols=[1,2,3,4,5,6,7,53]
      ;CLEAN_UP_MONTHS,times,cloud_base,cloud_base_mdi,subset_cols,qc_flags, last_year=last_year

      history=history+STRING(SYSTIME())+' Clean Up Months \n '     
  ENDIF

  ;
  ;************************************************************************************
  ;     START FILE OUTPUT
  ;************************************************************************************
  ; Output modified data to a new directory. Preserve the variable attributes and add 
  ; in the location info as a header field.
  ;************************************************************************************
  ; Write out to the output directory the clean version.
  ; Create as wilma
  ;************************************************************************************
  netcdf_outfile=outputdir+stations(st)+'_'+suffix+'.nc'

  wilma=NCDF_CREATE(netcdf_outfile,/clobber)
  
  tid=NCDF_DIMDEF(wilma,'time',nt)
  testid=NCDF_DIMDEF(wilma, 'test', 61)
  charid=NCDF_DIMDEF(wilma,'Character',12)
  flagid=NCDF_DIMDEF(wilma,'flagged',9)
  changesid=NCDF_DIMDEF(wilma,'changepoints',N_ELEMENTS(STRING(changenotes)))
  chptsid=NCDF_DIMDEF(wilma,'changes',4)
  
  chgntlen=STRLEN(changenotes)
  lcharid=NCDF_DIMDEF(wilma,'LongCharacter',MAX(chgntlen))
  
  timevar=NCDF_VARDEF(wilma,'time',[tid],/DOUBLE)
  stationsvar=NCDF_VARDEF(wilma,'input_station_id',[charid,tid],/CHAR)
  tempsvar=NCDF_VARDEF(wilma,'temperatures',[tid],/FLOAT)
  dewsvar=NCDF_VARDEF(wilma,'dewpoints',[tid],/FLOAT)
  tcvar=NCDF_VARDEF(wilma,'total_cloud_cover',[tid],/SHORT)
  lcvar=NCDF_VARDEF(wilma,'low_cloud_cover',[tid],/SHORT)
  mcvar=NCDF_VARDEF(wilma,'mid_cloud_cover',[tid],/SHORT)
  hcvar=NCDF_VARDEF(wilma,'high_cloud_cover',[tid],/SHORT)
  cbvar=NCDF_VARDEF(wilma,'cloud_base',[tid],/SHORT)
  wsvar=NCDF_VARDEF(wilma,'windspeeds',[tid],/FLOAT) 
  wdvar=NCDF_VARDEF(wilma,'winddirs',[tid],/SHORT)
  slpvar=NCDF_VARDEF(wilma,'slp',[tid],/FLOAT)
  sigwxvar=NCDF_VARDEF(wilma,'past_sigwx1',[tid],/SHORT)
  precipvar=NCDF_VARDEF(wilma,'precip1_depth',[tid],/FLOAT)
  qcvar=NCDF_VARDEF(wilma,'qc_flags',[testid,tid],/SHORT)
  flagvar=NCDF_VARDEF(wilma,'flagged_values',[flagid,tid],/FLOAT)
  changevar=NCDF_VARDEF(wilma,'change_points',[chptsid,changesid],/FLOAT)
  chntsvar=NCDF_VARDEF(wilma,'change_notes',[lcharid,changesid],/CHAR)
  
  NCDF_ATTPUT,wilma,'time','long_name','time',/CHAR
  NCDF_ATTPUT,wilma,'time','standard_name','time',/CHAR
  NCDF_ATTPUT,wilma,'time','units','days since 1973-01-01 00:00:00',/CHAR
  NCDF_ATTPUT,wilma,'time','axis','T',/CHAR
  NCDF_ATTPUT,wilma,'time','calendar','gregorian',/CHAR
  NCDF_ATTPUT,wilma,'time','valid_min',0.,/FLOAT
  NCDF_ATTPUT,wilma,'input_station_id','long_name','Primary source for timestep (may be multiple sources for composite stations). USAF - WBAN from ISD source',/CHAR
  NCDF_ATTPUT,wilma,'input_station_id','cf_role','timeseries id',/CHAR
  NCDF_ATTPUT,wilma,'temperatures','long_name','Dry bulb temperatures nearest to reporting hour',/CHAR
  NCDF_ATTPUT,wilma,'temperatures','standard_name','surface_temperature',/CHAR
  NCDF_ATTPUT,wilma,'temperatures','units','degreeC',/CHAR
  NCDF_ATTPUT,wilma,'temperatures','coordinates','time',/CHAR
  valid=WHERE(((temperatures NE temperatures_mdi) AND (temperatures NE temperatures_fdi)), tc)
  IF tc GE 1 THEN BEGIN
    min_t=MIN(temperatures(valid))
    max_t=MAX(temperatures(valid))
    NCDF_ATTPUT,wilma,'temperatures','valid_min',min_t(0),/FLOAT
    NCDF_ATTPUT,wilma,'temperatures','valid_max',max_t(0),/FLOAT
  ENDIF
  NCDF_ATTPUT,wilma,'temperatures','missing_value',temperatures_mdi,/FLOAT
  NCDF_ATTPUT,wilma,'temperatures','flagged_value',temperatures_fdi,/FLOAT
  NCDF_ATTPUT,wilma,'dewpoints','long_name','Dew point temperatures nearest to reporting hour',/CHAR
  NCDF_ATTPUT,wilma,'dewpoints','standard_name','dew_point_temperature',/CHAR
  NCDF_ATTPUT,wilma,'dewpoints','units','degreeC',/CHAR
  NCDF_ATTPUT,wilma,'dewpoints','coordinates','time',/CHAR
  valid=WHERE(((dewpoints NE dewpoints_mdi) AND (dewpoints NE dewpoints_fdi)),tc)
  IF tc GE 1 THEN BEGIN
    min_t=MIN(dewpoints(valid))
    max_t=MAX(dewpoints(valid))
    NCDF_ATTPUT,wilma,'dewpoints','valid_min',min_t(0),/FLOAT
    NCDF_ATTPUT,wilma,'dewpoints','valid_max',max_t(0),/FLOAT
  ENDIF
  NCDF_ATTPUT,wilma,'dewpoints','missing_value',dewpoints_mdi,/FLOAT
  NCDF_ATTPUT,wilma,'dewpoints','flagged_value',dewpoints_fdi,/FLOAT
  NCDF_ATTPUT,wilma,'total_cloud_cover','long_name','Total cloud cover derived in priority order GA, GF, GD - see ISD documentation.  Units are Oktas',/CHAR
  NCDF_ATTPUT,wilma,'total_cloud_cover','standard_name','cloud_area_fraction',/CHAR
  NCDF_ATTPUT,wilma,'total_cloud_cover','units','1',/CHAR
  NCDF_ATTPUT,wilma,'total_cloud_cover','valid_min',0,/SHORT
  NCDF_ATTPUT,wilma,'total_cloud_cover','valid_max',8,/SHORT
  NCDF_ATTPUT,wilma,'total_cloud_cover','missing_value',total_cloud_cover_mdi,/SHORT
  NCDF_ATTPUT,wilma,'total_cloud_cover','flagged_value',total_cloud_cover_fdi,/SHORT
  NCDF_ATTPUT,wilma,'total_cloud_cover','coordinates','time',/CHAR
  NCDF_ATTPUT,wilma,'low_cloud_cover','long_name','low cloud cover derived in priority order GA, GF, GD - see ISD documentation.  Units are Oktas',/CHAR
  NCDF_ATTPUT,wilma,'low_cloud_cover','units','1',/CHAR
  NCDF_ATTPUT,wilma,'low_cloud_cover','valid_min',0,/SHORT
  NCDF_ATTPUT,wilma,'low_cloud_cover','valid_max',8,/SHORT
  NCDF_ATTPUT,wilma,'low_cloud_cover','missing_value',low_cloud_cover_mdi,/SHORT
  NCDF_ATTPUT,wilma,'low_cloud_cover','flagged_value',low_cloud_cover_fdi,/SHORT
  NCDF_ATTPUT,wilma,'low_cloud_cover','coordinates','time',/CHAR
  NCDF_ATTPUT,wilma,'mid_cloud_cover','long_name','mid cloud cover derived in priority order GA, GF, GD - see ISD documentation.  Units are Oktas',/CHAR
  NCDF_ATTPUT,wilma,'mid_cloud_cover','units','1',/CHAR
  NCDF_ATTPUT,wilma,'mid_cloud_cover','valid_min',0,/SHORT
  NCDF_ATTPUT,wilma,'mid_cloud_cover','valid_max',8,/SHORT
  NCDF_ATTPUT,wilma,'mid_cloud_cover','missing_value',mid_cloud_cover_mdi,/SHORT
  NCDF_ATTPUT,wilma,'mid_cloud_cover','flagged_value',mid_cloud_cover_fdi,/SHORT
  NCDF_ATTPUT,wilma,'mid_cloud_cover','coordinates','time',/CHAR
  NCDF_ATTPUT,wilma,'high_cloud_cover','long_name','high cloud cover derived in priority order GA, GF, GD - see ISD documentation.  Units are Oktas',/CHAR
  NCDF_ATTPUT,wilma,'high_cloud_cover','units','1',/CHAR
  NCDF_ATTPUT,wilma,'high_cloud_cover','valid_min',0,/SHORT
  NCDF_ATTPUT,wilma,'high_cloud_cover','valid_max',8,/SHORT
  NCDF_ATTPUT,wilma,'high_cloud_cover','missing_value',high_cloud_cover_mdi,/SHORT
  NCDF_ATTPUT,wilma,'high_cloud_cover','flagged_value',high_cloud_cover_fdi,/SHORT
  NCDF_ATTPUT,wilma,'high_cloud_cover','coordinates','time',/CHAR
  NCDF_ATTPUT,wilma,'cloud_base','long_name','Cloud base of lowest cloud layer nearest to reporting hour',/CHAR
  NCDF_ATTPUT,wilma,'cloud_base','standard_name','cloud_base_altitude',/CHAR
  NCDF_ATTPUT,wilma,'cloud_base','units','metres',/CHAR
  NCDF_ATTPUT,wilma,'cloud_base','coordinates','time',/CHAR
  valid=WHERE(cloud_base NE -999,tc)
  IF tc GE 1 THEN BEGIN
    min_t=MIN(cloud_base(valid))
    max_t=MAX(cloud_base(valid))
    NCDF_ATTPUT,wilma,'cloud_base','valid_min',min_t(0),/FLOAT
    NCDF_ATTPUT,wilma,'cloud_base','valid_max',max_t(0),/FLOAT
  ENDIF
  NCDF_ATTPUT,wilma,'cloud_base','missing_value',cloud_base_mdi,/SHORT
  NCDF_ATTPUT,wilma,'windspeeds','long_name','Wind speed',/CHAR
  NCDF_ATTPUT,wilma,'windspeeds','standard_name','wind_speed',/CHAR
  NCDF_ATTPUT,wilma,'windspeeds','units','metres per second',/CHAR
  valid=WHERE(((windspeeds NE windspeeds_mdi) AND (windspeeds NE windspeeds_mdi)),tc)
  IF tc GE 1 THEN BEGIN
    min_t=MIN(windspeeds(valid))
    max_t=MAX(windspeeds(valid))
    NCDF_ATTPUT,wilma,'windspeeds','valid_min',min_t(0),/FLOAT
    NCDF_ATTPUT,wilma,'windspeeds','valid_max',max_t(0),/FLOAT
  ENDIF
  NCDF_ATTPUT,wilma,'windspeeds','missing_value',windspeeds_mdi,/FLOAT
  NCDF_ATTPUT,wilma,'windspeeds','flagged_value',windspeeds_fdi,/FLOAT
  NCDF_ATTPUT,wilma,'windspeeds','coordinates','time',/CHAR
  NCDF_ATTPUT,wilma,'winddirs','long_name','Wind Direction',/CHAR
  NCDF_ATTPUT,wilma,'winddirs','standard_name','wind_from_direction',/CHAR
  NCDF_ATTPUT,wilma,'winddirs','units','Degrees',/CHAR
  NCDF_ATTPUT,wilma,'winddirs','valid_min',0,/SHORT
  NCDF_ATTPUT,wilma,'winddirs','valid_max',360,/SHORT
  NCDF_ATTPUT,wilma,'winddirs','missing_value',winddirs_mdi,/SHORT
  NCDF_ATTPUT,wilma,'winddirs','flagged_value',winddirs_fdi,/SHORT
  NCDF_ATTPUT,wilma,'winddirs','coordinates','time',/CHAR
  NCDF_ATTPUT,wilma,'slp','long_name','Reported sea level pressure',/CHAR
  NCDF_ATTPUT,wilma,'slp','standard_name','air_pressure_at_sea_level',/CHAR
  NCDF_ATTPUT,wilma,'slp','units','hPa',/CHAR
  valid=WHERE(((slp NE slp_mdi) AND (slp NE slp_mdi)), tc)
  IF tc GE 1 THEN BEGIN
    min_t=MIN(slp(valid))
    max_t=MAX(slp(valid))
    NCDF_ATTPUT,wilma,'slp','valid_min',min_t(0),/FLOAT
    NCDF_ATTPUT,wilma,'slp','valid_max',max_t(0),/FLOAT
  ENDIF
  NCDF_ATTPUT,wilma,'slp','missing_value',slp_mdi,/FLOAT
  NCDF_ATTPUT,wilma,'slp','flagged_value',slp_fdi,/FLOAT
  NCDF_ATTPUT,wilma,'slp','coordinates','time',/CHAR
  NCDF_ATTPUT,wilma,'past_sigwx1','long_name','Station reports of past significant weather phenomena',/CHAR
  NCDF_ATTPUT,wilma,'past_sigwx1','units','1',/CHAR
  NCDF_ATTPUT,wilma,'past_sigwx1','valid_min',0s,/SHORT
  NCDF_ATTPUT,wilma,'past_sigwx1','valid_max',9s,/SHORT
  NCDF_ATTPUT,wilma,'past_sigwx1','missing_value',-999s
  NCDF_ATTPUT,wilma,'past_sigwx1','coordinates','time',/CHAR
  NCDF_ATTPUT,wilma,'precip1_depth','long_name','Depth of precipitation reported',/CHAR
  NCDF_ATTPUT,wilma,'precip1_depth','standard_name','lwe_thickness_of_precipitation_amount',/CHAR
  NCDF_ATTPUT,wilma,'precip1_depth','units','mm',/CHAR
  NCDF_ATTPUT,wilma,'precip1_depth','valid_min',0,/SHORT
  NCDF_ATTPUT,wilma,'precip1_depth','valid_max',9,/SHORT
  NCDF_ATTPUT,wilma,'precip1_depth','missing_value',-1.e+30,/FLOAT
  NCDF_ATTPUT,wilma,'precip1_depth','coordinates','time',/CHAR
  NCDF_ATTPUT,wilma,'qc_flags','long_name','QC status for individual obs',/CHAR
  NCDF_ATTPUT,wilma,'qc_flags','units','1',/CHAR
  NCDF_ATTPUT,wilma,'qc_flags','missing_value',-999,/SHORT
  NCDF_ATTPUT,wilma,'qc_flags','coordinates','time',/CHAR
  NCDF_ATTPUT,wilma,'flagged_values','long_name','observation values removed by flagging',/CHAR
  NCDF_ATTPUT,wilma,'flagged_values','units','1',/CHAR
  NCDF_ATTPUT,wilma,'flagged_values','missing_value',temperatures_mdi,/FLOAT ; not strictly correct
  NCDF_ATTPUT,wilma,'flagged_values','coordinates','time',/CHAR
  NCDF_ATTPUT,wilma,'change_points','long_name','Month and year of change in reporting accuracy or frequency, and associated values',/CHAR
  NCDF_ATTPUT,wilma,'change_points','units','day',/CHAR
  NCDF_ATTPUT,wilma,'change_notes','long_name','Any notes associated with the change points',/CHAR
  NCDF_ATTPUT,wilma,'change_notes','units','day',/CHAR

  ; Global Attributes
  NCDF_ATTPUT,wilma,/GLOBAL,'title',title,/CHAR
  NCDF_ATTPUT,wilma,/GLOBAL,'institution',institution,/CHAR
  NCDF_ATTPUT,wilma,/GLOBAL,'source',source,/CHAR
  NCDF_ATTPUT,wilma,/GLOBAL,'references',references,/CHAR
  NCDF_ATTPUT,wilma,/GLOBAL,'creator_name',creator_name,/CHAR
  NCDF_ATTPUT,wilma,/GLOBAL,'creator_url',creator_url,/CHAR
  NCDF_ATTPUT,wilma,/GLOBAL,'creator_email',creator_email,/CHAR
  NCDF_ATTPUT,wilma,/GLOBAL,'time_coverage_start',time_coverage_start,/CHAR
  NCDF_ATTPUT,wilma,/GLOBAL,'time_coverage_end',time_coverage_end,/CHAR
  NCDF_ATTPUT,wilma,/GLOBAL,'cdm_data_type',cdm_data_type,/CHAR
  NCDF_ATTPUT,wilma,/GLOBAL,'keywords',keywords,/CHAR
  NCDF_ATTPUT,wilma,/GLOBAL,'summary',summary,/CHAR
  NCDF_ATTPUT,wilma,/GLOBAL,'hadisd_version',hadisd_version,/CHAR
  NCDF_ATTPUT,wilma,/GLOBAL,'isd_version',isd_version,/CHAR
  NCDF_ATTPUT,wilma,/GLOBAL,'isd_date',isd_date,/CHAR
  NCDF_ATTPUT,wilma,/GLOBAL,'station_information','Where station is a composite the station id refers to the primary source \n used in the timestep and may not apply to all elements',/CHAR
  NCDF_ATTPUT,wilma,/GLOBAL,'station_id',stations(st),/CHAR
  NCDF_ATTPUT,wilma,/GLOBAL,'latitude',latitudes(st),/FLOAT
  NCDF_ATTPUT,wilma,/GLOBAL,'longitude',longitudes(st),/FLOAT
  NCDF_ATTPUT,wilma,/GLOBAL,'elevation',elevations(st),/FLOAT
  current_time=SYSTIME()
  PRINT,current_time
  NCDF_ATTPUT,wilma,/GLOBAL,'date_created',STRING(current_time),/CHAR
  NCDF_ATTPUT,wilma,/GLOBAL,'processing_start',processing_start,/CHAR
  NCDF_ATTPUT,wilma,/GLOBAL,'qc_code_version',svn_version,/CHAR
  NCDF_ATTPUT,wilma,/GLOBAL,'composite_notes',composite_notes,/CHAR
  NCDF_ATTPUT,wilma,/GLOBAL,'flag_notes',flag_notes,/CHAR
  NCDF_ATTPUT,wilma,/GLOBAL,'history',oldhistory+history,/CHAR
  NCDF_ATTPUT,wilma,/GLOBAL,'Conventions','CF-1.5',/CHAR
  NCDF_ATTPUT,wilma,/GLOBAL,'Metadata_Conventions','Unidata Dataset Discovery v1.0,CF Discrete Sampling Geometries Conventions',/CHAR
  NCDF_ATTPUT,wilma,/GLOBAL,'featureType','timeSeries',/CHAR
  IF suffix EQ 'external' THEN BEGIN
     NCDF_ATTPUT,wilma,/GLOBAL,'processing_level','Internal & External Checks ',/CHAR
  ENDIF ELSE IF suffix EQ 'external2' THEN BEGIN
     NCDF_ATTPUT,wilma,/GLOBAL,'processing_level','2x Internal & External Checks',/CHAR
  ENDIF
  WAIT,1
  NCDF_CONTROL,wilma,/ENDEF
  
  NCDF_VARPUT, wilma, timevar, times 
  NCDF_VARPUT, wilma, stationsvar, input_station_id
  NCDF_VARPUT, wilma, tempsvar, temperatures
  NCDF_VARPUT, wilma, dewsvar, dewpoints
  NCDF_VARPUT, wilma, tcvar, total_cloud_cover
  NCDF_VARPUT, wilma, lcvar, low_cloud_cover
  NCDF_VARPUT, wilma, mcvar, mid_cloud_cover
  NCDF_VARPUT, wilma, hcvar, high_cloud_cover
  NCDF_VARPUT, wilma, cbvar, cloud_base
  NCDF_VARPUT, wilma, wsvar, windspeeds
  NCDF_VARPUT, wilma, wdvar, winddirs
  NCDF_VARPUT, wilma, slpvar, slp
  NCDF_VARPUT, wilma, sigwxvar, past_sigwx1
  NCDF_VARPUT, wilma, precipvar, precip1
  NCDF_VARPUT, wilma, qcvar, qc_flags
  NCDF_VARPUT, wilma, flagvar, flagged_values
  NCDF_VARPUT, wilma, changevar, changepoints
  NCDF_VARPUT, wilma, chntsvar, STRING(changenotes)
  
  NCDF_CLOSE,wilma
  
  ;**************************************************
  ; DUP = duplicates 			= columns 0,
  ; DNL = diurnal 			= columns 4,
  ; GAP = TGP, DGP, SGP, gaps		= columns 5 (temp),6 (dewp),7 (slp) 
  ; REC = TRC, DRC, WRC, SRC, records	= columns 8 (temp), 9 (dewp), 10 (Wind), 11 (SLP),
  ; STS = TSS,DSS,WSS,PSS straight strngs	= columns 12 (temp), 13 (dewps), 14 (wind), 15 (SLP),
  ; HST = HTS,HDS,HWS,HPS hour strings    = columns 16 (temp), 17 (dewp), 18 (wind), 19 (slp)
  ; DST = DTS,DDS,DWS,DPS day strings	= columns 20 (temp), 21 (dewp), 22 (wind), 23 (slp)
  ; CLM = TCM,DCM,PCM lim			= columns 24 (temp), 25 (dewp), 26 (slp)
  ; SPK = TSP,DSP,PSPSpike		= columns 27 (temp), 28 (dewp), 29 (slp)
  ; SSS = supersaturation			= columns 30, 'ttw_sat'
  ; DPD = dew point depression		= columns 31, 'ttw_dpd'
  ; DCF = dew point cutoffs		= columns 32, 'ttw_cut'
  ; CUO = cloud unobservable		= columns 33, (total), 34 (low), 35 (Mid), 36 (High),'cloud_unobservable'
  ; CST = cloud small total		= columns 37, 'cloud_smalltot'
  ; FLW = full low cloud			= columns 38, 'cloud_fulllow'
  ; FMC = full mid cloud			= columns 39, 'cloud_fullmid'
  ; FVC = TFV,DFV,SFVfrequent values check= columns 1 (temp), 2 (dewp), 3 (slp), 
  ; NGC = Negative cloud value		= columns 40, if Total, low, mid or high cloud is negative
  ; LAST THREE COLUMNS BLANK FOR EXTERNAL
  ; OUT = TOT,DOT,SOTTemperature Outlier	= columns 41 (temp), 42 (dewp), 43 (slp)

  ; TMB = month cleared temperature 	= columns 44
  ; DMB = month cleared dewpoint	 	= columns 45
  ; SMB = month cleared slp	 	= columnss 46
  ; WMB/BBB = month cleared windspd/dir 	= columns 47 and 48 (direction)
  ; CMB = month cleared total cloud 	= columns 49
  ; LMB = month cleared low cloud 	= columns 50
  ; MMB = month cleared middle cloud 	= columns 51
  ; HMB = month cleared high cloud 	= columns 52
  ; BMB = month cleared cloud base 	= columns 53

  ; OCT = odd_cluster temperatures 	= columns 54
  ; OCD = odd_cluster dewpoints	 	= columns 55
  ; OCW = odd_cluster windspeeds	 	= columns 56
  ; OCS = odd_cluster slp		 	= columns 57 

  ; TVR = variance check temperatures	= columns 58
  ; DVR = variance check dewpoints	= columns 59
  ; SVR = variance check slp 		= columns 60
  ;**************************************************

  ;**************************************************
  ; Print out breakdown of total non-passes to file
  ;   adjusted to avoid double counting
  ;**************************************************
  compressed_flags=qc_flags
  no_neighbours=WHERE(compressed_flags EQ -1, nncnt)
  IF nncnt GE 1 THEN compressed_flags[no_neighbours]=0
  compressed_flags=TOTAL(compressed_flags,1)
  nonpass=WHERE(compressed_flags NE 0, failcount)

  ;**************************************************
  ; Adjust for 2 level flagging system in Gap and Climatological Checks
  ;**************************************************
  ; GAP - TEMP
  tgfc=WHERE(qc_flags(5,*) EQ 2, tgfcnt)
  IF tgfcnt GE 0 THEN tentative_gap_flags_temp=tgfcnt ELSE tentative_gap_flags_temp=0 
  fgfc=WHERE(qc_flags(5,*) EQ 1, fgfcnt)
  IF fgfcnt GE 0 THEN firm_gap_flags_temp=fgfcnt ELSE firm_gap_flags_temp=0 

  gap_flags_temp=tentative_gap_flags_temp+firm_gap_flags_temp

  ; GAP - DEWPT
  tgfc=WHERE(qc_flags(6,*) EQ 2, tgfcnt)
  IF tgfcnt GE 0 THEN tentative_gap_flags_dpt=tgfcnt ELSE tentative_gap_flags_dpt=0 
  fgfc=WHERE(qc_flags(6,*) EQ 1, fgfcnt)
  IF fgfcnt GE 0 THEN firm_gap_flags_dpt=fgfcnt ELSE firm_gap_flags_dpt=0 

  gap_flags_dpt=tentative_gap_flags_dpt+firm_gap_flags_dpt

  ; GAP - SLP
  tgfc=WHERE(qc_flags(7,*) EQ 2, tgfcnt)
  IF tgfcnt GE 0 THEN tentative_gap_flags_slp=tgfcnt ELSE tentative_gap_flags_slp=0 
  fgfc=WHERE(qc_flags(7,*) EQ 1, fgfcnt)
  IF fgfcnt GE 0 THEN firm_gap_flags_slp=fgfcnt ELSE firm_gap_flags_slp=0 

  gap_flags_slp=tentative_gap_flags_slp+firm_gap_flags_slp

  ; CLIMATE - TEMP
  tclc=WHERE(qc_flags(24,*) EQ 2, tcfcnt)
  IF tcfcnt GE 0 THEN tentative_climate_flags_temp=tcfcnt ELSE tentative_climate_flags_temp=0
  fclc=WHERE(qc_flags(24,*) EQ 1, fcfcnt)
  IF fcfcnt GE 0 THEN firm_climate_flags_temp=fcfcnt ELSE firm_climate_flags_temp=0

  climate_flags_temp=tentative_climate_flags_temp+firm_climate_flags_temp

  ; CLIMATE - DEWP
  tclc=WHERE(qc_flags(25,*) EQ 2, tcfcnt)
  IF tcfcnt GE 0 THEN tentative_climate_flags_dewp=tcfcnt ELSE tentative_climate_flags_dewp=0
  fclc=WHERE(qc_flags(25,*) EQ 1, fcfcnt)
  IF fcfcnt GE 0 THEN firm_climate_flags_dewp=fcfcnt ELSE firm_climate_flags_dewp=0

  climate_flags_dewp=tentative_climate_flags_dewp+firm_climate_flags_dewp


  ; OUTLIER - account for -1 for no-neighbours
  ; Temperature
  nntf=WHERE(qc_flags(41,*) EQ 1, nntcnt)

  ; Dewpoint
  nndf=WHERE(qc_flags(42,*) EQ 1, nndcnt)

  ; SLP
  nnsf=WHERE(qc_flags(43,*) EQ 1, nnscnt)




  PRINTF,1,stations(st),nt,failcount,TOTAL(qc_flags(0,*)),TOTAL(qc_flags(1,*)),TOTAL(qc_flags(2,*)),$
                   TOTAL(qc_flags(3,*)),TOTAL(qc_flags(4,*)),gap_flags_temp,gap_flags_dpt,$
		   gap_flags_slp,TOTAL(qc_flags(8,*)),TOTAL(qc_flags(9,*)),TOTAL(qc_flags(10,*)),$
		   TOTAL(qc_flags(11,*)),TOTAL(qc_flags(12,*)),TOTAL(qc_flags(13,*)),TOTAL(qc_flags(14,*)),$
		   TOTAL(qc_flags(15,*)),TOTAL(qc_flags(16,*)),TOTAL(qc_flags(17,*)),TOTAL(qc_flags(18,*)),$
		   TOTAL(qc_flags(19,*)),TOTAL(qc_flags(20,*)),TOTAL(qc_flags(21,*)),TOTAL(qc_flags(22,*)),$
		   TOTAL(qc_flags(23,*)),climate_flags_temp,climate_flags_dewp,TOTAL(qc_flags(26,*)),$
		   TOTAL(qc_flags(27,*)),TOTAL(qc_flags(28,*)),TOTAL(qc_flags(29,*)),TOTAL(qc_flags(30,*)),$
		   TOTAL(qc_flags(31,*)),TOTAL(qc_flags(32,*)),TOTAL(qc_flags(33,*)),TOTAL(qc_flags(34,*)),$
		   TOTAL(qc_flags(35,*)),TOTAL(qc_flags(36,*)),TOTAL(qc_flags(37,*)),TOTAL(qc_flags(38,*)),$
		   TOTAL(qc_flags(39,*)),TOTAL(qc_flags(40,*)),nntcnt,nndcnt,nnscnt,TOTAL(qc_flags(44,*)),$
		   TOTAL(qc_flags(45,*)),TOTAL(qc_flags(46,*)),TOTAL(qc_flags(47,*)),TOTAL(qc_flags(48,*)),$
		   TOTAL(qc_flags(49,*)),TOTAL(qc_flags(50,*)),TOTAL(qc_flags(51,*)),TOTAL(qc_flags(52,*)),$
		   TOTAL(qc_flags(53,*)),TOTAL(qc_flags(54,*)),TOTAL(qc_flags(55,*)),TOTAL(qc_flags(56,*)),$
		   TOTAL(qc_flags(57,*)),TOTAL(qc_flags(58,*)),TOTAL(qc_flags(59,*)),TOTAL(qc_flags(60,*)),$
		   FORMAT='(A12,63(I8))'


  IF KEYWORD_SET(plots) THEN BEGIN
      PRINT,'Plotting'
      ;**************************************************
      ; Producing a plot of all years for this station
      ; highlighting where the QC tests failed.
      ;**************************************************
      PLOT_QC_FAIL_ALL_YEARS_RD,stations(st),last_year,suffix=suffix, pflags=pflags
  ENDIF
  
  
ENDFOR ; close station loop.

test_arr=['DUP','TFV','DFV','SFV','DNL','TGP','DGP','SGP','TRC','DRC',$
          'WRC','PRC','TSS','DSS','WSS','PSS','HTS','HDS','HWS','HPS',$
          'DTS','DDS','DWS','DPS','TCM','DCM','PCM','TSP','DSP','PSP',$
          'SSS','DPD','DCF','CUOT','CUOL','CUOM','CUOH','CST','FLW','FMC',$
          'NGC','TOT','DOT','SOT','TMB','DMB','SMB','WMB','BBB','CMB',$
          'LMB','MMB','HMB','BMB','OCT','OCD','OCW','OCS','TVR','DVR',$
          'SVR']

PRINTF,1,'station','times','fails',test_arr,FORMAT='(A12,63(A8))'
CLOSE,1

IF KEYWORD_SET(diagnostics) THEN PRINT,systime(1)-total_starttime
print,systime()
PRINT,'FINISHED'

RETURN

END
;************************************************************************
;                                 END
;************************************************************************
