;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;British Crown Copyright (c) 2012, the Met Office All rights reserved.
;
;Redistribution and use in source and binary forms, with or without 
;modification, are permitted provided that the following conditions 
;are met:
;
;Redistributions of source code must retain the above copyright notice, 
;this list of conditions and the following disclaimer. 
;
;Redistributions in binary form must reproduce the above copyright 
;notice, this list of conditions and the following disclaimer in the 
;documentation and/or other materials provided with the distribution. 
;
;Neither the name of the Met Office nor the names of its contributors 
;may be used to endorse or promote products derived from this software 
;without specific prior written permission. 
;
;THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
;"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
;LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
;A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
;HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
;SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
;LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
;DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
;THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
;(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
;OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

PRO qc_all_vars_internal, last_year,processing_date, $
                          svn_version, restart_id=restart_id, end_id=end_id, test=test, only=only,$
                          dynamic_diurnal=dynamic_diurnal, datadir=datadir, outputdir=outputdir, $
                          diagnostics=diagnostics, flags=flags, all_tests=all_tests, mask=mask, $
                          duplicate=duplicate, odd=odd, frequent=frequent, diurnal=diurnal, $
                          gap=gap, records=records, streak=streak, climate=climate,$
                          spike=spike, ttw=ttw, clouds=clouds, variance=variance, plots=plots, $
                          suffix=suffix, pflags=pflags, parallel=parallel
;
; Program written to undertake qc checks internal to a given station's record
; borrowing heavily off the procedures developed for GHCND and HadCRUH development
; (see respective documentation for details). Undertakes the analysis through a DO 
; loop that can be restarted at any point based upon the restart_id keyword value.
; This is then followed by a subsequent program that runs meta checks across variables
; as a second step once nastiness in each file has been suitably discarded. The order 
; is deliberate such that we are finding the very worst offending data first and then 
; refining our analysis down and down and down to harder tests.
;
; NAME:
;     QC_ALL_VARS_INTERNAL
;
; PURPOSE:
;     Control routine for the quality control checks to be performed
;     on 1 station.  Also reads in and writes out the data.  Default
;     is that bad data are only flagged, and masking is done separately.
;
; CALLING SEQUENCE:
;     qc_all_vars_internal, last_year,processing_date,
;     svn_version, restart_id, end_id,
;     test, only, dynamic_diurnal, datadir, outputdir, diagnostics, all_tests, mask,
;     duplicate, odd, frequent, diurnal, gap, records, streak, climate, spike,
;     ttw, clouds, variance, plots, suffix, pflags, parallel
;
; INPUTS:
;     last_year -  final year in the data sequence (to be passed to the
;                  check scripts)
;
;     processing_date - date at which the run of processing began
;
;     svn_version - version number of this QC code
;
;     restart_id - starting station number (allows continuation of
;                  terminated run)
;
;     end_id - ending station number (allows termination of run)
;
;     test -       keyword for doing test checks on 1 station only
;
;     only -       keyword for doing test checks on a small subset of
;            stations. Requires 'station_info.txt' to exist.
;
;     dyamic_diurnal - keyword to be passed to check_diurnal.pro
;
;     datadir -   directory containing the input data
;
;     outputdir - directory into which the output files are written
;
;     diagnostics - should diagnostics be run (produces intermediate
;                   plots, and more verbose output). Default is NO
;
;     flags - keyword whether to read in the old flags or not
;
;     all_tests - should all the tests be run
;
;     mask - should the data be masked during the program (YES) or at
;            the end (NO)
;
;     duplicate, odd, frequent, diurnal, gap, records, streak, climate, spike,
;     ttw, clouds, variance
;           - keywords to run individual tests
;
;     plots - keyword to run the summary plots at the end of the script
;
;     suffix - label for the files to be read in and out of the script
;              depending on at which stage it is run in the QC proceedure
;
;     pflags - passed to annual plotting to plot flags or not on
;              output
;
;     parallel - being run in parallel mode - remove and write an
;                output file to allow testing of completion.  An
;                integer, different for each session being run (ie
;                1,2,3,4..). Also turns off output of summary files
;
; OUTPUTS:
;
;     None - files written to specified or current directory.
;
; RESTRICTIONS:
;
;     Assumes netcdf file inputs.  Requires file "kept_station_info.txt" to
;     read in the stations to be analysed.  kept_station_info.txt contains
;     a list of all stations in the following form:
;
;     Station ID     LAT     LONG       ELEV
;     718936-99999   49.9500 -125.2670  106.0000
;
;     Also requires attributes.dat containing input for Metadata in
;     Key Value pairs.
;
; HISTORY:
;     NB - not all revisions are noted here.
;     # 1 - contains the form of the program as obtained
;           October 2010 RJHD 
;     # 2 - initial editing of file paths and extra commenting
;          01 November 2010 RJHD
;     # 5 - added extra keywords (mask) and tidied check_duplicates call
;          03 November 2010 RJHD
;     # 8 - adjusted odd_cluster call to add an extra loop + case
;           structure
;          03 November 2010 RJHD
;     #13 - added individual test keywords
;     #14-30 - adjusted the call to each QC subroutine as they were
;              dealt with.
;     #40 - diagnostic plots have been added to most subroutines
;           10 November 2010 RJHD
;     #44 - output .txt file summarising the flagging done so far
;           11 November 2010 RJHD
;     #97 - Adjusted call to Gap_check to reflect changes in that routine
;           25 November 2010 RJHD
;     #108 - Added call to reporting accuracy per month, and return
;            value written to output file. Done for 3 variables and
;            their times.
;           29 November 2010 RJHD
;     #135 - Updated program to run in a wrapper
;            (hadisd_qc_control.pro).  Keywords added (suffix) which
;            allow the selection of the correct filenames for the
;            input and output files.  Added in sigwx1 and precip1 to
;            the output .nc file
;           7 December 2010 RJHD
;     #172 - Added in the filter_flags arrays and calls to qc-tests.
;           14 January 2011 RJHD
;     #174 - Added flags keyword to allow reading in of flags from
;            previous runs.  Also works on suffix keyword.
;           18 January 2011 RJHD
;     #178 - Filter flag bug in gap check - fixed
;           28 January 2011 RJHD
;     #179 - Filter flag bug - now flags being copied back to source
;            arrays correctly, and masking also updated as that was
;            also not working.
;           2 February 2011 RJHD
;     #200- changed last_year from optional keyword to required input
;           to remove the hardwiring of the end date
;           11 March 2011 RJHD
;     #233- qc_summary file output corrected. Files updated using
;           qc_summary_create code.
;           03 October 2011 RJHD
;     #237- SVN information added to header.  Removed WHILE loops for
;           NCDF_OPEN command.  Started addition of
;           flagged_values_array.  Added meta-data to headers.
;           Adjusted call to duplicate_months as now not needed to run
;           each time.
;           16 November 2011 RJHD
;     #237- Typos on /mask fixed.  Added end_id keyword.
;           18 November 2011 RJHD
;     #241- Parallelisation option added - removes and creates a file
;           to allow monitoring of completion
;           21 November 2011 RJHD
;     #242- Composites identified and added into metadata in header
;           21 November 2011 RJHD
;     #243- Changed numerical order of FIRM and TENTATIVE FLAGS
;           21 November 2011 RJHD
;     #245- Changed reporting accuracy format in netcdf files
;           23 November 2011 RJHD
;     #250- Adjusted type of MetaData and the way it is processed
;           29 November 2011 RJHD
;     #252- Added output of value of /parallel and a finish statement
;           to allow restarting of crashed runs automatically, at the
;           correct station.  Also adjusted netcdf variable names after
;           metadata changes.
;           9 December 2011 RJHD
;     #262- Adjusted the loop for the composite station info.
;           14 December 2011 RJHD
;
;
;************************************************************************
;                    SVN Info
;$Rev:: 316                                           $:  Revision of last commit
;$Author:: rdunn                                      $:  Author of last commit
;$Date:: 2012-08-29 17:44:25 +0100 (Wed, 29 Aug 2012) $:  Date of last commit
;************************************************************************
;                                 START
;************************************************************************

total_starttime=systime(1)
print,systime()
;*******************
; Check input and output directories have been set, if not, use
; default setting
;*******************

cd, "./", current=pwd
IF NOT KEYWORD_SET(datadir) THEN fulldatadir=pwd+'/' ELSE fulldatadir=pwd+'/'+datadir+'/'
IF NOT KEYWORD_SET(outputdir) THEN outputdir=fulldatadir
IF NOT KEYWORD_SET(suffix) THEN suffix='internal'



IF suffix EQ 'internal' THEN BEGIN
;*******************
; Read in meta-data information
;*******************
   metadatafile='attributes.dat'
   lines=NLINES(metadatafile)

   MetaDNames=STRARR(lines)
   MetaDValue=STRARR(lines)
   line=''

   lnctr=0
   OPENR,1,metadatafile
   WHILE NOT EOF(1) DO BEGIN
   
      READF,1,line, FORMAT='(A)'

      Words=STRSPLIT(line,/Extract)
      
      MetaDNames[lnctr]=Words[0]

      MetaDValue[lnctr]=STRJOIN(Words[1:N_ELEMENTS(Words)-1],' ')
      lnctr=lnctr+1
      
   ENDWHILE
   CLOSE,1
ENDIF

;*******************
; if running in parallel mode then remove 
; the completion file from any previous run
; should also be removed by the control script, but just in case.
;*******************
IF KEYWORD_SET(parallel) THEN BEGIN
   parallelfile='CompletedParallel'+STRCOMPRESS(parallel,/remove_all)+'.txt'
   FILE_DELETE,parallelfile,/ALLOW_NONEXISTENT
   PRINT,'Parallel Run Number '+STRCOMPRESS(parallel,/remove_all)

ENDIF

;*******************
; Check the diagnostics keyword
;*******************
IF NOT KEYWORD_SET(diagnostics) THEN diagnostics=0
IF KEYWORD_SET(diagnostics) THEN BEGIN
    RESTORE_COLOURS,'shortbow'
ENDIF

;*******************
; Extract the information about the Duplicates if appropriate
;*******************
IF suffix EQ 'internal' THEN BEGIN
    COMPOSITEINFO, 'duplicates_ordered.txt', CompositesList
    ; returns array of size #stations X #composites
ENDIF

;*******************
; Echo which tests should be run if specified
;*******************

IF NOT KEYWORD_SET(all_tests) THEN PRINT,"Running individual tests"
IF KEYWORD_SET(duplicates) THEN PRINT, "Duplicates"
IF KEYWORD_SET(odd) THEN PRINT, "Odd Values"
IF KEYWORD_SET(frequent) THEN PRINT, "Frequent Value"
IF KEYWORD_SET(diurnal) THEN PRINT, "Diurnal"
IF KEYWORD_SET(gap) THEN PRINT, "Gap"
IF KEYWORD_SET(records) THEN PRINT, "Records"
IF KEYWORD_SET(streak) THEN PRINT, "Streak"
IF KEYWORD_SET(climate) THEN PRINT, "Climate"
IF KEYWORD_SET(spike) THEN PRINT, "Spike"
IF KEYWORD_SET(ttw) THEN PRINT, "T & TW Cross Check"
IF KEYWORD_SET(clouds) THEN PRINT, "Cloud Cross Check"
IF KEYWORD_SET(variance) THEN PRINT, "Variance"

;*******************
; Delete log_file.
;*******************
IF NOT KEYWORD_SET(test) THEN BEGIN
    FILE_DELETE,'qc_summary_internal_only.txt',/ALLOW_NONEXISTENT
 ENDIF
test_arr=['DUP','TFV','DFV','SFV','DNL','TGP','DGP','SGP','TRC','DRC',$
          'WRC','PRC','TSS','DSS','WSS','PSS','HTS','HDS','HWS','HPS',$
          'DTS','DDS','DWS','DPS','TCM','DCM','PCM','TSP','DSP','PSP',$
          'SSS','DPD','DCF','CUOT','CUOL','CUOM','CUOH','CST','FLW','FMC',$
          'NGC','TOT','DOT','SOT','TMB','DMB','SMB','WMB','BBB','CMB',$
          'LMB','MMB','HMB','BMB','OCT','OCD','OCW','OCS','TVR','DVR',$
          'SVR']

;*******************
; Read in the list of stations to be considered.
;*******************
IF KEYWORD_SET(only) OR KEYWORD_SET(test) THEN BEGIN
    ; to allow a small subset of the stations to be internally QC'd
    Station_info_file=pwd+'/station_info.txt'
ENDIF ELSE BEGIN
    Station_info_file=pwd+'/kept_station_info.txt'
ENDELSE

nstation=nlines(Station_info_file)

station_ids=STRARR(nstation)
station_lats=FLTARR(nstation)
station_lons=FLTARR(nstation)
station_elevs=FLTARR(nstation)
id=''
st=0

OPENR,1,Station_info_file
WHILE NOT EOF(1) DO BEGIN
  READF,1,id,FORMAT='(A)'
  station_ids(st)=STRMID(id,0,12)
  station_lats(st)=FLOAT(STRCOMPRESS(STRMID(id,13,9),/remove_all))
  station_lons(st)=FLOAT(STRCOMPRESS(STRMID(id,23,9),/remove_all))
  station_elevs(st)=FLOAT(STRCOMPRESS(STRMID(id,33,9),/remove_all))
  st=st+1
ENDWHILE
CLOSE,1

;*********************
; Ascertain which station id we are starting at.
;*********************

IF KEYWORD_SET(restart_id) THEN BEGIN
  match=WHERE(station_ids EQ restart_id, c)
  IF c NE 1 THEN STOP, 'restart_id passed on call does not match valid id'
  start_loc=match(0)
ENDIF ELSE start_loc=0

IF KEYWORD_SET(end_id) THEN BEGIN
  PRINT,'Terminated run selected'
  match=WHERE(station_ids EQ end_id, c)
  IF c NE 1 THEN STOP, 'end_id passed on call does not match valid id'
  end_loc=match(0)
ENDIF ELSE end_loc=nstation-1


IF KEYWORD_SET(test) THEN BEGIN
  ; so that only a single station is run and all subroutines are run too
  nstation=1
  start_loc=0
  end_loc=0
  station_lats=[56.]
ENDIF

IF NOT KEYWORD_SET(parallel) THEN BEGIN
   IF NOT KEYWORD_SET(test) THEN BEGIN
      ; echo the current station being processed
      OPENW,1,pwd+'/qc_summary_'+suffix+'.txt',/append 
   ENDIF ELSE BEGIN             ; test is set
      FILE_DELETE,pwd+'/qc_summary_'+suffix+'_only_'+station_ids(0)+'.txt', /ALLOW_NONEXISTENT
      ; delete the individual station qc storage file
      OPENW,1,pwd+'/qc_summary_'+suffix+'_only_'+station_ids(0)+'.txt',/append 
   ENDELSE
ENDIF

;***************************************************************************
;    START THE STATION LOOP
;***************************************************************************
FOR st=start_loc,end_loc DO BEGIN

   PRINT,'Station Number :',st+1,"/",nstation, FORMAT='(A-40,I4,A1,I4)'
   printstring='Station Identifier :'
   PRINT,printstring,station_ids(st), FORMAT='(A-40,A15)'

  ;***************************************************************************
  ;    TEST FOR SENSIBLE LAT/LON
  ;***************************************************************************
  
  IF ABS(station_lats(st)) GT 90 THEN BEGIN
     
     OPENW, 100, 'spurious_stations.txt',/append
     PRINT, 100, 'Latitude Check', station_ids(st),'1973-'+strcompress(last_year,/remove_all),'-','All','Non-physical value: '+strcompress(station_lats(st),/remove_all),FORMAT='(A20,A20,A10,I4,A20,A)'
     ClOSE, 100

     PRINT, 'Station number ',st+1,station_ids(st),' not processed - bad latitude', FORMAT='(A-40,I4,A15,A)'

     CONTINUE
  ENDIF

  IF ABS(station_lons(st)) GT 180 THEN BEGIN
     
     OPENW, 100, 'spurious_stations.txt',/append
     PRINT, 100, 'Longitude Check', station_ids(st),'1973-'+strcompress(last_year,/remove_all),'-','All','Outside format bounds: '+strcompress(station_lons(st),/remove_all),FORMAT='(A20,A20,A10,I4,A20,A)'
     ClOSE, 100

     PRINT, 'Station number ',st+1,station_ids(st),' longitude outside format bounds', FORMAT='(A-40,I4,A15,A)'

  ENDIF
  ;***************************************************************************
  ; storage string for processing history
  ;***************************************************************************
  oldhistory=''
  history=STRING(SYSTIME())+'*** INTERNAL TESTS ***\n'
  IF suffix EQ 'internal2' THEN history=history+STRING(SYSTIME())+' Second pass through QC Suite \n '      

  ;***************************************************************************
  ;    START THE DATA READING
  ;***************************************************************************



  ;****************************************
  ; Read in from the netcdf file the variables that we will use 
  ;****************************************

  IF suffix EQ 'internal' THEN file=fulldatadir+station_ids(st)+'.nc' $
    ELSE IF suffix EQ 'internal2' THEN file=fulldatadir+station_ids(st)+'_mask.nc'

  ;****************************************
  ; "fred" is the datafile for the station
  ; "betty" contains information about
  ;              the structure of the netcdf file, fred
  ;****************************************

  IF file EQ '.nc' THEN BREAK ; leave station loop if reading empty file (extra CRs in station-info.txt)

  fred=NCDF_OPEN(file,/nowrite)
  NCDF_CONTROL,0,/verbose
  betty=NCDF_INQUIRE(fred)

  ;****************************************
  ;  obtain information on the dimensions
  ;****************************************

  tid=NCDF_DIMID(fred,'time')
  NCDF_DIMINQ,fred,tid,timedim,nt

  ;****************************************
  ;  read the variable information
  ;****************************************

  IF suffix EQ 'internal' THEN BEGIN
     timesinfo=NCDF_VARINQ(fred,'times')
  ENDIF ELSE IF suffix EQ 'internal2' THEN BEGIN
     timesinfo=NCDF_VARINQ(fred,'time')
  ENDIF
  tempsinfo=NCDF_VARINQ(fred,'temperatures')
  idsinfo=NCDF_VARINQ(fred, 'input_station_id')
  dewsinfo=NCDF_VARINQ(fred, 'dewpoints')
  totclinfo=NCDF_VARINQ(fred, 'total_cloud_cover')
  lowclinfo=NCDF_VARINQ(fred, 'low_cloud_cover')
  midclinfo=NCDF_VARINQ(fred, 'mid_cloud_cover')
  hiclinfo=NCDF_VARINQ(fred, 'high_cloud_cover')
  clbaseinfo=NCDF_VARINQ(fred, 'cloud_base')
  windspinfo=NCDF_VARINQ(fred, 'windspeeds')
  winddirinfo=NCDF_VARINQ(fred, 'winddirs')
  slpinfo=NCDF_VARINQ(fred, 'slp')               ; Sea level pressuure
  pastsigwx1info=NCDF_VARINQ(fred,'past_sigwx1') ; Significant weather flag
  precipvar=NCDF_VARINQ(fred,'precip1_depth')    ; precipitation depth
  
  IF KEYWORD_SET(flags) OR suffix EQ 'internal2' THEN BEGIN
      ;****************************************
      ; 18/1/11 RJHD - need to read in the flags from a previous run if
      ;   they exist in the .netcdf file 
      ;****************************************
      flag_test=NCDF_VARID(fred, 'qc_flags')
      IF flag_test NE -1 THEN BEGIN
          flaginfo=NCDF_VARINQ(fred, 'qc_flags')
      ENDIF

      fv_test=NCDF_VARID(fred, 'flagged_values')
      IF fv_test NE -1 THEN BEGIN
          fvinfo=NCDF_VARINQ(fred, 'flagged_values')
      ENDIF

      chpt_test=NCDF_VARID(fred,'change_points')
      IF chpt_test NE -1 THEN BEGIN
         chngpntinfo=NCDF_VARINQ(fred, 'change_points')  
      ENDIF

      chnt_test=NCDF_VARID(fred,'change_notes')
      IF chnt_test NE -1 THEN BEGIN
         chngntinfo=NCDF_VARINQ(fred, 'change_notes')  
      ENDIF

  ENDIF
  
  ;****************************************
  ;  read the variable values into the arrays
  ;****************************************

  IF suffix EQ 'internal' THEN BEGIN
     NCDF_VARGET,fred,'times',times
  ENDIF ELSE IF suffix EQ 'internal2' THEN BEGIN
     NCDF_VARGET,fred,'time',times
  ENDIF
  NCDF_VARGET,fred,'temperatures',temperatures
  NCDF_VARGET,fred,'input_station_id',input_station_id
  NCDF_VARGET,fred,'dewpoints',dewpoints
  NCDF_VARGET,fred,'total_cloud_cover',total_cloud_cover
  NCDF_VARGET,fred,'low_cloud_cover',low_cloud_cover
  NCDF_VARGET,fred,'mid_cloud_cover',mid_cloud_cover
  NCDF_VARGET,fred,'high_cloud_cover',high_cloud_cover
  NCDF_VARGET,fred,'cloud_base',cloud_base
  NCDF_VARGET,fred,'windspeeds',windspeeds
  NCDF_VARGET,fred,'winddirs',winddirs
  NCDF_VARGET,fred,'slp',slp
  NCDF_VARGET,fred,'past_sigwx1',past_sigwx1
  NCDF_VARGET,fred,'precip1_depth',precip1		; added 23/09/09 KATE

  ;****************************************
  ;  read the mdi (missing data indicator) values
  ;****************************************

  NCDF_ATTGET,fred,'temperatures','missing_value',temperatures_mdi
  NCDF_ATTGET,fred,'dewpoints','missing_value',dewpoints_mdi
  NCDF_ATTGET,fred,'total_cloud_cover','missing_value',total_cloud_cover_mdi
  NCDF_ATTGET,fred,'low_cloud_cover','missing_value',low_cloud_cover_mdi
  NCDF_ATTGET,fred,'mid_cloud_cover','missing_value',mid_cloud_cover_mdi
  NCDF_ATTGET,fred,'high_cloud_cover','missing_value',high_cloud_cover_mdi
  NCDF_ATTGET,fred,'cloud_base','missing_value',cloud_base_mdi
  NCDF_ATTGET,fred,'windspeeds','missing_value',windspeeds_mdi
  NCDF_ATTGET,fred,'winddirs','missing_value',winddirs_mdi
  NCDF_ATTGET,fred,'slp','missing_value',slp_mdi

  IF suffix EQ 'internal2' THEN BEGIN
     ; get metadata
     NCDF_ATTGET,fred,'title',title,/GLOBAL
     NCDF_ATTGET,fred,'institution',institution,/GLOBAL
     NCDF_ATTGET,fred,'source',source,/GLOBAL
     NCDF_ATTGET,fred,'references',references,/GLOBAL
     NCDF_ATTGET,fred,'creator_name',creator_name,/GLOBAL
     NCDF_ATTGET,fred,'creator_url',creator_url,/GLOBAL
     NCDF_ATTGET,fred,'creator_email',creator_email,/GLOBAL
     NCDF_ATTGET,fred,'time_coverage_start',time_coverage_start,/GLOBAL
     NCDF_ATTGET,fred,'time_coverage_end',time_coverage_end,/GLOBAL
     NCDF_ATTGET,fred,'cdm_data_type',cdm_data_type,/GLOBAL
     NCDF_ATTGET,fred,'keywords',keywords,/GLOBAL
     NCDF_ATTGET,fred,'summary',summary,/GLOBAL
     NCDF_ATTGET,fred,'hadisd_version',hadisd_version,/GLOBAL
     NCDF_ATTGET,fred,'isd_version',isd_version,/GLOBAL
     NCDF_ATTGET,fred,'isd_date',isd_date,/GLOBAL
     NCDF_ATTGET,fred,'composite_notes',composite_notes,/GLOBAL
     NCDF_ATTGET,fred,'flag_notes',flag_notes,/GLOBAL
     NCDF_ATTGET,fred,'history',bhistory,/GLOBAL ; byte array
     oldhistory=STRING(bhistory)
  ENDIF

  ;****************************************
  ;  initialise the QC flag array
  ;****************************************
  
  IF KEYWORD_SET(flags) OR suffix EQ 'internal2' THEN BEGIN
     ; get the set flags
     IF flag_test NE -1 THEN BEGIN
        NCDF_VARGET,fred,'qc_flags',qc_flags
        oldflags=1
     ENDIF ELSE BEGIN
        qc_flags=INTARR(61,nt)
        oldflags=0
     ENDELSE
     ; get the masked values
     IF fv_test NE -1 THEN BEGIN
        NCDF_VARGET,fred,'flagged_values',flagged_values
     ENDIF ELSE BEGIN
        flagged_values=DBLARR(9,nt)
        flagged_values[0,*]=temperatures_mdi
        flagged_values[1,*]=dewpoints_mdi
        flagged_values[2,*]=slp_mdi
        flagged_values[3,*]=total_cloud_cover_mdi
        flagged_values[4,*]=low_cloud_cover_mdi
        flagged_values[5,*]=mid_cloud_cover_mdi
        flagged_values[6,*]=high_cloud_cover_mdi
        flagged_values[7,*]=windspeeds_mdi
        flagged_values[8,*]=winddirs_mdi
     ENDELSE

     IF chpt_test NE -1 THEN NCDF_VARGET,fred,'change_points',changepoints
     IF chnt_test NE -1 THEN NCDF_VARGET,fred,'change_notes',changenotes

  ENDIF ELSE BEGIN
     ; initialise the QC flags array and the flagged values array
      qc_flags=INTARR(61,nt)
      flagged_values=DBLARR(9,nt)
      flagged_values[0,*]=temperatures_mdi
      flagged_values[1,*]=dewpoints_mdi
      flagged_values[2,*]=slp_mdi
      flagged_values[3,*]=total_cloud_cover_mdi
      flagged_values[4,*]=low_cloud_cover_mdi
      flagged_values[5,*]=mid_cloud_cover_mdi
      flagged_values[6,*]=high_cloud_cover_mdi
      flagged_values[7,*]=windspeeds_mdi
      flagged_values[8,*]=winddirs_mdi
  ENDELSE

  ;****************************************
  ; total flags arrays for T, DP, SLP
  ;****************************************

  t_qc_flags=INTARR(nt)
  dp_qc_flags=INTARR(nt)
  slp_qc_flags=INTARR(nt)
  w_qc_flags=INTARR(nt)

  ;****************************************
  ;  set a flagged data indicator, fdi
  ;****************************************
  IF KEYWORD_SET(flags) OR suffix EQ 'internal2' THEN BEGIN
      ;****************************************
      ; 16/11/11 RJHD - read in flagged data indicator if appropriate
      ;****************************************

      trep_test=NCDF_ATTINQ(fred, 'temperatures','flagged_value')
      IF trep_test.(0) NE 'UNKNOWN' THEN BEGIN
          NCDF_ATTGET,fred, 'temperatures','flagged_value',temperatures_fdi
      ENDIF ELSE temperatures_fdi=-2.e30

      drep_test=NCDF_ATTINQ(fred, 'dewpoints','flagged_value')
      IF drep_test.(0) NE 'UNKNOWN' THEN BEGIN
          NCDF_ATTGET,fred, 'dewpoints','flagged_value',dewpoints_fdi
      ENDIF ELSE dewpoints_fdi=-2.e30
   
      srep_test=NCDF_ATTINQ(fred, 'slp','flagged_value')
      IF srep_test.(0) NE 'UNKNOWN' THEN BEGIN
          NCDF_ATTGET,fred, 'slp','flagged_value',slp_fdi
      ENDIF ELSE slp_fdi=-2.e30

      tcrep_test=NCDF_ATTINQ(fred, 'total_cloud_cover','flagged_value')
      IF tcrep_test.(0) NE 'UNKNOWN' THEN BEGIN
          NCDF_ATTGET,fred, 'total_cloud_cover','flagged_value',total_cloud_cover_fdi
      ENDIF ELSE total_cloud_cover_fdi=-888

      lcrep_test=NCDF_ATTINQ(fred, 'low_cloud_cover','flagged_value')
      IF lcrep_test.(0) NE 'UNKNOWN' THEN BEGIN
          NCDF_ATTGET,fred, 'low_cloud_cover','flagged_value',low_cloud_cover_fdi
      ENDIF ELSE low_cloud_cover_fdi=-888

      mcrep_test=NCDF_ATTINQ(fred, 'mid_cloud_cover','flagged_value')
      IF mcrep_test.(0) NE 'UNKNOWN' THEN BEGIN
          NCDF_ATTGET,fred, 'mid_cloud_cover','flagged_value',mid_cloud_cover_fdi
      ENDIF ELSE mid_cloud_cover_fdi=-888

      hcrep_test=NCDF_ATTINQ(fred, 'high_cloud_cover','flagged_value')
      IF hcrep_test.(0) NE 'UNKNOWN' THEN BEGIN
          NCDF_ATTGET,fred, 'high_cloud_cover','flagged_value',high_cloud_cover_fdi
      ENDIF ELSE high_cloud_cover_fdi=-888
   
      wsrep_test=NCDF_ATTINQ(fred, 'windspeeds','flagged_value')
      IF wsrep_test.(0) NE 'UNKNOWN' THEN BEGIN
          NCDF_ATTGET,fred, 'windspeeds','flagged_value',windspeeds_fdi
      ENDIF ELSE windspeeds_fdi=-2.e30
      
      wdrep_test=NCDF_ATTINQ(fred, 'winddirs','flagged_value')
      IF wdrep_test.(0) NE 'UNKNOWN' THEN BEGIN
          NCDF_ATTGET,fred, 'winddirs','flagged_value',winddirs_fdi
      ENDIF ELSE winddirs=-888
   
   ENDIF ELSE BEGIN
      temperatures_fdi=-2.e30
      dewpoints_fdi=-2.e30
      slp_fdi=-2.e30
      total_cloud_cover_fdi=-888
      low_cloud_cover_fdi=-888
      mid_cloud_cover_fdi=-888
      high_cloud_cover_fdi=-888
      windspeeds_fdi=-2.e30
      winddirs_fdi=-888
   ENDELSE
   
  printstring='Total station record size: '
  PRINT,printstring,nt, FORMAT='(A-40,I10)'

  ;****************************************
  ;  if doing a diagnostic run then close the original file
  ;       can then check changes to data read, code running
  ;       or just not wishing to proceed as far as output
  ;****************************************

  IF KEYWORD_SET(diagnostics) THEN NCDF_CLOSE,fred   

  ;****************************************
  ; Set up the uncompressed arrays
  ;****************************************
   
  CREATE_FULLTIMES, last_year, temperatures, temperatures_mdi, dewpoints, slp,$
    windspeeds, winddirs, times, full_times, h_times, full_h_times, full_temperatures, full_dewpoints, full_slp,$ 
    full_windspeeds,full_winddirs,duplicate_qc_flags, dewpoints_mdi, slp_mdi,$
    windspeeds_mdi,winddirs_mdi,diagnostics, station_ids(st), flagged_values, $
    full_flagged_values, total_cloud_cover_mdi, low_cloud_cover_mdi, $
    mid_cloud_cover_mdi, high_cloud_cover_mdi

  ;************************************************************************************
  ;      Obtain the reporting accuracy for the observations 
  ;          And reporting frequency
  ;************************************************************************************
  

  REPORTING_CHANGES, last_year,times,temperatures,temperatures_mdi,temperatures_fdi,tchangepoints,tchangenotes, 'Temp'
  REPORTING_CHANGES, last_year,times,dewpoints,dewpoints_mdi,dewpoints_fdi,dchangepoints,dchangenotes, 'Dewp'
  REPORTING_CHANGES, last_year,times,slp,slp_mdi,slp_fdi,schangepoints,schangenotes, 'SLP '
  
  changepoints=[[tchangepoints],[dchangepoints],[schangepoints]]
  changenotes=[tchangenotes,dchangenotes,schangenotes]

  ;************************************************************************************
  ;      END OF DATA READING  ---  START OF THE QC SUBROUTINES
  ;************************************************************************************
  ;
  ; Call subroutines in roughly the order they are called in GHCND following
  ; intensive discussions with Imke Durre
  ;
  ;************************************************************************************
  ;      CHECK DUPLICATES
  ;
  ; Checking exact duplicates within a given year and between years for each calendar month.
  ; These can't be real and must be indicative of issues with the whole dataset in at least one of
  ; the periods. As we cannot know a priori which we'll simply ditch both. Only done for temperature 
  ; under assumption this'll catch all cases (minimises computational overhead).
  ;************************************************************************************

  IF KEYWORD_SET(all_tests) OR KEYWORD_SET(duplicate) THEN BEGIN

     PRINT,""
     starttime=systime(1)
     CHECK_DUPLICATES, last_year, temperatures, temperatures_mdi, temperatures_fdi, $
                       times, full_times, full_temperatures, duplicate_qc_flags, $
                       diagnostics, station_ids(st), flagged_values, full_flagged_values
     
     endtime=systime(1)
     
     IF KEYWORD_SET(flags) OR suffix EQ 'internal2' THEN BEGIN
        prev_flags=WHERE(qc_flags(0,*) GT 0, pfcnt)
     ENDIF

     ;**********************************
     ;     Set the flags
     ;**********************************
     points=WHERE(duplicate_qc_flags GT 0, bad_count)
     
     printstring="      Replication Flags :"
     IF KEYWORD_SET(flags) OR suffix EQ 'internal2' THEN BEGIN
        PRINT,printstring,bad_count-pfcnt,'  Old Flags:',pfcnt, FORMAT='(A-40,I8,A-15,I8)'
     ENDIF ELSE BEGIN
        PRINT,printstring,bad_count, FORMAT='(A-40,I8)'
     ENDELSE
     
     IF (bad_count GE 1) THEN BEGIN
       ;*************************************************
       ; there are duplications, so set up the flags
       ;*************************************************
       ; Only new flags set to 1
       ;   old flags not wipes
       ;*************************************************
        qc_flags(0,points)=1
        
        t_qc_flags(points)=1
        dp_qc_flags(points)=1
        slp_qc_flags(points)=1
        w_qc_flags(points)=1
        
        IF KEYWORD_SET(mask) THEN BEGIN
          
          ;*******************************************
          ; If masking specified, then do it
          ;*******************************************
          
          ; 1) move values to flagged_values array
          ; 2) put flagged data indicator in their place

           flagged_values[0,points]=temperatures(points)
           flagged_values[1,points]=dewpoints(points)
           flagged_values[2,points]=slp(points)
           flagged_values[3,points]=total_cloud_cover(points)
           flagged_values[4,points]=low_cloud_cover(points)
           flagged_values[5,points]=mid_cloud_cover(points)
           flagged_values[6,points]=high_cloud_cover(points)
           flagged_values[7,points]=windspeeds(points)
           flagged_values[8,points]=winddirs(points)       
           
           temperatures(points)=temperatures_fdi  
           dewpoints(points)=dewpoints_fdi  
           total_cloud_cover(points)=total_cloud_cover_fdi
           low_cloud_cover(points)=low_cloud_cover_fdi
           mid_cloud_cover(points)=mid_cloud_cover_fdi
           high_cloud_cover(points)=high_cloud_cover_fdi
           cloud_base(points)=cloud_base_mdi ; mdi on purpose - no QC done on cloudbase - just for cloud checks later 17/11/11 RJHD
           windspeeds(points)=windspeeds_fdi
           winddirs(points)=winddirs_fdi
           slp(points)=slp_fdi

           h_times=INT(times*24)
           full_h_times=INT(full_times*24.)
           MATCH, h_times(points), full_h_times, suba, subb

           full_temperatures(subb)=temperatures_fdi
           full_dewpoints(subb)=dewpoints_fdi
           full_slp(subb)=slp_fdi
           full_windspeeds(subb)=windspeeds_fdi
           full_winddirs(subb)=winddirs_fdi
        ENDIF
     ENDIF
     
     IF KEYWORD_SET(diagnostics) THEN BEGIN
        printstring="Check Duplicates took "
        PRINT, printstring, endtime-starttime, FORMAT='(A-40,F15.5)'   
     ENDIF
     history=history+STRING(SYSTIME())+' Duplicate Month Check \n '

  ENDIF ELSE PRINT, 'Duplicate Month Check not run'

  
  ;************************************************************************************
  ;      CHECK FOR ODD CLUSTERS
  ;************************************************************************************
  
  ; Check for random small clusters of data isolated from all other data - unreliable
  
  IF KEYWORD_SET(all_tests) OR KEYWORD_SET(odd) THEN BEGIN
      PRINT,""

      starttime=systime(1)

      PRINT, '0 - Temperature, 1 - Dewpoints, 2 - Windspeeds, 3 - SLP'

      FOR type=0,3 DO BEGIN
          ;*************************************************
          ; used to have all 4 variables inside subroutine
          ; and also inside the FOR loop.  This is clearer
          ; but is it quicker?
          ;*************************************************
          CASE type OF
              0: BEGIN
                  value=temperatures
                  full_value=full_temperatures
                  mdi=temperatures_mdi
                  fdi=temperatures_fdi
                  flagcol=54
                  filter_flags=t_qc_flags
              END
              1: BEGIN
                  value=dewpoints
                  full_value=full_dewpoints
                  mdi=dewpoints_mdi
                  fdi=dewpoints_fdi
                  flagcol=55
                  filter_flags=dp_qc_flags
              END
              2: BEGIN
                  value=windspeeds
                  full_value=full_windspeeds
                  mdi=windspeeds_mdi
                  fdi=windspeeds_fdi
                  flagcol=56
                  filter_flags=w_qc_flags
              END
              3: BEGIN
                  value=slp
                  full_value=full_slp
                  mdi=slp_mdi
                  fdi=slp_fdi
                  flagcol=57
                  filter_flags=slp_qc_flags
              END
          ENDCASE
                   
          IF KEYWORD_SET(flags) OR suffix EQ 'internal2' THEN BEGIN
              prev_flags=WHERE(qc_flags(flagcol,*) GT 0, pfcnt)
          ENDIF

          ODD_CLUSTER_CHECK, times, full_times, h_times, full_h_times, full_value, mdi, fdi, qc_flags, flagcol, filter_flags,$
            diagnostics, station_ids(st)

          points=WHERE(qc_flags(flagcol,*) GT 0, bad_count)

          printstring="       Odd Cluster Flags"
          IF KEYWORD_SET(flags) OR suffix EQ 'internal2' THEN BEGIN
              PRINT,printstring,type,':', bad_count-pfcnt,'  Old Flags:',pfcnt, FORMAT='(A-38,I1,A1,I8,A-15,I8)'
          ENDIF ELSE BEGIN
              PRINT,printstring,type,':', bad_count, FORMAT='(A-38,I1,A1,I8)'
          ENDELSE

          ;*************************************************
          ; Read out the filter flags
          ;*************************************************
          CASE type OF
              0: BEGIN
                  t_qc_flags=filter_flags
              END
              1: BEGIN
                  dp_qc_flags=filter_flags
              END
              2: BEGIN
                  w_qc_flags=filter_flags
              END
              3: BEGIN
                  slp_qc_flags=filter_flags
              END
          ENDCASE
          
          IF KEYWORD_SET(mask) THEN BEGIN              
              ;*************************************************
              ; If masking specified, then do it
              ;*************************************************
              IF bad_count GE 1 THEN BEGIN

                  MATCH, h_times(points), full_h_times, suba, subb
                  CASE type OF
                      0: BEGIN
                          flagged_values[0,points]=temperatures[points]
                          temperatures(points)=temperatures_fdi
                          full_temperatures(subb)=temperatures_fdi
                      END
                      1: BEGIN
                          flagged_values[1,points]=dewpoints[points]
                          dewpoints(points)=dewpoints_fdi
                          full_dewpoints(subb)=dewpoints_fdi
                      END
                      2: BEGIN
                          flagged_values[7,points]=windspeeds[points]
                          flagged_values[8,points]=winddirs[points]
                          winddirs(points)=winddirs_fdi
                          windspeeds(points)=windspeeds_fdi
                          full_winddirs(subb)=winddirs_fdi
                          full_windspeeds(subb)=windspeeds_fdi
                      END
                      3: BEGIN
                          flagged_values[2,points]=slp[points]
                          slp(points)=slp_fdi
                          full_slp(subb)=slp_fdi
                      END
                  ENDCASE
              ENDIF
          ENDIF 


      ENDFOR
      endtime=systime(1)
      IF KEYWORD_SET(diagnostics) THEN BEGIN
          printstring="Odd Cluster took "
          PRINT, printstring, endtime-starttime, FORMAT='(A-40,F15.5)'   
      ENDIF
     history=history+STRING(SYSTIME())+' Odd Cluster Check \n '
  ENDIF ELSE PRINT, "Odd Cluster not run"

  
  ;************************************************************************************
  ;      CHECK FOR FREQUENT VALUES
  ;************************************************************************************
  ;
  ; Check if there are grossly erroneous patterns of values that should be removed.

  IF KEYWORD_SET(all_tests) OR KEYWORD_SET(frequent) THEN BEGIN
      PRINT,""
      starttime=systime(1) 

      PRINT, '1 - Temperature, 2 - Dewpoints, 3 - SLP'

      FOR type=1,3 DO BEGIN

          CASE type OF
              1: BEGIN
                  value=temperatures
                  mdi=temperatures_mdi
                  fdi=temperatures_fdi
                  filter_flags=t_qc_flags
              END
              2: BEGIN
                  value=dewpoints
                  mdi=dewpoints_mdi
                  fdi=dewpoints_fdi
                  filter_flags=dp_qc_flags
              END
              3: BEGIN
                  value=slp
                  mdi=slp_mdi
                  fdi=slp_fdi
                  filter_flags=slp_qc_flags
              END

          ENDCASE

          IF KEYWORD_SET(flags) OR suffix EQ 'internal2' THEN BEGIN
              prev_flags=WHERE(qc_flags(type,*) GT 0, pfcnt)
          ENDIF

          FREQUENT_VALUE_CHECK_SEASONAL, last_year, value, times, mdi, fdi, qc_flags, filter_flags, type, diagnostics, station_ids(st)

          points=WHERE(qc_flags(type,*) GT 0, bad_count)

          printstring="    Frequent Value Flags"
          IF KEYWORD_SET(flags) OR suffix EQ 'internal2' THEN BEGIN
              PRINT,printstring,type,':',bad_count-pfcnt,'  Old Flags:',pfcnt, FORMAT='(A-38,I1,A1,I8,A-15,I8)'
          ENDIF ELSE BEGIN
              PRINT,printstring,type,':',bad_count, FORMAT='(A-38,I1,A1,I8)'
          ENDELSE

          ;*************************************************
          ; Read out the filter flags
          ;*************************************************
          CASE type OF
              1: BEGIN
                  t_qc_flags=filter_flags
              END
              2: BEGIN
                  dp_qc_flags=filter_flags
              END
              3: BEGIN
                  slp_qc_flags=filter_flags
              END
          ENDCASE
          IF KEYWORD_SET(mask) THEN BEGIN              

              ;*************************************************
              ; If masking specified, then do it
              ;*************************************************

              IF bad_count GE 1 THEN BEGIN

                  MATCH, h_times(points), full_h_times, suba, subb
                  CASE type OF
                      1: BEGIN
                          flagged_values[0,points]=temperatures[points]
                          temperatures(points)=temperatures_fdi
                          full_temperatures(subb)=temperatures_fdi
                          flagged_values[1,points]=dewpoints[points]
                          dewpoints(points)=dewpoints_fdi  
                          full_dewpoints(subb)=dewpoints_fdi
                                ; to ensure that filter flags for both
                                ; variables are set
                          t_qc_flags(points)=1
                          dp_qc_flags(points)=1
                      END
                      2: BEGIN
                          flagged_values[0,points]=temperatures[points]
                          temperatures(points)=temperatures_fdi
                          full_temperatures(subb)=temperatures_fdi
                          flagged_values[1,points]=dewpoints[points]
                          dewpoints(points)=dewpoints_fdi  
                          full_dewpoints(subb)=dewpoints_fdi
                                ; to ensure that filter flags for both
                                ; variables are set
                          t_qc_flags(points)=1
                          dp_qc_flags(points)=1
                      END
                      3: BEGIN
                          flagged_values[2,points]=slp[points]
                          slp(points)=slp_fdi
                          full_slp(subb)=slp_fdi
                     END
                  ENDCASE
              ENDIF
          ENDIF
      ENDFOR
 
      endtime=systime(1)
      IF KEYWORD_SET(diagnostics) THEN BEGIN
          printstring="Frequent Value took "
          PRINT, printstring, endtime-starttime, FORMAT='(A-40,F15.5)'   
      ENDIF
     history=history+STRING(SYSTIME())+' Frequent Value Check (Seasonal) \n '
  ENDIF ELSE PRINT, "Frequent Value not run"
  

  ;************************************************************************************
  ;      CHECK FOR A SENSIBLE DIURNAL CYCLE
  ;************************************************************************************
  ;
  ; Check if the diurnal cycle in temperature is stable and remove data that seems to be wrong local
  ; time. This was found to be an issue in HadCRUH and will pervade all variables so simply remove
  ; everything if we think time of obs is systematically wrong.
  ;
  ; Only call if the station is not a polar station (defined as <67 degrees latitude) as we
  ; wouldn't necessarilly expect a diurnal cycle in lats where there isn't a diurnal cycle in
  ; radiation.

  IF KEYWORD_SET(all_tests) or KEYWORD_SET(diurnal)  THEN BEGIN
      PRINT,""
      ;************************************************************************************
      ; Only call if the station is not a polar station (defined as <67 degrees latitude) as we
      ; wouldn't necessarilly expect a diurnal cycle in lats where there isn't a diurnal cycle in
      ; radiation.
      ;************************************************************************************

      IF ABS(station_lats(st)) LE 60. THEN BEGIN 
          ; Commented out when 5K range added - then back in because
          ;                                     too much was being removed >60 deg

          starttime=systime(1) 

          IF KEYWORD_SET(flags) OR suffix EQ 'internal2' THEN BEGIN
              prev_flags=WHERE(qc_flags(4,*) GT 0, pfcnt)
          ENDIF

          CHECK_DIURNAL, temperatures, temperatures_mdi,temperatures_fdi, times, diurnal_qc_flags, t_qc_flags, $
                   dynamic_diurnal=dynamic_diurnal, diagnostics, station_ids(st)
          

          points=WHERE(diurnal_qc_flags GT 0, bad_count)

          printstring="           Diurnal Flags :"
          IF KEYWORD_SET(flags) OR suffix EQ 'internal2' THEN BEGIN
              PRINT,printstring,bad_count,'  Old Flags:',pfcnt, FORMAT='(A-40,I8,A-15,I8)'
          ENDIF ELSE BEGIN
              PRINT,printstring,bad_count, FORMAT='(A-40,I8)'
          ENDELSE


          IF bad_count GE 1 THEN BEGIN 

              ;*******************************
              ; Only new flags set to 1
              ;   old flags not wipes
              ;*******************************
              qc_flags(4,points)=1

              ;*******************************
              ; Set the filtering flags
              ;*******************************
              t_qc_flags(points)=1
              dp_qc_flags(points)=1
              slp_qc_flags(points)=1
              w_qc_flags(points)=1
              
              IF KEYWORD_SET(mask) THEN BEGIN 

                  ;*************************************************
                  ; If masking specified, then do it
                  ;*************************************************

                 flagged_values[0,points]=temperatures(points)
                 flagged_values[1,points]=dewpoints(points)
                 flagged_values[2,points]=slp(points)
                 flagged_values[3,points]=total_cloud_cover(points)
                 flagged_values[4,points]=low_cloud_cover(points)
                 flagged_values[5,points]=mid_cloud_cover(points)
                 flagged_values[6,points]=high_cloud_cover(points)
                 flagged_values[7,points]=windspeeds(points)
                 flagged_values[8,points]=winddirs(points)       

                 temperatures(points)=temperatures_fdi  
                 dewpoints(points)=dewpoints_fdi  
                 total_cloud_cover(points)=total_cloud_cover_fdi
                 low_cloud_cover(points)=low_cloud_cover_fdi
                 mid_cloud_cover(points)=mid_cloud_cover_fdi
                 high_cloud_cover(points)=high_cloud_cover_fdi
                 cloud_base(points)=cloud_base_fdi ; mdi on purpose - no QC done on cloudbase - just for cloud checks later 17/11/11 RJHD
                 windspeeds(points)=windspeeds_fdi
                 winddirs(points)=winddirs_fdi
                 slp(points)=slp_fdi    
                 MATCH, h_times(points), full_h_times, suba, subb
                 full_temperatures(subb)=temperatures_fdi
                 full_dewpoints(subb)=dewpoints_fdi
                 full_slp(subb)=slp_fdi
                 full_windspeeds(subb)=windspeeds_fdi
                 full_winddirs(subb)=winddirs_fdi
              ENDIF
          ENDIF
          endtime=systime(1)
          IF KEYWORD_SET(diagnostics) THEN BEGIN
              printstring="Check Diurnal took "
              PRINT, printstring, endtime-starttime, FORMAT='(A-40,F15.5)'   
          ENDIF
          history=history+STRING(SYSTIME())+' Diurnal Cycle Check \n '
      ENDIF ELSE BEGIN
          history=history+STRING(SYSTIME())+' Diurnal Cycle Check not run as '+STRCOMPRESS(ABS(station_lats(st)),/remove_all)+'>60\n '
          PRINT, "Check Diurnal not run as ",ABS(station_lats(st)),">60"
      ENDELSE
  ENDIF

  ;************************************************************************************
  ;      CHECK FOR DISTRIBUTIONAL GAPS
  ;************************************************************************************
  ;
  ; Check for distributional outliers in those variables we may expect to be iid.
  ;
  IF KEYWORD_SET(all_tests) or KEYWORD_SET(gap) THEN BEGIN
      PRINT,""
      starttime=systime(1) 
      PRINT, '0 - Temperature, 1 - Dewpoints, 2 - SLP'

      FOR type=0,2 DO BEGIN

          CASE type OF
              0: BEGIN
                  value=temperatures
                  full_values=full_temperatures
                  mdi=temperatures_mdi
                  fdi=temperatures_fdi
                  column=5
                  filter_flags=t_qc_flags
              END
              1: BEGIN
                  value=dewpoints
                  full_values=full_dewpoints
                  mdi=dewpoints_mdi
                  fdi=dewpoints_fdi
                  column=6
                  filter_flags=dp_qc_flags
              END
              2: BEGIN
                  value=slp
                  full_values=full_slp
                  mdi=slp_mdi
                  fdi=slp_fdi
                  column=7
                  filter_flags=slp_qc_flags
              END

          ENDCASE
          
          IF KEYWORD_SET(flags) OR suffix EQ 'internal2' THEN BEGIN
              prev_flags=WHERE(qc_flags(column,*) GT 0, pfcnt)
          ENDIF

          IF type NE 2 THEN BEGIN
              GAP_CHECK, last_year, times, h_times, value, full_times, full_h_times, full_values, mdi, fdi, type, qc_flags_gap=qc_flags_gap,$
                filter_flags, diagnostics, station_ids(st)
          ENDIF ELSE BEGIN
              GAP_CHECK, last_year, times, h_times, value, full_times, full_h_times, full_values, mdi, fdi, type, qc_flags_gap=qc_flags_gap,$
                filter_flags, diagnostics, station_ids(st), windsp=windspeeds, full_windspeeds=full_windspeeds
          ENDELSE
           
          points=WHERE(qc_flags_gap GT 0, bad_count)


          ;*******************************
          ; Set the filtering flags
          ;*******************************
          IF bad_count GE 1 THEN BEGIN
              ; Only new flags set to 1
              ;*******************************
              ;   old flags not wipes
              ;*******************************
              qc_flags(column,points)=qc_flags_gap(points)

              CASE type OF
                  0: BEGIN
                      t_qc_flags(points)=1
                  END
                  1: BEGIN
                      dp_qc_flags(points)=1
                  END
                  2: BEGIN
                      slp_qc_flags(points)=1
                  END
              ENDCASE
          ENDIF
                   
          printstring="         Gap Check Flags"
          IF KEYWORD_SET(flags) OR suffix EQ 'internal2' THEN BEGIN
              PRINT,printstring,type,':',bad_count,'  Old Flags:',pfcnt, FORMAT='(A-38,I1,A1,I8,A-15,I8)'
          ENDIF ELSE BEGIN
              PRINT,printstring,type,':',bad_count, FORMAT='(A-38,I1,A1,I8)'
          ENDELSE
          
          IF bad_count GE 1 THEN BEGIN 
              
              IF KEYWORD_SET(mask) THEN BEGIN              
                  ;*************************************************
                  ; There were gaps so go through and assign as missing for 
                  ; all the variables.
                  ; If masking specified, then do it (also to
                  ; tentative flags) 
                  ;*************************************************
                  CASE type OF
                      0: BEGIN
                          flagged_values[0,points]=temperatures[points]
                          temperatures(points)=temperatures_fdi
                          full_temperatures(subb)=temperatures_fdi
                      END
                      1: BEGIN
                          flagged_values[1,points]=dewpoints[points]
                          dewpoints(points)=dewpoints_fdi
                          full_dewpoints(subb)=dewpoints_fdi
                      END
                      2: BEGIN
                          flagged_values[2,points]=slp[points]
                          slp(points)=slp_fdi
                          full_slp(subb)=slp_fdi
                      END
                  ENDCASE
               ENDIF
          ENDIF
          
      ENDFOR
      endtime=systime(1) 
      IF KEYWORD_SET(diagnostics) THEN BEGIN
          printstring="Gap Check took "
          PRINT, printstring, endtime-starttime, FORMAT='(A-40,F15.5)'   
      ENDIF
      history=history+STRING(SYSTIME())+' Distributional Gap Check \n '      
  ENDIF ELSE PRINT, "Gap Check not run"

  ;************************************************************************************
  ;      CHECK FOR EXCEEDANCE OF WORLD CLIMATE RECORDS
  ;************************************************************************************
  ;
  ; Check against documented world records or logical limits where appropriate. Does not apply
  ; to all variables.
  ;
  IF KEYWORD_SET(all_tests) or KEYWORD_SET(records) THEN BEGIN
      PRINT,""
      starttime=systime(1) 

      this_wmo_id=LONG(STRMID(station_ids(st),0,5))

      IF KEYWORD_SET(flags) OR suffix EQ 'internal2' THEN BEGIN
          prev_flags=WHERE(qc_flags(8:11,*) GT 0, pfcnt)
      ENDIF
      
      ; Assign issues to the full array ids where appropriate
      CHECK_RECORDS, temperatures, temperatures_fdi, temperatures_mdi, dewpoints, dewpoints_fdi, dewpoints_mdi, $ 
        windspeeds, windspeeds_fdi, windspeeds_mdi, slp, slp_mdi, slp_fdi, qc_flags, filter_flags, this_wmo_id, diagnostics

      points=WHERE(qc_flags(8:11,*) GT 0, bad_count)

      printstring="      Record Check Flags :"
      IF KEYWORD_SET(flags) OR suffix EQ 'internal2' THEN BEGIN
          PRINT,printstring,bad_count-pfcnt,'  Old Flags:',pfcnt, FORMAT='(A-40,I8,A-15,I8)'
      ENDIF ELSE BEGIN
          PRINT,printstring,bad_count, FORMAT='(A-40,I8)'
      ENDELSE


      IF KEYWORD_SET(mask) THEN BEGIN
          points=WHERE(qc_flags(8,*) GT 0, bad_count)

          IF bad_count GE 1 THEN BEGIN
             t_qc_flags(points)=1

             flagged_values[0,points]=temperatures[points]
             temperatures(points)=temperatures_fdi
             MATCH, h_times(points), full_h_times, suba, subb
             full_temperatures(subb)=temperatures_fdi

             ; if temperatures flagged, also flag dewpoints
             dp_qc_flags(points)=1
             
             flagged_values[1,points]=dewpoints[points]
             dewpoints(points)=dewpoints_fdi
             MATCH, h_times(points), full_h_times, suba, subb
             full_dewpoints(subb)=dewpoints_fdi
          ENDIF

          points=WHERE(qc_flags(9,*) GT 0, bad_count)

          IF bad_count GE 1 THEN BEGIN
              dp_qc_flags(points)=1

              flagged_values[1,points]=dewpoints[points]
              dewpoints(points)=dewpoints_fdi
              MATCH, h_times(points), full_h_times, suba, subb
              full_dewpoints(subb)=dewpoints_fdi
          ENDIF

          points=WHERE(qc_flags(10,*) GT 0, bad_count)

          IF bad_count GE 1 THEN BEGIN
              w_qc_flags(points)=1

              flagged_values[7,points]=windspeeds[points]
              flagged_values[8,points]=winddirs[points]
              winddirs(points)=winddirs_fdi
              windspeeds(points)=windspeeds_fdi
              MATCH, h_times(points), full_h_times, suba, subb
              full_windspeeds(subb)=windspeeds_fdi
              full_winddirs(subb)=winddirs_fdi
          ENDIF

          points=WHERE(qc_flags(11,*) GT 0, bad_count)

          IF bad_count GE 1 THEN BEGIN
              slp_qc_flags(points)=1

              flagged_values[2,points]=slp[points]
              slp(points)=slp_fdi
              MATCH, h_times(points), full_h_times, suba, subb
              full_slp(subb)=slp_fdi
          ENDIF

      ENDIF
      endtime=systime(1) 
      IF KEYWORD_SET(diagnostics) THEN BEGIN
          printstring="Record Check took "
          PRINT, printstring, endtime-starttime, FORMAT='(A-40,F15.5)'   
      ENDIF
      history=history+STRING(SYSTIME())+' Record Check \n '      
  ENDIF ELSE PRINT, "Record Check not run"

  ;************************************************************************************
  ;      CHECK FOR STREAKS
  ;************************************************************************************
  ;
  ; Check for streaks in those variables where it is appropriate to do so.
  ;
  IF KEYWORD_SET(all_tests) or KEYWORD_SET(streak) THEN BEGIN
      PRINT,""
      starttime=systime(1) 
      PRINT, '0 - Temperature, 1 - Dewpoints, 2,- Windspeeds, 3 - SLP'
      
      FOR type=0,3 DO BEGIN

          CASE type OF
              0: BEGIN
                  values=temperatures
                  full_values=full_temperatures
                  mdi=temperatures_mdi
                  fdi=temperatures_fdi
                  filter_flags=t_qc_flags
              END
              1: BEGIN
                  values=dewpoints
                  full_values=full_dewpoints
                  mdi=dewpoints_mdi
                  fdi=dewpoints_fdi
                  filter_flags=dp_qc_flags
              END
              2: BEGIN
                  values=windspeeds
                  full_values=full_windspeeds
                  mdi=windspeeds_mdi
                  fdi=windspeeds_fdi
                  filter_flags=w_qc_flags
              END
              3: BEGIN
                  values=slp
                  full_values=full_slp
                  mdi=slp_mdi
                  fdi=slp_fdi
                  filter_flags=slp_qc_flags
              END

          ENDCASE

          
          STREAK_CHECK, last_year, times, h_times, values, mdi, fdi, type, qc_flags, filter_flags, $
                        full_times, full_h_times, full_values, diagnostics, station_ids(st),flagged_values, mask=mask, oldflags=oldflags
          ; Masking done in subroutine
          ; flags such that only bad points are flagged, old ones not overwritten

          ;*************************************************
          ; Read out the filter flags
          ;  And read the masked data back in when appropriate
          ;*************************************************
          CASE type OF
              0: BEGIN
                  t_qc_flags=filter_flags
                  IF KEYWORD_SET(mask) THEN temperatures=values
              END
              1: BEGIN
                  dp_qc_flags=filter_flags
                  IF KEYWORD_SET(mask) THEN dewpoints=values
              END
              2: BEGIN
                  w_qc_flags=filter_flags
                  IF KEYWORD_SET(mask) THEN windspeeds=values
              END
              3: BEGIN
                  slp_qc_flags=filter_flags
                  IF KEYWORD_SET(mask) THEN slp=values
              END
          ENDCASE
      ENDFOR
      endtime=systime(1) 
      IF KEYWORD_SET(diagnostics) THEN BEGIN
          printstring="Streak Check took "
          PRINT, printstring, endtime-starttime, FORMAT='(A-40,F15.5)'   
      ENDIF
      history=history+STRING(SYSTIME())+' Streak Check \n '      
  ENDIF ELSE PRINT, "Streak Check not run"
  

  ;************************************************************************************
  ;      CHECK FOR CLIMATOLOGICAL OUTLIERS
  ;************************************************************************************
  ;
  ;
  ; Check for climatological outliers for appropriate variables
  ;
  IF KEYWORD_SET(all_tests) or KEYWORD_SET(climate) THEN BEGIN
      PRINT,""
      starttime=systime(1) 
      PRINT, '0 - Temperature, 1 - Dewpoints'
      
      FOR type=0,1 DO BEGIN

          CASE type OF
              0: BEGIN
                  values=temperatures
                  full_values=full_temperatures
                  mdi=temperatures_mdi
                  fdi=temperatures_fdi
                  columns=24
                  filter_flags=t_qc_flags
              END
              1: BEGIN
                  values=dewpoints
                  full_values=full_dewpoints
                  mdi=dewpoints_mdi
                  fdi=dewpoints_fdi
                  columns=25
                  filter_flags=dp_qc_flags
              END
          ENDCASE
          CLIMATOLOGICAL_CHECK, last_year,times, full_times, h_times, full_h_times, values, $
            full_values, mdi, fdi, columns, qc_flags, filter_flags,  $
            diagnostics, station_ids(st),flagged_values,mask=mask, oldflags=oldflags
          ; Masking done in subroutine

          ;*************************************************
          ; Read out the filter flags
          ;  And read the masked data back in when appropriate
          ;*************************************************
          CASE type OF
              0: BEGIN
                  t_qc_flags=filter_flags
                  IF KEYWORD_SET(mask) THEN temperatures=values
              END
              1: BEGIN
                  dp_qc_flags=filter_flags
                  IF KEYWORD_SET(mask) THEN dewpoints=values
              END
          ENDCASE
      ENDFOR
      endtime=systime(1) 
      IF KEYWORD_SET(diagnostics) THEN BEGIN
          printstring="Climatological Check took "
          PRINT, printstring, endtime-starttime, FORMAT='(A-40,F15.5)'   
      ENDIF
      history=history+STRING(SYSTIME())+' Climatological Outlier Check \n '      
  ENDIF ELSE PRINT, "Climatological Check not run"
  ; Assign issues to the full arrays where appropriate

  ;************************************************************************************
  ;      CHECK FOR SPIKES
  ;************************************************************************************
  ;
  ; Check for spikes
  ;
  IF KEYWORD_SET(all_tests) or KEYWORD_SET(spike) THEN BEGIN
      PRINT,""
      starttime=systime(1) 
      PRINT, '0 - Temperature, 1 - Dewpoints, 2 - SLP'
      
      FOR type=0,2 DO BEGIN

          CASE type OF
              0: BEGIN
                  values=temperatures
                  full_values=full_temperatures
                  mdi=temperatures_mdi
                  fdi=temperatures_fdi
                  columns=27
                  filter_flags=t_qc_flags
              END
              1: BEGIN
                  values=dewpoints
                  full_values=full_dewpoints
                  mdi=dewpoints_mdi
                  fdi=dewpoints_fdi
                  columns=28
                  filter_flags=dp_qc_flags
              END
              2: BEGIN
                  values=slp
                  full_values=full_slp
                  mdi=slp_mdi
                  fdi=slp_fdi
                  columns=29
                  filter_flags=slp_qc_flags
              END
          ENDCASE

          IF KEYWORD_SET(flags) OR suffix EQ 'internal2' THEN BEGIN
              prev_flags=WHERE(qc_flags(columns,*) GT 0, pfcnt)
          ENDIF

          SPIKE_CHECK, last_year, times, full_times, values, mdi, fdi, columns, qc_flags, filter_flags, $
             diagnostics, station_ids(st)
          ; Assign issues to the full arrays where appropriate

           
          points=WHERE(qc_flags(columns,*) GT 0, bad_count)

          IF KEYWORD_SET(flags) OR suffix EQ 'internal2' THEN BEGIN
              printstring="       Spike Check Flags"
              PRINT,printstring,type,':',bad_count-pfcnt,'  Old Flags:',pfcnt, FORMAT='(A-38,I1,A1,I8,A-15,I8)'
          ENDIF ELSE BEGIN
              printstring="       Spike Check Flags"
              PRINT,printstring,type,':', bad_count, FORMAT='(A-38,I1,A1,I8)'
          ENDELSE
          

          ;*************************************************
          ; Read out the filter flags
          ;*************************************************
          CASE type OF
              0: BEGIN
                  t_qc_flags=filter_flags
              END
              1: BEGIN
                  dp_qc_flags=filter_flags
              END
              2: BEGIN
                  slp_qc_flags=filter_flags
              END
          ENDCASE

          IF bad_count GE 1 THEN BEGIN
              IF KEYWORD_SET(mask) THEN BEGIN
                  MATCH, h_times(points), full_h_times, suba, subb
                  CASE type OF
                      0: BEGIN
                          flagged_values[0,points]=temperatures[points]
                          temperatures(points)=temperatures_fdi
                          full_temperatures(subb)=temperatures_fdi
                      END
                      1: BEGIN
                          flagged_values[1,points]=dewpoints[points]
                          dewpoints(points)=dewpoints_fdi  
                          full_dewpoints(subb)=dewpoints_fdi
                      END
                      2: BEGIN
                          flagged_values[2,points]=slp[points]
                          slp(points)=slp_fdi
                          full_slp(subb)=slp_fdi
                      END
                  ENDCASE
              ENDIF
          ENDIF
      ENDFOR
      endtime=systime(1) 
      IF KEYWORD_SET(diagnostics) THEN BEGIN
          printstring="Spike Check took "
          PRINT, printstring, endtime-starttime, FORMAT='(A-40,F15.5)'   
      ENDIF
      history=history+STRING(SYSTIME())+' Spike Check \n '      
  ENDIF ELSE PRINT, "Spike Check not run"



  ;************************************************************************************
  ;      CROSS CHECK TEMPERATURE AND DEWPOINT MEASUREMENTS
  ;************************************************************************************
  ;
  ; Cross check temperature and dewpoint measurements
  ;
  IF KEYWORD_SET(all_tests) or KEYWORD_SET(ttw) THEN BEGIN
      PRINT,""

      starttime=systime(1) 

      CROSS_CHECK_T_TW, last_year, times, temperatures, temperatures_mdi, temperatures_fdi,$
        dewpoints, dewpoints_mdi, dewpoints_fdi, qc_flags, t_qc_flags, dp_qc_flags, $
        past_sigwx1, precip1,cloud_base, flagged_values, mask=mask,$
        diagnostics, station_ids(st), oldflags=oldflags



      ; Assign issues to the full arrays where appropriate
      IF KEYWORD_SET(mask) THEN BEGIN
          points=WHERE(qc_flags(30,*) GT 0, bad_count)
          IF bad_count GE 1 THEN BEGIN
              MATCH, h_times(points), full_h_times, suba, subb
              full_dewpoints(subb)=dewpoints_fdi
              dp_qc_flags(points)=1
          ENDIF
          points=WHERE(qc_flags(31,*) GT 0, bad_count)
          IF bad_count GE 1 THEN BEGIN
              MATCH, h_times(points), full_h_times, suba, subb
              full_dewpoints(subb)=dewpoints_fdi
              dp_qc_flags(points)=1
          ENDIF
          points=WHERE(qc_flags(32,*) GT 0, bad_count)
          IF bad_count GE 1 THEN BEGIN
              MATCH, h_times(points), full_h_times, suba, subb
              full_dewpoints(subb)=dewpoints_fdi	       
              dp_qc_flags(points)=1
          ENDIF
      ENDIF
      endtime=systime(1) 
      IF KEYWORD_SET(diagnostics) THEN BEGIN
          printstring="T & TW Cross Check took "
          PRINT, printstring, endtime-starttime, FORMAT='(A-40,F15.5)'   
      ENDIF

      history=history+STRING(SYSTIME())+' Temperature-Dewpoint Cross Check \n '      
  ENDIF ELSE PRINT, "T & TW Cross Check not run"
  
  ;************************************************************************************
  ;      CROSS CHECK CLOUD DATA FOR CONSISTENCY
  ;************************************************************************************
  ;
  ; Cross check on cloud data
  ;
  IF KEYWORD_SET(all_tests) or KEYWORD_SET(clouds) THEN BEGIN
      PRINT,""

      starttime=systime(1) 

      CROSS_CHECK_CLOUDS, times, total_cloud_cover, total_cloud_cover_mdi, total_cloud_cover_fdi, low_cloud_cover,$
                      low_cloud_cover_mdi, low_cloud_cover_fdi, mid_cloud_cover, mid_cloud_cover_mdi, mid_cloud_cover_fdi, $
                      high_cloud_cover, high_cloud_cover_mdi, high_cloud_cover_fdi, cloud_base,$
                      cloud_base_mdi, qc_flags, flagged_values, mask=mask, diagnostics

      endtime=systime(1) 
      IF KEYWORD_SET(diagnostics) THEN BEGIN
          printstring="Cloud Cross Check took "
          PRINT, printstring, endtime-starttime, FORMAT='(A-40,F15.5)'   
      ENDIF

  
      history=history+STRING(SYSTIME())+' Cloud Cross Check \n '      
  ENDIF ELSE PRINT, "Cloud Cross Check not run"
  
  ;************************************************************************************
  ;      CHECK FOR YEARS WITH EXCESS VARIANCE
  ;************************************************************************************
  ;
  ; Check for years where variance is significantly different to other years - using IQR in the standard
  ; deviation in the anomalised (from the median) dataset
  ;
  IF KEYWORD_SET(all_tests) or KEYWORD_SET(variance) THEN BEGIN
      PRINT,""
      starttime=systime(1) 

      PRINT, '0 - Temperature, 1 - Dewpoints, 2 - SLP'
      
      FOR type=0,2 DO BEGIN

          CASE type OF
              0: BEGIN
                  values=temperatures
                  full_values=full_temperatures
                  mdi=temperatures_mdi
                  fdi=temperatures_fdi
                  columns=58
                  filter_flags=t_qc_flags
              END
              1: BEGIN
                  values=dewpoints
                  full_values=full_dewpoints
                  mdi=dewpoints_mdi
                  fdi=dewpoints_fdi
                  columns=59
                  filter_flags=dp_qc_flags
              END
              2: BEGIN
                  values=slp
                  full_values=full_slp
                  mdi=slp_mdi
                  fdi=slp_fdi
                  columns=60
                  filter_flags=slp_qc_flags
              END
          ENDCASE

          IF KEYWORD_SET(flags) OR suffix EQ 'internal2' THEN BEGIN
              prev_flags=WHERE(qc_flags(columns,*) GT 0, pfcnt)
          ENDIF
  
          IF type NE 2 THEN BEGIN
              VARIANCE_CHECK, last_year, times, full_times, h_times, full_h_times, values, full_values, mdi,fdi,$
                qc_flags, filter_flags, columns, flagged_values, mask=mask, diagnostics, station_ids(st)
          ENDIF ELSE BEGIN
              VARIANCE_CHECK, last_year,times, full_times, h_times, full_h_times, values, full_values, mdi,fdi,$
                qc_flags, filter_flags, columns, flagged_values, mask=mask, diagnostics, station_ids(st), windsp=windspeeds, full_windspeeds=full_windspeeds
          ENDELSE
          
          points=WHERE(qc_flags(columns,*) GT 0, bad_count)

          IF KEYWORD_SET(flags) OR suffix EQ 'internal2' THEN BEGIN
              printstring="    Variance Check Flags"
              PRINT,printstring,type,':',bad_count-pfcnt,'  Old Flags:',pfcnt, FORMAT='(A-38,I1,A1,I8,A-15,I8)'
          ENDIF ELSE BEGIN
              printstring="    Variance Check Flags"
              PRINT,printstring,type,':',bad_count, FORMAT='(A-38,I1,A1,I8)'
          ENDELSE

          ;*************************************************
          ; Read out the filter flags
          ;  And read the masked data back in when appropriate
          ;*************************************************
          CASE type OF
              0: BEGIN
                  t_qc_flags=filter_flags
                  IF KEYWORD_SET(mask) THEN temperatures=values
              END
              1: BEGIN
                  dp_qc_flags=filter_flags
                  IF KEYWORD_SET(mask) THEN dewpoints=values
              END
              2: BEGIN
                  slp_qc_flags=filter_flags
                  IF KEYWORD_SET(mask) THEN slp=values
               END
           
          ENDCASE
          
      ENDFOR
      endtime=systime(1) 
      
      IF KEYWORD_SET(diagnostics) THEN BEGIN
          printstring="Variance Check took "
          PRINT, printstring, endtime-starttime, FORMAT='(A-40,F15.5)'   
      ENDIF
      history=history+STRING(SYSTIME())+' Variance Check \n '      
      
  ENDIF ELSE PRINT, "Variance not run"
  PRINT,""
  
  ;************************************************************************************
  ;     START FILE OUTPUT
  ;************************************************************************************
  ; Output modified data to a new directory. Preserve the variable attributes and add 
  ; in the location info as a header field.
  ;
  ; Create as wilma (don't ask!)

   
  netcdf_outfile=outputdir+station_ids(st)+'_'+suffix+'.nc'
  
  wilma=NCDF_CREATE(netcdf_outfile,/clobber)
  
  tid=NCDF_DIMDEF(wilma,'time',nt)
  testid=NCDF_DIMDEF(wilma, 'test', 61)
  charid=NCDF_DIMDEF(wilma,'Character',12)
  flagid=NCDF_DIMDEF(wilma,'flagged',9)
  changesid=NCDF_DIMDEF(wilma,'changepoints',N_ELEMENTS(STRING(changenotes)))
  chptsid=NCDF_DIMDEF(wilma,'changes',4)
  
  chgntlen=STRLEN(changenotes)
  lcharid=NCDF_DIMDEF(wilma,'LongCharacter',MAX(chgntlen))
  
  timevar=NCDF_VARDEF(wilma,'time',[tid],/DOUBLE)
  stationsvar=NCDF_VARDEF(wilma,'input_station_id',[charid,tid],/CHAR)
  tempsvar=NCDF_VARDEF(wilma,'temperatures',[tid],/FLOAT)
  dewsvar=NCDF_VARDEF(wilma,'dewpoints',[tid],/FLOAT)
  tcvar=NCDF_VARDEF(wilma,'total_cloud_cover',[tid],/SHORT)
  lcvar=NCDF_VARDEF(wilma,'low_cloud_cover',[tid],/SHORT)
  mcvar=NCDF_VARDEF(wilma,'mid_cloud_cover',[tid],/SHORT)
  hcvar=NCDF_VARDEF(wilma,'high_cloud_cover',[tid],/SHORT)
  cbvar=NCDF_VARDEF(wilma,'cloud_base',[tid],/SHORT)
  wsvar=NCDF_VARDEF(wilma,'windspeeds',[tid],/FLOAT) 
  wdvar=NCDF_VARDEF(wilma,'winddirs',[tid],/SHORT)
  slpvar=NCDF_VARDEF(wilma,'slp',[tid],/FLOAT)
  qcvar=NCDF_VARDEF(wilma,'qc_flags',[testid,tid],/SHORT)
  sigwxvar=NCDF_VARDEF(wilma,'past_sigwx1',[tid],/SHORT)
  precipvar=NCDF_VARDEF(wilma,'precip1_depth',[tid],/FLOAT)
  flagvar=NCDF_VARDEF(wilma,'flagged_values',[flagid,tid],/FLOAT)
  changevar=NCDF_VARDEF(wilma,'change_points',[chptsid,changesid],/FLOAT)
  chntsvar=NCDF_VARDEF(wilma,'change_notes',[lcharid,changesid],/CHAR)
  

  NCDF_ATTPUT,wilma,'time','long_name','time',/CHAR
  NCDF_ATTPUT,wilma,'time','standard_name','time',/CHAR
  NCDF_ATTPUT,wilma,'time','units','days since 1973-01-01 00:00:00',/CHAR
  NCDF_ATTPUT,wilma,'time','axis','T',/CHAR
  NCDF_ATTPUT,wilma,'time','calendar','gregorian',/CHAR
  NCDF_ATTPUT,wilma,'time','valid_min',0.,/FLOAT
  NCDF_ATTPUT,wilma,'input_station_id','long_name','Primary source for timetep (may be multiple sources for composite stations). USAF - WBAN from ISD source',/CHAR
  NCDF_ATTPUT,wilma,'input_station_id','cf_role','timeseries id',/CHAR
  NCDF_ATTPUT,wilma,'temperatures','long_name','Dry bulb temperatures nearest to reporting hour',/CHAR
  NCDF_ATTPUT,wilma,'temperatures','standard_name','surface_temperature',/CHAR
  NCDF_ATTPUT,wilma,'temperatures','units','degreeC',/CHAR
  NCDF_ATTPUT,wilma,'temperatures','coordinates','time',/CHAR
  valid=WHERE(((temperatures NE temperatures_mdi) AND (temperatures NE temperatures_fdi)), tc)
  IF tc GE 1 THEN BEGIN
    min_t=MIN(temperatures(valid))
    max_t=MAX(temperatures(valid))
    NCDF_ATTPUT,wilma,'temperatures','valid_min',min_t(0),/FLOAT
    NCDF_ATTPUT,wilma,'temperatures','valid_max',max_t(0),/FLOAT
  ENDIF
  NCDF_ATTPUT,wilma,'temperatures','missing_value',temperatures_mdi,/FLOAT
  NCDF_ATTPUT,wilma,'temperatures','flagged_value',temperatures_fdi,/FLOAT
  NCDF_ATTPUT,wilma,'dewpoints','long_name','Dew point temperatures nearest to reporting hour',/CHAR
  NCDF_ATTPUT,wilma,'dewpoints','standard_name','dew_point_temperature',/CHAR
  NCDF_ATTPUT,wilma,'dewpoints','units','degreeC',/CHAR
  NCDF_ATTPUT,wilma,'dewpoints','coordinates','time',/CHAR
  valid=WHERE(((dewpoints NE dewpoints_mdi) AND (dewpoints NE dewpoints_fdi)),tc)
  IF tc GE 1 THEN BEGIN
    min_t=MIN(dewpoints(valid))
    max_t=MAX(dewpoints(valid))
    NCDF_ATTPUT,wilma,'dewpoints','valid_min',min_t(0),/FLOAT
    NCDF_ATTPUT,wilma,'dewpoints','valid_max',max_t(0),/FLOAT
  ENDIF
  NCDF_ATTPUT,wilma,'dewpoints','missing_value',dewpoints_mdi,/FLOAT
  NCDF_ATTPUT,wilma,'dewpoints','flagged_value',dewpoints_fdi,/FLOAT
  NCDF_ATTPUT,wilma,'total_cloud_cover','long_name','Total cloud cover derived in priority order GA, GF, GD - see ISD documentation.  Units are Oktas',/CHAR
  NCDF_ATTPUT,wilma,'total_cloud_cover','standard_name','cloud_area_fraction',/CHAR
  NCDF_ATTPUT,wilma,'total_cloud_cover','units','1',/CHAR
  NCDF_ATTPUT,wilma,'total_cloud_cover','valid_min',0,/SHORT
  NCDF_ATTPUT,wilma,'total_cloud_cover','valid_max',8,/SHORT
  NCDF_ATTPUT,wilma,'total_cloud_cover','missing_value',total_cloud_cover_mdi,/SHORT
  NCDF_ATTPUT,wilma,'total_cloud_cover','flagged_value',total_cloud_cover_fdi,/SHORT
  NCDF_ATTPUT,wilma,'total_cloud_cover','coordinates','time',/CHAR
  NCDF_ATTPUT,wilma,'low_cloud_cover','long_name','low cloud cover derived in priority order GA, GF, GD - see ISD documentation.  Units are Oktas',/CHAR
  NCDF_ATTPUT,wilma,'low_cloud_cover','units','1',/CHAR
  NCDF_ATTPUT,wilma,'low_cloud_cover','valid_min',0,/SHORT
  NCDF_ATTPUT,wilma,'low_cloud_cover','valid_max',8,/SHORT
  NCDF_ATTPUT,wilma,'low_cloud_cover','missing_value',low_cloud_cover_mdi,/SHORT
  NCDF_ATTPUT,wilma,'low_cloud_cover','flagged_value',low_cloud_cover_fdi,/SHORT
  NCDF_ATTPUT,wilma,'low_cloud_cover','coordinates','time',/CHAR
  NCDF_ATTPUT,wilma,'mid_cloud_cover','long_name','mid cloud cover derived in priority order GA, GF, GD - see ISD documentation.  Units are Oktas',/CHAR
  NCDF_ATTPUT,wilma,'mid_cloud_cover','units','1',/CHAR
  NCDF_ATTPUT,wilma,'mid_cloud_cover','valid_min',0,/SHORT
  NCDF_ATTPUT,wilma,'mid_cloud_cover','valid_max',8,/SHORT
  NCDF_ATTPUT,wilma,'mid_cloud_cover','missing_value',mid_cloud_cover_mdi,/SHORT
  NCDF_ATTPUT,wilma,'mid_cloud_cover','flagged_value',mid_cloud_cover_fdi,/SHORT
  NCDF_ATTPUT,wilma,'mid_cloud_cover','coordinates','time',/CHAR
  NCDF_ATTPUT,wilma,'high_cloud_cover','long_name','high cloud cover derived in priority order GA, GF, GD - see ISD documentation.  Units are Oktas',/CHAR
  NCDF_ATTPUT,wilma,'high_cloud_cover','units','1',/CHAR
  NCDF_ATTPUT,wilma,'high_cloud_cover','valid_min',0,/SHORT
  NCDF_ATTPUT,wilma,'high_cloud_cover','valid_max',8,/SHORT
  NCDF_ATTPUT,wilma,'high_cloud_cover','missing_value',high_cloud_cover_mdi,/SHORT
  NCDF_ATTPUT,wilma,'high_cloud_cover','flagged_value',high_cloud_cover_fdi,/SHORT
  NCDF_ATTPUT,wilma,'high_cloud_cover','coordinates','time',/CHAR
  NCDF_ATTPUT,wilma,'cloud_base','long_name','Cloud base of lowest cloud layer nearest to reporting hour',/CHAR
  NCDF_ATTPUT,wilma,'cloud_base','standard_name','cloud_base_altitude',/CHAR
  NCDF_ATTPUT,wilma,'cloud_base','units','metres',/CHAR
  NCDF_ATTPUT,wilma,'cloud_base','coordinates','time',/CHAR
  valid=WHERE(cloud_base NE cloud_base_mdi,tc)
  IF tc GE 1 THEN BEGIN
    min_t=MIN(cloud_base(valid))
    max_t=MAX(cloud_base(valid))
    NCDF_ATTPUT,wilma,'cloud_base','valid_min',min_t(0),/FLOAT
    NCDF_ATTPUT,wilma,'cloud_base','valid_max',max_t(0),/FLOAT
  ENDIF
  NCDF_ATTPUT,wilma,'cloud_base','missing_value',cloud_base_mdi,/SHORT
  NCDF_ATTPUT,wilma,'windspeeds','long_name','Wind speed',/CHAR
  NCDF_ATTPUT,wilma,'windspeeds','standard_name','wind_speed',/CHAR
  NCDF_ATTPUT,wilma,'windspeeds','units','metres per second',/CHAR
  valid=WHERE(((windspeeds NE windspeeds_mdi) AND (windspeeds NE windspeeds_fdi)),tc)
  IF tc GE 1 THEN BEGIN
    min_t=MIN(windspeeds(valid))
    max_t=MAX(windspeeds(valid))
    NCDF_ATTPUT,wilma,'windspeeds','valid_min',min_t(0),/FLOAT
    NCDF_ATTPUT,wilma,'windspeeds','valid_max',max_t(0),/FLOAT
  ENDIF
  NCDF_ATTPUT,wilma,'windspeeds','missing_value',windspeeds_mdi,/FLOAT
  NCDF_ATTPUT,wilma,'windspeeds','flagged_value',windspeeds_fdi,/FLOAT
  NCDF_ATTPUT,wilma,'windspeeds','coordinates','time',/CHAR
  NCDF_ATTPUT,wilma,'winddirs','long_name','Wind Direction',/CHAR
  NCDF_ATTPUT,wilma,'winddirs','standard_name','wind_from_direction',/CHAR
  NCDF_ATTPUT,wilma,'winddirs','units','Degrees',/CHAR
  NCDF_ATTPUT,wilma,'winddirs','valid_min',0,/SHORT
  NCDF_ATTPUT,wilma,'winddirs','valid_max',360,/SHORT
  NCDF_ATTPUT,wilma,'winddirs','missing_value',winddirs_mdi,/SHORT
  NCDF_ATTPUT,wilma,'winddirs','flagged_value',winddirs_fdi,/SHORT
  NCDF_ATTPUT,wilma,'winddirs','coordinates','time',/CHAR
  NCDF_ATTPUT,wilma,'slp','long_name','Reported sea level pressure',/CHAR
  NCDF_ATTPUT,wilma,'slp','standard_name','air_pressure_at_sea_level',/CHAR
  NCDF_ATTPUT,wilma,'slp','units','hPa',/CHAR
  valid=WHERE(((slp NE slp_mdi) AND (slp NE slp_fdi)), tc)
  IF tc GE 1 THEN BEGIN
    min_t=MIN(slp(valid))
    max_t=MAX(slp(valid))
    NCDF_ATTPUT,wilma,'slp','valid_min',min_t(0),/FLOAT
    NCDF_ATTPUT,wilma,'slp','valid_max',max_t(0),/FLOAT
  ENDIF
  NCDF_ATTPUT,wilma,'slp','missing_value',slp_mdi,/FLOAT
  NCDF_ATTPUT,wilma,'slp','flagged_value',slp_fdi,/FLOAT
  NCDF_ATTPUT,wilma,'slp','coordinates','time',/CHAR
  NCDF_ATTPUT,wilma,'past_sigwx1','long_name','Station reports of past significant weather phenomena',/CHAR
  NCDF_ATTPUT,wilma,'past_sigwx1','units','1',/CHAR
  NCDF_ATTPUT,wilma,'past_sigwx1','valid_min',0s,/SHORT
  NCDF_ATTPUT,wilma,'past_sigwx1','valid_max',9s,/SHORT
  NCDF_ATTPUT,wilma,'past_sigwx1','missing_value',-999s,/SHORT
  NCDF_ATTPUT,wilma,'past_sigwx1','coordinates','time',/CHAR
  NCDF_ATTPUT,wilma,'precip1_depth','long_name','Depth of precipitation reported',/CHAR
  NCDF_ATTPUT,wilma,'precip1_depth','standard_name','lwe_thickness_of_precipitation_amount',/CHAR
  NCDF_ATTPUT,wilma,'precip1_depth','units','mm',/CHAR
  NCDF_ATTPUT,wilma,'precip1_depth','valid_min',0,/SHORT
  NCDF_ATTPUT,wilma,'precip1_depth','valid_max',9,/SHORT
  NCDF_ATTPUT,wilma,'precip1_depth','missing_value',-1.e+30,/FLOAT
  NCDF_ATTPUT,wilma,'precip1_depth','coordinates','time',/CHAR
  NCDF_ATTPUT,wilma,'qc_flags','long_name','QC status for individual obs, #tests x time',/CHAR
  NCDF_ATTPUT,wilma,'qc_flags','units','1',/CHAR
  NCDF_ATTPUT,wilma,'qc_flags','missing_value',-999,/SHORT
  NCDF_ATTPUT,wilma,'qc_flags','coordinates','time',/CHAR
  NCDF_ATTPUT,wilma,'flagged_values','long_name','observation values removed by flagging, #variable x time',/CHAR
  NCDF_ATTPUT,wilma,'flagged_values','missing_value',temperatures_mdi,/FLOAT ; not strictly correct
  NCDF_ATTPUT,wilma,'flagged_values','units','1',/CHAR
  NCDF_ATTPUT,wilma,'flagged_values','coordinates','time',/CHAR
  NCDF_ATTPUT,wilma,'change_points','long_name','Month and year of change in reporting accuracy or frequency, and associated values',/CHAR
  NCDF_ATTPUT,wilma,'change_points','units','day',/CHAR
  NCDF_ATTPUT,wilma,'change_notes','long_name','Any notes associated with the change points',/CHAR
  NCDF_ATTPUT,wilma,'change_notes','units','day',/CHAR



  IF suffix EQ 'internal' THEN BEGIN

     FOR attrib=0,N_ELEMENTS(MetaDNames)-1 DO BEGIN

        MDname=MetaDNames[attrib]
        MDvalue=MetaDValue[attrib]
        
        attput_string="NCDF_ATTPUT,wilma,/GLOBAL,'"+STRING(MDname)+"','"+STRING(MDvalue)+"',/CHAR"
        
        exec_run=EXECUTE(attput_string)
     ENDFOR
    
  ENDIF ELSE IF suffix EQ 'internal2' THEN BEGIN
     NCDF_ATTPUT,wilma,/GLOBAL,'title',title,/CHAR
     NCDF_ATTPUT,wilma,/GLOBAL,'institution',institution,/CHAR
     NCDF_ATTPUT,wilma,/GLOBAL,'source',source,/CHAR
     NCDF_ATTPUT,wilma,/GLOBAL,'references',references,/CHAR
     NCDF_ATTPUT,wilma,/GLOBAL,'creator_name',creator_name,/CHAR
     NCDF_ATTPUT,wilma,/GLOBAL,'creator_url',creator_url,/CHAR
     NCDF_ATTPUT,wilma,/GLOBAL,'creator_email',creator_email,/CHAR
     NCDF_ATTPUT,wilma,/GLOBAL,'time_coverage_start',time_coverage_start,/CHAR
     NCDF_ATTPUT,wilma,/GLOBAL,'time_coverage_end',time_coverage_end,/CHAR
     NCDF_ATTPUT,wilma,/GLOBAL,'cdm_data_type',cdm_data_type,/CHAR
     NCDF_ATTPUT,wilma,/GLOBAL,'keywords',keywords,/CHAR
     NCDF_ATTPUT,wilma,/GLOBAL,'summary',summary,/CHAR
     NCDF_ATTPUT,wilma,/GLOBAL,'hadisd_version',hadisd_version,/CHAR
     NCDF_ATTPUT,wilma,/GLOBAL,'isd_version',isd_version,/CHAR
     NCDF_ATTPUT,wilma,/GLOBAL,'isd_date',isd_date,/CHAR

  ENDIF
  NCDF_ATTPUT,wilma,/GLOBAL,'station_id',station_ids(st),/CHAR
  NCDF_ATTPUT,wilma,/GLOBAL,'latitude',station_lats(st),/FLOAT
  NCDF_ATTPUT,wilma,/GLOBAL,'longitude',station_lons(st),/FLOAT
  NCDF_ATTPUT,wilma,/GLOBAL,'elevation',station_elevs(st),/FLOAT
  current_time=SYSTIME()
  PRINT,current_time
  NCDF_ATTPUT,wilma,/GLOBAL,'date_created',STRING(current_time),/CHAR
  NCDF_ATTPUT,wilma,/GLOBAL,'qc_code_version',svn_version,/CHAR
  NCDF_ATTPUT,wilma,/GLOBAL,'station_information','Where station is a composite the station id refers to the primary source \n used in the timeseries and may not apply to all elements',/CHAR

  NCDF_ATTPUT,wilma,/GLOBAL,'Conventions','CF-1.5',/CHAR
  NCDF_ATTPUT,wilma,/GLOBAL,'Metadata_Conventions','Unidata Dataset Discovery v1.0,CF Discrete Sampling Geometries Conventions',/CHAR
  NCDF_ATTPUT,wilma,/GLOBAL,'featureType','timeSeries',/CHAR
  NCDF_ATTPUT,wilma,/GLOBAL,'processing_date',processing_date,/CHAR
  NCDF_ATTPUT,wilma,/GLOBAL,'history',oldhistory+history,/CHAR
 
  IF suffix EQ 'internal' THEN BEGIN
      ; Using listing obtained at beginning of script
      ;  Test if the station is a composite
      ;  if it is, create a string listing the stations it is made
      ;  out of and place this into the header.
     
      IsComposite=WHERE(CompositesList[*,0] EQ station_ids(st), iscnt)

      IF iscnt EQ 1 THEN BEGIN
         CompositeStations=CompositesList[IsComposite,*]

         composite_notes='Station is a composite made up of the following members: '

         cscntr=0
         WHILE 1 DO BEGIN
            IF CompositeStations[cscntr] NE '-' AND cscntr LT N_ELEMENTS(compositestations) THEN BEGIN
               composite_notes=composite_notes+CompositeStations[cscntr]+' '
               cscntr=cscntr+1
            ENDIF ELSE BREAK         
           
        ENDWHILE
        
     ENDIF ELSE IF iscnt EQ 0 THEN BEGIN
        composite_notes='None'
     ENDIF ELSE STOP, 'more than one match for composite station, stopping'
 
     NCDF_ATTPUT,wilma,/GLOBAL,'composite_notes',composite_notes,/CHAR
     NCDF_ATTPUT,wilma,/GLOBAL,'flag_notes','flagged values contains the values removed by the QC process.  A 9 column array with each column for one variable.  The order as follows: Temp, DewT, SLP, TotC, LowC, MidC, HiC, Winds, Windd',/CHAR
     NCDF_ATTPUT,wilma,/GLOBAL,'processing_level','Internal Checks only',/CHAR


     IF composite_notes NE 'None' THEN BEGIN
        OPENW,6,'composites.dat',/Append
        PRINTF,6,composite_notes
        CLOSE,6
     ENDIF
  ENDIF ELSE BEGIN
     NCDF_ATTPUT,wilma,/GLOBAL,'composite_notes',composite_notes,/CHAR
     NCDF_ATTPUT,wilma,/GLOBAL,'flag_notes',flag_notes,/CHAR
     NCDF_ATTPUT,wilma,/GLOBAL,'processing_level','Internal, External, Second Internal Checks',/CHAR
  ENDELSE
  ; wait command to allow file writing to finish
  ;   - see if this helps with the errors on the /ENDEF
  WAIT,1
  NCDF_CONTROL,wilma,/ENDEF

  NCDF_VARPUT, wilma, timevar, times 
  NCDF_VARPUT, wilma, stationsvar, input_station_id
  NCDF_VARPUT, wilma, tempsvar, temperatures
  NCDF_VARPUT, wilma, dewsvar, dewpoints
  NCDF_VARPUT, wilma, tcvar, total_cloud_cover
  NCDF_VARPUT, wilma, lcvar, low_cloud_cover
  NCDF_VARPUT, wilma, mcvar, mid_cloud_cover
  NCDF_VARPUT, wilma, hcvar, high_cloud_cover
  NCDF_VARPUT, wilma, cbvar, cloud_base
  NCDF_VARPUT, wilma, wsvar, windspeeds
  NCDF_VARPUT, wilma, wdvar, winddirs
  NCDF_VARPUT, wilma, slpvar, slp
  NCDF_VARPUT, wilma, sigwxvar, past_sigwx1
  NCDF_VARPUT, wilma, precipvar, precip1
  NCDF_VARPUT, wilma, qcvar, qc_flags
  NCDF_VARPUT, wilma, flagvar, flagged_values
  NCDF_VARPUT, wilma, changevar, changepoints
  NCDF_VARPUT, wilma, chntsvar, STRING(changenotes)
  
  NCDF_CLOSE,wilma
  IF NOT KEYWORD_SET(diagnostics) THEN NCDF_CLOSE,fred

;**************************************************
; DUP = duplicates 			= columns 0,
; DNL = diurnal 			= columns 4,
; GAP = TGP, DGP, SGP, gaps		= columns 5( temp),6 (dewp),7 (slp) 
; REC = TRC, DRC, WRC, SRC, records	= columns 8 (temp), 9 (dewp), 10 (Wind), 11 (SLP),
; STS = TSS,DSS,WSS,PSS straight strngs	= columns 12 (temp), 13 (dewps), 14 (wind), 15 (SLP),
; HST = HTS,HDS,HWS,HPS hour strings    = columns 16 (temp), 17 (dewp), 18 (wind), 19 (slp)
; DST = DTS,DDS,DWS,DPS day strings	= columns 20 (temp), 21 (dewp), 22 (wind), 23 (slp)
; CLM = TCM,DCM,PCM lim			= columns 24 (temp), 25 (dewp), 26 (slp)
; SPK = TSP,DSP,PSPSpike		= columns 27 (temp), 28 (dewp), 29 (slp)
; SSS = supersaturation			= columns 30, 'ttw_sat'
; DPD = dew point depression		= columns 31, 'ttw_dpd'
; DCF = dew point cutoffs		= columns 32, 'ttw_cut'
; CUO = cloud unobservable		= columns 33, (total), 34 (low), 35 (Mid), 36 (High),'cloud_unobservable'
; CST = cloud small total		= columns 37, 'cloud_smalltot'
; FLW = full low cloud			= columns 38, 'cloud_fulllow'
; FMC = full mid cloud			= columns 39, 'cloud_fullmid'
; FVC = TFV,DFV,SFVfrequent values check= columns 1 (temp), 2 (dewp), 3 (slp), 
; NGC = Negative cloud value		= columns 40, if Total, low, mid or high cloud is negative
; LAST THREE COLUMNS BLANK FOR EXTERNAL
; OUT = TOT,DOT,SOTTemperature Outlier	= columns 41 (temp), 42 (dewp), 43 (slp)

; TMB = month cleared temperature 	= columns 44
; DMB = month cleared dewpoint	 	= columns 45
; SMB = month cleared slp	 	= columnss 46
; WMB/BBB = month cleared windspd/dir 	= columns 47 and 48 (direction)
; CMB = month cleared total cloud 	= columns 49
; LMB = month cleared low cloud 	= columns 50
; MMB = month cleared middle cloud 	= columns 51
; HMB = month cleared high cloud 	= columns 52
; BMB = month cleared cloud base 	= columns 53

; OCT = odd_cluster temperatures 	= columns 54
; OCD = odd_cluster dewpoints	 	= columns 55
; OCW = odd_cluster windspeeds	 	= columns 56
; OCS = odd_cluster slp		 	= columns 57 

; TVR = variance check temperatures	= columns 58
; DVR = variance check dewpoints	= columns 59
; SVR = variance check slp 		= columns 60
;**************************************************

  IF NOT KEYWORD_SET(parallel) THEN BEGIN

     ;**************************************************
     ; Print out breakdown of total non-passes to file
     ;**************************************************
     nonpass=WHERE(qc_flags NE 0, failcount)

     ;**************************************************
     ; Adjust for 2 level flagging system in Gap and Climatological Checks
     ;**************************************************
     ; GAP - TEMP
     tgfc=WHERE(qc_flags(5,*) EQ 2, tgfcnt)
     IF tgfcnt GE 0 THEN tentative_gap_flags_temp=tgfcnt ELSE tentative_gap_flags_temp=0 
     fgfc=WHERE(qc_flags(5,*) EQ 1, fgfcnt)
     IF fgfcnt GE 0 THEN firm_gap_flags_temp=fgfcnt ELSE firm_gap_flags_temp=0 

     gap_flags_temp=tentative_gap_flags_temp+firm_gap_flags_temp

     ; GAP - DEWPT
     tgfc=WHERE(qc_flags(6,*) EQ 2, tgfcnt)
     IF tgfcnt GE 0 THEN tentative_gap_flags_dpt=tgfcnt ELSE tentative_gap_flags_dpt=0 
     fgfc=WHERE(qc_flags(6,*) EQ 1, fgfcnt)
     IF fgfcnt GE 0 THEN firm_gap_flags_dpt=fgfcnt ELSE firm_gap_flags_dpt=0 

     gap_flags_dpt=tentative_gap_flags_dpt+firm_gap_flags_dpt

     ; GAP - SLP
     tgfc=WHERE(qc_flags(7,*) EQ 2, tgfcnt)
     IF tgfcnt GE 0 THEN tentative_gap_flags_slp=tgfcnt ELSE tentative_gap_flags_slp=0 
     fgfc=WHERE(qc_flags(7,*) EQ 1, fgfcnt)
     IF fgfcnt GE 0 THEN firm_gap_flags_slp=fgfcnt ELSE firm_gap_flags_slp=0 

     gap_flags_slp=tentative_gap_flags_slp+firm_gap_flags_slp

     ; CLIMATE - TEMP
     tclc=WHERE(qc_flags(24,*) EQ 2, tcfcnt)
     IF tcfcnt GE 0 THEN tentative_climate_flags_temp=tcfcnt ELSE tentative_climate_flags_temp=0
     fclc=WHERE(qc_flags(24,*) EQ 1, fcfcnt)
     IF fcfcnt GE 0 THEN firm_climate_flags_temp=fcfcnt ELSE firm_climate_flags_temp=0

     climate_flags_temp=tentative_climate_flags_temp+firm_climate_flags_temp

     ; CLIMATE - DEWP
     tclc=WHERE(qc_flags(25,*) EQ 2, tcfcnt)
     IF tcfcnt GE 0 THEN tentative_climate_flags_dewp=tcfcnt ELSE tentative_climate_flags_dewp=0
     fclc=WHERE(qc_flags(25,*) EQ 1, fcfcnt)
     IF fcfcnt GE 0 THEN firm_climate_flags_dewp=fcfcnt ELSE firm_climate_flags_dewp=0
     
     climate_flags_dewp=tentative_climate_flags_dewp+firm_climate_flags_dewp


  ; OUTLIER - account for -1 for no-neighbours
  ; Temperature
  nntf=WHERE(qc_flags(41,*) EQ 1, nntcnt)

  ; Dewpoint
  nndf=WHERE(qc_flags(42,*) EQ 1, nndcnt)

  ; SLP
  nnsf=WHERE(qc_flags(43,*) EQ 1, nnscnt)


     PRINTF,1,station_ids(st),nt,failcount, TOTAL(qc_flags(0,*)),TOTAL(qc_flags(1,*)),TOTAL(qc_flags(2,*)),$
                   TOTAL(qc_flags(3,*)),TOTAL(qc_flags(4,*)),gap_flags_temp,gap_flags_dpt,$
		   gap_flags_slp,TOTAL(qc_flags(8,*)),TOTAL(qc_flags(9,*)),TOTAL(qc_flags(10,*)),$
		   TOTAL(qc_flags(11,*)),TOTAL(qc_flags(12,*)),TOTAL(qc_flags(13,*)),TOTAL(qc_flags(14,*)),$
		   TOTAL(qc_flags(15,*)),TOTAL(qc_flags(16,*)),TOTAL(qc_flags(17,*)),TOTAL(qc_flags(18,*)),$
		   TOTAL(qc_flags(19,*)),TOTAL(qc_flags(20,*)),TOTAL(qc_flags(21,*)),TOTAL(qc_flags(22,*)),$
		   TOTAL(qc_flags(23,*)),climate_flags_temp,climate_flags_dewp,TOTAL(qc_flags(26,*)),$
		   TOTAL(qc_flags(27,*)),TOTAL(qc_flags(28,*)),TOTAL(qc_flags(29,*)),TOTAL(qc_flags(30,*)),$
		   TOTAL(qc_flags(31,*)),TOTAL(qc_flags(32,*)),TOTAL(qc_flags(33,*)),TOTAL(qc_flags(34,*)),$
		   TOTAL(qc_flags(35,*)),TOTAL(qc_flags(36,*)),TOTAL(qc_flags(37,*)),TOTAL(qc_flags(38,*)),$
		   TOTAL(qc_flags(39,*)),TOTAL(qc_flags(40,*)),nntcnt,nndcnt,nnscnt,TOTAL(qc_flags(44,*)),$
		   TOTAL(qc_flags(45,*)),TOTAL(qc_flags(46,*)),TOTAL(qc_flags(47,*)),TOTAL(qc_flags(48,*)),$
		   TOTAL(qc_flags(49,*)),TOTAL(qc_flags(50,*)),TOTAL(qc_flags(51,*)),TOTAL(qc_flags(52,*)),$
		   TOTAL(qc_flags(53,*)),TOTAL(qc_flags(54,*)),TOTAL(qc_flags(55,*)),TOTAL(qc_flags(56,*)),$
		   TOTAL(qc_flags(57,*)),TOTAL(qc_flags(58,*)),TOTAL(qc_flags(59,*)),TOTAL(qc_flags(60,*)),$
		   FORMAT='(A12,63(I8))'
  ENDIF


  IF KEYWORD_SET(plots) THEN BEGIN
      PRINT,'Plotting'
      ;**************************************************
      ; Producing a plot of all years for this station
      ; highlighting where the QC tests failed.
      ;**************************************************
      PLOT_QC_FAIL_ALL_YEARS_RD,station_ids(st),last_year,suffix=suffix, pflags=pflags, datadir=datadir
  ENDIF

ENDFOR ; close station loop

IF NOT KEYWORD_SET(parallel) THEN BEGIN
   PRINTF,1,'station','times','fails',test_arr,FORMAT='(A12,63(A8))'
   CLOSE,1
ENDIF

IF KEYWORD_SET(diagnostics) THEN PRINT,systime(1)-total_starttime

IF KEYWORD_SET(diagnostics) AND KEYWORD_SET(test) THEN STOP

IF KEYWORD_SET(parallel) THEN BEGIN
   OPENW,1,parallelfile  ; set at beginning when it was deleted
   PRINTF,1,'Completed Run from '+restart_id+' to '+end_id, FORMAT='(A)'
   PRINTF,1,systime()
   CLOSE,1
ENDIF
print,systime()
PRINT,'FINISHED'

RETURN

END
;************************************************************************
;                                 END
;************************************************************************
