;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;British Crown Copyright (c) 2012, the Met Office All rights reserved.
;
;Redistribution and use in source and binary forms, with or without 
;modification, are permitted provided that the following conditions 
;are met:
;
;Redistributions of source code must retain the above copyright notice, 
;this list of conditions and the following disclaimer. 
;
;Redistributions in binary form must reproduce the above copyright 
;notice, this list of conditions and the following disclaimer in the 
;documentation and/or other materials provided with the distribution. 
;
;Neither the name of the Met Office nor the names of its contributors 
;may be used to endorse or promote products derived from this software 
;without specific prior written permission. 
;
;THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
;"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
;LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
;A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
;HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
;SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
;LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
;DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
;THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
;(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
;OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

PRO QC_MASKING,last_year,processing_start,  $
               svn_version,only=only, restart_id=restart_id,datadir=datadir,$
               outputdir=outputdir,plots=plots,test=test, suffix=suffix, pflags=pflags

;************************************************************************
; NAME: QC_MASKING
;     
;
; PURPOSE: To select and perform the masking of the data after all
; flags have been set - both by internal and by external flagging
;                       proceedures. 
;     
;
; CALLING SEQUENCE:
;     qc_masking,last_year,processing_start, 
;     svn_version,restart_id=restart_id,datadir=datadir, output_dir=outputdir, plots=plots,test=test,suffix=suffix
;
; INPUTS:
;     last_year - keyword to pass to plotting routine
;
;     processing_start - date at which the run of processing began
;
;     svn_version - version number of this QC code
;
;     only - keyword for doing test checks on a small subset of
;            stations. Requires 'station_info.txt' to exist.
;
;     restart_id - starting station number (allows continuation of
;                  terminated run)
;
;     input_dir - directory from which the input files are read
;
;     output_dir - directory into which the output files are written
;
;     plots - keyword to run the summary plots at the end of the script
;
;     test - keyword to run a single station as a test run
;
;     suffix - label for the files to be read in and out of the script
;              depending on at which stage it is run in the QC proceedure
;
;     pflags - keyword to plot flags on yearly plots
;
; OUTPUTS:
;     netcdf file with the masked data, and all the flags.
;     
;
; RESTRICTIONS:
;     Requires either kept_station_into.txt or station_info.txt
;     containing:
;       Station_name lat lon elev
;               one per line.
;     Assumes netcdf file inputs and parameter names as outlined in
;     the code.
;   
; HISTORY:
;     NB - not all revisions are noted here.
;     # 1   - creation
;     # 135 - created to do the masking of the data, if not already
;             applied in the individual scripts.  Masks all values
;             which have been flagged by any appropriate flags.
;             Output file written as a netcdf file containing the
;             masked data, and all the flags.
;             7 December 2010 RJHD
;     # 200 - Corrected column selection for each data type.
;             11 March 2011 RJHD
;     # 237 - SVN information added to header
;             16 November 2011 RJHD
;     # 245 - Changed reporting accuracy format in netcdf files
;             23 November 2011 RJHD
;     # 250 - Adjusted type of MetaData and the way it is processed
;             29 November 2011 RJHD
;     # 252 - Added finish statement for completion monitoring.
;             Adjusted netcdf variable names after metadata additions
;             9 December 2011 RJHD
;     # 292 - Cloud data now masked along with all others.
;             24 January 2012 RJHD
;
;************************************************************************
;                    SVN Info
;$Rev:: 309                                           $:  Revision of last commit
;$Author:: rdunn                                      $:  Author of last commit
;$Date:: 2012-07-13 15:33:22 +0100 (Fri, 13 Jul 2012) $:  Date of last commit
;************************************************************************
;                                 START
;************************************************************************

total_starttime=systime(1)
print,systime()

;*******************
; Read in the list of stations to be considered.
;*******************
cd, "./", current=pwd
IF NOT KEYWORD_SET(datadir) THEN fulldatadir=pwd+'/' ELSE fulldatadir=pwd+'/'+datadir+'/'
IF NOT KEYWORD_SET(outputdir) THEN outputdir=fulldatadir
IF NOT KEYWORD_SET(suffix) THEN suffix='mask'

IF KEYWORD_SET(only) THEN BEGIN
    ; to allow a small subset of the stations to be internally QC'd
    Station_info_file=pwd+'/station_info.txt'
ENDIF ELSE BEGIN
    Station_info_file=pwd+'/kept_station_info.txt'
ENDELSE

nstation=nlines(Station_info_file)

station_ids=STRARR(nstation)
station_lats=FLTARR(nstation)
station_lons=FLTARR(nstation)
station_elevs=FLTARR(nstation)
id=''
st=0

OPENR,1,Station_info_file
WHILE NOT EOF(1) DO BEGIN
  READF,1,id,FORMAT='(A)'
  station_ids(st)=STRMID(id,0,12)
  station_lats(st)=FLOAT(STRCOMPRESS(STRMID(id,13,9),/remove_all))
  station_lons(st)=FLOAT(STRCOMPRESS(STRMID(id,23,9),/remove_all))
  station_elevs(st)=FLOAT(STRCOMPRESS(STRMID(id,33,9),/remove_all))
  st=st+1
ENDWHILE
CLOSE,1

;*********************
; Ascertain which station id we are starting at.
;*********************

IF KEYWORD_SET(restart_id) THEN BEGIN
  match=WHERE(station_ids EQ restart_id, c)
  IF c NE 1 THEN STOP, 'restart_id passed on call does not match valid id'
  start_loc=match(0)
ENDIF ELSE start_loc=0

IF KEYWORD_SET(test) THEN BEGIN
  ; so that only a single station is run and all subroutines are run too
  nstation=1
  start_loc=0
ENDIF

FOR st=start_loc,nstation-1 DO BEGIN

    IF suffix EQ 'mask' THEN file=fulldatadir+station_ids(st)+'_external.nc' $
      ELSE IF suffix EQ 'mask2' THEN file=fulldatadir+station_ids(st)+'_external2.nc'


    PRINT,'Station Number :',st+1,"/",nstation, FORMAT='(A-40,I4,A1,I4)'
    printstring='Station Identifier :'
    PRINT,printstring,station_ids(st), FORMAT='(A-40,A15)'

    fred=NCDF_OPEN(file,/nowrite)
    NCDF_CONTROL,0,/verbose
    betty=NCDF_INQUIRE(fred)
    tid=NCDF_DIMID(fred,'time')
    NCDF_DIMINQ,fred,tid,timedim,nt
    chptid=NCDF_DIMID(fred,'changepoints')
    NCDF_DIMINQ,fred,chptid,chptdim,nchpts
    chgsid=NCDF_DIMID(fred,'changes')
    NCDF_DIMINQ,fred,chgsid,chgsdim,nchgs


    timesinfo=NCDF_VARINQ(fred,'time')
    tempsinfo=NCDF_VARINQ(fred,'temperatures')
    idsinfo=NCDF_VARINQ(fred, 'input_station_id')
    dewsinfo=NCDF_VARINQ(fred, 'dewpoints')
    totclinfo=NCDF_VARINQ(fred, 'total_cloud_cover')
    lowclinfo=NCDF_VARINQ(fred, 'low_cloud_cover')
    midclinfo=NCDF_VARINQ(fred, 'mid_cloud_cover')
    hiclinfo=NCDF_VARINQ(fred, 'high_cloud_cover')
    clbaseinfo=NCDF_VARINQ(fred, 'cloud_base')
    windspinfo=NCDF_VARINQ(fred, 'windspeeds')
    winddirinfo=NCDF_VARINQ(fred, 'winddirs')
    slpinfo=NCDF_VARINQ(fred, 'slp')
    pastsigwx1info=NCDF_VARINQ(fred,'past_sigwx1') 
    precipvar=NCDF_VARINQ(fred,'precip1_depth') 
    flaginfo=NCDF_VARINQ(fred, 'qc_flags')
    fvinfo=NCDF_VARINQ(fred, 'flagged_values')  
    chngpntinfo=NCDF_VARINQ(fred, 'change_points')  
    chngntinfo=NCDF_VARINQ(fred, 'change_notes')  
 
    NCDF_VARGET,fred,'time',times
    NCDF_VARGET,fred,'temperatures',temperatures
    NCDF_VARGET,fred,'input_station_id',input_station_id
    NCDF_VARGET,fred,'dewpoints',dewpoints
    NCDF_VARGET,fred,'total_cloud_cover',total_cloud_cover
    NCDF_VARGET,fred,'low_cloud_cover',low_cloud_cover
    NCDF_VARGET,fred,'mid_cloud_cover',mid_cloud_cover
    NCDF_VARGET,fred,'high_cloud_cover',high_cloud_cover
    NCDF_VARGET,fred,'cloud_base',cloud_base
    NCDF_VARGET,fred,'windspeeds',windspeeds
    NCDF_VARGET,fred,'winddirs',winddirs
    NCDF_VARGET,fred,'slp',slp
    NCDF_VARGET,fred,'qc_flags',qc_flags
    NCDF_VARGET,fred,'past_sigwx1',past_sigwx1
    NCDF_VARGET,fred,'precip1_depth',precip1
    NCDF_VARGET,fred,'flagged_values',flagged_values
    NCDF_VARGET,fred,'change_points',changepoints
    NCDF_VARGET,fred,'change_notes',changenotes
    
    ;
    ; get the mdi values
    NCDF_ATTGET,fred,'temperatures','missing_value',temperatures_mdi
    NCDF_ATTGET,fred,'dewpoints','missing_value',dewpoints_mdi
    NCDF_ATTGET,fred,'total_cloud_cover','missing_value',total_cloud_cover_mdi
    NCDF_ATTGET,fred,'low_cloud_cover','missing_value',low_cloud_cover_mdi
    NCDF_ATTGET,fred,'mid_cloud_cover','missing_value',mid_cloud_cover_mdi
    NCDF_ATTGET,fred,'high_cloud_cover','missing_value',high_cloud_cover_mdi
    NCDF_ATTGET,fred,'cloud_base','missing_value',cloud_base_mdi
    NCDF_ATTGET,fred,'windspeeds','missing_value',windspeeds_mdi
    NCDF_ATTGET,fred,'winddirs','missing_value',winddirs_mdi
    NCDF_ATTGET,fred,'slp','missing_value',slp_mdi
    ; get the fdi values
    NCDF_ATTGET,fred, 'temperatures','flagged_value',temperatures_fdi
    NCDF_ATTGET,fred, 'dewpoints','flagged_value',dewpoints_fdi
    NCDF_ATTGET,fred, 'slp','flagged_value',slp_fdi
    NCDF_ATTGET,fred, 'total_cloud_cover','flagged_value',total_cloud_cover_fdi
    NCDF_ATTGET,fred, 'low_cloud_cover','flagged_value',low_cloud_cover_fdi
    NCDF_ATTGET,fred, 'mid_cloud_cover','flagged_value',mid_cloud_cover_fdi
    NCDF_ATTGET,fred, 'high_cloud_cover','flagged_value',high_cloud_cover_fdi
    NCDF_ATTGET,fred, 'windspeeds','flagged_value',windspeeds_fdi
    NCDF_ATTGET,fred, 'winddirs','flagged_value',winddirs_fdi
    ; get metadata
    ;
    NCDF_ATTGET,fred,'title',title,/GLOBAL
    NCDF_ATTGET,fred,'institution',institution,/GLOBAL
    NCDF_ATTGET,fred,'source',source,/GLOBAL
    NCDF_ATTGET,fred,'references',references,/GLOBAL
    NCDF_ATTGET,fred,'creator_name',creator_name,/GLOBAL
    NCDF_ATTGET,fred,'creator_url',creator_url,/GLOBAL
    NCDF_ATTGET,fred,'creator_email',creator_email,/GLOBAL
    NCDF_ATTGET,fred,'time_coverage_start',time_coverage_start,/GLOBAL
    NCDF_ATTGET,fred,'time_coverage_end',time_coverage_end,/GLOBAL
    NCDF_ATTGET,fred,'cdm_data_type',cdm_data_type,/GLOBAL
    NCDF_ATTGET,fred,'keywords',keywords,/GLOBAL
    NCDF_ATTGET,fred,'summary',summary,/GLOBAL
    NCDF_ATTGET,fred,'hadisd_version',hadisd_version,/GLOBAL
    NCDF_ATTGET,fred,'isd_version',isd_version,/GLOBAL
    NCDF_ATTGET,fred,'isd_date',isd_date,/GLOBAL
    NCDF_ATTGET,fred,'composite_notes',composite_notes,/GLOBAL
    NCDF_ATTGET,fred,'flag_notes',flag_notes,/GLOBAL
    NCDF_ATTGET,fred,'history',bhistory,/GLOBAL 
    oldhistory=STRING(bhistory)
    history=STRING(SYSTIME())+'*** MASKING ***\n'
    
    NCDF_CLOSE,fred

    
  ;**************************************************
  ; DUP = duplicates 			= columns 0,
  ; DNL = diurnal 			= columns 4,
  ; GAP = TGP, DGP, SGP, gaps		= columns 5 (temp),6 (dewp),7 (slp) 
  ; REC = TRC, DRC, WRC, SRC, records	= columns 8 (temp), 9 (dewp), 10 (Wind), 11 (SLP),
  ; STS = TSS,DSS,WSS,PSS straight strngs	= columns 12 (temp), 13 (dewps), 14 (wind), 15 (SLP),
  ; HST = HTS,HDS,HWS,HPS hour strings  = columns 16 (temp), 17 (dewp), 18 (wind), 19 (slp)
  ; DST = DTS,DDS,DWS,DPS day strings	= columns 20 (temp), 21 (dewp), 22 (wind), 23 (slp)
  ; CLM = TCM,DCM,PCM lim		= columns 24 (temp), 25 (dewp), 26 (slp)
  ; SPK = TSP,DSP,PSPSpike		= columns 27 (temp), 28 (dewp), 29 (slp)
  ; SSS = supersaturation		= columns 30, 'ttw_sat'
  ; DPD = dew point depression		= columns 31, 'ttw_dpd'
  ; DCF = dew point cutoffs		= columns 32, 'ttw_cut'
  ; CUO = cloud unobservable		= columns 33, (total), 34 (low), 35 (Mid), 36 (High),'cloud_unobservable'
  ; CST = cloud small total		= columns 37, 'cloud_smalltot'
  ; FLW = full low cloud		= columns 38, 'cloud_fulllow'
  ; FMC = full mid cloud		= columns 39, 'cloud_fullmid'
  ; FVC = TFV,DFV,SFVfrequent values check= columns 1 (temp), 2 (dewp), 3 (slp), 
  ; NGC = Negative cloud value		= columns 40, if Total, low, mid or high cloud is negative
  ; LAST THREE COLUMNS BLANK FOR EXTERNAL
  ; OUT = TOT,DOT,SOTTemperature Outlier	= columns 41 (temp), 42 (dewp), 43 (slp)

  ; TMB = month cleared temperature 	= columns 44
  ; DMB = month cleared dewpoint	= columns 45
  ; SMB = month cleared slp	 	= columnss 46
  ; WMB/BBB = month cleared windspd/dir = columns 47 and 48 (direction)
  ; CMB = month cleared total cloud 	= columns 49
  ; LMB = month cleared low cloud 	= columns 50
  ; MMB = month cleared middle cloud 	= columns 51
  ; HMB = month cleared high cloud 	= columns 52
  ; BMB = month cleared cloud base 	= columns 53

  ; OCT = odd_cluster temperatures 	= columns 54
  ; OCD = odd_cluster dewpoints	 	= columns 55
  ; OCW = odd_cluster windspeeds	= columns 56
  ; OCS = odd_cluster slp		= columns 57 

  ; TVR = variance check temperatures	= columns 58
  ; DVR = variance check dewpoints	= columns 59
  ; SVR = variance check slp 		= columns 60
  ;**************************************************


    
    ;**************************************************
    ; need to compress the flags appropriate for each variable
    ; which needs masking.
    ;**************************************************
    ;**************************************************
    ; Change 27/1/11 RJHD & KW - Do not flag the temperatures 
    ;  with the t_tw cross check - SSS/DPD/Cutoffs
    ;**************************************************
   
    ; temperature -   0,1,4,5,8,12,16,20,24,27,41,44,54,58
    qc_temp=qc_flags([0,1,4,5,8,12,16,20,24,27,41,44,54,58],*)

    ;**************************************************
    ; Change 19/1/11 RJHD - If temperatures exceed a record, then
    ;         flag the dewpoints as well
    ;**************************************************

    ; dewpoints -     0,2,4,6,8,9,13,17,21,25,28,30,31,32,42,45,55,59
    qc_dewp=qc_flags([0,2,4,6,8,9,13,17,21,25,28,30,31,32,42,45,55,59],*)

    ; windspeeds -    0,4,10,14,18,22,47,56
    qc_windspd=qc_flags([0,4,10,14,18,22,47,56],*)

    ; winddirection -    0,4,48,56
    qc_winddir=qc_flags([0,4,48,56],*)

    ; slp -          0,3,4,7,11,15,19,23,26,29,43,46,57,60
    qc_slp=qc_flags([0,3,4,7,11,15,19,23,26,29,43,46,57,60],*)

    ; cloud - separate total,low,mid & high.  Negative Cloud
    ;      Value applied to all
    ;   added 23-Jan-2012
    qc_tcloud=qc_flags([33,37,40,49],*)
    qc_lcloud=qc_flags([34,38,40,50],*)
    qc_mcloud=qc_flags([35,39,40,51],*)
    qc_hcloud=qc_flags([36,40,52],*)


    ;**************************************************
    ; apply the flags
    ;   2 level flagging system accounted for as using GT 0, not EQ 1
    ;**************************************************

    ; temperature
    ; adjust for "-1" in neighbour checking
    neighbours=WHERE(qc_temp EQ -1, ncnt)
    IF ncnt GE 1 THEN qc_temp[neighbours]=0

    qc_temp=TOTAL(qc_temp,1)
    temp_flags=WHERE(qc_temp GT 0,tcnt)

    IF tcnt GE 1 THEN BEGIN
        flagged_values[0,temp_flags]=temperatures(temp_flags)
        temperatures(temp_flags)=temperatures_fdi
    ENDIF
            
    ; dewpoint
    ; adjust for "-1" in neighbour checking
    neighbours=WHERE(qc_dewp EQ -1, ncnt)
    IF ncnt GE 1 THEN qc_dewp[neighbours]=0

    qc_dewp=TOTAL(qc_dewp,1)
    dewp_flags=WHERE(qc_dewp GT 0,dcnt)

    IF dcnt GE 1 THEN BEGIN
        flagged_values[1,dewp_flags]=dewpoints(dewp_flags)
        dewpoints(dewp_flags)=dewpoints_fdi
    ENDIF

    ; wind speed
    ; adjust for "-1" in neighbour checking
    neighbours=WHERE(qc_windspd EQ -1, ncnt)
    IF ncnt GE 1 THEN qc_windspd[neighbours]=0

    qc_windspd=TOTAL(qc_windspd,1)
    wind_flags=WHERE(qc_windspd GT 0,wcnt)

    IF wcnt GE 1 THEN BEGIN
        flagged_values[7,wind_flags]=windspeeds(wind_flags)
        windspeeds(wind_flags)=windspeeds_fdi
    ENDIF
    
    ; wind direction
    ; adjust for "-1" in neighbour checking
    neighbours=WHERE(qc_winddir EQ -1, ncnt)
    IF ncnt GE 1 THEN qc_winddir[neighbours]=0

    qc_winddir=TOTAL(qc_winddir,1)
    wind_flags=WHERE(qc_winddir GT 0,wcnt)

    IF wcnt GE 1 THEN BEGIN
       flagged_values[8,wind_flags]=winddirs(wind_flags)
       winddirs(wind_flags)=winddirs_fdi
    ENDIF

    ; slp
    ; adjust for "-1" in neighbour checking
    neighbours=WHERE(qc_slp EQ -1, ncnt)
    IF ncnt GE 1 THEN qc_slp[neighbours]=0

    qc_slp=TOTAL(qc_slp,1)
    slp_flags=WHERE(qc_slp GT 0,scnt)

    IF scnt GE 1 THEN BEGIN
        flagged_values[2,slp_flags]=slp(slp_flags)
        slp(slp_flags)=slp_fdi
    ENDIF
    
    ; total cloud
    ; adjust for "-1" in neighbour checking
    neighbours=WHERE(qc_tcloud EQ -1, ncnt)
    IF ncnt GE 1 THEN qc_tcloud[neighbours]=0

    qc_tcloud=TOTAL(qc_tcloud,1)
    tcloud_flags=WHERE(qc_tcloud GT 0,tccnt)

    IF tccnt GE 1 THEN BEGIN
        flagged_values[3,tcloud_flags]=total_cloud_cover(tcloud_flags)
        total_cloud_cover(tcloud_flags)=total_cloud_cover_fdi
    ENDIF
    
    ; low cloud
    ; adjust for "-1" in neighbour checking
    neighbours=WHERE(qc_lcloud EQ -1, ncnt)
    IF ncnt GE 1 THEN qc_lcloud[neighbours]=0

    qc_lcloud=TOTAL(qc_lcloud,1)
    lcloud_flags=WHERE(qc_lcloud GT 0,lccnt)

    IF lccnt GE 1 THEN BEGIN
        flagged_values[4,lcloud_flags]=low_cloud_cover(lcloud_flags)
        low_cloud_cover(lcloud_flags)=low_cloud_cover_fdi
    ENDIF
    
    ; mid cloud
    ; adjust for "-1" in neighbour checking
    neighbours=WHERE(qc_mcloud EQ -1, ncnt)
    IF ncnt GE 1 THEN qc_mcloud[neighbours]=0

    qc_mcloud=TOTAL(qc_mcloud,1)
    mcloud_flags=WHERE(qc_mcloud NE 0,mccnt)

    IF mccnt GE 1 THEN BEGIN
        flagged_values[5,mcloud_flags]=mid_cloud_cover(mcloud_flags)
        mid_cloud_cover(mcloud_flags)=mid_cloud_cover_fdi
    ENDIF
    
    ; high cloud
    ; adjust for "-1" in neighbour checking
    neighbours=WHERE(qc_hcloud EQ -1, ncnt)
    IF ncnt GE 1 THEN qc_hcloud[neighbours]=0

    qc_hcloud=TOTAL(qc_hcloud,1)
    hcloud_flags=WHERE(qc_hcloud NE 0,hccnt)

    IF hccnt GE 1 THEN BEGIN
        flagged_values[6,hcloud_flags]=high_cloud_cover(hcloud_flags)
        high_cloud_cover(hcloud_flags)=high_cloud_cover_fdi
    ENDIF
    




    PRINT,'Masking Complete'
    history=history+STRING(SYSTIME())+' Flags Applied \n '      
    ;**************************************************
    ; output the new file
    ;**************************************************

    IF suffix EQ 'mask' THEN netcdf_outfile=outputdir+station_ids(st)+'_mask.nc' $
      ELSE IF suffix EQ 'mask2' THEN netcdf_outfile=outputdir+station_ids(st)+'_mask2.nc'
  
    wilma=NCDF_CREATE(netcdf_outfile,/clobber)
  
    tid=NCDF_DIMDEF(wilma,'time',nt)
    testid=NCDF_DIMDEF(wilma, 'test', 61)
    charid=NCDF_DIMDEF(wilma,'Character',12)
    flagid=NCDF_DIMDEF(wilma,'flagged',9)
    changesid=NCDF_DIMDEF(wilma,'changepoints',N_ELEMENTS(STRING(changenotes)))
    chptsid=NCDF_DIMDEF(wilma,'changes',4)
  
    chgntlen=STRLEN(changenotes)
    lcharid=NCDF_DIMDEF(wilma,'LongCharacter',MAX(chgntlen))

    IF suffix EQ 'mask' THEN $
       timevar=NCDF_VARDEF(wilma,'time',[tid],/DOUBLE) $
    ELSE IF suffix EQ 'mask2' THEN $
       timevar=NCDF_VARDEF(wilma,'time',[tid],/LONG)

    stationsvar=NCDF_VARDEF(wilma,'input_station_id',[charid,tid],/CHAR)
    tempsvar=NCDF_VARDEF(wilma,'temperatures',[tid],/FLOAT)
    dewsvar=NCDF_VARDEF(wilma,'dewpoints',[tid],/FLOAT)
    tcvar=NCDF_VARDEF(wilma,'total_cloud_cover',[tid],/SHORT)
    lcvar=NCDF_VARDEF(wilma,'low_cloud_cover',[tid],/SHORT)
    mcvar=NCDF_VARDEF(wilma,'mid_cloud_cover',[tid],/SHORT)
    hcvar=NCDF_VARDEF(wilma,'high_cloud_cover',[tid],/SHORT)
    cbvar=NCDF_VARDEF(wilma,'cloud_base',[tid],/SHORT)
    wsvar=NCDF_VARDEF(wilma,'windspeeds',[tid],/FLOAT) 
    wdvar=NCDF_VARDEF(wilma,'winddirs',[tid],/SHORT)
    slpvar=NCDF_VARDEF(wilma,'slp',[tid],/FLOAT)
    sigwxvar=NCDF_VARDEF(wilma,'past_sigwx1',[tid],/SHORT)
    precipvar=NCDF_VARDEF(wilma,'precip1_depth',[tid],/FLOAT)
    qcvar=NCDF_VARDEF(wilma,'qc_flags',[testid,tid],/SHORT)
    flagvar=NCDF_VARDEF(wilma,'flagged_values',[flagid,tid],/FLOAT)
    changevar=NCDF_VARDEF(wilma,'change_points',[chptsid,changesid],/FLOAT)
    chntsvar=NCDF_VARDEF(wilma,'change_notes',[lcharid,changesid],/CHAR)
 
    
    NCDF_ATTPUT,wilma,'time','long_name','time',/CHAR
    NCDF_ATTPUT,wilma,'time','standard_name','time',/CHAR

    IF suffix EQ 'mask' THEN $
       NCDF_ATTPUT,wilma,'time','units','days since 1973-01-01 00:00:00',/CHAR $
    ELSE IF suffix EQ 'mask2' THEN $
       NCDF_ATTPUT,wilma,'time','units','hours since 1973-01-01 00:00:00',/CHAR

    NCDF_ATTPUT,wilma,'time','axis','T',/CHAR
    NCDF_ATTPUT,wilma,'time','calendar','gregorian',/CHAR
    NCDF_ATTPUT,wilma,'time','valid_min',0.,/LONG
    NCDF_ATTPUT,wilma,'input_station_id','long_name','Primary source for timestep (may be multiple sources for composite stations). USAF - WBAN from ISD source',/CHAR
    NCDF_ATTPUT,wilma,'input_station_id','cf_role','timeseries id',/CHAR
    NCDF_ATTPUT,wilma,'temperatures','long_name','Dry bulb temperatures nearest to reporting hour',/CHAR
    NCDF_ATTPUT,wilma,'temperatures','standard_name','surface_temperature',/CHAR
    NCDF_ATTPUT,wilma,'temperatures','units','degreeC',/CHAR
    NCDF_ATTPUT,wilma,'temperatures','coordinates','time',/CHAR
    valid=WHERE(((temperatures NE temperatures_mdi) AND (temperatures NE temperatures_fdi)), tc)
    IF tc GE 1 THEN BEGIN
        min_t=MIN(temperatures(valid))
        max_t=MAX(temperatures(valid))
        NCDF_ATTPUT,wilma,'temperatures','valid_min',min_t(0),/FLOAT
        NCDF_ATTPUT,wilma,'temperatures','valid_max',max_t(0),/FLOAT
    ENDIF
    NCDF_ATTPUT,wilma,'temperatures','missing_value',temperatures_mdi,/FLOAT
    NCDF_ATTPUT,wilma,'temperatures','flagged_value',temperatures_fdi,/FLOAT
    NCDF_ATTPUT,wilma,'dewpoints','long_name','Dew point temperatures nearest to reporting hour',/CHAR
    NCDF_ATTPUT,wilma,'dewpoints','standard_name','dew_point_temperature',/CHAR
    NCDF_ATTPUT,wilma,'dewpoints','units','degreeC',/CHAR
    valid=WHERE(((dewpoints NE dewpoints_mdi) AND (dewpoints NE dewpoints_fdi)),tc)
    IF tc GE 1 THEN BEGIN
        min_t=MIN(dewpoints(valid))
        max_t=MAX(dewpoints(valid))
        NCDF_ATTPUT,wilma,'dewpoints','valid_min',min_t(0),/FLOAT
        NCDF_ATTPUT,wilma,'dewpoints','valid_max',max_t(0),/FLOAT
    ENDIF
    NCDF_ATTPUT,wilma,'dewpoints','missing_value',dewpoints_mdi,/FLOAT
    NCDF_ATTPUT,wilma,'dewpoints','flagged_value',dewpoints_fdi,/FLOAT
    NCDF_ATTPUT,wilma,'dewpoints','coordinates','time',/CHAR
    NCDF_ATTPUT,wilma,'total_cloud_cover','long_name','Total cloud cover derived in priority order GA, GF, GD - see ISD documentation',/CHAR
    NCDF_ATTPUT,wilma,'total_cloud_cover','standard_name','cloud_area_fraction',/CHAR
    NCDF_ATTPUT,wilma,'total_cloud_cover','units','1',/CHAR
    NCDF_ATTPUT,wilma,'total_cloud_cover','valid_min',0,/SHORT
    NCDF_ATTPUT,wilma,'total_cloud_cover','valid_max',8,/SHORT
    NCDF_ATTPUT,wilma,'total_cloud_cover','missing_value',total_cloud_cover_mdi,/SHORT
    NCDF_ATTPUT,wilma,'total_cloud_cover','flagged_value',total_cloud_cover_fdi,/SHORT
    NCDF_ATTPUT,wilma,'total_cloud_cover','coordinates','time',/CHAR
    NCDF_ATTPUT,wilma,'low_cloud_cover','long_name','low cloud cover derived in priority order GA, GF, GD - see ISD documentation.  Units are Oktas',/CHAR
    NCDF_ATTPUT,wilma,'low_cloud_cover','units','1',/CHAR
    NCDF_ATTPUT,wilma,'low_cloud_cover','valid_min',0,/SHORT
    NCDF_ATTPUT,wilma,'low_cloud_cover','valid_max',8,/SHORT
    NCDF_ATTPUT,wilma,'low_cloud_cover','missing_value',low_cloud_cover_mdi,/SHORT
    NCDF_ATTPUT,wilma,'low_cloud_cover','flagged_value',low_cloud_cover_fdi,/SHORT
    NCDF_ATTPUT,wilma,'low_cloud_cover','coordinates','time',/CHAR
    NCDF_ATTPUT,wilma,'mid_cloud_cover','long_name','mid cloud cover derived in priority order GA, GF, GD - see ISD documentation.  Units are Oktas',/CHAR
    NCDF_ATTPUT,wilma,'mid_cloud_cover','units','1',/CHAR
    NCDF_ATTPUT,wilma,'mid_cloud_cover','valid_min',0,/SHORT
    NCDF_ATTPUT,wilma,'mid_cloud_cover','valid_max',8,/SHORT
    NCDF_ATTPUT,wilma,'mid_cloud_cover','missing_value',mid_cloud_cover_mdi,/SHORT
    NCDF_ATTPUT,wilma,'mid_cloud_cover','flagged_value',mid_cloud_cover_fdi,/SHORT
    NCDF_ATTPUT,wilma,'mid_cloud_cover','coordinates','time',/CHAR
    NCDF_ATTPUT,wilma,'high_cloud_cover','long_name','high cloud cover derived in priority order GA, GF, GD - see ISD documentation.  Units are Oktas',/CHAR
    NCDF_ATTPUT,wilma,'high_cloud_cover','units','1',/CHAR
    NCDF_ATTPUT,wilma,'high_cloud_cover','valid_min',0,/SHORT
    NCDF_ATTPUT,wilma,'high_cloud_cover','valid_max',8,/SHORT
    NCDF_ATTPUT,wilma,'high_cloud_cover','missing_value',high_cloud_cover_mdi,/SHORT
    NCDF_ATTPUT,wilma,'high_cloud_cover','flagged_value',high_cloud_cover_fdi,/SHORT
    NCDF_ATTPUT,wilma,'high_cloud_cover','coordinates','time',/CHAR
    NCDF_ATTPUT,wilma,'cloud_base','long_name','Cloud base of lowest cloud layer nearest to reporting hour',/CHAR
    NCDF_ATTPUT,wilma,'cloud_base','standard_name','cloud_base_altitude',/CHAR
    NCDF_ATTPUT,wilma,'cloud_base','units','metres',/CHAR
    valid=WHERE(cloud_base NE cloud_base_mdi,tc)
    IF tc GE 1 THEN BEGIN
        min_t=MIN(cloud_base(valid))
        max_t=MAX(cloud_base(valid))
        NCDF_ATTPUT,wilma,'cloud_base','valid_min',min_t(0),/FLOAT
        NCDF_ATTPUT,wilma,'cloud_base','valid_max',max_t(0),/FLOAT
    ENDIF
    NCDF_ATTPUT,wilma,'cloud_base','missing_value',cloud_base_mdi,/SHORT
    NCDF_ATTPUT,wilma,'cloud_base','coordinates','time',/CHAR
    NCDF_ATTPUT,wilma,'windspeeds','long_name','Wind speed',/CHAR
    NCDF_ATTPUT,wilma,'windspeeds','units','metres per second',/CHAR
    NCDF_ATTPUT,wilma,'windspeeds','standard_name','wind_speed',/CHAR
    valid=WHERE(((windspeeds NE windspeeds_mdi) AND (windspeeds NE windspeeds_fdi)),tc)
    IF tc GE 1 THEN BEGIN
        min_t=MIN(windspeeds(valid))
        max_t=MAX(windspeeds(valid))
        NCDF_ATTPUT,wilma,'windspeeds','valid_min',min_t(0),/FLOAT
        NCDF_ATTPUT,wilma,'windspeeds','valid_max',max_t(0),/FLOAT
    ENDIF
    NCDF_ATTPUT,wilma,'windspeeds','missing_value',windspeeds_mdi,/FLOAT
    NCDF_ATTPUT,wilma,'windspeeds','flagged_value',windspeeds_fdi,/FLOAT
    NCDF_ATTPUT,wilma,'windspeeds','coordinates','time',/CHAR
    NCDF_ATTPUT,wilma,'winddirs','long_name','Wind Direction',/CHAR
    NCDF_ATTPUT,wilma,'winddirs','standard_name','wind_from_direction',/CHAR
    NCDF_ATTPUT,wilma,'winddirs','units','Degrees',/CHAR
    NCDF_ATTPUT,wilma,'winddirs','valid_min',0,/SHORT
    NCDF_ATTPUT,wilma,'winddirs','valid_max',360,/SHORT
    NCDF_ATTPUT,wilma,'winddirs','missing_value',winddirs_mdi,/SHORT
    NCDF_ATTPUT,wilma,'winddirs','flagged_value',winddirs_fdi,/SHORT
    NCDF_ATTPUT,wilma,'winddirs','coordinates','time',/CHAR
    NCDF_ATTPUT,wilma,'slp','long_name','Reported sea level pressure',/CHAR
    NCDF_ATTPUT,wilma,'slp','standard_name','air_pressure_at_sea_level',/CHAR
    NCDF_ATTPUT,wilma,'slp','units','hPa',/CHAR
    NCDF_ATTPUT,wilma,'slp','coordinates','time',/CHAR
    valid=WHERE(((slp NE slp_mdi) AND (slp NE slp_fdi)), tc)
    IF tc GE 1 THEN BEGIN
        min_t=MIN(slp(valid))
        max_t=MAX(slp(valid))
        NCDF_ATTPUT,wilma,'slp','valid_min',min_t(0),/FLOAT
        NCDF_ATTPUT,wilma,'slp','valid_max',max_t(0),/FLOAT
    ENDIF
    NCDF_ATTPUT,wilma,'slp','missing_value',slp_mdi,/FLOAT
    NCDF_ATTPUT,wilma,'slp','flagged_value',slp_fdi,/FLOAT
    NCDF_ATTPUT,wilma,'past_sigwx1','long_name','Station reports of past significant weather phenomena',/CHAR
    NCDF_ATTPUT,wilma,'past_sigwx1','units','1',/CHAR
    NCDF_ATTPUT,wilma,'past_sigwx1','valid_min',0s,/SHORT
    NCDF_ATTPUT,wilma,'past_sigwx1','valid_max',9s,/SHORT
    NCDF_ATTPUT,wilma,'past_sigwx1','missing_value',-999s,/SHORT
    NCDF_ATTPUT,wilma,'past_sigwx1','coordinates','time',/CHAR
    NCDF_ATTPUT,wilma,'precip1_depth','long_name','Depth of precipitation reported',/CHAR
    NCDF_ATTPUT,wilma,'precip1_depth','standard_name','lwe_thickness_of_precipitation_amount',/CHAR
    NCDF_ATTPUT,wilma,'precip1_depth','units','mm',/CHAR
    NCDF_ATTPUT,wilma,'precip1_depth','valid_min',0,/SHORT
    NCDF_ATTPUT,wilma,'precip1_depth','valid_max',9,/SHORT
    NCDF_ATTPUT,wilma,'precip1_depth','missing_value',-1.e+30,/FLOAT
    NCDF_ATTPUT,wilma,'precip1_depth','coordinates','time',/CHAR
    NCDF_ATTPUT,wilma,'qc_flags','long_name','QC status for individual obs',/CHAR
    NCDF_ATTPUT,wilma,'qc_flags','units','1',/CHAR
    NCDF_ATTPUT,wilma,'qc_flags','missing_value',-999,/SHORT
    NCDF_ATTPUT,wilma,'qc_flags','coordinates','time',/CHAR
    NCDF_ATTPUT,wilma,'flagged_values','long_name','observation values removed by flagging',/CHAR
    NCDF_ATTPUT,wilma,'flagged_values','missing_value',temperatures_mdi,/FLOAT ; not strictly correct
    NCDF_ATTPUT,wilma,'flagged_values','units','1',/CHAR
    NCDF_ATTPUT,wilma,'flagged_values','coordinates','time',/CHAR
    NCDF_ATTPUT,wilma,'change_points','long_name','Month and year of change in reporting accuracy or frequency, and associated values',/CHAR
    NCDF_ATTPUT,wilma,'change_points','units','day',/CHAR
    NCDF_ATTPUT,wilma,'change_notes','long_name','Any notes associated with the change points',/CHAR
    NCDF_ATTPUT,wilma,'change_notes','units','day',/CHAR

  ; Global Attributes
    NCDF_ATTPUT,wilma,/GLOBAL,'title',title,/CHAR
    NCDF_ATTPUT,wilma,/GLOBAL,'institution',institution,/CHAR
    NCDF_ATTPUT,wilma,/GLOBAL,'source',source,/CHAR
    NCDF_ATTPUT,wilma,/GLOBAL,'references',references,/CHAR
    NCDF_ATTPUT,wilma,/GLOBAL,'creator_name',creator_name,/CHAR
    NCDF_ATTPUT,wilma,/GLOBAL,'creator_url',creator_url,/CHAR
    NCDF_ATTPUT,wilma,/GLOBAL,'creator_email',creator_email,/CHAR
    NCDF_ATTPUT,wilma,/GLOBAL,'time_coverage_start',time_coverage_start,/CHAR
    NCDF_ATTPUT,wilma,/GLOBAL,'time_coverage_end',time_coverage_end,/CHAR
    NCDF_ATTPUT,wilma,/GLOBAL,'cdm_data_type',cdm_data_type,/CHAR
    NCDF_ATTPUT,wilma,/GLOBAL,'keywords',keywords,/CHAR
    NCDF_ATTPUT,wilma,/GLOBAL,'summary',summary,/CHAR
    NCDF_ATTPUT,wilma,/GLOBAL,'hadisd_version',hadisd_version,/CHAR
    NCDF_ATTPUT,wilma,/GLOBAL,'isd_version',isd_version,/CHAR
    NCDF_ATTPUT,wilma,/GLOBAL,'isd_date',isd_date,/CHAR
    NCDF_ATTPUT,wilma,/GLOBAL,'station_information','Where station is a composite the station id refers to the primary source \n used in the timeseries and may not apply to all elements',/CHAR
    NCDF_ATTPUT,wilma,/GLOBAL,'station_id',station_ids(st),/CHAR
    NCDF_ATTPUT,wilma,/GLOBAL,'latitude',station_lats(st),/FLOAT
    NCDF_ATTPUT,wilma,/GLOBAL,'longitude',station_lons(st),/FLOAT
    NCDF_ATTPUT,wilma,/GLOBAL,'elevation',station_elevs(st),/FLOAT
    current_time=SYSTIME()
    PRINT,current_time
    NCDF_ATTPUT,wilma,/GLOBAL,'date_created',STRING(current_time),/CHAR
    NCDF_ATTPUT,wilma,/GLOBAL,'processing_start',processing_start,/CHAR
    NCDF_ATTPUT,wilma,/GLOBAL,'qc_code_version',svn_version,/CHAR
    NCDF_ATTPUT,wilma,/GLOBAL,'composite_notes',composite_notes,/CHAR
    NCDF_ATTPUT,wilma,/GLOBAL,'flag_notes',flag_notes,/CHAR
    NCDF_ATTPUT,wilma,/GLOBAL,'history',oldhistory+history,/CHAR
    NCDF_ATTPUT,wilma,/GLOBAL,'Conventions','CF-1.5',/CHAR
    NCDF_ATTPUT,wilma,/GLOBAL,'Metadata_Conventions','Unidata Dataset Discovery v1.0CF Discrete Sampling Geometries Conventions',/CHAR
    NCDF_ATTPUT,wilma,/GLOBAL,'featureType','timeSeries',/CHAR

    IF suffix EQ 'mask' THEN BEGIN
       NCDF_ATTPUT,wilma,/GLOBAL,'processing_level','Internal & External Checks ',/CHAR
    ENDIF ELSE IF suffix EQ 'mask2' THEN BEGIN
       NCDF_ATTPUT,wilma,/GLOBAL,'processing_level','2x Internal & External Checks',/CHAR
    ENDIF
    WAIT, 1
    NCDF_CONTROL,wilma,/ENDEF
  
    IF suffix EQ 'mask' THEN $
       NCDF_VARPUT, wilma, timevar, times $
    ELSE IF suffix EQ 'mask2' THEN BEGIN
       h_times=ROUND(times*24.)
       NCDF_VARPUT, wilma, timevar, h_times 
    ENDIF
 
    NCDF_VARPUT, wilma, stationsvar, input_station_id
    NCDF_VARPUT, wilma, tempsvar, temperatures
    NCDF_VARPUT, wilma, dewsvar, dewpoints
    NCDF_VARPUT, wilma, tcvar, total_cloud_cover
    NCDF_VARPUT, wilma, lcvar, low_cloud_cover
    NCDF_VARPUT, wilma, mcvar, mid_cloud_cover
    NCDF_VARPUT, wilma, hcvar, high_cloud_cover
    NCDF_VARPUT, wilma, cbvar, cloud_base
    NCDF_VARPUT, wilma, wsvar, windspeeds
    NCDF_VARPUT, wilma, wdvar, winddirs
    NCDF_VARPUT, wilma, slpvar, slp
    NCDF_VARPUT, wilma, sigwxvar, past_sigwx1
    NCDF_VARPUT, wilma, precipvar, precip1
    NCDF_VARPUT, wilma, qcvar, qc_flags
    NCDF_VARPUT, wilma, flagvar, flagged_values
    NCDF_VARPUT, wilma, changevar, changepoints
    NCDF_VARPUT, wilma, chntsvar, STRING(changenotes)
  
    NCDF_CLOSE, wilma

    IF KEYWORD_SET(plots) THEN BEGIN
      ;**************************************************
      ; Producing a plot of all years for this station
      ; highlighting where the QC tests failed.
      ;**************************************************
        PLOT_QC_FAIL_ALL_YEARS_RD,station_ids(st),last_year, suffix=suffix, pflags=pflags
    ENDIF

ENDFOR ;close station loop
print,systime()
PRINT,'FINISHED'

RETURN

END


;************************************************************************
;                                 END
;************************************************************************
