;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;British Crown Copyright (c) 2012, the Met Office All rights reserved.
;
;Redistribution and use in source and binary forms, with or without 
;modification, are permitted provided that the following conditions 
;are met:
;
;Redistributions of source code must retain the above copyright notice, 
;this list of conditions and the following disclaimer. 
;
;Redistributions in binary form must reproduce the above copyright 
;notice, this list of conditions and the following disclaimer in the 
;documentation and/or other materials provided with the distribution. 
;
;Neither the name of the Met Office nor the names of its contributors 
;may be used to endorse or promote products derived from this software 
;without specific prior written permission. 
;
;THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
;"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
;LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
;A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
;HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
;SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
;LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
;DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
;THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
;(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
;OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

PRO qc_summary_create,restart_id=restart_id, test=test, only=only, suffix=suffix, $
                      datadir=datadir, outputdir=outputdir

; NAME:
;     QC_SUMMARY_CREATE
;
; PURPOSE:
;     To output the qc_summary.txt table of fails.  Written to save
;     re-running complete procedure again to account for two level
;     flagging.  Future plans are to incorporate this into the QC
;     programmes.
;
; CALLING SEQUENCE:
;     qc_summary_create,restart_id=restart_id, test=test,
;     suffix=suffix, datadir=datadir, outputdir=outputdir
;
; INPUTS:
;     restart_id - station ID for single station run, or to pick up
;                  where left off
;
;     test - run on a single station only
;
;     only - run on small number of stations only
;
;     suffix - which stage of QC do we want an output for
;
;     datadir - location of netcdf files
;
;     outputdir - location of output .txt file
;
; OUTPUTS:
;     .txt file with summary of QC fails for each test and station
;
; RESTRICTIONS:
;

;
; HISTORY:
;     # 232 - Created from qc_all_vars_internal
;             3 October 2011 RJHD
;     # 237 - SVN information added to header
;             16 November 2011 RJHD
;     # 244 - Adjusted numerical order of 2-level flagging scheme.
;             Added datadir & outputdir keywords. 
;             22 November 2011 RJHD
;     # 252 - Adjusted netcdf variable names after metadata changes
;             9 December 2011 RJHD
;
;************************************************************************
;                    SVN Info
;$Rev:: 327                                           $:  Revision of last commit
;$Author:: rdunn                                      $:  Author of last commit
;$Date:: 2013-01-24 15:28:30 +0000 (Thu, 24 Jan 2013) $:  Date of last commit
;************************************************************************
;                                 START
;************************************************************************

cd, "./", current=pwd
IF NOT KEYWORD_SET(datadir) THEN datadir=pwd+'/' ELSE datadir=pwd+'/'+datadir+'/'
IF NOT KEYWORD_SET(outputdir) THEN outputdir=datadir
IF NOT KEYWORD_SET(suffix) THEN suffix='internal'

test_arr=['DUP','TFV','DFV','SFV','DNL','TGP','DGP','SGP','TRC','DRC',$
          'WRC','PRC','TSS','DSS','WSS','PSS','HTS','HDS','HWS','HPS',$
          'DTS','DDS','DWS','DPS','TCM','DCM','PCM','TSP','DSP','PSP',$
          'SSS','DPD','DCF','CUOT','CUOL','CUOM','CUOH','CST','FLW','FMC',$
          'NGC','TOT','DOT','SOT','TMB','DMB','SMB','WMB','BBB','CMB',$
          'LMB','MMB','HMB','BMB','OCT','OCD','OCW','OCS','TVR','DVR',$
          'SVR']

;*******************
; Read in the list of stations to be considered.
;*******************
IF KEYWORD_SET(only) OR KEYWORD_SET(test) THEN BEGIN
    ; to allow a small subset of the stations to be internally QC'd
    Station_info_file=pwd+'/station_info.txt'
ENDIF ELSE BEGIN
    Station_info_file=pwd+'/kept_station_info.txt'
ENDELSE

nstation=nlines(Station_info_file)

station_ids=STRARR(nstation)
station_lats=FLTARR(nstation)
station_lons=FLTARR(nstation)
station_elevs=FLTARR(nstation)
id=''
st=0

OPENR,1,Station_info_file
WHILE NOT EOF(1) DO BEGIN
  READF,1,id,FORMAT='(A)'
  station_ids(st)=STRMID(id,0,12)
  station_lats(st)=FLOAT(STRCOMPRESS(STRMID(id,13,9),/remove_all))
  station_lons(st)=FLOAT(STRCOMPRESS(STRMID(id,23,9),/remove_all))
  station_elevs(st)=FLOAT(STRCOMPRESS(STRMID(id,33,9),/remove_all))
  st=st+1
ENDWHILE
CLOSE,1

;*********************
; Ascertain which station id we are starting at.
;*********************

IF KEYWORD_SET(restart_id) THEN BEGIN
  match=WHERE(station_ids EQ restart_id, c)
  IF c NE 1 THEN STOP, 'restart_id passed on call does not match valid id'
  start_loc=match(0)
ENDIF ELSE start_loc=0

IF KEYWORD_SET(test) THEN BEGIN
  ; so that only a single station is run and all subroutines are run too
  nstation=1
  start_loc=0
ENDIF


IF NOT KEYWORD_SET(test) THEN BEGIN
                                ; echo the current station being processed
    OPENW,1,pwd+'/qc_summary_'+suffix+'.txt',/append 
ENDIF ELSE BEGIN                ; test is set
    FILE_DELETE,pwd+'/qc_summary_'+suffix+'_only_'+station_ids(0)+'.txt', /ALLOW_NONEXISTENT
                                ; delete the individual station qc storage file
    OPENW,1,pwd+'/qc_summary_'+suffix+'_only_'+station_ids(0)+'.txt',/append 
ENDELSE
    

;***************************************************************************
;    START THE STATION LOOP
;***************************************************************************
FOR st=start_loc,nstation-1 DO BEGIN
  IF NOT KEYWORD_SET(test) THEN BEGIN
      PRINT,'Station Number :',st+1,"/",nstation, FORMAT='(A-40,I4,A1,I4)'
      printstring='Station Identifier :'
      PRINT,printstring,station_ids(st), FORMAT='(A-40,A15)'
  ENDIF
  ;***************************************************************************
  ;    START THE DATA READING
  ;***************************************************************************
  file=datadir+station_ids(st)+'_'+suffix+'.nc'

  fred=NCDF_OPEN(file,/nowrite)

  NCDF_CONTROL,0,/verbose
  betty=NCDF_INQUIRE(fred)

  ;****************************************
  ;  obtain information on the dimensions
  ;****************************************

  tid=NCDF_DIMID(fred,'time')
  NCDF_DIMINQ,fred,tid,timedim,nt

  ;****************************************
  ;  read the variable information
  ;****************************************

  timesinfo=NCDF_VARINQ(fred,'time')
  
  flag_test=NCDF_VARID(fred, 'qc_flags')
  IF flag_test NE -1 THEN BEGIN
      flaginfo=NCDF_VARINQ(fred, 'qc_flags')
  ENDIF

  
  ;****************************************
  ;  read the variable values into the arrays
  ;****************************************

  NCDF_VARGET,fred,'time',times


  ;****************************************
  ;  initialise the QC flag array
  ;****************************************

  IF flag_test NE -1 THEN BEGIN
      NCDF_VARGET,fred,'qc_flags',qc_flags
      oldflags=1
  ENDIF ELSE BEGIN
      qc_flags=INTARR(61,nt)
      oldflags=0
  ENDELSE


  NCDF_CLOSE,fred   


  ;**************************************************
  ; Print out breakdown of total non-passes to file
  ;   adjusted to avoid double counting
  ;**************************************************
  compressed_flags=qc_flags
  no_neighbours=WHERE(compressed_flags EQ -1, nncnt)
  IF nncnt GE 1 THEN compressed_flags[no_neighbours]=0
  compressed_flags=total(compressed_flags,1)
  nonpass=WHERE(compressed_flags NE 0, failcount)

  ;**************************************************
  ; Adjust for 2 level flagging system in Gap and Climatological Checks
  ;**************************************************
  ; GAP - TEMP
  tgfc=WHERE(qc_flags(5,*) EQ 2, tgfcnt)
  IF tgfcnt GE 0 THEN tentative_gap_flags_temp=tgfcnt ELSE tentative_gap_flags_temp=0 
  fgfc=WHERE(qc_flags(5,*) EQ 1, fgfcnt)
  IF fgfcnt GE 0 THEN firm_gap_flags_temp=fgfcnt ELSE firm_gap_flags_temp=0 

  gap_flags_temp=tentative_gap_flags_temp+firm_gap_flags_temp

  ; GAP - DEWPT
  tgfc=WHERE(qc_flags(6,*) EQ 2, tgfcnt)
  IF tgfcnt GE 0 THEN tentative_gap_flags_dpt=tgfcnt ELSE tentative_gap_flags_dpt=0 
  fgfc=WHERE(qc_flags(6,*) EQ 1, fgfcnt)
  IF fgfcnt GE 0 THEN firm_gap_flags_dpt=fgfcnt ELSE firm_gap_flags_dpt=0 

  gap_flags_dpt=tentative_gap_flags_dpt+firm_gap_flags_dpt

  ; GAP - SLP
  tgfc=WHERE(qc_flags(7,*) EQ 2, tgfcnt)
  IF tgfcnt GE 0 THEN tentative_gap_flags_slp=tgfcnt ELSE tentative_gap_flags_slp=0 
  fgfc=WHERE(qc_flags(7,*) EQ 1, fgfcnt)
  IF fgfcnt GE 0 THEN firm_gap_flags_slp=fgfcnt ELSE firm_gap_flags_slp=0 

  gap_flags_slp=tentative_gap_flags_slp+firm_gap_flags_slp

  ; CLIMATE - TEMP
  tclc=WHERE(qc_flags(24,*) EQ 2, tcfcnt)
  IF tcfcnt GE 0 THEN tentative_climate_flags_temp=tcfcnt ELSE tentative_climate_flags_temp=0
  fclc=WHERE(qc_flags(24,*) EQ 1, fcfcnt)
  IF fcfcnt GE 0 THEN firm_climate_flags_temp=fcfcnt ELSE firm_climate_flags_temp=0

  climate_flags_temp=tentative_climate_flags_temp+firm_climate_flags_temp

  ; CLIMATE - DEWP
  tclc=WHERE(qc_flags(25,*) EQ 2, tcfcnt)
  IF tcfcnt GE 0 THEN tentative_climate_flags_dewp=tcfcnt ELSE tentative_climate_flags_dewp=0
  fclc=WHERE(qc_flags(25,*) EQ 1, fcfcnt)
  IF fcfcnt GE 0 THEN firm_climate_flags_dewp=fcfcnt ELSE firm_climate_flags_dewp=0

  climate_flags_dewp=tentative_climate_flags_dewp+firm_climate_flags_dewp


  ;**************************************************
  ; Adjust for commenting in neighbour check where flags = -1
  ; 2 July 2012 RJHD
  ;**************************************************
  ; Temperature
  nntf=WHERE(qc_flags(41,*) EQ 1, nntcnt)

  ; Dewpoint
  nndf=WHERE(qc_flags(42,*) EQ 1, nndcnt)

  ; SLP
  nnsf=WHERE(qc_flags(43,*) EQ 1, nnscnt)


  PRINTF,1,station_ids(st),nt,failcount, TOTAL(qc_flags(0,*)),TOTAL(qc_flags(1,*)),TOTAL(qc_flags(2,*)),$
                   TOTAL(qc_flags(3,*)),TOTAL(qc_flags(4,*)),gap_flags_temp,gap_flags_dpt,$
		   gap_flags_slp,TOTAL(qc_flags(8,*)),TOTAL(qc_flags(9,*)),TOTAL(qc_flags(10,*)),$
		   TOTAL(qc_flags(11,*)),TOTAL(qc_flags(12,*)),TOTAL(qc_flags(13,*)),TOTAL(qc_flags(14,*)),$
		   TOTAL(qc_flags(15,*)),TOTAL(qc_flags(16,*)),TOTAL(qc_flags(17,*)),TOTAL(qc_flags(18,*)),$
		   TOTAL(qc_flags(19,*)),TOTAL(qc_flags(20,*)),TOTAL(qc_flags(21,*)),TOTAL(qc_flags(22,*)),$
		   TOTAL(qc_flags(23,*)),climate_flags_temp,climate_flags_dewp,TOTAL(qc_flags(26,*)),$
		   TOTAL(qc_flags(27,*)),TOTAL(qc_flags(28,*)),TOTAL(qc_flags(29,*)),TOTAL(qc_flags(30,*)),$
		   TOTAL(qc_flags(31,*)),TOTAL(qc_flags(32,*)),TOTAL(qc_flags(33,*)),TOTAL(qc_flags(34,*)),$
		   TOTAL(qc_flags(35,*)),TOTAL(qc_flags(36,*)),TOTAL(qc_flags(37,*)),TOTAL(qc_flags(38,*)),$
		   TOTAL(qc_flags(39,*)),TOTAL(qc_flags(40,*)),nntcnt,nndcnt,nnscnt,TOTAL(qc_flags(44,*)),$
		   TOTAL(qc_flags(45,*)),TOTAL(qc_flags(46,*)),TOTAL(qc_flags(47,*)),TOTAL(qc_flags(48,*)),$
		   TOTAL(qc_flags(49,*)),TOTAL(qc_flags(50,*)),TOTAL(qc_flags(51,*)),TOTAL(qc_flags(52,*)),$
		   TOTAL(qc_flags(53,*)),TOTAL(qc_flags(54,*)),TOTAL(qc_flags(55,*)),TOTAL(qc_flags(56,*)),$
		   TOTAL(qc_flags(57,*)),TOTAL(qc_flags(58,*)),TOTAL(qc_flags(59,*)),TOTAL(qc_flags(60,*)),$
		   FORMAT='(A12,63(I8))'


ENDFOR ; close station loop
PRINTF,1,'station','times','fails',test_arr,FORMAT='(A12,63(A8))'
CLOSE,1

RETURN

END
;************************************************************************
;                                 END
;************************************************************************
