;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;British Crown Copyright (c) 2012, the Met Office All rights reserved.
;
;Redistribution and use in source and binary forms, with or without 
;modification, are permitted provided that the following conditions 
;are met:
;
;Redistributions of source code must retain the above copyright notice, 
;this list of conditions and the following disclaimer. 
;
;Redistributions in binary form must reproduce the above copyright 
;notice, this list of conditions and the following disclaimer in the 
;documentation and/or other materials provided with the distribution. 
;
;Neither the name of the Met Office nor the names of its contributors 
;may be used to endorse or promote products derived from this software 
;without specific prior written permission. 
;
;THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
;"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
;LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
;A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
;HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
;SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
;LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
;DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
;THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
;(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
;OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

PRO REPORTING_ACCURACY, last_year, values, mdi, fdi, vres, remhist, times, vfreq, get_changes=get_changes, obs_accuracy
;*************************************************************

; NAME:
;     REPORTING_ACCURACY
;
; PURPOSE:
;     To calculate the reporting accuracy of the instrument.
;
; CALLING SEQUENCE:
;     reporting_accuracy, last_year, values, mdi, vres, remhist, get_changes=get_changes, obs_accuracy
;
; INPUTS:
;     last_year - keyword to set the last year of the observations
;
;     values - array containing the values of the observations
;
;     mdi - missing data indicator
;
;     fdi - flagged data indicator
;
;     vres - resolution of the instrument (reporting accuracy)
;
;     remhist - histogram of the remainders after integer subtraction
;
;     get_changes - keyword whether to activate the collection of when
;                   the reporting accuracy changed
;
;     obs_accuracy - array to contain the average reporting accuracy for each
;                    month for each observation
;
;
; METHOD:
;     Compares the ABS with the FIX value to get a remainder and
;     analyses the distribution
;
;
; OUTPUTS:
;
;     None - arrays returned to control script
;
; RESTRICTIONS:
;
;
;   
; HISTORY:
;      #24 - Created from the climatological_check script by Peter
;            Thorne and Kate Willett.
;            7 November 2010 RJHD
;      #33 - added the hourly reporting loop from the excess variance
;            check.
;      #107 - Added sequence to obtain the accuracy for each month,
;             and stored to array to then output into file at the end
;             9 November 2010 RJHD
;      #200 - changed last_year from optional keyword to required input
;             to remove the hardwiring of the end date
;             11 March 2011 RJHD
;      #237 - SVN information added to header
;             Added flagged data indicator tests
;             16 November 2011 RJHD
;      #245 - Adjusted reporting accuracy test for .0 and .5
;             23 November 2011 RJHD
;
;************************************************************************
;                    SVN Info
;$Rev:: 303                                           $:  Revision of last commit
;$Author:: rdunn                                      $:  Author of last commit
;$Date:: 2012-06-14 14:02:44 +0100 (Thu, 14 Jun 2012) $:  Date of last commit
;************************************************************************
;                                 START
;************************************************************************



;************************************************************************
; Just obtain the reporting accuracy for a QC subroutine
;************************************************************************


IF NOT KEYWORD_SET(get_changes) THEN BEGIN

    legal_values=WHERE(((values NE mdi) AND (values NE fdi)), legalcnt)
    
    IF legalcnt GE 1 THEN BEGIN
        remainders=ABS(values(legal_values)-FIX(ABS(values(legal_values)))) ; KATE SEP 09 - added ABS
                                ;RJHD 29 NOV 10 - added second ABS
        remhist=HISTOGRAM(remainders,min=-0.05,max=0.95,binsize=0.1)
        remhist=FLOAT(remhist)/FLOAT(TOTAL(remhist))
        
        ; Define based upon prevalence of reporting to .0 and .5 
        ;  As testing only for whole or 0.5 reporting,
        ;  Then either all is in .0 or half-half .0-.5
        ;  Hence upped limits from 0.2 to 0.3 
        IF remhist(0) GE 0.3 THEN BEGIN
            IF remhist(5) GE 0.15 THEN BEGIN
                ; Reporting most frequently to half degree precision 
                vres=0.5
            ENDIF ELSE BEGIN 
                ; No secondary peak at 0.5 so reporting largely to whole numbers
                vres=1.
            ENDELSE
        ENDIF ELSE BEGIN
            ; As the reporting to .0 is not greater than 20% we should treat as reporting evenly across all decimal places.
            vres=0.1
        ENDELSE
        ; sussing hour reporting frequency   
        hourrecs=FIX((times(legal_values)-DOUBLE(FLOOR(times(legal_values))))*24)
        histo=histogram(hourrecs,binsize=1,min=0,max=23)
        hourgots=WHERE((histo GT TOTAL(histo)/48.),cthrgts)
        IF (cthrgts EQ 24) THEN BEGIN
            vfreq=1
        ENDIF ELSE IF (cthrgts GE 12) THEN BEGIN
            vfreq=2
        ENDIF ELSE IF (cthrgts GE 8) THEN BEGIN
            vfreq=3
        ENDIF ELSE vfreq=4
    ENDIF ELSE  vres=0
    
;************************************************************************
; Obtain the reporting accuracy for each month in the data series
;  To be stored in output file.
;************************************************************************

ENDIF ELSE IF KEYWORD_SET(get_changes) THEN BEGIN

    obs_accuracy=FLTARR(2,N_ELEMENTS(values))

    MONTH_SETUP,last_year,nmonths,nyears,times,times,monthly_segment_starts,$
      monthly_segment_ends, monthly_segment_lengths
    
    monthly_segment_ends(nmonths-1)=monthly_segment_ends(nmonths-1)+1
    monthly_segment_starts=REFORM(monthly_segment_starts,12,nyears)
    monthly_segment_ends=REFORM(monthly_segment_ends,12,nyears)
    monthly_segment_lengths=REFORM(monthly_segment_lengths,12,nyears)
    
    FOR yr=0,nyears-1 DO BEGIN
        FOR m=0,11 DO BEGIN
            vres=0
            vfreq=0
            IF monthly_segment_lengths(m,yr) GT 0 THEN BEGIN
                temp_values=values(monthly_segment_starts(m,yr):monthly_segment_ends(m,yr))

                legal_values=WHERE(((temp_values NE mdi) AND (temp_values NE fdi)), legalcnt)

                IF legalcnt GE 1 THEN BEGIN
                
                    remainders=ABS(temp_values(legal_values)-FIX(ABS(temp_values(legal_values)))) ; KATE SEP 09 - added ABS
                    ;RJHD 29 NOV 10 - added second ABS
                    remhist=HISTOGRAM(remainders,min=-0.05,max=0.95,binsize=0.1)

                    IF TOTAL(remhist) NE 0 THEN BEGIN
                        ;;****************************
                        ; if the entire month not only to whole numbers
                        ;;****************************
                        remhist=FLOAT(remhist)/FLOAT(TOTAL(remhist))
                
                        ; Define based upon prevalence of reporting to .0 and .5 
                        ;  As testing only for whole or 0.5 reporting,
                        ;  Then either all is in .0 or half-half .0-.5
                        ;  Hence upped limits from 0.2 to 0.3 
                        IF remhist(0) GE 0.3 THEN BEGIN
                            IF remhist(5) GE 0.15 THEN BEGIN
                                ; Reporting most frequently to half degree precision 
                                vres=0.5
                            ENDIF ELSE BEGIN 
                                ; No secondary peak at 0.5 so reporting largely to whole numbers
                                vres=1.
                            ENDELSE
                        ENDIF ELSE BEGIN 
                            ; As the reporting to .0 is not greater than 20% we should treat as reporting evenly across all decimal places.
                            vres=0.1
                        ENDELSE
                    ENDIF ELSE vres=1.
                    ;****************************
                    ; Otherwise reporting only to whole numbers
                    ;****************************

                    ; sussing hour reporting frequency   
                    hourrecs=FIX((times(legal_values)-DOUBLE(FLOOR(times(legal_values))))*24)
                    histo=histogram(hourrecs,binsize=1,min=0,max=23)
                    hourgots=WHERE((histo GT TOTAL(histo)/48.),cthrgts)
                    IF (cthrgts EQ 24) THEN BEGIN
                        vfreq=1
                    ENDIF ELSE IF (cthrgts GE 12) THEN BEGIN
                        vfreq=2
                    ENDIF ELSE IF (cthrgts GE 8) THEN BEGIN
                        vfreq=3
                    ENDIF ELSE vfreq=4
                ENDIF
                obs_accuracy(0,monthly_segment_starts(m,yr):monthly_segment_ends(m,yr))=vres
                obs_accuracy(1,monthly_segment_starts(m,yr):monthly_segment_ends(m,yr))=vfreq
            ENDIF                    
        ENDFOR
    ENDFOR

ENDIF

RETURN

END

;************************************************************************
;                                 END
;************************************************************************
