;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;British Crown Copyright (c) 2012, the Met Office All rights reserved.
;
;Redistribution and use in source and binary forms, with or without 
;modification, are permitted provided that the following conditions 
;are met:
;
;Redistributions of source code must retain the above copyright notice, 
;this list of conditions and the following disclaimer. 
;
;Redistributions in binary form must reproduce the above copyright 
;notice, this list of conditions and the following disclaimer in the 
;documentation and/or other materials provided with the distribution. 
;
;Neither the name of the Met Office nor the names of its contributors 
;may be used to endorse or promote products derived from this software 
;without specific prior written permission. 
;
;THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
;"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
;LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
;A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
;HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
;SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
;LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
;DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
;THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
;(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
;OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

PRO REPORTING_CHANGES, last_year, times, values, mdi, fdi, changepoints, changenotes, type

;*************************************************************
; NAME:
;     REPORTING_CHANGES
;
; PURPOSE:
;     To calculate the reporting accuracy of the instrument.
;
; CALLING SEQUENCE:
;     reporting_accuracy, last_year, values, mdi, vres, changepoints, changenotes, type
;
; INPUTS:
;     last_year - keyword to set the last year of the observations
;
;     times - array containing the times of the observations
;
;     values - array containing the values of the observations
;
;     mdi - missing data indicator
;
;     fdi - flagged data indicator
;
;     changepoints - array containing information about change points.
;
;     changenotes - string array containing notes about change points.
;
;     type - name of variable being scanned
;
; METHOD:
;     Compares the ABS with the FIX value to get a remainder and
;     analyses the distribution
;
;
; OUTPUTS:
;
;     None - arrays returned to control script
;
; RESTRICTIONS:
;
;
;   
; HISTORY:
;      #246 - Created from Reporting_Accuracy.pro.  Outputs list of
;             change points when reporting resolution or frequency
;             changes
;             23 November 2011 RJHD
;     # 266 - Adjusted output to distinguish between no values and
;             insufficent values to calculate the accuracy
;             16 December 2011 RJHD
;     # 295 - Ensured no blank output for change-notes
;             19 March 2012 RJHD
;
;************************************************************************
;                    SVN Info
;$Rev:: 308                                           $:  Revision of last commit
;$Author:: rdunn                                      $:  Author of last commit
;$Date:: 2012-07-04 14:47:15 +0100 (Wed, 04 Jul 2012) $:  Date of last commit
;************************************************************************
;                                 START
;************************************************************************



;************************************************************************
; Obtain the reporting accuracy for each month in the data series
;  To be stored in output file.
;************************************************************************

  obs_accuracy=FLTARR(2,N_ELEMENTS(values))
  
  MONTH_SETUP,last_year,nmonths,nyears,times,times,monthly_segment_starts,$
              monthly_segment_ends, monthly_segment_lengths
  
  monthly_segment_ends(nmonths-1)=monthly_segment_ends(nmonths-1)+1
  monthly_segment_starts=REFORM(monthly_segment_starts,12,nyears)
  monthly_segment_ends=REFORM(monthly_segment_ends,12,nyears)
  monthly_segment_lengths=REFORM(monthly_segment_lengths,12,nyears)
  

  changepoints=FLTARR(4)

  changenotes=STRARR(1)
  changenotes='dummy'

  pvres=0.
  pvfreq=0
  phourgots=[0.]
  notes=''
  FOR yr=0,nyears-1 DO BEGIN
     FOR m=0,11 DO BEGIN
        vres=0.
        vfreq=0
        hourgots=[0.]
        IF monthly_segment_lengths(m,yr) GT 0 THEN BEGIN
           temp_values=values(monthly_segment_starts(m,yr):monthly_segment_ends(m,yr))
           
           legal_values=WHERE(((temp_values NE mdi) AND (temp_values NE fdi)), legalcnt)
           
           IF legalcnt GE 10 THEN BEGIN
              
              remainders=ABS(temp_values(legal_values)-FIX(ABS(temp_values(legal_values)))) 
              ; KATE SEP 09 - added ABS
              ; RJHD 29 NOV 10 - added second ABS

              remhist=HISTOGRAM(remainders,min=-0.05,max=0.95,binsize=0.1)
              IF TOTAL(remhist) NE 0 THEN BEGIN
                 ;****************************
                 ; if the entire month not only to whole numbers
                 ;****************************
                 remhist=FLOAT(remhist)/FLOAT(TOTAL(remhist))
                
                 ; Define based upon prevalence of reporting to .0 and .5 
                 ;  As testing only for whole or 0.5 reporting,
                 ;  Then either all is in .0 or half-half .0-.5
                 ;  Hence upped limits from 0.2 to 0.3 each
                 IF remhist(0) GE 0.3 THEN BEGIN
                    IF remhist(5) GE 0.15 THEN BEGIN
                       ; Reporting most frequently to half degree precision 
                       vres=0.5
                    ENDIF ELSE BEGIN 
                       ; No secondary peak at 0.5 so reporting largely to whole numbers
                       vres=1.
                    ENDELSE
                 ENDIF ELSE BEGIN 
                    ; As the reporting to .0 is not greater than 20% we should treat as reporting evenly across all decimal places.
                    vres=0.1
                 ENDELSE
              ENDIF ELSE vres=1.
              ;****************************
              ; Otherwise reporting only to whole numbers
              ;****************************

              ; sussing hour reporting frequency   
              hourrecs=FIX((times(legal_values)-DOUBLE(FLOOR(times(legal_values))))*24)
              histo=histogram(hourrecs,binsize=1,min=0,max=23)
              hourgots=WHERE((histo GT TOTAL(histo)/48.),cthrgts)
              
              IF cthrgts GE 2 THEN BEGIN

                 DiffSeries=hourgots[1:cthrgts-1]-hourgots[0:cthrgts-2]
                 notes=''
                 
                 UniqDiffs=UNIQUE(DiffSeries)
                 
                 IF N_ELEMENTS(UniqDiffs) EQ 1 THEN BEGIN
                    ; have a single hourly reporting frequency.
                    vfreq=UniqDiffs

                    ;  Check that all hours of that frequency are present.
                    ;    Will be missing one difference using above calculation
                    ;    Hence the "-1" in the test below.

                    IF N_ELEMENTS(hourgots) NE (24./UniqDiffs)-1 THEN BEGIN
                       ; all hours are not present
                       notes=type+' Observations at (GMT):'
                       FOR hr=0,N_ELEMENTS(hourgots)-1 DO BEGIN
                          notes=notes+' '+STRCOMPRESS(hourgots[hr],/remove_all)
                       ENDFOR
                    ENDIF ELSE notes=type+' All observations present at indicated frequency'
                    
                 ENDIF ELSE BEGIN
                    ; extract number of individual observations.
                    Intervals=FLTARR(N_ELEMENTS(UniqDiffs))
                    IntervalNo=N_ELEMENTS(UniqDiffs)
                    
                    FOR i=0,IntervalNo-1 DO BEGIN
                       dummy=WHERE(DiffSeries EQ UniqDiffs[i],udcnt)
                       Intervals[i]=udcnt
                    ENDFOR
                    
                    ; set the reporting frequency to that interval which
                    ;  appears the most often for this month

                    IF N_ELEMENTS(WHERE(Intervals EQ MAX(Intervals))) EQ 1 THEN BEGIN
                       ; if there is a clear most common interval
                       ;    take it
                       vfreq=UniqDiffs[WHERE(Intervals EQ MAX(Intervals))]
                    ENDIF ELSE BEGIN
                       ; a number of equally present intervals.
                       ;  Take the highest Interval value
                       ;  ie from 1,2&3 hrly, take 3hrly
                       vfreq=MAX(UniqDiffs[WHERE(Intervals EQ MAX(Intervals))])
                    ENDELSE
                    
                    notes=type+' Observations at (GMT):'
                    FOR hr=0,N_ELEMENTS(hourgots)-1 DO BEGIN
                       notes=notes+' '+STRCOMPRESS(hourgots[hr],/remove_all)
                    ENDFOR
                    
                 ENDELSE
              ENDIF ELSE IF cthrgts EQ 1 THEN BEGIN
                 ; only one peak in histogram - single observation
                 ;   per day

                 vfreq=24
                 notes=type+ 'Observations at (GMT):'
                 FOR hr=0,N_ELEMENTS(hourgots)-1 DO BEGIN
                    notes=notes+' '+STRCOMPRESS(hourgots[hr],/remove_all)
                 ENDFOR

              ENDIF ELSE BEGIN
                 ; no strong peak in histogram
                 vres=0
                 vfreq=0
                 notes=type
              ENDELSE

           ENDIF ELSE BEGIN
              ; not sufficient values
              vres=0
              vfreq=0
              notes=type
           ENDELSE
             
        ENDIF ELSE BEGIN
           ; no values in that month
           vres=-1
           vfreq=-1
           notes=type
        ENDELSE
        
        ; test if there has been a change from the previous month
        IF (Pvres NE vres) OR (pvfreq NE vfreq) OR NOT ARRAY_EQUAL(phourgots, hourgots) THEN BEGIN
           ;PRINT,m+1,yr+1973,vres,vfreq,'   ', notes

           ; echo the changes into the storage arrays
           IF changenotes[0] EQ 'dummy' THEN BEGIN
              changepoints=[m+1,yr+1973,vres,vfreq]
              changenotes=[notes]
           ENDIF ELSE BEGIN
              changepoints=[[changepoints],[m+1,yr+1973,vres,vfreq]]
              changenotes=[[changenotes],[notes]]

           ENDELSE
        ENDIF
        pvres=vres
        pvfreq=vfreq
        phourgots=hourgots
     ENDFOR
  ENDFOR

changenotes=REFORM(changenotes)
RETURN

END

;************************************************************************
;                                 END
;************************************************************************
