;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;British Crown Copyright (c) 2012, the Met Office All rights reserved.
;
;Redistribution and use in source and binary forms, with or without 
;modification, are permitted provided that the following conditions 
;are met:
;
;Redistributions of source code must retain the above copyright notice, 
;this list of conditions and the following disclaimer. 
;
;Redistributions in binary form must reproduce the above copyright 
;notice, this list of conditions and the following disclaimer in the 
;documentation and/or other materials provided with the distribution. 
;
;Neither the name of the Met Office nor the names of its contributors 
;may be used to endorse or promote products derived from this software 
;without specific prior written permission. 
;
;THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
;"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
;LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
;A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
;HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
;SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
;LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
;DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
;THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
;(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
;OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

PRO REPORTING_DISTRIBUTION, limit=limit, suffix=suffix, accept=accept,datadir=datadir

; NAME:
;     REPORTING_DISTRIBUTION
;
; PURPOSE:
;     Determine the percentage of observations which have different
;     reporting accuracies and frequencies.  The raw values are
;     calculated on a monthly level, so if a month contains, day 0.5
;     and 1.0 reporting, only one of these will be returned for the
;     entire month depending on the balance.  However with sufficient
;     years and stations, it should give a reasonable overview of the
;     spread of values.
;
; CALLING SEQUENCE:
;     REPORTING_DISTRIBUTION, limit=limit, suffix=suffix,
;     accept=accept, datadir=datadir
;
; INPUTS:
;     limit - keyword to limit to only a few stations and use the
;             station_info.txt file.
;
;     suffix - keyword to select which input files to use
;
;     accept - use only the accepted files - give filename as input.
;
;     datadir - netcdf data directory
;
; OUTPUTS:
;
;     None - files written to specified or current directory.
;
; RESTRICTIONS:
;
;
; HISTORY:
;     # 204 - Created
;             15 March 2011 RJHD
;     # 205 - Added "0" reporting accuracy for missing values and
;             updated header
;             15 March 2011 RJHD
;     # 237 - SVN information added to header
;             16 November 2011 RJHD
;     # 266 - Adjusted to work with new reporting accuray format.
;             16 December 2011 RJHD
;     # 289 - Added accept keyword to run on filtered stations only
;             23 January 2012 RJHD   
;   
;************************************************************************
;                    SVN Info
;$Rev:: 323                                           $:  Revision of last commit
;$Author:: rdunn                                      $:  Author of last commit
;$Date:: 2012-11-02 17:27:23 +0000 (Fri, 02 Nov 2012) $:  Date of last commit
;************************************************************************
;                                 START
;************************************************************************

cd, "./", current=pwd

;*******************
; Read in the list of stations to be considered.
;*******************
IF KEYWORD_SET(limit) THEN BEGIN
    Station_info_file=pwd+'/station_info.txt'
ENDIF ELSE BEGIN
    Station_info_file=pwd+'/kept_station_info.txt'
ENDELSE

nstation=nlines(Station_info_file)

station_ids=STRARR(nstation)
station_lats=FLTARR(nstation)
station_lons=FLTARR(nstation)
station_elevs=FLTARR(nstation)
id=''
st=0

OPENR,1,Station_info_file
WHILE NOT EOF(1) DO BEGIN
  READF,1,id,FORMAT='(A)'
  station_ids(st)=STRMID(id,0,12)
  station_lats(st)=FLOAT(STRCOMPRESS(STRMID(id,13,9),/remove_all))
  station_lons(st)=FLOAT(STRCOMPRESS(STRMID(id,23,9),/remove_all))
  station_elevs(st)=FLOAT(STRCOMPRESS(STRMID(id,33,9),/remove_all))
  st=st+1
ENDWHILE
CLOSE,1

;***************************************************************************
;    Use only the accepted files if switch activated
;***************************************************************************
IF KEYWORD_SET(accept) THEN BEGIN
   accept_file=pwd+'/'+accept

   nastation=NLINES(accept_file)

   accept_sts=STRARR(nastation)
   
   ast=0
   OPENR,1,accept_file
   WHILE NOT EOF(1) DO BEGIN
      READF,1,id,FORMAT='(A)'
      accept_sts(ast)=STRCOMPRESS(id,/remove_all)
      ast=ast+1
      
   ENDWHILE
   
   CLOSE,1

   MATCH,accept_sts,station_ids,suba,subb

   station_ids=station_ids(subb)
   station_lats=station_lats(subb)
   station_lons=station_lons(subb)
   station_elevs=station_elevs(subb)
   
   nstation=N_ELEMENTS(station_ids)

ENDIF

Ttemperature_accuracy=0.
Tdewpoint_accuracy=0.
Tslp_accuracy=0.

Ttemperature_Taccuracy=0.
Tdewpoint_Taccuracy=0.
Tslp_Taccuracy=0.



datadir=pwd+'/'+datadir+'/'

FOR st=0,nstation-1 DO BEGIN
    PRINT,'Station Number :',st+1,"/",nstation, FORMAT='(A-40,I4,A1,I4)'
    file=datadir+station_ids(st)+'_'+suffix+'.nc'

    fred=NCDF_OPEN(file,/nowrite)

    NCDF_CONTROL,0,/verbose
    betty=NCDF_INQUIRE(fred)


                                ;****************************************
                                ;  obtain information on the dimensions
                                ;****************************************
    
    tid=NCDF_DIMID(fred,'time')
    NCDF_DIMINQ,fred,tid,timedim,nt
    
    NCDF_VARGET,fred,'change_points',change_points
    NCDF_VARGET,fred,'change_notes',change_notes
    NCDF_CLOSE,fred  

    change_notes=STRING(change_notes)

    changes=N_ELEMENTS(change_points[0,*])


    ; values
    temperature_accuracy=FLTARR(4)
    dewpoint_accuracy=FLTARR(4)
    slp_accuracy=FLTARR(4)

    ; reporting
    temperature_Taccuracy=FLTARR(5)
    dewpoint_Taccuracy=FLTARR(5)
    slp_Taccuracy=FLTARR(5)

    NO_T=0
    NO_D=0
    NO_S=0
    

    FOR ch=0,changes-1 DO BEGIN
       entry=STRSPLIT(change_notes[ch],/extract)
       IF entry[0] EQ "Temp" THEN BEGIN

          ; print,change_points[*,ch], change_notes[ch]
          ; IF change_points[0,ch] EQ 1 AND change_points[1,ch] EQ 1973 THEN BEGIN
          IF NO_T EQ 0 THEN BEGIN
             ; first entry
             lastTtime=JULDAY(change_points[0,ch],1,change_points[1,ch])
             currentTacc=change_points[2,ch]
             currentTrep=change_points[3,ch]
             NO_T=1
             
          ENDIF ELSE BEGIN
             deltaT=JULDAY(change_points[0,ch],1,change_points[1,ch])-lastTtime
             lastTtime=JULDAY(change_points[0,ch],1,change_points[1,ch])
             ; 0.0 - unable to determine, -1.0 - no points to determine
             IF currentTacc EQ 0.0 THEN temperature_accuracy[0]=temperature_accuracy[0]+deltaT
             IF currentTacc EQ 0.1 THEN temperature_accuracy[1]=temperature_accuracy[1]+deltaT
             IF currentTacc EQ 0.5 THEN temperature_accuracy[2]=temperature_accuracy[2]+deltaT
             IF currentTacc EQ 1.0 THEN temperature_accuracy[3]=temperature_accuracy[3]+deltaT

             IF currentTrep EQ 0.0 THEN temperature_Taccuracy[0]=temperature_Taccuracy[0]+deltaT
             IF currentTrep EQ 1.0 THEN temperature_Taccuracy[1]=temperature_Taccuracy[1]+deltaT
             IF currentTrep EQ 2.0 THEN temperature_Taccuracy[2]=temperature_Taccuracy[2]+deltaT
             IF currentTrep EQ 3.0 THEN temperature_Taccuracy[3]=temperature_Taccuracy[3]+deltaT
             IF currentTrep EQ 4.0 THEN temperature_Taccuracy[4]=temperature_Taccuracy[4]+deltaT
             currentTacc=change_points[2,ch]
             currentTrep=change_points[3,ch]
          ENDELSE
             
       ENDIF

       IF entry[0] EQ "Dewp" THEN BEGIN
          ; print,change_points[*,ch], change_notes[ch]
          ; IF change_points[0,ch] EQ 1 AND change_points[1,ch] EQ 1973 THEN BEGIN
          IF NO_D EQ 0 THEN BEGIN
             ; first entry
             lastDtime=JULDAY(change_points[0,ch],1,change_points[1,ch])
             currentDacc=change_points[2,ch]
             currentDrep=change_points[3,ch]
             NO_D=1
          ENDIF ELSE BEGIN
             deltaT=JULDAY(change_points[0,ch],1,change_points[1,ch])-lastDtime
             lastDtime=JULDAY(change_points[0,ch],1,change_points[1,ch])
             IF currentDacc EQ 0.0 THEN dewpoint_accuracy[0]=dewpoint_accuracy[0]+deltaT
             IF currentDacc EQ 0.1 THEN dewpoint_accuracy[1]=dewpoint_accuracy[1]+deltaT
             IF currentDacc EQ 0.5 THEN dewpoint_accuracy[2]=dewpoint_accuracy[2]+deltaT
             IF currentDacc EQ 1.0 THEN dewpoint_accuracy[3]=dewpoint_accuracy[3]+deltaT

             IF currentDrep EQ 0.0 THEN dewpoint_Taccuracy[0]=dewpoint_Taccuracy[0]+deltaT
             IF currentDrep EQ 1.0 THEN dewpoint_Taccuracy[1]=dewpoint_Taccuracy[1]+deltaT
             IF currentDrep EQ 2.0 THEN dewpoint_Taccuracy[2]=dewpoint_Taccuracy[2]+deltaT
             IF currentDrep EQ 3.0 THEN dewpoint_Taccuracy[3]=dewpoint_Taccuracy[3]+deltaT
             IF currentDrep EQ 4.0 THEN dewpoint_Taccuracy[4]=dewpoint_Taccuracy[4]+deltaT
             currentDacc=change_points[2,ch]
             currentDrep=change_points[3,ch]
             
          ENDELSE
          
       ENDIF
       IF entry[0] EQ "SLP" THEN BEGIN
          ; print,change_points[*,ch], change_notes[ch]
          ; IF change_points[0,ch] EQ 1 AND change_points[1,ch] EQ 1973 THEN BEGIN
          IF NO_S EQ 0 THEN BEGIN
             ; first entry
             lastStime=JULDAY(change_points[0,ch],1,change_points[1,ch])
             currentSacc=change_points[2,ch]
             currentSrep=change_points[3,ch]
             NO_S=1
          ENDIF ELSE BEGIN
             deltaT=JULDAY(change_points[0,ch],1,change_points[1,ch])-lastStime
             lastDtime=JULDAY(change_points[0,ch],1,change_points[1,ch])
             IF currentSacc EQ 0.0 THEN slp_accuracy[0]=slp_accuracy[0]+deltaT
             IF currentSacc EQ 0.1 THEN slp_accuracy[1]=slp_accuracy[1]+deltaT
             IF currentSacc EQ 0.5 THEN slp_accuracy[2]=slp_accuracy[2]+deltaT
             IF currentSacc EQ 1.0 THEN slp_accuracy[3]=slp_accuracy[3]+deltaT

             IF currentSrep EQ 0.0 THEN slp_Taccuracy[0]=slp_Taccuracy[0]+deltaT
             IF currentSrep EQ 1.0 THEN slp_Taccuracy[1]=slp_Taccuracy[1]+deltaT
             IF currentSrep EQ 2.0 THEN slp_Taccuracy[2]=slp_Taccuracy[2]+deltaT
             IF currentSrep EQ 3.0 THEN slp_Taccuracy[3]=slp_Taccuracy[3]+deltaT
             IF currentSrep EQ 4.0 THEN slp_Taccuracy[4]=slp_Taccuracy[4]+deltaT
             currentSacc=change_points[2,ch]
             currentSrep=change_points[3,ch]
             
          ENDELSE
             
       ENDIF

    ENDFOR


; Normalise

    IF TOTAL(temperature_accuracy) GT 0 THEN $
       temperature_accuracy=temperature_accuracy/TOTAL(temperature_accuracy)
    IF TOTAL(dewpoint_accuracy) GT 0 THEN $
       dewpoint_accuracy=dewpoint_accuracy/TOTAL(dewpoint_accuracy)
    IF TOTAL(slp_accuracy) GT 0 THEN $
       slp_accuracy=slp_accuracy/TOTAL(slp_accuracy)

    IF TOTAL(temperature_Taccuracy) GT 0 THEN $
       temperature_Taccuracy=temperature_Taccuracy/TOTAL(temperature_Taccuracy)
    IF TOTAL(dewpoint_Taccuracy) GT 0 THEN $
       dewpoint_Taccuracy=dewpoint_Taccuracy/TOTAL(dewpoint_Taccuracy)
    IF TOTAL(slp_Taccuracy) GT 0 THEN $
       slp_Taccuracy=slp_Taccuracy/TOTAL(slp_Taccuracy)



    Ttemperature_accuracy=Ttemperature_accuracy+temperature_accuracy
    Tdewpoint_accuracy=Tdewpoint_accuracy+dewpoint_accuracy
    Tslp_accuracy=Tslp_accuracy+slp_accuracy

    Ttemperature_Taccuracy=Ttemperature_Taccuracy+temperature_Taccuracy
    Tdewpoint_Taccuracy=Tdewpoint_Taccuracy+dewpoint_Taccuracy
    Tslp_Taccuracy=Tslp_Taccuracy+slp_Taccuracy
    
                                ;****************************************
                                ;  count where the accuracy matches te
                                ;  relevant criteria and sum to containers.
                                ;****************************************
        
ENDFOR


PRINT, 'TEMPERATURE'

totaltd=100.*Ttemperature_accuracy/TOTAL(Ttemperature_accuracy)
totaltt=100.*Ttemperature_Taccuracy/TOTAL(Ttemperature_Taccuracy)

PRINT, '0.0C Reporting  : ', totaltd[0], ' %', FORMAT='(A,F5.1, A)'
PRINT, '0.1C Reporting  : ', totaltd[1], ' %', FORMAT='(A,F5.1, A)'
PRINT, '0.5C Reporting  : ', totaltd[2], ' %', FORMAT='(A,F5.1, A)'
PRINT, '1.0C Reporting  : ', totaltd[3], ' %', FORMAT='(A,F5.1, A)'
PRINT, '0 hr Reporting  : ', totaltt[0], ' %', FORMAT='(A,F5.1, A)'
PRINT, '1 hr Reporting  : ', totaltt[1], ' %', FORMAT='(A,F5.1, A)'
PRINT, '2 hr Reporting  : ', totaltt[2], ' %', FORMAT='(A,F5.1, A)'
PRINT, '3 hr Reporting  : ', totaltt[3], ' %', FORMAT='(A,F5.1, A)'
PRINT, '4 hr Reporting  : ', totaltt[4], ' %', FORMAT='(A,F5.1, A)'

PRINT, 'DEWPOINT'

totaldd=100.*Tdewpoint_accuracy/TOTAL(Tdewpoint_accuracy)
totaldt=100.*Tdewpoint_Taccuracy/TOTAL(Tdewpoint_Taccuracy)

PRINT, '0.0C Reporting  : ', totaldd[0], ' %', FORMAT='(A,F5.1, A)'
PRINT, '0.1C Reporting  : ', totaldd[1], ' %', FORMAT='(A,F5.1, A)'
PRINT, '0.5C Reporting  : ', totaldd[2], ' %', FORMAT='(A,F5.1, A)'
PRINT, '1.0C Reporting  : ', totaldd[3], ' %', FORMAT='(A,F5.1, A)'
PRINT, '0 hr Reporting  : ', totaldt[0], ' %', FORMAT='(A,F5.1, A)'
PRINT, '1 hr Reporting  : ', totaldt[1], ' %', FORMAT='(A,F5.1, A)'
PRINT, '2 hr Reporting  : ', totaldt[2], ' %', FORMAT='(A,F5.1, A)'
PRINT, '3 hr Reporting  : ', totaldt[3], ' %', FORMAT='(A,F5.1, A)'
PRINT, '4 hr Reporting  : ', totaldt[4], ' %', FORMAT='(A,F5.1, A)'

PRINT, 'SLP'

totalsd=100.*Tslp_accuracy/TOTAL(Tslp_accuracy)
totalst=100.*Tslp_Taccuracy/TOTAL(Tslp_Taccuracy)

PRINT, '0.0hPa Reporting: ', totalsd[0], ' %', FORMAT='(A,F5.1, A)'
PRINT, '0.1hPa Reporting: ', totalsd[1], ' %', FORMAT='(A,F5.1, A)'
PRINT, '0.5hPa Reporting: ', totalsd[2], ' %', FORMAT='(A,F5.1, A)'
PRINT, '1.0hPa Reporting: ', totalsd[3], ' %', FORMAT='(A,F5.1, A)'
PRINT, '0 hr Reporting  : ', totalst[0], ' %', FORMAT='(A,F5.1, A)'
PRINT, '1 hr Reporting  : ', totalst[1], ' %', FORMAT='(A,F5.1, A)'
PRINT, '2 hr Reporting  : ', totalst[2], ' %', FORMAT='(A,F5.1, A)'
PRINT, '3 hr Reporting  : ', totalst[3], ' %', FORMAT='(A,F5.1, A)'
PRINT, '4 hr Reporting  : ', totalst[4], ' %', FORMAT='(A,F5.1, A)'

stop
RETURN

END
;************************************************************************
;                                 END
;************************************************************************
