;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;British Crown Copyright (c) 2012, the Met Office All rights reserved.
;
;Redistribution and use in source and binary forms, with or without 
;modification, are permitted provided that the following conditions 
;are met:
;
;Redistributions of source code must retain the above copyright notice, 
;this list of conditions and the following disclaimer. 
;
;Redistributions in binary form must reproduce the above copyright 
;notice, this list of conditions and the following disclaimer in the 
;documentation and/or other materials provided with the distribution. 
;
;Neither the name of the Met Office nor the names of its contributors 
;may be used to endorse or promote products derived from this software 
;without specific prior written permission. 
;
;THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
;"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
;LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
;A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
;HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
;SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
;LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
;DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
;THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
;(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
;OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

PRO SPIKE_PLOT,i,n,ntimes,columns,station_id,values,times,mdi,fdi,FigCounter

; NAME:
;     SPIKE_PLOT
;
; PURPOSE:
;     To search plot spikes found in the timeseries
;
; CALLING SEQUENCE:
;     spike_plot,i,n,ntimes,columns,station_id,values,times,mdi,FigCounter
;
; INPUTS:
;     i - array counter
;
;     n - counter as to how many points to highlight
;
;     ntimes - number of observations
;
;     columns - flag column identifier
;
;     station_id - station name
;
;     values - array of the values
;
;     times - array of the times
;
;     mdi - missing data indicator
;
;     fdi - flagged data indicator
;
;     FigCounter - Figure counter
;
; HISTORY:
;     #87 - creation - 19 November 2010 RJHD
;     #237 - added fdi - 17 November 2011 RJHD
;************************************************************************
;                                 START
;************************************************************************

RANGE=30

IF i<RANGE THEN MIN_X=0 ELSE MIN_X=i-RANGE
IF ntimes-1-i<RANGE THEN MAX_X=ntimes-1 ELSE MAX_X=i+RANGE

CASE columns OF
    27: BEGIN
        label="temperature"
    END
    28: BEGIN
        label="dewpoint"
    END
    29: BEGIN
        label="slp"
    END
ENDCASE

set_plot,'PS'
tvlct,0,0,0,0                   ; black for kept datapoints
cd, "./", current=outdirplot
device,filename=outdirplot+'/images/'+station_id+'_SpikeCheck_'+label+'_'+string(FigCounter,FORMAT='(I2.2)')+'.eps'$
  ,/color,/encapsulate,/helvetica,/bold,/landscape,ysize=20,xsize=26
!P.Font=0
!P.Thick=4
!P.Multi=[0,1,1]
plotsym,3,1.2
temp_values=values(MIN_X:MAX_X)
useful_values=WHERE(((temp_values NE mdi) AND (temp_values NE fdi)), usefulcnt)
IF usefulcnt GE 1 THEN BEGIN
    MINY=MIN(temp_values(useful_values))
    MAXY=MAX(temp_values(useful_values))
    IF label EQ 'SLP' THEN BEGIN
        MINY=MINY*1.01
        MAXY=MAXY*0.99
    ENDIF ELSE BEGIN
        IF MINY LT 0 THEN MINY=MINY*1.1 ELSE MINY=MINY*0.9
        IF MAXY LT 0 THEN MAXY=MAXY*0.9 ELSE MAXY=MAXY*1.1
    ENDELSE
    
    dummy=LABEL_DATE(DATE_FORMAT=['%D/%N/%Y'])
    start_date=JULDAY(1, 1, 1973, 0)

    plot,times(MIN_X:MAX_X)+start_date,values(MIN_X:MAX_X),min_value=-100,xstyle=1,ystyle=1,ytitle=label,xtitle="time",xtickformat='LABEL_DATE',$
      title="Spike_Check - Diagnostic Timeseries - "+label,charsize=1,ythick=4,xthick=4,psym=2,yrange=[MINY,MAXY], POSITION=[0.10, 0.10, 0.90, 0.90]
    oplot,times(MIN_X:MAX_X)+start_date,values(MIN_X:MAX_X),min_value=-100,xstyle=1,ystyle=1,ytitle=label,xtitle="time",xtickformat='LABEL_DATE',$
      title="Spike_Check - Diagnostic Timeseries - "+label,charsize=1,ythick=4,xthick=4,psym=0,yrange=[MINY,MAXY], POSITION=[0.10, 0.10, 0.90, 0.90]
    oplot,[times(i-n:i)+start_date],[values(i-n:i)],min_value=-100,xstyle=1,ystyle=1,ytitle=label,xtitle="time",xtickformat='LABEL_DATE',$
      title="Spike_Check - Diagnostic Timeseries - "+label,symsize=2,charsize=1,ythick=4,xthick=4,psym=4,color=239,yrange=[MINY,MAXY], POSITION=[0.10, 0.10, 0.90, 0.90]
    device,/close
    FigCounter=FigCounter+1
ENDIF
RETURN

END
;************************************************************************
;                                 END
;************************************************************************


;************************************************************************
;                                 START
;************************************************************************
;June 2009: Written by Chris Beaumont
;November 2010 downloaded by RJHD
function sign, x 
compile_opt idl2
on_error, 2 
return, 0 + (x gt 0) - (x lt 0) 
end 
;************************************************************************
;                                 END
;************************************************************************




PRO spike_check, last_year, times, full_times, values, mdi, fdi, columns, qc_flags, filter_flags, $
                 diagnostics, station_id

;*************************************************************
; program to read in data and check for large spikes occurring over short timescales
;*************************************************************
; train these tests to each individual station
; get all hourly to 3 hourly first differences
; rank them and get monthly interquartile ranges for all 
; uses 6 interquartile ranges to chuck out
;*************************************************************
; NAME:
;     SPIKE_CHECK
;
; PURPOSE:
;     To search for spikes in the data timeseries on short timescales
;
; CALLING SEQUENCE:
;     spike_check, last_year, times, values, mdi, fdi, columns, qc_flags,
;     diagnostics, station_id
;
; INPUTS:
;     last_year - keyword containing the last year of the data.
;
;     times - array containing the times
;
;     full_times - array containing the uncompressed times
;
;     values - array containing the values
;
;     mdi - missing data indicator
;
;     fdi - flagged data indicator
;
;     columns - integer to indicate which column in qc_flags to fill
;
;     qc_flags - array containing the flags for the station dataset
;
;     filter_flags - array containing all flags previously set on values
;
;     diagnostics - keyword to turn on diagnostic plots
;
;     station_id - the name of the station - for the plot files
;
; METHOD:
;     Uses a difference method to work out places where there is a
;     rapid change in one direction followed by a rapid change in the
;     other immediately thereafter.  Adjusted to the climatological
;     behaviour of the station.
;
; OUTPUTS:
;
;     None - files written to specified directory
;
; RESTRICTIONS:
;
;     
;   
; HISTORY:
;     #25 - initial upload - 8 November 2010 RJHD
;     #26 - sorted to run once per variable
;           8 November 2010 RJHD
;     #27 - commented fully and added diagnostic plots
;           9 November 2010 RJHD
;     #33 - adjusted reporting_accuracy call
;           9 November 2010 RJHD
;     #40 - added colour to diagnostic plots
;           10 November 2010 RJHD
;     #50 - updated diagnostic plots
;           15 November 2010 RJHD
;     #87 - removed plotting into separate function
;           19 November 2010 RJHD
;     #88 - now set to extract spikes of length 2, with twice the
;           critical value and time difference of 6hrs
;           19 November 2010 RJHD
;     #89 - now set to extract spikes of up to length 3, the same
;           critical value and time difference of up to 9hrs.  Also
;           adjusted the comparison to the following value, that the
;           difference be of opposite sign.  Added Function SIGN.
;           19 November 2010 RJHD
;     #90 - looks for single bad values at the beginning of data
;           runs. Gap has to be larger than 10 days
;           19 November 2010 RJHD
;     #109 - small adjustment to the search for spikes at the beginning
;            of datastreams
;            29 November 2010 RJHD
;     #117 - added code to search for spikes at the end of a data stream
;            as well.
;            1 December 2010 RJHD
;     #137 - Changed minimum critical value for time differences of 1
;            hr to be 6, even if the IQR is zero.  All other hours
;            left.  Station 718936 had >0.5 of all dewpoint
;            differences in Decembers = 0, and so spike check could
;            not be run
;            8 December 2010 RJHD
;     #163 - Added further test - that the differences either side of
;            the ones which are above critical (and as such define the
;            spike) must be below critical.  Also made the
;            requirement that the differences within a spike are small
;            more explicit.
;            6 January 2011 RJHD
;     #167 - Further IF loop to make sure that the ends of the data
;            are not overrun for n>1
;            11 January 2011 RJHD
;     #171 - Added filter flags to pass through the total flags set
;            for each value
;            14 January 2011 RJHD
;     #173 - Used filtered values to find spikes in timeseries
;            17 January 2011 RJHD
;     #178 - BUG - the check to make sure the search for the
;            differences surrounding the spikes didn't include n=1.
;            Fixed and amended
;            28 January 2011 RJHD
;     #181 - BUG - untouched values was being copied back, but the
;            difference series was not being recreated. Now done.
;            8 February 2011 RJHD
;     #200 - changed last_year from optional keyword to required input
;            to remove the hardwiring of the end date
;            11 March 2011 RJHD
;     #237 - SVN information added to header
;            Added fdi to mdi tests
;            16 November 2011 RJHD
;     #267 - Differences outside spike have to be under 1/2 critical
;            value - as had been documented, but not implemented
;            16 December 2011 RJHD
;     #300 - Value difference arrays set to double
;            7 June 2012 RJHD
;
;************************************************************************
;                    SVN Info
;$Rev:: 308                                           $:  Revision of last commit
;$Author:: rdunn                                      $:  Author of last commit
;$Date:: 2012-07-04 14:47:15 +0100 (Wed, 04 Jul 2012) $:  Date of last commit
;************************************************************************
;                                 START
;************************************************************************


;******************************************
; Mask values with past flags
;******************************************
untouched_values=values

past_flags=WHERE(filter_flags GT 0, pfcnt)

IF pfcnt GE 1 THEN BEGIN
    values(past_flags)=mdi
ENDIF

;**************************************************************
; Obtain the reporting accuracy of the instrument
;**************************************************************


REPORTING_ACCURACY, last_year, values,mdi,fdi,vres,remhist,times, vfreq


;**************************************************************
; Set up the monthly values
;**************************************************************

MONTH_SETUP,last_year,nmonths,nyears,times,full_times,monthly_segment_starts,$
  monthly_segment_ends, monthly_segment_lengths



ntimes=N_ELEMENTS(times)
monthly_segment_starts=REFORM(monthly_segment_starts,12,nyears)
monthly_segment_ends=REFORM(monthly_segment_ends,12,nyears)
monthly_segment_lengths=REFORM(monthly_segment_lengths,12,nyears)


;**************************************************************
; get the hour diffs 
;**************************************************************

tim_diffs=make_array(ntimes-1,/long)
value_diffs=make_array(ntimes-1,/double,value=mdi)

hour_val=1./24.

FOR i=1L,ntimes-1 DO BEGIN
;**************************************************************
; get difference series of all elements (in hours)
;**************************************************************
  tim_diffs(i-1)=ROUND((times(i)-times(i-1))/hour_val)
  IF ((values(i-1) NE mdi) AND (values(i) NE mdi)) AND ((values(i-1) NE fdi) AND (values(i) NE fdi)) THEN value_diffs(i-1)=values(i)-values(i-1)
ENDFOR

;**************************************************************
; critical differences for up to 9hr separations
;**************************************************************
critvals=FLTARR(9,12) ; leave as float as in units of IQR
critvals(*)=mdi

monthlocs=INTARR(ntimes)


;**************************************************************
; for each month
;**************************************************************
FOR m=0,11 DO BEGIN
    valid_yrs=WHERE(monthly_segment_lengths(m,*) NE -1, yrcnt)
    IF yrcnt GE 1 THEN BEGIN
        ;**************************************************************
        ; extract the differences for that months' data
        ;**************************************************************
        monthlocs(monthly_segment_starts(m,valid_yrs(0)):monthly_segment_ends(m,valid_yrs(0)))=m
        months_value_diffs=value_diffs(monthly_segment_starts(m,valid_yrs(0)):monthly_segment_ends(m,valid_yrs(0)))
        months_tim_diffs=tim_diffs(monthly_segment_starts(m,valid_yrs(0)):monthly_segment_ends(m,valid_yrs(0)))
        IF yrcnt GE 2 THEN BEGIN
            FOR y=1,yrcnt-1 DO BEGIN
                monthlocs(monthly_segment_starts(m,valid_yrs(y)):monthly_segment_ends(m,valid_yrs(y)))=m
                months_value_diffs=[months_value_diffs,value_diffs(monthly_segment_starts(m,valid_yrs(y)):monthly_segment_ends(m,valid_yrs(y)))]
                months_tim_diffs=[months_tim_diffs,tim_diffs(monthly_segment_starts(m,valid_yrs(y)):monthly_segment_ends(m,valid_yrs(y)))]
            ENDFOR
        ENDIF

        FOR delta=1,9 DO BEGIN
            ;**************************************************************
            ; find the distribution of value-differences
            ;**************************************************************
            datapoints=WHERE(months_tim_diffs EQ delta,tim1cnt)
            IF (tim1cnt GE 1) THEN BEGIN
                these_values=months_value_diffs(datapoints)
                tp=WHERE(((these_values NE mdi) AND (these_values NE fdi)), tc)
                IF tc GE 1 THEN value_dist1=these_values(tp) ELSE value_dist1=[mdi]
                
                pres=WHERE(((value_dist1 NE mdi) AND (value_dist1 NE fdi)),count)
                IF (count GE 100) THEN BEGIN
                    rnk_tmp=SORT(value_dist1)
                    value_dist1=value_dist1(rnk_tmp)
                    ;**************************************************************
                    ; critical value is 6x IQR for 1, 2 and 3 hourly differences
                    ;**************************************************************

                    critvals(delta-1,m)=6.*(value_dist1(ROUND(3.*FLOAT(count)/4.))-value_dist1(ROUND(FLOAT(count)/4.)))
                    
                    IF critvals(delta-1,m) EQ 0 AND delta EQ 1 THEN critvals(delta-1,m)=6. $
                    ELSE IF critvals(delta-1,m) EQ 0 THEN critvals(delta-1,m)=mdi
                ENDIF ELSE critvals(delta-1,m)=mdi
            ENDIF
        ENDFOR
    ENDIF
ENDFOR


;******************************************************
; Make these ranges no smaller than 5 times the typical reporting accuracy in many stations
;******************************************************
lowpoints=WHERE(ABS(critvals) LE 5.*vres, cl)
IF cl GE 1 THEN critvals(lowpoints)=5.*vres

;******************************************************
; Check hourly against 2 hourly. If its less than 2/3 of the value ramp it up.
; We get a crazy rejection rate otherwise and it makes no sense for hourly to be that much smaller!
;******************************************************
FOR m=0,11 DO BEGIN
  IF ((critvals(0,m) NE mdi) AND (critvals(1,m) NE mdi)) AND ((critvals(0,m) NE fdi) AND (critvals(1,m) NE fdi)) THEN BEGIN
    IF critvals(0,m)/critvals(1,m) LE 0.66 THEN critvals(0,m)=0.66*critvals(1,m)
  ENDIF
ENDFOR

FigCounter=0

;******************************************
; Find spikes on un_filtered values so revert
;  Critical values have been created with cleaned data
;******************************************
values=untouched_values
value_diffs=make_array(ntimes-1,/double,value=mdi)
FOR i=1L,ntimes-1 DO BEGIN
;**************************************************************
; get difference series of all elements (in hours)
;**************************************************************
  IF ((values(i-1) NE mdi) AND (values(i) NE mdi)) AND ((values(i-1) NE fdi) AND (values(i) NE fdi)) THEN value_diffs(i-1)=values(i)-values(i-1)
ENDFOR

;******************************************************
; go through dataset and pull out spikes exceeding set values
; aim to try and get spikes at the beginnings and ends of datasets
; do this before the main search.  Does not work for the very start or
; very end of the dataset.
;******************************************************
FOR i=1L,ntimes-2 DO BEGIN
    IF ((values(i) NE mdi) AND (values(i) NE fdi)) THEN BEGIN
        IF (ABS(tim_diffs(i-1)) GT 240.) AND (ABS(tim_diffs(i)) LT 3.) THEN BEGIN 
            ;******************************************************
            ; >10 day time difference before the set of the data, and a short one
            ; after.  Cannot do this test if it is an isolated point
            ;******************************************************

            ;******************************************************
            ; Get the average of the next 20 values
            ;******************************************************
            next_values=values(i+1:*)
            next_values_locs=WHERE(((next_values NE mdi) AND (next_values NE fdi)), next_cnt)

            IF next_cnt GE 1. THEN BEGIN
                
                temp_values=next_values(next_values_locs)
                IF N_ELEMENTS(temp_values) LT 11 THEN BEGIN
                    next_median=MEDIAN(temp_values)
                ENDIF ELSE next_median=MEDIAN(temp_values(0:10))
                
                next_diff=values(i+1)-values(i)
                median_diff=next_median-values(i)
                
                IF (critvals(tim_diffs(i)-1,monthlocs(i)) NE mdi) AND (critvals(tim_diffs(i)-1,monthlocs(i)) NE fdi) THEN BEGIN
                    ;******************************************************
                    ; If the critical values are well defined - can only 
                    ; use the following side of the possible spike
                    ;******************************************************
                    IF (ABS(median_diff) GT 2.*critvals(tim_diffs(i)-1,monthlocs(i))) AND $
                      (ABS(next_diff) GT critvals(tim_diffs(i)-1,monthlocs(i)))THEN BEGIN
                        ;******************************************************
                        ; If differences are greater than the critical values
                        ;   only bad values flagged - old flags not wiped
                        ;******************************************************
                        qc_flags(columns,i)=1
                        filter_flags(i)=1
                        IF KEYWORD_SET(diagnostics) THEN BEGIN
                            SPIKE_PLOT,i,0,ntimes,columns,station_id,values,times,mdi,fdi,FigCounter
                        ENDIF
                    ENDIF
                ENDIF
            ENDIF
        ENDIF
        IF (ABS(tim_diffs(i-1)) LT 3.) AND (ABS(tim_diffs(i)) GT 240.) THEN BEGIN 
            ;******************************************************
            ; >10 day time difference after the set of the data, and a short one
            ; before.  Cannot do this test if it is an isolated point
            ;******************************************************

            ;******************************************************
            ; Get the average of the previous and next 10 values
            ;******************************************************

            prev_values=values(0:i-1)
            prev_values_locs=WHERE(((prev_values ne mdi) AND (prev_values ne fdi)), prev_cnt)
            
            IF prev_cnt GE 1. THEN BEGIN
                temp_values=reverse(prev_values(prev_values_locs))
                IF N_ELEMENTS(temp_values) LT 11 THEN BEGIN
                    prev_median=MEDIAN(temp_values)
                ENDIF ELSE prev_median=MEDIAN(temp_values(0:10))
                
                median_diff=values(i)-prev_median
                prev_diff=values(i)-values(i-1)
                
                IF (critvals(tim_diffs(i-1)-1,monthlocs(i)) NE mdi) AND (critvals(tim_diffs(i-1)-1,monthlocs(i)) NE fdi) THEN BEGIN
                    ;******************************************************
                    ; If the critical values are well defined - can only 
                    ; use the leading side of the possible spike
                    ;******************************************************
                    IF (ABS(median_diff) GT 2.*critvals(tim_diffs(i-1)-1,monthlocs(i))) AND $
                      (ABS(prev_diff) GT critvals(tim_diffs(i-1)-1,monthlocs(i)))THEN BEGIN
                        ;******************************************************
                        ; If differences are greater than the critical values
                        ;   only bad values flagged - old flags not wiped
                        ;******************************************************
                        qc_flags(columns,i)=1
                        filter_flags(i)=1
                        IF KEYWORD_SET(diagnostics) THEN BEGIN
                            SPIKE_PLOT,i,0,ntimes,columns,station_id,values,times,mdi,fdi,FigCounter
                        ENDIF
                    ENDIF
                ENDIF
            ENDIF
        ENDIF
    ENDIF
ENDFOR



;******************************************************
; go through dataset and pull out spikes exceeding set values
;   point of interest is i = difference between i+1 and i
;******************************************************
FOR i=1L,ntimes-2 DO BEGIN
    ;******************************************************
    ; find spikes of length 1, 2 and 3 observations which exceed 
    ; the critical value and have time differences of 3, 6 and 9h
    ;******************************************************
    FOR n=1,3 DO BEGIN
        IF i GE n  THEN BEGIN

            ;******************************************************
            ; prevent overrun at eend of the array
            ; when n=2 or n=3
            ;******************************************************
            IF (n EQ 1 AND I LT ntimes-2 AND I GE 2) OR (n EQ 2 AND I LT ntimes-3 AND I GE 3) OR (n EQ 3 AND I LT ntimes-4 AND I GE 4) THEN BEGIN
                ;******************************************************
                ; if time differences <n*3hrs (else can have too much variation)
                ; and (if multi-point) the time differences within the spike
                ; are also small enough (including the ones outside of 
                ; the spike.  Further separations cannot be reliably investigated
                ;******************************************************
                IF (ABS(tim_diffs(i-n)) LE n*3.) AND (ABS(tim_diffs(i)) LE n*3.) AND (tim_diffs(i-n-1)-1 LT n*3.) AND (tim_diffs(i+1)-1 LT n*3.) AND $
                  ((n EQ 1) OR $
                   ((n EQ 2) AND (ABS(tim_diffs(i-n+1)) LE n*3.)) OR $
                   ((n EQ 3) AND (ABS(tim_diffs(i-n+1)) LE n*3.) AND (ABS(tim_diffs(i-n+2)) LE n*3.))) THEN BEGIN
                    ;******************************************************
                    ; if valid differences
                    ;******************************************************
                    IF (value_diffs(i-n) NE mdi) AND (critvals(tim_diffs(i-n)-1,monthlocs(i)) NE mdi) AND $
                       (value_diffs(i-n) NE fdi) AND (critvals(tim_diffs(i-n)-1,monthlocs(i)) NE fdi) THEN BEGIN
                        ;******************************************************
                        ; if exceed the critical differences
                        ;******************************************************
                        IF ABS(value_diffs(i-n)) GE critvals(tim_diffs(i-n)-1,monthlocs(i)) THEN BEGIN
                            ;******************************************************
                            ; Check that the following obs is in the 
                            ; other direction
                            ;******************************************************
                            IF SIGN(value_diffs(i)) NE SIGN(value_diffs(i-n))  THEN BEGIN
                                ;******************************************************
                                ; Check that the differences within
                                ; the spike are small - half the critical
                                ;   made explicit n=1 or 2 or 3 (6/1/11 RJHD)
                                ;******************************************************

                                IF (n EQ 1) $
                                  OR $
                                  ((n EQ 2) AND (ABS(value_diffs(i-n+1)) LT (critvals(tim_diffs(i-n+1)-1,monthlocs(i)) /2.))) $
                                  OR $
                                  ((n EQ 3) AND (ABS(value_diffs(i-n+1)) LT (critvals(tim_diffs(i-n+1)-1,monthlocs(i)) /2.)) AND $
                                   (ABS(value_diffs(i-n+2)) LT (critvals(tim_diffs(i-n+2)-1,monthlocs(i)) /2.)) ) THEN BEGIN

                                    IF (value_diffs(i) NE mdi) AND (critvals(tim_diffs(i)-1,monthlocs(i)) NE mdi) AND $
                                       (value_diffs(i) NE fdi) AND (critvals(tim_diffs(i)-1,monthlocs(i)) NE fdi) THEN BEGIN

                                       ;******************************************************
                                       ; Check that the following obs is at least half of the critval
                                       ;******************************************************
                                        IF ABS(value_diffs(i)) GE critvals(tim_diffs(i)-1,monthlocs(i)) /2. THEN BEGIN

                                            ;******************************************************
                                            ; new 6 Jan 2011 RJHD.  If differences
                                            ; surrounding the critical ones are
                                            ; small enough (to prevent curves
                                            ; being caught) - < 1/2 critical
                                            ;******************************************************
                                            IF ABS(value_diffs(i-n-1)) LE critvals(tim_diffs(i-n-1)-1,monthlocs(i))/2. THEN BEGIN
                                                IF ABS(value_diffs(i+1)) LE critvals(tim_diffs(i+1)-1,monthlocs(i))/2. THEN BEGIN


                                                    FOR col=0,n-1 DO BEGIN
                                                        ;****************************************
                                                        ; only bad values flagged - old ones not wiped
                                                        ;****************************************
                                                        qc_flags(columns,i-col)=1
                                                        filter_flags(i-col)=1
                                                    ENDFOR
                                                    IF KEYWORD_SET(diagnostics) THEN BEGIN
                                                        
                                                        SPIKE_PLOT,i,n-1,ntimes,columns,station_id,values,times,mdi,fdi,FigCounter
                                                    ENDIF
                                                    
                                                ENDIF
                                            ENDIF
                                        ENDIF
                                    ENDIF
                                ENDIF
                            ENDIF
                        ENDIF
                    ENDIF
                ENDIF
            ENDIF
        ENDIF
    ENDFOR

ENDFOR


RETURN

END   
	
;************************************************************************
;                                 END
;************************************************************************
