;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;British Crown Copyright (c) 2012, the Met Office All rights reserved.
;
;Redistribution and use in source and binary forms, with or without 
;modification, are permitted provided that the following conditions 
;are met:
;
;Redistributions of source code must retain the above copyright notice, 
;this list of conditions and the following disclaimer. 
;
;Redistributions in binary form must reproduce the above copyright 
;notice, this list of conditions and the following disclaimer in the 
;documentation and/or other materials provided with the distribution. 
;
;Neither the name of the Met Office nor the names of its contributors 
;may be used to endorse or promote products derived from this software 
;without specific prior written permission. 
;
;THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
;"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
;LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
;A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
;HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
;SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
;LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
;DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
;THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
;(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
;OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

PRO streak_check, last_year, times, h_times, values, mdi, fdi, type, qc_flags, filter_flags, full_times, full_h_times, $
		 full_values, diagnostics, station_id, flagged_values, mask=mask, oldflags=oldflags


; Program looks for streaks of either repeating values or repeating day strings
; and removes these if found. Looks for exact matches of strings (within tolerance) 
; either over a long period or a large number of values (depending on time resolution).
;
;*************************************************************
;*************************************************************
; Firstly ascertain the *typical* recording precision based upon the prevalence of given 
; decimal points within the legal values of the series for the recorded parameters. This then 
; has an implication for the streak length required to assign a real streak rather than a plausible
; string for all the tests.
;*************************************************************
;*************************************************************

; NAME:
;     STREAK_CHECK
;
; PURPOSE:
;     To check for streaks of repeating values or repeating day strings.
;
; CALLING SEQUENCE:
;     streak_check, last_year, times, values, mdi, fdi, type, qc_flags, full_times,
;     full_values, diagnostics, station_id, flagged_values,
;     mask=mask , oldflags=oldflags
;
; INPUTS:
;     last_year - keyword to set the last year of the observations
;
;     times - array containing the times of the observations in
;             fractions of days
;
;     times - array containing he times of the observations in hours
;
;     values - array containing the values of the observations
;
;     mdi - missing data indicator
;
;     fdi - flagged data indicator
;
;     type - integer describing which type of data is being
;               analysed 
;
;     qc_flags_gap - array containing the values of the flags for the
;                    test
;
;     filter_flags - array containing all flags previously set on values
;
;     full_times - array containing the uncompressed times of the
;                  observations in fractions of a day
;
;     full_h_times - array containing the uncompressed times of the
;                    observations in hours
;
;     full_values - array containing the uncompressed values of the observations
;
;     diagnostics - keyword for running diagnostic tests
;
;     station_id - station name for diagnostic files
;
;     flagged_values - array of values removed by the flags on masking
;
;     mask - keyword as to whether any masking rather than just
;            flagging of the data is performed.
;
;     oldflags - keyword to calculate the number of streak check flags
;                already set
;
; METHOD:
;     1) checks for consecutive repeating values
;     2) checks if one year has more repeating strings than expected
;     3) checks for repeats at a given hour across a number of days
;     4) checks for repeats for whole days - all 24 hourly values
;
;
; OUTPUTS:
;
;     None - arrays returned to control script
;
; RESTRICTIONS:
;
;
;   
; HISTORY:
;      #21 - Initial version as received in October 2010 RJHD
;      #22 - Commented up and set to work with a single value
;              Looped through in control script
;            5 November 2010 RJHD
;      #24 - added reporting_accuracy call.
;            7 November 2010 RJHD
;      #34 - updated reporting_accuracy call.
;            9 November 2010 RJHD
;      #40 - Added diagnostic plots - though very rudimentary at the moment
;            10 November 2010 RJHD
;      #50 - BUG - reordered qc_flags assignment.  Also updated the
;            diagnostic plots.
;            15 November 2010 RJHD
;      #124- Some BUGs in the plotting parts, esp for data in
;            the last year of the data acquisition.
;            03 December 2010 RJHD
;      #129- Added further diagnostic plot as the wiping of all
;            strings in a year seemed a little drastic.  A single
;            month view as an example can put minds at rest. 
;            06 December 2010 RJHD
;     #171 - Added filter flags to pass through the total flags set
;            for each value
;            14 January 2011 RJHD
;     #174 - Added oldflags keyword to allow the calculation of the
;            previously set flags.
;            18 January 2011 RJHD
;     #200 - changed last_year from optional keyword to required input
;            to remove the hardwiring of the end date
;            11 March 2011 RJHD
;     #237 - SVN information added to header
;            added flagged data indicator and flagged_values array functionality
;            16 November 2011 RJHD
;     #238 - Bugfix - line 686 - change AND to OR for testing EQ mdi
;            and fdi.
;            18 November 2011 RJHD
;     #299 - Added in times in hours for Matching
;            6 June 2012 RJHD
;
;************************************************************************
;                    SVN Info
;$Rev:: 308                                           $:  Revision of last commit
;$Author:: rdunn                                      $:  Author of last commit
;$Date:: 2012-07-04 14:47:15 +0100 (Wed, 04 Jul 2012) $:  Date of last commit
;************************************************************************
;                                 START
;************************************************************************

;*************************************************************
; Define based upon prevalence of reporting to .0 and .5 
;    and set up the bounding values. NB Wind has extra one.
;*************************************************************

REPORTING_ACCURACY, last_year, values,mdi,fdi,vres,remhist, times, vfreq

mdi_test=WHERE(((values NE mdi) AND (values NE fdi)), mdicnt)

IF mdicnt GE 1 THEN BEGIN

    wind_min_value=0.0          ; default for non-wind values

    CASE type OF
        0: BEGIN
        ; TEMPERATURES
            label="temperature"
            column=12
            columns2=16
            columns3=20
            IF remhist(0) GE 0.2 THEN BEGIN
                IF remhist(5) GE 0.15 THEN BEGIN
                ; Reporting most frequently to half degree precision 
                    nv_elements=30
                    ndays=10
                    nhr_repeats=20
                    ndayrpts=7
                ENDIF ELSE BEGIN ; No secondary peak at 0.5 so reporting largely to whole numbers
                    nv_elements=40
                    ndays=14
                    nhr_repeats=25
                    ndayrpts=10
                ENDELSE
            ENDIF ELSE BEGIN ; As the reporting to .0 is not greater than 20% we should treat as reporting evenly across all decimal places.
                nv_elements=24
                ndays=7
                nhr_repeats=15
                ndayrpts=5
            ENDELSE
            flagged_col=0
        END
        
        1: BEGIN
        ; DEWPOINTS
            label="dewpoints"
            column=13
            columns2=17
            columns3=21
            IF remhist(0) GE 0.2 THEN BEGIN
                IF remhist(5) GE 0.15 THEN BEGIN
                ; Reporting most frequently to half degree precision 
                    nv_elements=60
                    ndays=10
                    nhr_repeats=20
                    ndayrpts=7
                ENDIF ELSE BEGIN ; No secondary peak at 0.5 so reporting largely to whole numbers
                    nv_elements=80
                    ndays=14
                    nhr_repeats=25
                    ndayrpts=10
                ENDELSE
            ENDIF ELSE BEGIN ; As the reporting to .0 is not greater than 20% we should treat as reporting evenly across all decimal places.
                nv_elements=48
                ndays=7
                nhr_repeats=15
                ndayrpts=5
            ENDELSE
            flagged_col=1
        END
    
        2: BEGIN
        ; WINDSPEEDS
            label="wind"
            column=14
            columns2=18
            columns3=22
            IF remhist(0) GE 0.2 THEN BEGIN
                IF remhist(5) GE 0.15 THEN BEGIN
                ; Reporting most frequently to half degree precision 
                    nv_elements=30
                    ndays=10
                    nhr_repeats=20
                    ndayrpts=7
                    wind_min_value=0.5
                ENDIF ELSE BEGIN ; No secondary peak at 0.5 so reporting largely to whole numbers
                    nv_elements=40
                    ndays=14
                    nhr_repeats=25
                    ndayrpts=10
                    wind_min_value=1.0
                ENDELSE
            ENDIF ELSE BEGIN ; As the reporting to .0 is not greater than 20% we should treat as reporting evenly across all decimal places.
                nv_elements=24
                ndays=7
                nhr_repeats=15
                ndayrpts=5
                wind_min_value=0.5
            ENDELSE
            flagged_col=7
        END
        
        3: BEGIN
        ; SLP
            label="slp"
            column=15
            columns2=19
            columns3=23
            IF remhist(0) GE 0.2 THEN BEGIN
                IF remhist(5) GE 0.15 THEN BEGIN
                ; Reporting most frequently to half degree precision 
                    nv_elements=100
                    ndays=21
                    nhr_repeats=20
                    ndayrpts=7
                ENDIF ELSE BEGIN ; No secondary peak at 0.5 so reporting largely to whole numbers
                    nv_elements=120
                    ndays=28
                    nhr_repeats=25
                    ndayrpts=10
                ENDELSE
            ENDIF ELSE BEGIN ; As the reporting to .0 is not greater than 20% we should treat as reporting evenly across all decimal places.
                nv_elements=72
                ndays=14
                nhr_repeats=15
                ndayrpts=5
            ENDELSE
            flagged_col=2
        END
    ENDCASE



    IF KEYWORD_SET(oldflags) THEN BEGIN
        prev_flags1=WHERE(qc_flags(column,*) GT 0, pfcnt1)
        prev_flags2=WHERE(qc_flags(columns2,*) GT 0, pfcnt2)
        prev_flags3=WHERE(qc_flags(columns3,*) GT 0, pfcnt3)
    ENDIF

    

;*************************************************************
; For straight strings we want to ascertain their changing prevalence over time and remove all strings that
; are clustered together more than we would expect. To do this we need to initialise some variables here
; and then do some analysis later.
;*************************************************************

    value_starts=[0]
    value_lengths=[0]
    
    nt=N_ELEMENTS(times)

;*************************************************************
; Repeat of exact values
;
; Temperatures:
;      The same for either 24 consecutive valid values or for over a week
;
; Dewpoints:
;      The same for either 48 consecutive valid values or for over a week.
; Larger because dewpoint is more conservative than temperature.
; Difference set in initial CASE structure.
;
; Windspeeds:
;      The same for either 24 consecutive valid values or for over a week
; discounting any windspeeds under 0.5 m/s
;
; SLP:
;      Same for either 72 consecutive valid values or for over 2 weeks
;
;*************************************************************
    FigCounter=0
    this_value=values(0)
    same_val_points=[0]
    same_vals=[values(0)]
    FOR t=1L,nt-1 DO BEGIN

        IF (values(t) NE mdi) AND (values(t) NE fdi) THEN BEGIN
        ; for all observations, for all valid values..
            
            IF values(t) NE this_value THEN BEGIN
            ;********************************
            ; is the just after a set of repeating values
            ;********************************
                stringlength=N_ELEMENTS(same_val_points)
            
                IF stringlength GE 10 THEN BEGIN
                ; more than 10 points having the same value

                    IF type NE 2 OR (type EQ 2 AND this_value GT wind_min_value) THEN BEGIN ; was GE KATE
                        value_starts=[value_starts,times(same_val_points(0))] ; start of string
                        value_lengths=[value_lengths,stringlength] ; length of string
                        time_diff=times(same_val_points(stringlength-1))-times(same_val_points(0)) ; duration of string

                    ;********************************
                    ; Is the string of repeating values either longer than n_elements or ndays 
                    ; - if so assign as missing. 
                    ;********************************
                        IF (stringlength GE nv_elements) OR (time_diff GE ndays) THEN BEGIN 
                            qc_flags(column,same_val_points)=1
                        
                            filter_flags(same_val_points)=1

                        ;********************************
                        ; diagnostic plot per streak
                        ;********************************
                            IF KEYWORD_SET(diagnostics) THEN BEGIN
                                
                                RANGE=1000
                                
                                IF same_val_points(0)-RANGE LT 0 THEN MIN_X=0 ELSE MIN_X=same_val_points(0)-RANGE
                                IF (nt-1-t) LT RANGE THEN MAX_X=nt-1 ELSE MAX_X=t+RANGE
                                
                                set_plot,'PS'
                                cd, "./", current=outdirplot
                                device,filename=outdirplot+'/images/'+station_id+'_Streak_'+label+'_'+string(FigCounter,FORMAT='(I2.2)')+'.eps'$
                                  ,/color,/encapsulate,/helvetica,/bold,/landscape,ysize=20,xsize=26

                                dummy=LABEL_DATE(DATE_FORMAT=['%D/%N/%Y'])
                                start_date=JULDAY(1, 1, 1973, 0)

                                !P.Font=0
                                !P.Thick=4
                                plotsym,3,1.2
                                !P.Multi=[0,1,1]
                                plot,times(MIN_X:MAX_X)+start_date,values(MIN_X:MAX_X),min_value=-100,xstyle=1,ystyle=1,ytitle=label,xtitle="time",xtickformat='LABEL_DATE',$
                                  title="Streak - Diagnostic Timeseries - "+label,charsize=1,ythick=4,xthick=4,psym=2, POSITION=[0.10, 0.10, 0.90, 0.90]
                                oplot,times(MIN_X:MAX_X)+start_date,values(MIN_X:MAX_X),min_value=-100,xstyle=1,ystyle=1,ytitle=label,xtitle="time",xtickformat='LABEL_DATE',$
                                  title="Streak - Diagnostic Timeseries - "+label,charsize=1,ythick=4,xthick=4,psym=0, POSITION=[0.10, 0.10, 0.90, 0.90]
                                oplot,times(same_val_points)+start_date,values(same_val_points),min_value=-100,xstyle=1,ystyle=1,ytitle=label,xtitle="time",xtickformat='LABEL_DATE',$
                                  title="Streak - Diagnostic Timeseries - "+label,charsize=1,ythick=4,xthick=4,psym=2, POSITION=[0.10, 0.10, 0.90, 0.90], color=239
                                
                                device,/close
                                
                                FigCounter=FigCounter+1
                                
                            ENDIF
                         ;********************************
                        ; if masking selected, apply
                        ;********************************
                            IF KEYWORD_SET(mask) THEN BEGIN
                               flagged_values[flagged_col, same_val_points]=values[same_val_points]
                               values(same_val_points)=fdi
                            ENDIF
                        ENDIF
                    ENDIF
                ENDIF
            ;********************************
            ; Reset string length and last values
            ;********************************
                this_value=values(t)
                same_val_points=[t]
                same_vals=[values(t)]
            ENDIF ELSE BEGIN
                same_val_points=[same_val_points,t]
                same_vals=[same_vals,values(t)]
            ENDELSE
        ENDIF
    ENDFOR


;**************************************************************
; Ascertain whether prevalence in a given year is more than expected
; for each string type. 
; Based as a proportion of the string data
; to the total data for each variable in each year.
;**************************************************************

    nyears=last_year-1973

    year_divides=DBLARR(nyears+2)
    FOR y=0,(nyears+1) DO year_divides(y)=JULDAY(1, 1, 1974+y, 0)-JULDAY(1, 1, 1973, 0)
    year_proportions_value=FLTARR(nyears)
    year_proportions_value(*)=-999.

;**************************************************************
; to extract a single year:
; all data >= year_x and all data <= year_x +1
;    ie >2001 and <2002 gets all of 2001's data
;**************************************************************


    FOR y=0,nyears DO BEGIN
        yearpoints=WHERE(times GE year_divides(y), ycnt)

        IF ycnt GE 1 THEN BEGIN
            yearpoints2=WHERE(times(yearpoints) LE year_divides(y+1), ycnt1)

            IF ycnt1 GE 1 THEN BEGIN
            ;******************************
            ; Were there any data in the extracted year
            ;******************************
                legalvalues=WHERE(((values(yearpoints(yearpoints2)) NE mdi) AND (values(yearpoints(yearpoints2)) NE fdi)), tyc)

                IF tyc GE 200 THEN BEGIN
                ;********************************************************
                ; Did any strings start during this year?
                ;********************************************************
                    strings1=WHERE(value_starts GE year_divides(y), ts1)
                    
                    IF ts1 GE 1 THEN BEGIN
                        strings2=WHERE(value_starts(strings1) LE year_divides(y+1), ts2)

                    ;********************************************************
                    ; how much of the year's data were in repeating strings
                    ;********************************************************
                        IF ts2 GE 1 THEN year_proportions_value(y)=FLOAT(TOTAL(value_lengths(strings1(strings2))))/FLOAT(tyc)$
                        ELSE year_proportions_value(y)=0.
                    ENDIF ELSE year_proportions_value(y)=0.
                ENDIF
            ENDIF
        ENDIF
    ENDFOR
    
    plot_points=[-1]

;**************************************************************
; Are any years substantially unusual in their propensity for strings 
; when compared to all other years? If so delete all
; the strings that existed within that year.
;**************************************************************
    years=WHERE(year_proportions_value NE -999., value_c)
    IF value_c GE 10 THEN BEGIN
     ;*****************************************************
     ; find the average proportion a year has repeating strings
     ;*****************************************************
        medval=MEDIAN(year_proportions_value(years))

        IF medval LE 0.005 THEN medval=0.005 ; if small, then 0
    
        badyears=WHERE(year_proportions_value(years) GE 5.*medval,bad_value_c)
     ;*****************************************************
     ; if more than 5x the average, then flag
     ;*****************************************************
        IF bad_value_c GE 1 THEN BEGIN
        
            FOR bad=0,bad_value_c-1 DO BEGIN
            
      ;*****************************************************
      ; Find all the streak identified values in this year and flag them.
      ;*****************************************************
                stringids=WHERE(value_starts GE year_divides(years(badyears(bad))))
                stringids2=WHERE(value_starts(stringids) LE year_divides(years(badyears(bad))+1), stdel)
                
                FOR strid=0,stdel-1 DO BEGIN
                    tloc=WHERE(times EQ value_starts(stringids(stringids2(strid))))
                    
                    IF qc_flags(column,tloc) NE 1 THEN BEGIN
                        same_val_points=[tloc]
                        loc_on=1
                        
                        WHILE N_ELEMENTS(same_val_points) NE value_lengths(stringids(stringids2(strid))) DO BEGIN
                            IF (values(tloc+loc_on) NE mdi) AND (values(tloc+loc_on) NE fdi) THEN same_val_points=[same_val_points,tloc+loc_on]
                            loc_on=loc_on+1
                        ENDWHILE
                        qc_flags(column,same_val_points)=1
                        
                        filter_flags(same_val_points)=1
                        
                        IF KEYWORD_SET(diagnostics) THEN BEGIN
                    ;********************************
                    ; Read streaks into array for plotting
                    ;********************************
                            if (n_elements(plot_points) eq 0) or (plot_points(0) eq -1) then plot_points=[same_val_points] else plot_points=[plot_points,same_val_points]
                        ENDIF
                    
                    ;********************************
                    ; if masking selected, apply
                    ;********************************
                        IF KEYWORD_SET(mask) THEN BEGIN
                           flagged_values[flagged_col, same_val_points]=values[same_val_points]
                           values(same_val_points)=fdi
                        ENDIF
                     ENDIF
                ENDFOR
          ;********************************
          ; diagnostic plot per streaky year
          ;********************************
                IF KEYWORD_SET(diagnostics) THEN BEGIN
                    
                    PLTyearpoints=WHERE(times GE year_divides(years(badyears(bad))), ycnt)
                    IF badyears(bad)+1 LT nyears THEN BEGIN
                        PLTyearpoints2=WHERE(times(PLTyearpoints) LE year_divides(years(badyears(bad)+1)), ycnt1)
                        temp=PLTyearpoints(PLTyearpoints2)
                    ENDIF ELSE BEGIN
                        temp=PLTyearpoints
                    END
                
              ;******************************
              ; Were there any data in the extracted year
              ;******************************
 
                    PLTlegalvalues=values(temp) 
                    PLTlegaltimes=times(temp)
                    
                    useful_values=WHERE((PLTlegalvalues NE mdi) AND (PLTlegalvalues NE fdi))
                    MINY=MIN(PLTlegalvalues(useful_values))
                    MAXY=MAX(PLTlegalvalues(useful_values))
                    
                    set_plot,'PS'
                    
                    tvlct,0,0,0,0 ; black for kept datapoints
                    cd, "./", current=outdirplot
                    device,filename=outdirplot+'/images/'+station_id+'_Streak_Year_'+label+'_'+string(badyears(bad)+1973,FORMAT='(I4)')+'.eps'$
                      ,/color,/encapsulate,/helvetica,/bold,/landscape,ysize=20,xsize=26
                    !P.Font=0
                    !P.Thick=4
                    !P.Multi=[0,1,1]
                    plotsym,3,1.2
                    start_date=JULDAY(1, 1, 1973, 0)
                    plot,PLTlegaltimes+start_date,PLTlegalvalues,min_value=-100,xstyle=1,ystyle=1,ytitle=label,xtitle="time",xtickformat='LABEL_DATE',$
                      title="Streak - Diagnostic Timeseries - "+label,charsize=1,ythick=4,xthick=4,psym=2,yrange=[0.9*MINY,1.1*MAXY], POSITION=[0.10, 0.10, 0.90, 0.90]
                    oplot,PLTlegaltimes+start_date,PLTlegalvalues,min_value=-100,xstyle=1,ystyle=1,ytitle=label,xtitle="time",xtickformat='LABEL_DATE',$
                      title="Streak - Diagnostic Timeseries - "+label,charsize=1,ythick=4,xthick=4,psym=0,yrange=[0.9*MINY,1.1*MAXY], POSITION=[0.10, 0.10, 0.90, 0.90]
                    oplot,times(plot_points)+start_date,values(plot_points),min_value=-100,xstyle=1,ystyle=1,ytitle=label,xtitle="time",xtickformat='LABEL_DATE',$
                      title="Streak - Diagnostic Timeseries - "+label,charsize=1,ythick=4,xthick=4,psym=4,color=239,yrange=[0.9*MINY,1.1*MAXY], POSITION=[0.10, 0.10, 0.90, 0.90]
                    
                    device,/close

                 ;******************************
                 ; Streaks highlighted in Red
                 ;******************************
                    
                    FigCounter=FigCounter+1
               
                    set_plot,'PS'


                    tvlct,0,0,0,0 ; black for kept datapoints
                    cd, "./", current=outdirplot
                    device,filename=outdirplot+'/images/'+station_id+'_Streak_Month_'+label+'_'+string(badyears(bad)+1973,FORMAT='(I4)')+'.eps'$
                      ,/color,/encapsulate,/helvetica,/bold,/landscape,ysize=20,xsize=26
                    !P.Font=0
                    !P.Thick=4
                    !P.Multi=[0,1,1]
                    plotsym,3,1.2
                    start_date=JULDAY(1, 1, 1973, 0)

                    plot,PLTlegaltimes[0:400]+start_date,PLTlegalvalues[0:400],min_value=-100,xstyle=1,ystyle=1,ytitle=label,xtitle="time",xtickformat='LABEL_DATE',$
                      title="Streak - Diagnostic Timeseries - "+label,charsize=1,ythick=4,xthick=4,psym=2,yrange=[0.9*MINY,1.1*MAXY], POSITION=[0.10, 0.10, 0.90, 0.90]
                    oplot,PLTlegaltimes[0:400]+start_date,PLTlegalvalues[0:400],min_value=-100,xstyle=1,ystyle=1,ytitle=label,xtitle="time",xtickformat='LABEL_DATE',$
                      title="Streak - Diagnostic Timeseries - "+label,charsize=1,ythick=4,xthick=4,psym=0,yrange=[0.9*MINY,1.1*MAXY], POSITION=[0.10, 0.10, 0.90, 0.90]
                    oplot,times(plot_points)+start_date,values(plot_points),min_value=-100,xstyle=1,ystyle=1,ytitle=label,xtitle="time",xtickformat='LABEL_DATE',$
                      title="Streak - Diagnostic Timeseries - "+label,charsize=1,ythick=4,xthick=4,psym=4,color=239,yrange=[0.9*MINY,1.1*MAXY], POSITION=[0.10, 0.10, 0.90, 0.90]
                    
                    device,/close
                    
                    FigCounter=FigCounter+1
                    
                    plot_points=[-1]
                ENDIF
                
            ENDFOR
        ENDIF
    ENDIF
    FigCounter=0

;*************************************************************
;   if masking selected, apply to full_values too
;*************************************************************
    IF KEYWORD_SET(mask) THEN BEGIN
        points=WHERE(qc_flags(column,*) GT 0, bad_count)
        IF bad_count GE 1 THEN BEGIN
            MATCH, h_times(points), full_h_times, suba, subb
            full_values(subb)=fdi
        ENDIF
    ENDIF



;**************************************************************
; Use the keyword last_year to define how many hours and months we are
; to consider.
;**************************************************************
    start_date = JULDAY(1, 1, 1973, 0)
    end_date = JULDAY(1, 1, last_year+1, 0)
    ndays=LONG(end_date-start_date)

;*************************************************************
; Reform the unpacked arrays passed upon call so that they are 24 by ndays
;*************************************************************
    full_times=REFORM(full_times,24,ndays)
    full_values=REFORM(full_values,24,ndays)


;*************************************************************
; Repeat across hours
;*************************************************************

; Find cases where they repeat for a given hour for greater than 15 days

    FOR h=0,23 DO BEGIN
  ;*************************************
  ; for each hour
  ;*************************************
        match_values=-1.e+30
        value_match_times=[0]
        FOR d=0,ndays-1 DO BEGIN
      ;*************************************
      ; look at each day
      ;*************************************
            IF (full_values(h,d) NE mdi) AND (full_values(h,d) NE fdi) THEN BEGIN
                IF full_values(h,d) NE match_values THEN BEGIN
            ;*************************************
            ; if the values is different from the previous
            ;*************************************
                    IF N_ELEMENTS(value_match_times) GE nhr_repeats THEN BEGIN
                    ;*************************************
                    ; if the number of repeating ones > critical ,set to flag
                    ;*************************************
                        FOR t=0,N_ELEMENTS(value_match_times)-1 DO BEGIN
                        
                            badloc=WHERE(times EQ value_match_times(t), blcnt)
                            IF blcnt GE 1 THEN BEGIN
                               qc_flags(columns2,badloc(0))=1
                               filter_flags(badloc(0))=1
                               ;********************************
                               ; if masking selected, apply
                               ;********************************
                               IF KEYWORD_SET(mask) THEN BEGIN
                                  flagged_values[flagged_col, badloc(0)]=values[badloc(0)]
                                  values(badloc(0))=fdi
                                  badloc=WHERE(full_times EQ value_match_times(t))
                                  full_values(badloc(0))=fdi
                               ENDIF
                            ENDIF
                        
                            
                        ENDFOR


                    ;********************************
                    ; diagnostic plot per streaky hour - to be implemented
                    ;********************************

                    ENDIF
                    match_values=full_values(h,d)
                    value_match_times=[full_times(h,d)]
                ENDIF ELSE value_match_times=[value_match_times,full_times(h,d)]  
            ENDIF
        ENDFOR
    ENDFOR
    

;*************************************************************
; Whole day repeats
;*************************************************************
;
; Find exact repeats that are >5 days and remove these
;
    day_values=full_values(*,0)
    nday=1
    
    FOR d=1,ndays-1 DO BEGIN
        this_day_values=full_values(*,d)
        matches=WHERE(this_day_values EQ day_values, mc)
        
        IF mc NE 24 THEN BEGIN
        ;*************************************
        ; at least 24 hours of match - ie all match
        ;*************************************
            IF nday GE ndayrpts THEN BEGIN
            ;*************************************
            ; Check not all missing
            ;*************************************
                mia=WHERE(((day_values EQ mdi) OR (day_values EQ fdi)), mia_cnt)
                IF mia_cnt NE 24 THEN BEGIN
                ;*************************************
                ; Real case of days being exactly replicated
                ;*************************************
                    deltimes=full_times(*,d-nday:d-1)
                    deltimes=REFORM(deltimes,24*nday)
                    FOR bt=0,nday*24-1 DO BEGIN
                        badloc=WHERE(times EQ deltimes(bt), delpoint)
                        IF delpoint EQ 1 THEN BEGIN
                            qc_flags(columns3,badloc(0))=1
                        ;********************************
                        ; if masking selected, apply
                        ;********************************
                            IF KEYWORD_SET(mask) THEN BEGIN
                               flagged_values[flagged_col, badloc(0)]=values[badloc(0)]
                               values(badloc(0))=fdi
                            ENDIF
                        ENDIF
                        badloc=WHERE(full_times EQ deltimes(bt), delpoint)
                    ;********************************
                    ; if masking selected, apply - full values only
                    ;********************************
                        IF KEYWORD_SET(mask) THEN full_values(badloc(0))=fdi

                    ;********************************
                    ; diagnostic plot per day - to be implemented
                    ;********************************

                    ENDFOR
                ENDIF
            ENDIF
            day_values=this_day_values
            nday=1
        ENDIF ELSE nday=nday+1
    ENDFOR
    
    full_times=REFORM(full_times,24*ndays)
    full_values=REFORM(full_values,24*ndays)
    
    printstring="      Streak Check Flags  "
    
    IF KEYWORD_SET(oldflags) THEN BEGIN
        points=WHERE(qc_flags(column,*) GT 0, bad_count)
        PRINT,printstring,type,':',bad_count-pfcnt1,'  Old Flags:',pfcnt1,FORMAT='(A-38,I1,A1,I8,A-15,I8)'
        points=WHERE(qc_flags(columns2,*) GT 0, bad_count)
        PRINT,printstring,type,':',bad_count-pfcnt2,'  Old Flags:',pfcnt2,FORMAT='(A-38,I1,A1,I8,A-15,I8)'
        points=WHERE(qc_flags(columns3,*) GT 0, bad_count)
        PRINT,printstring,type,':',bad_count-pfcnt3,'  Old Flags:',pfcnt3,FORMAT='(A-38,I1,A1,I8,A-15,I8)'
    ENDIF ELSE BEGIN
        points=WHERE(qc_flags(column,*) GT 0, bad_count)
        PRINT,printstring,type,':',bad_count, FORMAT='(A-38,I1,A1,I8)'
        points=WHERE(qc_flags(columns2,*) GT 0, bad_count)
        PRINT,printstring,type,':',bad_count, FORMAT='(A-38,I1,A1,I8)'
        points=WHERE(qc_flags(columns3,*) GT 0, bad_count)
        PRINT,printstring,type,':',bad_count, FORMAT='(A-38,I1,A1,I8)'
    ENDELSE
    
ENDIF

RETURN

END
;************************************************************************
;                                 END
;************************************************************************

