;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;British Crown Copyright (c) 2012, the Met Office All rights reserved.
;
;Redistribution and use in source and binary forms, with or without 
;modification, are permitted provided that the following conditions 
;are met:
;
;Redistributions of source code must retain the above copyright notice, 
;this list of conditions and the following disclaimer. 
;
;Redistributions in binary form must reproduce the above copyright 
;notice, this list of conditions and the following disclaimer in the 
;documentation and/or other materials provided with the distribution. 
;
;Neither the name of the Met Office nor the names of its contributors 
;may be used to endorse or promote products derived from this software 
;without specific prior written permission. 
;
;THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
;"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
;LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
;A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
;HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
;SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
;LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
;DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
;THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
;(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
;OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

PRO TEMPERATURE_TIMES_v2,last_year,  type=type, months=months, suffix=suffix, days_in_month=days_in_month, deltat=deltat, accept=accept

;************************************************************
; NAME:
;     TEMPERATURE_TIMES_v2
;
; PURPOSE:
;     Produce a plot showing the time of the maximum temperatures for
;     either JJA or DJF depending on input.  
;
; CALLING SEQUENCE:
;     TEMPERATURE_TIMES_v2, last_year, type=type, months=months, suffix=suffix,
;     days_in_month=days_in_month, deltat=deltat, accept=accept
;
; INPUTS:
;     type - array of ['MAX','MIN'] (default)
;
;     months - array of ['DJF','MAM','JJA','SON']
;              Default is ['DJF','JJA']
;
;     last_year - last year of data series
;
;     suffix - suffix of netcdffiles to use
;
;     days_in_month - number of days in 3 month set which have to have
;                     valid time values to make station count in final
;                     output.  Default is 60.
;
;     deltat - create plot of delta time between JJA and DJF for
;              extremes
;
;     accept - run using only 3375 accepted stations.  Give filename
;              as keyword
;
; METHOD:
;
; OUTPUTS:
;     Single postscript file.
;     
; RESTRICTIONS:
;
;   
; HISTORY:
;      281 - Based on hadisd_humidity.pro
;            10 January 2012 RJHD   
;      285 - Updates and reordering to improve speed using more array
;            like operations
;            12 January 2012 RJHD   
;      286 - Runs all months and types in one.  Allows optional output
;            of delta-time between times of extreme for DJF and JJA.
;            13 January 2012 RJHD   
;      289 - Added accept keyword to run on filtered stations only
;            23 January 2012 RJHD   
;************************************************************************
;                    SVN Info
;$Rev:: 323                                           $:  Revision of last commit
;$Author:: rdunn                                      $:  Author of last commit
;$Date:: 2012-11-02 17:27:23 +0000 (Fri, 02 Nov 2012) $:  Date of last commit
;************************************************************************
;                                 START
;************************************************************************

; Set defaults
IF NOT KEYWORD_SET(days_in_month) THEN days_in_month=60
IF NOT KEYWORD_SET(type) THEN type=['MAX','MIN']
IF NOT KEYWORD_SET(months) THEN months=['DJF','MAM','JJA','SON']


;*******************
; Check input and output directories have been set, if not, use
; default setting
;*******************

cd, "./", current=pwd
datadir=pwd+'/../netcdffiles_2012/'
outputdir=pwd+'/'
IF NOT KEYWORD_SET(suffix) THEN suffix='mask2'

;*******************
; Read in the list of stations to be considered.
;*******************
Station_info_file=pwd+'/kept_station_info.txt'


nstation=nlines(Station_info_file)

station_ids=STRARR(nstation)
station_lats=FLTARR(nstation)
station_lons=FLTARR(nstation)
station_elevs=FLTARR(nstation)
station_times=FLTARR(nstation,N_ELEMENTS(type),N_ELEMENTS(months))
station_dtrs=FLTARR(nstation,N_ELEMENTS(months))
station_times[*]=-1.e30
station_dtrs[*]=-1.e30
id=''
st=0

OPENR,1,Station_info_file
WHILE NOT EOF(1) DO BEGIN
  READF,1,id,FORMAT='(A)'
  station_ids(st)=STRMID(id,0,12)
  station_lats(st)=FLOAT(STRCOMPRESS(STRMID(id,13,9),/remove_all))
  station_lons(st)=FLOAT(STRCOMPRESS(STRMID(id,23,9),/remove_all))
  station_elevs(st)=FLOAT(STRCOMPRESS(STRMID(id,33,9),/remove_all))
  st=st+1
ENDWHILE
CLOSE,1

;***************************************************************************
;    Use only the accepted files if switch activated
;***************************************************************************
IF KEYWORD_SET(accept) THEN BEGIN
   accept_file=pwd+'/'+accept

   nastation=NLINES(accept_file)

   accept_sts=STRARR(nastation)
   
   ast=0
   OPENR,1,accept_file
   WHILE NOT EOF(1) DO BEGIN
      READF,1,id,FORMAT='(A)'
      accept_sts(ast)=STRCOMPRESS(id,/remove_all)
      ast=ast+1
   ENDWHILE
   
   CLOSE,1

   MATCH,accept_sts,station_ids,suba,subb

   station_ids=station_ids(subb)
   station_lats=station_lats(subb)
   station_lons=station_lons(subb)
   station_elevs=station_elevs(subb)
   
   nstation=N_ELEMENTS(station_ids)

ENDIF

;***************************************************************************
;    START THE STATION LOOP
;***************************************************************************
FOR st=0,nstation-1 DO BEGIN

  IF NOT KEYWORD_SET(test) THEN BEGIN
      PRINT,'Station Number :',st+1,"/",nstation, FORMAT='(A-40,I4,A1,I4)'
      printstring='Station Identifier :'
      PRINT,printstring,station_ids(st), FORMAT='(A-40,A15)'
  ENDIF
  ;***************************************************************************
  ;    START THE DATA READING
  ;***************************************************************************

  file=datadir+station_ids(st)+'_'+suffix+'.nc' 

  ;****************************************
  ; "fred" is the datafile for the station
  ; "betty" contains information about
  ;              the structure of the netcdf file, fred
  ;****************************************

  IF file EQ '.nc' THEN BREAK ; leave station loop if reading empty file (extra CRs in station-info.txt)

  fred=NCDF_OPEN(file,/nowrite)

  NCDF_CONTROL,0,/verbose
  betty=NCDF_INQUIRE(fred)

  ;****************************************
  ;  obtain information on the dimensions
  ;****************************************

  tid=NCDF_DIMID(fred,'time')
  NCDF_DIMINQ,fred,tid,timedim,nt

  ;****************************************
  ;  read the variable information
  ;****************************************

  timesinfo=NCDF_VARINQ(fred,'time')
  tempsinfo=NCDF_VARINQ(fred,'temperatures')
  
  ;****************************************
  ;  read the variable values into the arrays
  ;****************************************

  NCDF_VARGET,fred,'time',times
  NCDF_VARGET,fred,'temperatures',temperatures

  ;****************************************
  ;  read the mdi (missing data indicator) values
  ;****************************************

  NCDF_ATTGET,fred,'temperatures','missing_value',temperatures_mdi
  NCDF_ATTGET,fred,'temperatures','flagged_value',temperatures_fdi

  printstring='Total station record size: '
  PRINT,printstring,nt, FORMAT='(A-40,I10)'

  mdi=temperatures_mdi

  fdiloc=WHERE(temperatures EQ temperatures_fdi, fdicnt)

  IF fdicnt GE 1 THEN BEGIN
     temperatures[fdiloc]=temperatures_mdi
  ENDIF

  NCDF_CLOSE,fred   


  ;************************************************************************
  ; Next we need to unpack the compressed time axis that the netcdf
  ; data file was held in and map the temperatures onto hourly times.
  ;************************************************************************

  ;************************************************************************
  ; Create the hourly index
  ;************************************************************************

  start_date = JULDAY(1, 1, 1973, 0)
  end_date = JULDAY(1, 1, last_year+1, 0)
  nyears=last_year-1973+1
  nhours=LONG((end_date-start_date)*24.)
  full_times = TIMEGEN(nhours, UNIT = 'Hours',  START = JULDAY(1, 1, 1973, 0))
  current_time=JULDAY(1, 1, 1973, 0, 0)
  full_times_reset=full_times-current_time

  ;************************************************************************
  ; Find the locs and fill them into full_temperatures array.
  ;************************************************************************
  full_temperatures=FLTARR(nhours)
  full_temperatures(*)=temperatures_mdi
  loc=0L

  IF suffix EQ 'mask2' THEN BEGIN
     ; adjust for new time-axis (hours-since) 2 July 2012 RJHD
     full_times_reset=ROUND(full_times_reset*24.)
  ENDIF

  MATCH,times,full_times_reset, suba, subb
  
  IF suffix EQ 'mask2' THEN BEGIN
     ; revert back from new time-axis to days-since  (2 July 2012 RJHD)
     full_times_reset=FLOAT(full_times_reset)/24.
     times=FLOAT(times)/24.
  ENDIF

  full_temperatures(subb)=temperatures(suba)

  ndays=N_ELEMENTS(full_temperatures)/24


  
  ;************************************************************************
  ; Reform into 24h by ndays array and find extreme of each day
  ;************************************************************************
  ;daily_times=REFORM(full_times,24,ndays)
  daily_temperature=REFORM(full_temperatures,24,ndays)

  FOR typei=0,N_ELEMENTS(type)-1 DO BEGIN

     ; account for mdi's - swap to +ve value
     inverted=daily_temperature
     inverted[WHERE(inverted EQ mdi)]=-mdi

     IF type[typei] EQ 'MAX' THEN BEGIN
        daily_extreme=MAX(daily_temperature,DIMENSION=0)
     ENDIF ELSE IF type[typei] EQ 'MIN' THEN BEGIN
        daily_extreme=MIN(inverted,DIMENSION=0)
     ENDIF

     ; DTR code, 30 May 2012 RJHD
     
     maxes=MAX(daily_temperature,DIMENSION=0)
     mins=MIN(inverted,DIMENSION=0)

     dtr=FLTARR(N_ELEMENTS(daily_extreme))
     dtr[*]=mdi

     goodlocs=WHERE((maxes NE mdi) AND (mins NE -mdi))

     dtr[goodlocs]=maxes[goodlocs]-mins[goodlocs] ; 1 x ndays array

     

     daily_times=FLTARR(ndays)
     daily_times[*]=mdi

  
  ;************************************************************************
  ; If sufficient observations, well spread then find hour of extreme
  ;   Observations have to be present in all 4 quartiles of the day.
  ;************************************************************************
     FOR day=0,ndays-1 DO BEGIN
        quartiles=0
        hr=0
        FOR quarter=0,3 DO BEGIN
                                ; find if observations present in each quartile
           qtrloc=WHERE(daily_temperature[hr:hr+5,day] NE mdi,qtrcnt)
           IF qtrcnt GE 1 THEN quartiles=quartiles+1
           hr=hr+6
        ENDFOR

        IF (daily_extreme[day] NE mdi) AND (quartiles EQ 4) THEN BEGIN
           ; take latest observation of maximum
           daily_times[day]=MAX(WHERE(daily_temperature[*,day] EQ daily_extreme[day]))
        ENDIF
     ENDFOR                     ; total days
  
  ;************************************************************************
  ; create array of 366days by Nyears to hold hour location
  ;************************************************************************
     startday=0
     
     daily_times_years=FLTARR(366,nyears)
     daily_times_years[*]=mdi
  
     dtr_years=FLTARR(366,nyears)
     dtr_years[*]=mdi

     FOR yr=0,nyears-1 DO BEGIN
        
        IF ISLEAPYEAR(1973+yr) EQ 1 THEN BEGIN
           yeardays=366
           daily_times_years[0:yeardays-1,yr]=daily_times[startday:startday+yeardays-1]
           dtr_years[0:yeardays-1,yr]=dtr[startday:startday+yeardays-1]

        ENDIF ELSE IF ISLEAPYEAR(1973+yr) EQ 0 THEN BEGIN
           yeardays=365
           daily_times_years[0:yeardays-1,yr]=daily_times[startday:startday+yeardays-1]
           daily_times_years[60:365,yr]=daily_times_years[59:364,yr]
           daily_times_years[59,yr]=mdi

           dtr_years[0:yeardays-1,yr]=dtr[startday:startday+yeardays-1]
           dtr_years[60:365,yr]=dtr_years[59:364,yr]
           dtr_years[59,yr]=mdi
        ENDIF
        startday=startday+yeardays
        
     ENDFOR                     ; years

  ;************************************************************************
  ; obtain average over all years of hour location
  ; need at least 10 years
  ;************************************************************************
     daily_times_average=FLTARR(366) 
     daily_times_average[*]=mdi

     dtr_average=FLTARR(366) 
     dtr_average[*]=mdi
     
     FOR d = 0,365 DO BEGIN
        non_mdi=WHERE(daily_times_years[d,*] NE mdi, mdicnt)
        
        IF mdicnt GE 10 THEN daily_times_average[d]=MEDIAN(daily_times_years[d,non_mdi])
        
        non_mdi=WHERE(dtr_years[d,*] NE mdi, mdicnt)
        
        IF mdicnt GE 10 THEN dtr_average[d]=MEDIAN(dtr_years[d,non_mdi])
        
     ENDFOR                     ; year days
     
  ;************************************************************************
  ; Select the months.  Need at least 2/3 of observations to count.
  ;************************************************************************
     month_times_average=mdi
     month_dtr_average=mdi
     
     FOR monthi=0,N_ELEMENTS(months)-1 DO BEGIN
        
        IF months[monthi] EQ 'DJF' THEN BEGIN
                                ; 1 December is 336th day of year for
                                ; LY, 29th February is 60th 
           selected=[daily_times_average[336:365],daily_times_average[0:59]]
           
           goodlocs=WHERE(selected NE mdi, gdcnt)
           
           IF gdcnt GE days_in_month THEN BEGIN
              
              month_times_average=MEDIAN(selected[goodlocs])
              
           ENDIF
           
           selected=[dtr_average[336:365],dtr_average[0:59]]
           
           goodlocs=WHERE(selected NE mdi, gdcnt)
           
           IF gdcnt GE days_in_month THEN BEGIN
              
              month_dtr_average=MEDIAN(selected[goodlocs])
              
           ENDIF
           
        ENDIF ELSE IF months[monthi] EQ 'MAM' THEN BEGIN
                                ; 1 MARCH = 61 day of year for LY,
                                ; 31 May is 152nd day
           
           selected=daily_times_average[60:151]
           goodlocs=WHERE(selected NE mdi, gdcnt)
           
           IF gdcnt GE days_in_month THEN BEGIN
              
              month_times_average=MEDIAN(selected[goodlocs])
           
           ENDIF
           selected=dtr_average[60:151]
           goodlocs=WHERE(selected NE mdi, gdcnt)
           
           IF gdcnt GE days_in_month THEN BEGIN
              
              month_dtr_average=MEDIAN(selected[goodlocs])
           
           ENDIF
        ENDIF ELSE IF months[monthi] EQ 'JJA' THEN BEGIN
                                ; 1 June = 153rd day of year for LY,
                                ; 31 August is 244th day
           
           selected=daily_times_average[152:243]
           goodlocs=WHERE(selected NE mdi, gdcnt)
           
           IF gdcnt GE days_in_month THEN BEGIN
              
              month_times_average=MEDIAN(selected[goodlocs])
              
           ENDIF
           selected=dtr_average[152:243]
           goodlocs=WHERE(selected NE mdi, gdcnt)
           
           IF gdcnt GE days_in_month THEN BEGIN
              
              month_dtr_average=MEDIAN(selected[goodlocs])
              
           ENDIF
        ENDIF ELSE IF months[monthi] EQ 'SON' THEN BEGIN
                                ; 1 June = 245 day of year for LY,
                                ; 30 November is 335th day
           
           selected=daily_times_average[244:335]
           goodlocs=WHERE(selected NE mdi, gdcnt)
           
           IF gdcnt GE days_in_month THEN BEGIN
              
              month_times_average=MEDIAN(selected[goodlocs])
              
           ENDIF
           selected=dtr_average[244:335]
           goodlocs=WHERE(selected NE mdi, gdcnt)
           
           IF gdcnt GE days_in_month THEN BEGIN
              
              month_dtr_average=MEDIAN(selected[goodlocs])
              
           ENDIF
        ENDIF
        
        station_times[st,typei,monthi]=month_times_average
        station_dtrs[st,monthi]=month_dtr_average

     ENDFOR                     ; month sets (DJF,MAM,JJA,SON)
  ENDFOR                        ; types (MAX,MIN)
  
ENDFOR                          ; station


;************************************************************************
; Set up the plots.
;************************************************************************

FOR typei=0,N_ELEMENTS(type)-1 DO BEGIN

   FOR monthi=0,N_ELEMENTS(months)-1 DO BEGIN

      IF type[typei] EQ 'MAX' THEN BEGIN
         outplot='temperatures_max_'+months[monthi]+'_v2.ps'
         plottitle='Time of Maximum Temperature ('+months[monthi]+')'
      ENDIF ELSE IF type[typei] EQ 'MIN' THEN BEGIN
         outplot='temperatures_min_'+months[monthi]+'_v2.ps'
         plottitle='Time of Minimum Temperature ('+months[monthi]+')'
      ENDIF
      
; blue - yellow - red - purple - blue
; blues
      restore_colours,'shortbow'
      tvlct,41,10,216,1
      tvlct,38,77,255,2
      tvlct,63,160,255,3
      tvlct,114,217,255,4
      tvlct,170,247,255,5
;reds
      tvlct,255,255,191,6
      tvlct,255,173,114,7
      tvlct,247,109,94,8
      tvlct,216,38,50,9
      tvlct,185,0,33,10
;purples
      tvlct,150,0,150,11
      tvlct,138,3,255,12
      
      colsarr=[1,2,3,4,5,6,7,8,9,10,11,12]
      labsarr=['0000','0200','0400','0600','0800','1000','1200','1400','1600','1800','2000','2200','2400']
      coltitle='Universal Time (UT)'
      
      SET_PLOT,'ps'
      Device,file=outputdir+'images/'+outplot,/color,/helvetica,xsize=26,ysize=20
      !P.font=0
      !P.Position=[0.08,0.16,0.96,0.94]
      plotsym,0,0.55,/fill
      Map_set,0,0,limit=[-90,-180,90,180],/noborder,color=0,/noerase,/robinson,/grid, glinestyle=2
      map_continents,/coasts, color=0
      
      hour_counter=0
      FOR hour=0,22,2 DO BEGIN
         bin=WHERE((station_times[*,typei,monthi] GE hour) AND (station_times[*,typei,monthi] LT hour+2), cbin)
         
         IF (cbin GE 1) THEN plots,station_lons[bin],station_lats[bin],min_value=-180.,psym=8,color=colsarr[hour_counter], symsize=0.8
         print,hour, cbin
         
         hour_counter=hour_counter+1
      ENDFOR                    ; hour
      PLOTS,[-179.9,0],[89.9,89.9],color=0
      PLOTS,[0,179.9],[89.9,89.9],color=0
      PLOTS,[-179.9,0],[-89.9,-89.9],color=0
      PLOTS,[0,179.9],[-89.9,-89.9],color=0
      
      XYOUTS,-180,85,'90N ',alignment=1,charsize=0.9,color=0
      XYOUTS,-180,43,'45N ',alignment=1,charsize=0.9,color=0
      XYOUTS,-180,-2,'0 ',alignment=1,charsize=0.9,color=0
      XYOUTS,-180,-47,'45S  ',alignment=1,charsize=0.9,color=0
      XYOUTS,-180,-89,'90S      ',alignment=1,charsize=0.9,color=0
      
      XYOUTS,0.26,0.12,'180W',/normal,charsize=0.9,color=0
      XYOUTS,0.38,0.12,'90W',/normal,charsize=0.9,color=0
      XYOUTS,0.512,0.12,'0',/normal,charsize=0.9,color=0
      XYOUTS,0.64,0.12,'90E',/normal,alignment=1,charsize=0.9,color=0
      XYOUTS,0.78,0.12,'180E',/normal,alignment=1,charsize=0.9,color=0
      
      MAKE_KEY,0.1,0.08,0.8,0.02,0.00,-0.03,/NORMAL,COLORS=colsarr,labels=labsarr,charsize=1.,charthick=4,bcolor=0
      XYOUTS, 0.5, 0.01, coltitle, alignment=0.5,charsize=1.0, color=0,/normal
      XYOUTS, 0.5, 0.95, plottitle, alignment=0.5,charsize=1.0, color=0,/normal
      
      device,/close_file
      
   ENDFOR                       ; months
ENDFOR                          ; types

;*******************
; Create plot of delta-t between DJF and JJA for extremes
;*******************

djfloc=WHERE(months EQ 'DJF',djfcnt)
jjaloc=WHERE(months EQ 'JJA',jjacnt)

IF KEYWORD_SET(deltat) AND (djfcnt EQ 1) AND (jjacnt EQ 1) THEN BEGIN

   FOR typei=0,N_ELEMENTS(type)-1 DO BEGIN

      IF type[typei] EQ 'MAX' THEN BEGIN
         outplot='differences_'+type[typei]+'_v2.ps'
         plottitle='Difference Time of Maximum Temperature (DJF-JJA)'
      ENDIF ELSE IF type[typei] EQ 'MIN' THEN BEGIN
         outplot='differences_'+type[typei]+'_v2.ps'
         plottitle='Difference in Time of Minimum Temperature (DJF-JJA)'
      ENDIF
      

      difference_times=station_times[*,typei,djfloc]-station_times[*,typei,jjaloc]

      tvlct,0,0,0,0     
; blue - green - magenta - blue
      tvlct,62,100,124, 1     
      tvlct,62,124,62, 2     
      tvlct,0,187,0,3
      tvlct,0,241,0,4
      tvlct,80,255,80,5
      tvlct,187,255,187,6
      tvlct,255,187,255,7
      tvlct,255,80,255,8
      tvlct,241,0,241,9
      tvlct,187,0,187,10
      tvlct,124,62,124,11
      tvlct,62,62,124,12

      colsarr=[1,2,3,4,5,6,7,8,9,10,11,12]
      labsarr=['-12','-10','-8','-6','-4','-2','0','2','4','6','8','10','12']
      coltitle='Time Difference (hr)'
      
      SET_PLOT,'ps'
      Device,file=outputdir+'images/'+outplot,/color,/helvetica,xsize=26,ysize=20
      !P.font=0
      !P.Position=[0.08,0.16,0.96,0.94]
      plotsym,0,0.55,/fill
      Map_set,0,0,limit=[-90,-180,90,180],/noborder,color=0,/noerase,/robinson,/grid, glinestyle=2
      map_continents,/coasts, color=0
      
      hour_counter=0
      FOR hour=-12,10,2 DO BEGIN
         bin=WHERE((difference_times GE hour) AND (difference_times LT hour+2), cbin)
         
         IF (cbin GE 1) THEN plots,station_lons[bin],station_lats[bin],min_value=-180.,psym=8,color=colsarr[hour_counter], symsize=0.8
         print,hour, cbin
         
         hour_counter=hour_counter+1
      ENDFOR                    ; hour
      PLOTS,[-179.9,0],[89.9,89.9],color=0
      PLOTS,[0,179.9],[89.9,89.9],color=0
      PLOTS,[-179.9,0],[-89.9,-89.9],color=0
      PLOTS,[0,179.9],[-89.9,-89.9],color=0
      
      XYOUTS,-180,85,'90N ',alignment=1,charsize=0.9,color=0
      XYOUTS,-180,43,'45N ',alignment=1,charsize=0.9,color=0
      XYOUTS,-180,-2,'0 ',alignment=1,charsize=0.9,color=0
      XYOUTS,-180,-47,'45S  ',alignment=1,charsize=0.9,color=0
      XYOUTS,-180,-89,'90S      ',alignment=1,charsize=0.9,color=0
      
      XYOUTS,0.26,0.12,'180W',/normal,charsize=0.9,color=0
      XYOUTS,0.38,0.12,'90W',/normal,charsize=0.9,color=0
      XYOUTS,0.512,0.12,'0',/normal,charsize=0.9,color=0
      XYOUTS,0.64,0.12,'90E',/normal,alignment=1,charsize=0.9,color=0
      XYOUTS,0.78,0.12,'180E',/normal,alignment=1,charsize=0.9,color=0
      
      MAKE_KEY,0.1,0.08,0.8,0.02,0.00,-0.03,/NORMAL,COLORS=colsarr,labels=labsarr,charsize=1.,charthick=4,bcolor=0
      XYOUTS, 0.5, 0.01, coltitle, alignment=0.5,charsize=1.0, color=0,/normal
      XYOUTS, 0.5, 0.95, plottitle, alignment=0.5,charsize=1.0, color=0,/normal
      
      device,/close_file
       
  


   ENDFOR                       ; type
ENDIF

FOR monthi=0,N_ELEMENTS(months)-1 DO BEGIN
   
   outplot='dtr_'+months[monthi]+'.ps'
   plottitle='Diurnal Temperature Range ('+months[monthi]+')'
      
; blue - yellow - red - purple - blue
; blues
   restore_colours,'shortbow'
   tvlct,41,10,216,1
   tvlct,38,77,255,2
   tvlct,63,160,255,3
   tvlct,114,217,255,4
   tvlct,170,247,255,5
;reds
   tvlct,255,255,191,6
   tvlct,255,173,114,7
   tvlct,247,109,94,8
   tvlct,216,38,50,9
   tvlct,185,0,33,10
;purples
   tvlct,150,0,150,11
   tvlct,138,3,255,12
   
   colsarr=[1,2,3,4,5,6,7,8,9,10,11,12]
   labsarr=['0','2','4','6','8','10','12','14','16','18','20','22','24']
   coltitle='DTR (C)'
      
   SET_PLOT,'ps'
   Device,file=outputdir+'images/'+outplot,/color,/helvetica,xsize=26,ysize=20
   !P.font=0
   !P.Position=[0.08,0.16,0.96,0.94]
   plotsym,0,0.55,/fill
   Map_set,0,0,limit=[-90,-180,90,180],/noborder,color=0,/noerase,/robinson,/grid, glinestyle=2
   map_continents,/coasts, color=0
   
   range_counter=0
   FOR range=0,36,2 DO BEGIN
      bin=WHERE((station_dtrs[*,monthi] GE range) AND (station_dtrs[*,monthi] LT range+2), cbin)
      
      IF (cbin GE 1) THEN plots,station_lons[bin],station_lats[bin],min_value=-180.,psym=8,color=colsarr[range_counter], symsize=0.8
      print,range, cbin
      
      range_counter=range_counter+1
   ENDFOR                       ; hour
   PLOTS,[-179.9,0],[89.9,89.9],color=0
   PLOTS,[0,179.9],[89.9,89.9],color=0
   PLOTS,[-179.9,0],[-89.9,-89.9],color=0
   PLOTS,[0,179.9],[-89.9,-89.9],color=0
      
   XYOUTS,-180,85,'90N ',alignment=1,charsize=0.9,color=0
   XYOUTS,-180,43,'45N ',alignment=1,charsize=0.9,color=0
   XYOUTS,-180,-2,'0 ',alignment=1,charsize=0.9,color=0
   XYOUTS,-180,-47,'45S  ',alignment=1,charsize=0.9,color=0
   XYOUTS,-180,-89,'90S      ',alignment=1,charsize=0.9,color=0
   
   XYOUTS,0.26,0.12,'180W',/normal,charsize=0.9,color=0
   XYOUTS,0.38,0.12,'90W',/normal,charsize=0.9,color=0
   XYOUTS,0.512,0.12,'0',/normal,charsize=0.9,color=0
   XYOUTS,0.64,0.12,'90E',/normal,alignment=1,charsize=0.9,color=0
   XYOUTS,0.78,0.12,'180E',/normal,alignment=1,charsize=0.9,color=0
   
   MAKE_KEY,0.1,0.08,0.8,0.02,0.00,-0.03,/NORMAL,COLORS=colsarr,labels=labsarr,charsize=1.,charthick=4,bcolor=0
   XYOUTS, 0.5, 0.01, coltitle, alignment=0.5,charsize=1.0, color=0,/normal
   XYOUTS, 0.5, 0.95, plottitle, alignment=0.5,charsize=1.0, color=0,/normal
   
   device,/close_file
      
ENDFOR                          ; months
STOP

RETURN

END
;************************************************************************
;                            END
;************************************************************************
