;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;British Crown Copyright (c) 2012, the Met Office All rights reserved.
;
;Redistribution and use in source and binary forms, with or without 
;modification, are permitted provided that the following conditions 
;are met:
;
;Redistributions of source code must retain the above copyright notice, 
;this list of conditions and the following disclaimer. 
;
;Redistributions in binary form must reproduce the above copyright 
;notice, this list of conditions and the following disclaimer in the 
;documentation and/or other materials provided with the distribution. 
;
;Neither the name of the Met Office nor the names of its contributors 
;may be used to endorse or promote products derived from this software 
;without specific prior written permission. 
;
;THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
;"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
;LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
;A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
;HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
;SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
;LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
;DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
;THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
;(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
;OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
PRO thin_network_v3, input_dir, suffix, last_year, select=select, doplots=doplots
;------------------------------------------------------------------------
; NAME:
;     thin_network_v3
;
; PURPOSE:
;     To decide whether a station matches certain criteria after QC to
;     aid in final selection criteria based on coverage and completeness.
;
; CALLING SEQUENCE:
;     thin_network_v3, input_dir, suffix, last_year, select=select, doplots=doplots
;
;
; INPUTS:
;     input_dir - where the .nc files are stored
;
;     suffix - suffix of netcdffiles
;
;     last_year - last_year of data 
;
;     select - run selection section
;
;     doplots - run plots section.  Give the root of the accept and
;             reject files as entry to the keyword
;
; METHOD:
;     Lists directory, runs through a number of tests.  If a station
;     fails a test, no further tests are completed.  Also outputs a
;     final plot of the distribution - of the fails and the accepts
;
; OUTPUTS:
;     Two text files (accept and reject) and one 2-panel plot (accept
;     and reject locations)
;
; RESTRICTIONS:
;     ;     
;   
; HISTORY:
;      #280 - adapted from thin_network.pro (PWT)
;             23 December 2011 - 6 January 2012 RJHD
;************************************************************************
;                    SVN Info
;$Rev:: 327                                           $:  Revision of last commit
;$Author:: rdunn                                      $:  Author of last commit
;$Date:: 2013-01-24 15:28:30 +0000 (Thu, 24 Jan 2013) $:  Date of last commit
;
;**************************************************************************************************
;
; Program creates a thinned down final network set to work from for the HadISD database depending upon
; the settings for the keywords which are tunable parameters for the system to enable us to play with
; assumptions in assigning a final network set. The basic idea is to weed out the short / patchy / apparently
; dubious and retain only the better stations following QC.
;
;************************************************************************
;                                 START
;************************************************************************

; read in the station information
Station_info_file='kept_station_info.txt'
nstation=nlines(Station_info_file)

station_ids=STRARR(nstation)
station_lats=FLTARR(nstation)
station_lons=FLTARR(nstation)
station_elevs=FLTARR(nstation)

id=''
st=0

OPENR,1,Station_info_file
WHILE NOT EOF(1) DO BEGIN
  READF,1,id,FORMAT='(A)'
  station_ids(st)=STRMID(id,0,12)
  station_lats(st)=FLOAT(STRCOMPRESS(STRMID(id,13,9),/remove_all))
  station_lons(st)=FLOAT(STRCOMPRESS(STRMID(id,23,9),/remove_all))
  station_elevs(st)=FLOAT(STRCOMPRESS(STRMID(id,33,9),/remove_all))
  st=st+1
  IF st GT nstation-1 THEN BREAK
ENDWHILE
CLOSE,1

IF KEYWORD_SET(select) THEN BEGIN

;************************************************************************
; Determine the months and years arrays outside the station loop as these don't change
;
months=INDGEN(12)+1
years=INDGEN(last_year-1973+1)+1973
leap_years=[1976,1980,1984,1988,1992,1996,2000,2004]
mlengths=[31,28,31,30,31,30,31,31,30,31,30,31]
;************************************************************************

; set up the structure to hold the information for testing
station_criteria={cstn,st_fyear:FLTARR(nstation), st_eyear:FLTARR(nstation), st_rfreq:FLTARR(nstation), $
                  st_rej:FLTARR(nstation,10), st_complete:INTARR(nstation,(last_year-1973+1),12,31)}



FOR st=0,nstation-1 DO BEGIN   ; Iterates over all the stations
  PRINT, station_ids(st), st+1,' of ',nstation
  ;********************************************************************************************************************
  ; Read in the netcdf file.
  ;
  file=input_dir+'/'+station_ids(st)+'_'+suffix+'.nc'
  fred=NCDF_OPEN(file,/nowrite)
  NCDF_CONTROL,fred,/verbose
  betty=NCDF_INQUIRE(fred)
  tid=NCDF_DIMID(fred,'time')
  NCDF_DIMINQ,fred,tid,timedim,nt
  charid=NCDF_DIMID(fred,'Character')
  NCDF_DIMINQ,fred,charid,chardim,nchar
  testid=NCDF_DIMID(fred,'test')
  NCDF_DIMINQ,fred,testid,testdim,ntests
  timesinfo=NCDF_VARINQ(fred,'time')
  tempsinfo=NCDF_VARINQ(fred,'temperatures')
  idsinfo=NCDF_VARINQ(fred, 'input_station_id')
  dewsinfo=NCDF_VARINQ(fred, 'dewpoints')
  totclinfo=NCDF_VARINQ(fred, 'total_cloud_cover')
  lowclinfo=NCDF_VARINQ(fred, 'low_cloud_cover')
  midclinfo=NCDF_VARINQ(fred, 'mid_cloud_cover')
  hiclinfo=NCDF_VARINQ(fred, 'high_cloud_cover')
  clbaseinfo=NCDF_VARINQ(fred, 'cloud_base')
  windspinfo=NCDF_VARINQ(fred, 'windspeeds')
  winddirinfo=NCDF_VARINQ(fred, 'winddirs')
  slpinfo=NCDF_VARINQ(fred, 'slp')
  flaginfo=NCDF_VARINQ(fred, 'qc_flags')
  
  NCDF_VARGET,fred,'time',times
  IF suffix EQ 'mask2' THEN times=FLOAT(times)/24.
  NCDF_VARGET,fred,'temperatures',temperatures
  NCDF_VARGET,fred,'input_station_id',ids
  NCDF_VARGET,fred,'dewpoints',dewpoints
  NCDF_VARGET,fred,'total_cloud_cover',total_cloud_cover
  NCDF_VARGET,fred,'low_cloud_cover',low_cloud_cover
  NCDF_VARGET,fred,'mid_cloud_cover',mid_cloud_cover
  NCDF_VARGET,fred,'high_cloud_cover',high_cloud_cover
  NCDF_VARGET,fred,'cloud_base',cloud_base
  NCDF_VARGET,fred,'windspeeds',windspeeds
  NCDF_VARGET,fred,'winddirs',winddirs
  NCDF_VARGET,fred,'slp',slp
  NCDF_VARGET,fred,'qc_flags',qc_flags
  ;
  ; get the mdi values
  NCDF_ATTGET,fred,'temperatures','missing_value',temperatures_mdi
  NCDF_ATTGET,fred,'dewpoints','missing_value',dewpoints_mdi
  NCDF_ATTGET,fred,'total_cloud_cover','missing_value',total_cloud_cover_mdi
  NCDF_ATTGET,fred,'low_cloud_cover','missing_value',low_cloud_cover_mdi
  NCDF_ATTGET,fred,'mid_cloud_cover','missing_value',mid_cloud_cover_mdi
  NCDF_ATTGET,fred,'high_cloud_cover','missing_value',high_cloud_cover_mdi
  NCDF_ATTGET,fred,'cloud_base','missing_value',cloud_base_mdi
  NCDF_ATTGET,fred,'windspeeds','missing_value',windspeeds_mdi
  NCDF_ATTGET,fred,'winddirs','missing_value',winddirs_mdi
  NCDF_ATTGET,fred,'slp','missing_value',slp_mdi
  ; get the fdi values
  NCDF_ATTGET,fred,'temperatures','flagged_value',temperatures_fdi
  NCDF_ATTGET,fred,'dewpoints','flagged_value',dewpoints_fdi
  NCDF_ATTGET,fred,'total_cloud_cover','flagged_value',total_cloud_cover_fdi
  NCDF_ATTGET,fred,'low_cloud_cover','flagged_value',low_cloud_cover_fdi
  NCDF_ATTGET,fred,'mid_cloud_cover','flagged_value',mid_cloud_cover_fdi
  NCDF_ATTGET,fred,'high_cloud_cover','flagged_value',high_cloud_cover_fdi
  cloud_base_fdi=-2.e30
  NCDF_ATTGET,fred,'windspeeds','flagged_value',windspeeds_fdi
  NCDF_ATTGET,fred,'winddirs','flagged_value',winddirs_fdi
  NCDF_ATTGET,fred,'slp','flagged_value',slp_fdi
  ;
  NCDF_CLOSE,fred


  

  ;*********************************************************************************************************************
  ;
  ; First test to complete which is easiest computationally is whether the first year and last year criteria are met
  ; as these are very simple binary decision aids.
  station_criteria.st_fyear(st)=times(0)
  station_criteria.st_eyear(st)=times(nt-1)

  ;*********************************************************************************************************************
  ; Next test set logically is based upon number of years meeting the month inclusion criteria.
  ; So obtain the base information (diurnal quartiles filled) and 
  ;   base the test on those after the file reading
  year_month_day=INTARR((last_year-1973+1),12,31)
  year_month_day[*]=-1

  monthdaycounts=INTARR(12*(last_year-1973+1))  ; 12 months, 35 years.
  mloc=0
  yloc=0
  FOR m=0,12*(last_year-1973)-1 DO BEGIN

     ; find number of days present within that month
     start_loc=JULDAY(months(mloc), 1, years(yloc), 0, 0)-JULDAY(1,1,1973,0,0)
     end_loc=JULDAY(months(mloc), mlengths(mloc), years(yloc),23,0)-JULDAY(1,1,1973,0,0)
     ndays_in_month=mlengths(mloc)
     IF mloc EQ 1 THEN BEGIN
        lc=WHERE(leap_years EQ years(yloc),lccnt)
        IF lccnt EQ 1 THEN BEGIN
           end_loc=JULDAY(months(mloc), 29, years(yloc),23,0)-JULDAY(1,1,1973,0,0)
           ndays_in_month=29
        ENDIF
     ENDIF
    ;
    ; Are there any timestamps within this month?
     totpoints=0
     points=WHERE(times GE start_loc, cnt1)
     IF cnt1 GE 1 THEN points2=WHERE(times(points) LE end_loc, totpoints)
     IF totpoints GE 1 THEN BEGIN
      ; There are at least some data points in this month.
        subset_times=times(points(points2))
        subset_times=subset_times-(JULDAY(months(mloc), 1, years(yloc), 0, 0)-JULDAY(1,1,1973,0,0))
      ; Now we have a set of timestamp points within the month that hold data starting from 0 and going up to
      ; ndays_in_month as a double precision. Firstly collapse into a present data array for the four different
      ; quadrants of the day and by day.
        day_mask=INTARR(4,ndays_in_month)
        FOR i=0,totpoints-1 DO BEGIN
           dayloc=FIX(subset_times(i))
           remainder=subset_times(i)-dayloc
           IF remainder LE 0.249 THEN day_mask(0,dayloc)=day_mask(0,dayloc)+1
           IF (remainder GE 0.25) AND (remainder LE 0.499) THEN day_mask(1,dayloc)=day_mask(1,dayloc)+1
           IF (remainder GE 0.5) AND (remainder LE 0.749) THEN day_mask(2,dayloc)=day_mask(2,dayloc)+1
           IF (remainder GE 0.75) THEN day_mask(3,dayloc)=day_mask(3,dayloc)+1
        ENDFOR
        
        present_mask=INTARR(4,ndays_in_month)
        present_mask[*]=0
        
        present_mask[WHERE(day_mask NE 0)]=1
        
        year_month_day[yloc,mloc,0:ndays_in_month-1]=TOTAL(present_mask, 0)
        
     ENDIF 
     
    ; change over at end of years
     mloc=mloc+1
     IF mloc EQ 12 THEN BEGIN
        mloc=0
        yloc=yloc+1
     ENDIF
  ENDFOR
  
  station_criteria.st_complete(st,*,*,*)=year_month_day 

  ;**********************************************************************************************************
  ; Next logical check is gap_check criteria seeing as we 
  ; have the month presence criteria handy at this
  ; point in time. We will use the months that exceeded the 
  ; month inclusion criteria as our decision aid here
  ; rather than allowing months with odd data points to ensure that we are conservative.
  ;**********************************************************************************************************

  ; THIS DEPENDS ON SELECTION APPLIED TO ABOVE TEST SO DONE DURING
  ;       TESTING PHASE.

  
  ;**********************************************************************************************************
  ; Reporting frequency
  ; This will be easiest by simply taking the median of the 
  ; differences created by shift by one element of the time array and the
  ;**********************************************************************************************************

  ; original time array.
  tdiffs=SHIFT(times,-1)-times
  medval=MEDIAN(tdiffs)

  station_criteria.st_rfreq(st)=medval
  
   ;**********************************************************************************************************
  ; Rejection rates
  ;
  ; These are on a variable specific set of settings, but the overall 
  ; methodology is variable independent so it can be looped to save code repetition.
  ;**********************************************************************************************************

  varnames=['temperature','dewpoint','SLP','windspeed','wind direction','cloud base','cloud total','low cloud','medium cloud','high cloud']
  reject_flag=0
  FOR v=0,9 DO BEGIN
    CASE v OF
      0: BEGIN
        varvals=temperatures
        varmdi=temperatures_mdi
        varfdi=temperatures_fdi
        qc_flag_locs=[0,1,4,5,8,12,16,20,24,27,41,44,54,58]
      END
      1: BEGIN
         varvals=dewpoints
         varmdi=dewpoints_mdi
         varfdi=dewpoints_fdi
         qc_flag_locs=[0,2,4,6,8,9,13,17,21,25,28,30,31,32,42,45,55,59]
      END
      2: BEGIN
         varvals=slp
         varmdi=slp_mdi
         varfdi=slp_fdi
         qc_flag_locs=[0,3,4,7,11,15,19,23,26,29,43,46,57,60]
      END
      3: BEGIN
         varvals=windspeeds
         varmdi=windspeeds_mdi
         varfdi=windspeeds_fdi
         qc_flag_locs=[0,4,10,14,18,22,56,47]
      END
      4: BEGIN
         varvals=winddirs
         varmdi=winddirs_mdi
         varfdi=winddirs_fdi
         qc_flag_locs=[0,4,56,48]
     END
      5: BEGIN
         varvals=cloud_base
         varmdi=cloud_base_mdi
         varfdi=cloud_base_fdi
         qc_flag_locs=[0,4,53]
      END
      6: BEGIN
         varvals=total_cloud_cover
         varmdi=total_cloud_cover_mdi
         varfdi=total_cloud_cover_fdi
         qc_flag_locs=[0,4,33,37,40,49]
      END
      7: BEGIN
         varvals=low_cloud_cover
         varmdi=low_cloud_cover_mdi
         varfdi=low_cloud_cover_fdi
         qc_flag_locs=[0,4,34,37,38,40,50]
      END
      8: BEGIN
         varvals=mid_cloud_cover
         varmdi=mid_cloud_cover_mdi
         varfdi=mid_cloud_cover_fdi
         qc_flag_locs=[0,4,35,37,38,39,40,51]
      END
      9: BEGIN
         varvals=high_cloud_cover
         varmdi=high_cloud_cover_mdi
         varfdi=high_cloud_cover_fdi
         qc_flag_locs=[0,4,36,37,38,39,40,52]
      END
    ENDCASE
    ;
    ; Were there more than 20% of initial obs for the current variable.

    ; compress flags to one column so compare flagged obs with clean obs
    ; so don't double count flags
    compressed_flags=qc_flags(qc_flag_locs,*)
    no_neighbours=WHERE(compressed_flags EQ -1, nncnt)
    IF nncnt GE 1 THEN compressed_flags[no_neighbours]=0
    compressed_flags=TOTAL(compressed_flags,1)

    flagged=WHERE(compressed_flags GE 1, failcount)
    varrejrate=0
    IF failcount GE 1 THEN BEGIN 
      ; If there were no fails then we are not going to continue interrogtaing this variable.
       not_flagged=WHERE(((varvals NE varmdi) AND (varvals NE varfdi)), varcnt)
       IF varcnt GE 1 THEN BEGIN
          totalorig=varcnt+failcount
          proportion=FLOAT(totalorig)/FLOAT(nt)
          IF proportion GE 0.2 THEN BEGIN 
	; Only continue if mnore than 20% of obs had data for the variable.
             varrejrate=FLOAT(failcount)/FLOAT(totalorig)
             station_criteria.st_rej(st,v)=varrejrate
          ENDIF      
       ENDIF ELSE BEGIN  
        ; All values for this variable were deleted. Are deletions 
        ; greater than 20% of the total observation points?
          proportion=FLOAT(failcount)/FLOAT(nt)
          IF proportion GE 0.2 THEN BEGIN
             station_criteria.st_rej(st,v)=-1
          ENDIF
       ENDELSE
    ENDIF ELSE station_criteria.st_rej(st,v)=0
    
 ENDFOR                         ; v (variable)
ENDFOR                          ; station


CLOSE,1

; now have got all the criteria stored in structure.  Can loop over
; different input criteria to see what matches.

;**********************************************************************
; First set of tests
; first and last input identical, and the most unrestrictive.
;**********************************************************************
nyears=[10,15,20,25,30,10]
first_year=[1980,1975,1985,1990,1980,1980,1980,1980] ; these two to work in step
final_year=[2000,2000,2000,2000,1990,1995,2005,2000]

ncalmonths=[9,10,11,12,9]
max_gap=[48,0,12,24,36,48]
rep_freq=[6,1,3,6]
diurnal=[2,3,4,2]
daysinmonth=[12,20,28,12]
;[T, Td, SLP, ws, wd, cb, ct, lc, mc, hc]
rej_rates=[$
[0.10,0.10,0.10,0.2,0.2,0.2,1.0,1.0,1.0,1.0],$ ; All low
[0.05,0.05,0.05,0.1,0.1,0.2,0.5,0.5,0.5,0.5],$ ; default
[0.05,0.05,0.05,0.1,0.1,0.2,1.0,1.0,1.0,1.0],$ ; no cloud restrictions
[0.01,0.01,0.01,0.1,0.1,0.2,0.5,0.5,0.5,0.5],$ ; high T,Td,S restrictions
[0.10,0.10,0.10,0.1,0.1,0.2,0.5,0.5,0.5,0.5],$ ; low T,Td,S restrictions
[0.05,0.05,0.05,0.05,0.05,0.2,0.5,0.5,0.5,0.5],$ ; high wind restrictions
[0.05,0.05,0.05,0.2,0.2,0.2,0.5,0.5,0.5,0.5],$  ; low wind restrictions
[0.10,0.10,0.10,0.2,0.2,0.2,1.0,1.0,1.0,1.0]$ ; Alllow
]
rej_name=['AllLow','Default','NoCloud','HighTTdS','LowTTdS','HighWind','LowWind','AllLow']

;**********************************************************************
; Second set of tests
; even though smaller number, all combinations would still be 100s realisations
;**********************************************************************

nyears=[20,25,20]
first_year=[1980,1980,1975,1975,1980] ; these two to work in step
final_year=[2000,2005,2005,2000,2000]

ncalmonths=[12]
max_gap=[24,12,24]
rep_freq=[3]
diurnal=[3,4,3]
daysinmonth=[20]
;[T, Td, SLP, ws, wd, cb, ct, lc, mc, hc]
rej_rates=[$
[0.10,0.10,0.10,0.2,0.2,0.2,1.0,1.0,1.0,1.0],$ ; All low (10%)
[0.01,0.01,0.01,0.1,0.1,0.2,0.5,0.5,0.5,0.5],$ ; high T,Td,S restrictions 1%
[0.02,0.02,0.02,0.1,0.1,0.2,0.5,0.5,0.5,0.5],$ ; lower T,Td,S restrictions 2%
[0.05,0.05,0.05,0.1,0.1,0.2,0.5,0.5,0.5,0.5],$ ; lower T,Td,S restrictions 5%
[0.01,0.01,0.01,1.0,1.0,1.0,1.0,1.0,1.0,1.0],$ ; high T,Td,S restrictions 1%, no wind or cloud restrictions
[0.02,0.02,0.02,1.0,1.0,1.0,1.0,1.0,1.0,1.0],$ ; lower T,Td,S restrictions 2%, no wind or cloud restrictions
[0.05,0.05,0.05,1.0,1.0,1.0,1.0,1.0,1.0,1.0],$ ; lower T,Td,S restrictions 5%, no wind or cloud restrictions
[0.10,0.10,0.10,0.2,0.2,0.2,1.0,1.0,1.0,1.0]$ ; All low (10%)
]
rej_name=['AllLow','1pcWC','2pcWC','5pcWC','1pc','2pc','5pc','AllLow']

;**********************************************************************
; Final set of tests
; even though smaller number, all combinations would still be 100s realisations
;**********************************************************************

nyears=[20]
first_year=[1980] ; these two to work in step
final_year=[2000]

ncalmonths=[12]
max_gap=[24]
rep_freq=[3]
diurnal=[3]
daysinmonth=[20]
;[T, Td, SLP, ws, wd, cb, ct, lc, mc, hc]
rej_rates=[$
[0.05,0.05,0.05,1.0,1.0,1.0,1.0,1.0,1.0,1.0]$ ; lower T,Td,S restrictions 5%, no wind or cloud restrictions
]
rej_name=['5pc']

N_nyears=N_ELEMENTS(nyears)
N_ftyear= N_ELEMENTS(first_year)
N_flyear= N_ELEMENTS(final_year)
N_months=N_ELEMENTS(ncalmonths)
N_gap   =N_ELEMENTS(max_gap)
N_freq  =N_ELEMENTS(rep_freq)
N_diurnal=N_ELEMENTS(diurnal)
N_days  =N_ELEMENTS(daysinmonth)
N_rej  =N_ELEMENTS(rej_name)

; set up flags to show which stations are bad.
flagged=INTARR(nstation)
flagged[*]=0

ny=0
fty=0
fly=0
mth=0
gap=0
freq=0
diur=0
day=0
rej=0

WHILE 1 DO BEGIN

file_root=STRCOMPRESS(nyears[ny],/remove_all)+'_'+STRCOMPRESS(first_year[fty],/remove_all)+'_'+$
             STRCOMPRESS(final_year[fly],/remove_all)+'_'+STRCOMPRESS(ncalmonths[mth],/remove_all)+'_'+$
             STRCOMPRESS(max_gap[gap],/remove_all)+'_'+STRCOMPRESS(rep_freq[freq],/remove_all)+'_'+$
             STRCOMPRESS(diurnal[diur],/remove_all)+'_'+STRCOMPRESS(daysinmonth[day],/remove_all)+'_'+rej_name[rej]

PRINT,file_root

accept_file='accept_'+file_root+'.txt'

reject_file='reject_'+file_root+'.txt'

plot_file='plot_'+file_root+'.ps'

FILE_DELETE, accept_file,/allow_nonexistent
FILE_DELETE, reject_file,/allow_nonexistent


FOR st=0,nstation-1 DO BEGIN

   first_obs_before=JULDAY(12, 31, first_year[fty], 0, 0)-JULDAY(1,1,1973,0,0)
   last_obs_after=JULDAY(1, 1, final_year[fly], 0, 0)-JULDAY(1,1,1973,0,0)

   ;**********************************************************
   ; check which stations have first year before criterion
   ;**********************************************************
   IF station_criteria.st_fyear(st) GT first_obs_before THEN BEGIN
    ; first observation is after the assigned first 
    ;  obs date so the test has failed.
      CALDAT,station_criteria.st_fyear(st)+JULDAY(1,1,1973,0,0),dm,dd,dy
      OPENW,2,reject_file,/append
      PRINTF,2,station_ids(st),'First timestep occurs after assigned minimum start date. ', dy,FORMAT='(A15,2x,A,I)'
      CLOSE,2
      CONTINUE                  ; to next station
   ENDIF


   ;**********************************************************
   ; check which unflagged stations have last year after criterion
   ;**********************************************************
   IF station_criteria.st_eyear(st) LT last_obs_after THEN BEGIN
    ; last observation is before the assigned last 
    ; obs date so the test has failed.
      CALDAT,station_criteria.st_fyear(st)+JULDAY(1,1,1973,0,0),dm,dd,dy
      OPENW,2,reject_file,/append
      PRINTF,2,station_ids(st),'Last timestep occurs before assigned minimum end date. ',dy,FORMAT='(A15,2x,A,I)'
      CLOSE,2
      CONTINUE                  ; to next station
   ENDIF

   ;**********************************************************
   ; check which unflagged stations meet hour/day/month criteria
   ;   Followed by 
   ; check which unflagged stations have small enough gaps
   ;**********************************************************

   monthdaycounts=INTARR(12*(last_year-1973+1)) ; 12 months, 35 years.
   mloc=0
   yloc=0
   
      ; for each month in record
   FOR m=0,12*(last_year-1973+1)-1 DO BEGIN

         ; Now ascertain how many days of month meet the quadrant criteria set by diurnal
      diurnal_days=station_criteria.st_complete(st,yloc,mloc,*)
      
      meet_diurnal_loc=WHERE(diurnal_days GE diurnal[diur],dircnt)
      
      monthdaycounts[m]=dircnt
         ; change over at end of years
      mloc=mloc+1
      IF mloc EQ 12 THEN BEGIN
         mloc=0
         yloc=yloc+1
      ENDIF
   ENDFOR                       ; m
                               
      ; Reform monthdaycounts and then ascertain for how many years for each month these count criteria were met.
   monthdaycounts=REFORM(monthdaycounts,12,(last_year-1973+1))
   years_month_criteria_met=INTARR(12)
   FOR m=0,11 DO BEGIN
      sdim=WHERE(monthdaycounts(m,*) GE daysinmonth[day], mcnt) ; Number of occurences of obs occuring more than crit val number of times
      years_month_criteria_met(m)=mcnt
   ENDFOR                       ; m

      ; Now compare the number of months against the critical number of years required through keyword nyears
   exceedances=WHERE(years_month_criteria_met GE nyears[ny], ec)

      ; Does the number of months exceeding the criteria itself exceed the ncalmonths criteria?
   IF ec LT ncalmonths[mth] THEN BEGIN
      OPENW,2,reject_file,/append
      PRINTF,2,station_ids(st),'Insufficient number of reports to meet stated diurnal / monthly / multiannual specified requirements.',FORMAT='(A15,2x,A)'
      CLOSE,2
      CONTINUE                  ; to next station
   ENDIF
   
  ;**********************************************************************************************************
  ; Next logical check is gap_check criteria seeing as we 
  ; have the month presence criteria handy at this
  ; point in time. We will use the months that exceeded the 
  ; month inclusion criteria as our decision aid here
  ; rather than allowing months with odd data points to ensure that we are conservative.
  ;**********************************************************************************************************
   monthdaycounts=REFORM(monthdaycounts,12*(last_year-1973+1))
   matchpoints=WHERE(monthdaycounts GE daysinmonth[day], complement=nonmatchpoints, ncomplement=nf)
   monthdaycounts(matchpoints)=1
   IF nf GE 1 THEN monthdaycounts(nonmatchpoints)=0
  ; Now we have a binary array value set where 1 passes and 0 fails.
  ;
   max_missing=0
   missing_string=0
   data_present=0

   ; for each month in the record
   ;   scroll through to find max period of missing
   broken=0
   FOR m=0,12*(last_year-1973+1)-1 DO BEGIN 
      IF monthdaycounts(m) EQ 1 THEN BEGIN
         IF missing_string GT max_gap[gap] THEN BEGIN
            ; There is a gap in data availability longer than specified maximum permissible gap. So reject this station.
            OPENW,2,reject_file,/append
            PRINTF,2,station_ids(st),'Too large a continuous gap in the timeseries relative to criteria that were set. ',missing_string,FORMAT='(A15,2x,A,I)'
            CLOSE,2
            broken=1
            BREAK               ;  out of this loop and onto next station
         ENDIF ELSE BEGIN
            missing_string=0
            data_present=1
         ENDELSE
      ENDIF ELSE BEGIN
         IF data_present EQ 1 THEN missing_string=missing_string+1  
         ; Only iterates if there was a pre-existing value to avoid issues
         ; with stations that commenced later than 1973
      ENDELSE
   ENDFOR
   IF broken EQ 1 THEN CONTINUE

   ;**********************************************************
   ; check which unflagged stations have good reporting frequencies
   ;**********************************************************
   IF station_criteria.st_rfreq(st) GT FLOAT(rep_freq[freq])/24. THEN BEGIN 
   ; There is too large a typical reporting frequency 
   ; vis-a-vis set requirements on call
      OPENW,2,reject_file,/append
      PRINTF,2,station_ids(st),'The median gap between consecutive observations is larger than assigned critical value. ',ROUND(station_criteria.st_rfreq(st)*24),FORMAT='(A15,2x,A,I)'
      CLOSE,2
      CONTINUE
   ENDIF

   ;**********************************************************
   ; check which variables pass on which unflagged stations
   ;**********************************************************
   rej_rate=rej_rates[*,rej]
   broken=0
   FOR v=0,9 DO BEGIN
      IF station_criteria.st_rej(st,v) GT rej_rate(v) THEN BEGIN 
         OPENW,2,reject_file,/append
         PRINTF,2,station_ids(st),'> critical proportion of the original observations of '+varnames(v)+' were removed ',station_criteria.st_rej(st,v),FORMAT='(A15,2x,A,F)'
         CLOSE,2
         broken=1
         BREAK
      ENDIF
      IF (station_criteria.st_rej(st,v) EQ -1) AND (flagged[st] NE 1) THEN BEGIN 
         OPENW,2,reject_file,/append
         PRINTF,2,station_ids(st),'All observations of '+varnames(v)+' were removed and it originally constituted more than 20% of data points.',FORMAT='(A15,2x,A)'
         CLOSE,2
         broken=1
         BREAK
      ENDIF
   ENDFOR

   IF broken EQ 1 THEN CONTINUE

   ;**********************************************************
   ; all unflagged stations are good.  Write out
   ;**********************************************************  
   OPENW,2,accept_file,/append
   PRINTF,2,station_ids(st),FORMAT='(A15)'
   CLOSE,2

   
ENDFOR                          ; station



;************************************************************************
; Having created the accepted and rejected lists, plot them out.
;************************************************************************

; read in list of accepted files

nastats=NLINES(accept_file)

IF nastats GE 1 THEN BEGIN

   acc_station_ids=STRARR(nastats)
   acc_lats=FLTARR(nastats)
   acc_lons=FLTARR(nastats)
   acc_elevs=FLTARR(nastats)


   OPENR,1,accept_file
   line=''
   stn=0
   WHILE NOT EOF(1) DO BEGIN
      READF,1,line
      split_line=STRSPLIT(line,/extract)
      acc_station_ids(stn)=split_line[0]
      
      stat_loc=WHERE(station_ids EQ acc_station_ids[stn],slcnt)
      
      IF slcnt EQ 1 THEN BEGIN
         acc_lats[stn]=station_lats[stat_loc]
         acc_lons[stn]=station_lons[stat_loc]
         acc_elevs[stn]=station_elevs[stat_loc]
         
      ENDIF ELSE STOP, 'cannot find accepted station in list'
      
      stn=stn+1
   ENDWHILE
   CLOSE,1
ENDIF

; Read in list of rejected files

nrstats=nstation-nastats

IF nrstats GE 1 THEN BEGIN

   rej_station_ids=STRARR(nrstats)
   rej_lats=FLTARR(nrstats)
   rej_lons=FLTARR(nrstats)
   rej_elevs=FLTARR(nrstats)
   
   
   OPENR,1,reject_file
   line=''
   stn=0
   prev_rej_stat_id=''
   WHILE NOT EOF(1) DO BEGIN
      READF,1,line
      split_line=STRSPLIT(line,/extract)
      IF split_line[0] NE prev_rej_stat_id THEN BEGIN
         
         rej_station_ids(stn)=split_line[0]
         stat_loc=WHERE(station_ids EQ rej_station_ids[stn],slcnt)
         
         IF slcnt EQ 1 THEN BEGIN
            rej_lats[stn]=station_lats[stat_loc]
            rej_lons[stn]=station_lons[stat_loc]
            rej_elevs[stn]=station_elevs[stat_loc]
            
         ENDIF ELSE STOP, 'cannot find rejected station in list'
         prev_rej_stat_id=rej_station_ids[stn]
         
         stn=stn+1
      ENDIF 
      
   ENDWHILE
   CLOSE,1
ENDIF
; create 2 panel plot showing distribution of the stations.
 
SET_PLOT,'ps'
Device,file='./images/'+plot_file,/color,/helvetica, xsize=20, ysize=28, /portrait
!P.font=0
plotsym,0,0.55,/fill
RESTORE_COLOURS,'shortbow'


!P.Position=[0.05,0.05,0.95,0.47]

Map_set,/continents,/noborder,color=0,/noerase,/robinson,/grid, glinestyle=2
IF nrstats GE 1 THEN PLOTS,rej_lons,rej_lats,psym=8,color=250, symsize=0.5
PLOTS,[-179.9,0],[89.9,89.9],color=0
PLOTS,[0,179.9],[89.9,89.9],color=0
PLOTS,[-179.9,0],[-89.9,-89.9],color=0
PLOTS,[0,179.9],[-89.9,-89.9],color=0

XYOUTS,-180,85,'90N ',alignment=1,charsize=1.0,color=0
XYOUTS,-180,43,'45N ',alignment=1,charsize=1.0,color=0
XYOUTS,-180,-2,'0 ',alignment=1,charsize=1.0,color=0
XYOUTS,-180,-47,'45S  ',alignment=1,charsize=1.0,color=0
XYOUTS,-180,-89,'90S      ',alignment=1,charsize=1.0,color=0

XYOUTS,0.25,0.03,'180W',/normal,charsize=1.0,color=0
XYOUTS,0.37,0.03,'90W',/normal,charsize=1.0,color=0
XYOUTS,0.50,0.03,'0',/normal,charsize=1.0,color=0
XYOUTS,0.64,0.03,'90E',/normal,alignment=1,charsize=1.0,color=0
XYOUTS,0.78,0.03,'180E',/normal,alignment=1,charsize=1.0,color=0

XYOUTS,0.02,0.02,'Total stations: '+STRCOMPRESS(STRING(nrstats),/remove_all)+$
       '/'+STRCOMPRESS(STRING(nstation),/remove_all),/normal
XYOUTS,0.5,0.48,'Rejected Stations',/normal,alignment=0.5

!P.Position=[0.05,0.55,0.95,0.95]
Map_set,/continents,/noborder,color=0,/noerase,/robinson,/grid, glinestyle=2
IF nastats GE 1 THEN PLOTS,acc_lons,acc_lats,psym=8,color=10, symsize=0.5
PLOTS,[-179.9,0],[89.9,89.9],color=0
PLOTS,[0,179.9],[89.9,89.9],color=0
PLOTS,[-179.9,0],[-89.9,-89.9],color=0
PLOTS,[0,179.9],[-89.9,-89.9],color=0

XYOUTS,-180,85,'90N ',alignment=1,charsize=1.0,color=0
XYOUTS,-180,43,'45N ',alignment=1,charsize=1.0,color=0
XYOUTS,-180,-2,'0 ',alignment=1,charsize=1.0,color=0
XYOUTS,-180,-47,'45S  ',alignment=1,charsize=1.0,color=0
XYOUTS,-180,-89,'90S      ',alignment=1,charsize=1.0,color=0

XYOUTS,0.25,0.53,'180W',/normal,charsize=1.0,color=0
XYOUTS,0.37,0.53,'90W',/normal,charsize=1.0,color=0
XYOUTS,0.50,0.53,'0',/normal,charsize=1.0,color=0
XYOUTS,0.64,0.53,'90E',/normal,alignment=1,charsize=1.0,color=0
XYOUTS,0.78,0.53,'180E',/normal,alignment=1,charsize=1.0,color=0


XYOUTS,0.02,0.51,'Total stations: '+STRCOMPRESS(STRING(nastats),/remove_all)+$
       '/'+STRCOMPRESS(STRING(nstation),/remove_all),/normal
XYOUTS,0.5,0.98,'Accepted Stations',/normal,alignment=0.5

DEVICE,/close_file

PRINT, 'Rejected stations: '+STRCOMPRESS(STRING(nrstats),/remove_all)+'  Accepted stations:  '+STRCOMPRESS(STRING(nastats),/remove_all)

; increment the counters.

IF fty LT N_ftyear-1 THEN BEGIN
   fty=fty+1
;ENDIF ELSE IF fly LT N_flyear-1 THEN BEGIN
   fly=fly+1
ENDIF ELSE IF freq LT N_freq-1 THEN BEGIN
   freq=freq+1
ENDIF ELSE IF rej LT N_rej-1 THEN BEGIN
   rej=rej+1
ENDIF ELSE IF diur LT N_diurnal-1 THEN BEGIN
   diur=diur+1
ENDIF ELSE IF day LT N_days-1 THEN BEGIN
   day=day+1
ENDIF ELSE IF mth LT N_months-1 THEN BEGIN
   mth=mth+1
ENDIF ELSE IF ny LT N_nyears-1 THEN BEGIN
   ny=ny+1
ENDIF ELSE IF gap LT N_gap-1 THEN BEGIN
   gap=gap+1
ENDIF ELSE BREAK

ENDWHILE

ENDIF ; keyword select

IF KEYWORD_SET(doplots) THEN BEGIN

;************************************************************************
; Having created the accepted and rejected lists, plot them out.
;************************************************************************
file_root=doplots
accept_file='accept_'+file_root+'.txt'
reject_file='reject_'+file_root+'.txt'
plot_file='plot_'+file_root+'.ps'
; read in list of accepted files

nastats=NLINES(accept_file)

IF nastats GE 1 THEN BEGIN

   acc_station_ids=STRARR(nastats)
   acc_lats=FLTARR(nastats)
   acc_lons=FLTARR(nastats)
   acc_elevs=FLTARR(nastats)


   OPENR,1,accept_file
   line=''
   stn=0
   WHILE NOT EOF(1) DO BEGIN
      READF,1,line
      split_line=STRSPLIT(line,/extract)
      acc_station_ids(stn)=split_line[0]
      
      stat_loc=WHERE(station_ids EQ acc_station_ids[stn],slcnt)
      
      IF slcnt EQ 1 THEN BEGIN
         acc_lats[stn]=station_lats[stat_loc]
         acc_lons[stn]=station_lons[stat_loc]
         acc_elevs[stn]=station_elevs[stat_loc]
         
      ENDIF ELSE STOP, 'cannot find accepted station in list'
      
      stn=stn+1
   ENDWHILE
   CLOSE,1
ENDIF

; Read in list of rejected files

nrstats=nstation-nastats

IF nrstats GE 1 THEN BEGIN

   rej_station_ids=STRARR(nrstats)
   rej_lats=FLTARR(nrstats)
   rej_lons=FLTARR(nrstats)
   rej_elevs=FLTARR(nrstats)
   rej_reasons=STRARR(nrstats)
   
   
   OPENR,1,reject_file
   line=''
   stn=0
   prev_rej_stat_id=''
   WHILE NOT EOF(1) DO BEGIN
      READF,1,line
      split_line=STRSPLIT(line,/extract)
      IF split_line[0] NE prev_rej_stat_id THEN BEGIN
         
         rej_station_ids(stn)=split_line[0]
         rej_reasons(stn)=split_line[1]
         stat_loc=WHERE(station_ids EQ rej_station_ids[stn],slcnt)
         
         IF slcnt EQ 1 THEN BEGIN
            rej_lats[stn]=station_lats[stat_loc]
            rej_lons[stn]=station_lons[stat_loc]
            rej_elevs[stn]=station_elevs[stat_loc]
            
         ENDIF ELSE STOP, 'cannot find rejected station in list'
         prev_rej_stat_id=rej_station_ids[stn]
         
         stn=stn+1
      ENDIF 
      
   ENDWHILE
   CLOSE,1
ENDIF
; create 2 panel plot showing distribution of the stations.
 
SET_PLOT,'ps'
Device,file='./images/'+plot_file,/color,/helvetica, xsize=20, ysize=28, /portrait,decomposed=0
tek_color
!P.font=0
plotsym,0,0.55,/fill
tvlct,0,0,0,20   ; black
tvlct,0,0,255,1   ; blue
tvlct,255,255,0,2 ; yellow
tvlct,255,100,255,3  ; pink
tvlct,255,0,0,4   ; red
tvlct,0,255,0,5 ; green
tvlct,255,0,0,6 ; red
tvlct,255,255,255,10 ;white


!P.Position=[0.05,0.05,0.95,0.47]

Map_set,0,0,/noborder,color=20,/noerase,/robinson,/grid, glinestyle=2
map_continents,/coasts, color=20 ;map_ne_coastlines,color=20

IF nrstats GE 1 THEN BEGIN
   sellocs=WHERE((rej_reasons EQ 'Insufficient'), selcnt)
   IF selcnt GE 1 THEN BEGIN
      PRINT,'Diurnal/Monthly/Annual Requirement failures ',selcnt
      PLOTS,rej_lons[sellocs],rej_lats[sellocs],psym=8,color=1, symsize=0.8
   ENDIF
   yearlocs=WHERE((rej_reasons EQ 'First') OR (rej_reasons EQ 'Last'), ylcnt)
   IF ylcnt GE 1 THEN BEGIN
      PRINT,'Record start and end failures ',ylcnt
      PLOTS,rej_lons[yearlocs],rej_lats[yearlocs],psym=8,color=2, symsize=0.8
   ENDIF
   gaplocs=WHERE((rej_reasons EQ 'Too'), gapcnt)
   IF gapcnt GE 1 THEN BEGIN
      PRINT,'Gap size failures ',gapcnt
      PLOTS,rej_lons[gaplocs],rej_lats[gaplocs],psym=8,color=3, symsize=0.8
   ENDIF
   replocs=WHERE((rej_reasons EQ 'The'), repcnt)
   IF repcnt GE 1 THEN BEGIN
      PRINT,'Reporting time failures ',repcnt
      PLOTS,rej_lons[replocs],rej_lats[replocs],psym=8,color=5, symsize=0.8
   ENDIF
   rejlocs=WHERE((rej_reasons EQ '>') OR (rej_reasons EQ 'All'), rejcnt)
   IF rejcnt GE 1 THEN BEGIN
      PRINT,'Flag rate failures ',rejcnt
      PLOTS,rej_lons[rejlocs],rej_lats[rejlocs],psym=8,color=4, symsize=0.8
   ENDIF


ENDIF
PLOTS,[-179.9,0],[89.9,89.9],color=0
PLOTS,[0,179.9],[89.9,89.9],color=0
PLOTS,[-179.9,0],[-89.9,-89.9],color=0
PLOTS,[0,179.9],[-89.9,-89.9],color=0

XYOUTS,-180,85,'90N ',alignment=1,charsize=1.0,color=0
XYOUTS,-180,43,'45N ',alignment=1,charsize=1.0,color=0
XYOUTS,-180,-2,'0 ',alignment=1,charsize=1.0,color=0
XYOUTS,-180,-47,'45S  ',alignment=1,charsize=1.0,color=0
XYOUTS,-180,-89,'90S      ',alignment=1,charsize=1.0,color=0

XYOUTS,0.25,0.03,'180W',/normal,charsize=1.0,color=0
XYOUTS,0.37,0.03,'90W',/normal,charsize=1.0,color=0
XYOUTS,0.50,0.03,'0',/normal,charsize=1.0,color=0
XYOUTS,0.64,0.03,'90E',/normal,alignment=1,charsize=1.0,color=0
XYOUTS,0.78,0.03,'180E',/normal,alignment=1,charsize=1.0,color=0

polyfill,[0.04,0.04,0.25,0.25],[0.03,0.13,0.13,0.03],color=10,/normal
XYOUTS,0.04,0.12,'D/M/A/IA fails ('+STRCOMPRESS(selcnt,/remove_all)+')',color=0,charsize=0.8,/normal
plots,[0.02],[0.125],psym=8,color=1,symsize=2,/normal
XYOUTS,0.04,0.10,'Record start/stop fails ('+STRCOMPRESS(ylcnt,/remove_all)+')',color=0,charsize=0.8,/normal
plots,[0.02],[0.105],psym=8,color=2,symsize=2,/normal
XYOUTS,0.04,0.08,'Gap Size fails ('+STRCOMPRESS(gapcnt,/remove_all)+')',color=0,charsize=0.8,/normal
plots,[0.02],[0.085],psym=8,color=3,symsize=2,/normal
XYOUTS,0.04,0.06,'Reporting Interval fails ('+STRCOMPRESS(repcnt,/remove_all)+')',color=0,charsize=0.8,/normal
plots,[0.02],[0.065],psym=8,color=5,symsize=2,/normal
XYOUTS,0.04,0.04,'Flag level fails ('+STRCOMPRESS(rejcnt,/remove_all)+')',color=0,charsize=0.8,/normal
plots,[0.02],[0.045],psym=8,color=4,symsize=2,/normal

XYOUTS,0.02,0.02,'Total stations: '+STRCOMPRESS(STRING(nrstats),/remove_all)+$
       '/'+STRCOMPRESS(STRING(nstation),/remove_all),/normal
XYOUTS,0.5,0.48,'Rejected Stations',/normal,alignment=0.5

!P.Position=[0.05,0.55,0.95,0.95]
Map_set,0,0,/noborder,color=20,/noerase,/robinson,/grid, glinestyle=2
map_continents,/coasts, color=20 ;map_ne_coastlines,color=20
IF nastats GE 1 THEN PLOTS,acc_lons,acc_lats,psym=8,color=1, symsize=0.8

OPENR,1,'duplicates_ordered.txt'
lat=0.
lon=0.
station_id=''
total_stns=0
composite_total=0
WHILE NOT EOF(1) DO BEGIN
  ; Read in the current line
  READF,1,station_id,FORMAT='(A12)'
  station_id=STRMID(station_id,0,6)+'-'+STRMID(station_id,7,5)
  match=WHERE(acc_station_ids EQ station_id, c)
  IF c EQ 1 THEN BEGIN
     PLOTS,acc_lons(match(0)),acc_lats(match(0)),psym=8,color= 6
     composite_total=composite_total+1
  ENDIF
  total_stns=total_stns+1  
ENDWHILE
CLOSE,1

PRINT, 'Number of composite stations in filtered dataset is ',composite_total


PLOTS,[-179.9,0],[89.9,89.9],color=0
PLOTS,[0,179.9],[89.9,89.9],color=0
PLOTS,[-179.9,0],[-89.9,-89.9],color=0
PLOTS,[0,179.9],[-89.9,-89.9],color=0

XYOUTS,-180,85,'90N ',alignment=1,charsize=1.0,color=0
XYOUTS,-180,43,'45N ',alignment=1,charsize=1.0,color=0
XYOUTS,-180,-2,'0 ',alignment=1,charsize=1.0,color=0
XYOUTS,-180,-47,'45S  ',alignment=1,charsize=1.0,color=0
XYOUTS,-180,-89,'90S      ',alignment=1,charsize=1.0,color=0

XYOUTS,0.25,0.53,'180W',/normal,charsize=1.0,color=0
XYOUTS,0.37,0.53,'90W',/normal,charsize=1.0,color=0
XYOUTS,0.50,0.53,'0',/normal,charsize=1.0,color=0
XYOUTS,0.64,0.53,'90E',/normal,alignment=1,charsize=1.0,color=0
XYOUTS,0.78,0.53,'180E',/normal,alignment=1,charsize=1.0,color=0

polyfill,[0.04,0.04,0.25,0.25],[0.56,0.60,0.60,0.56],color=10,/normal
XYOUTS,0.04,0.59,'Single stations ('+STRCOMPRESS(nastats-composite_total,/remove_all)+')',color=0,charsize=0.8,/normal
plots,[0.02],[0.595],psym=8,color=1,symsize=2,/normal
XYOUTS,0.04,0.57,'Composite stations ('+STRCOMPRESS(composite_total,/remove_all)+')',color=0,charsize=0.8,/normal
plots,[0.02],[0.575],psym=8,color=6,symsize=2,/normal

XYOUTS,0.02,0.51,'Total stations: '+STRCOMPRESS(STRING(nastats),/remove_all)+$
       '/'+STRCOMPRESS(STRING(nstation),/remove_all),/normal
XYOUTS,0.5,0.98,'Accepted Stations',/normal,alignment=0.5

DEVICE,/close_file

PRINT, 'Rejected stations: '+STRCOMPRESS(STRING(nrstats),/remove_all)+'  Accepted stations:  '+STRCOMPRESS(STRING(nastats),/remove_all)
stop
ENDIF

RETURN

END
;************************************************************************
;                                 END
;************************************************************************
