;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;British Crown Copyright (c) 2012, the Met Office All rights reserved.
;
;Redistribution and use in source and binary forms, with or without 
;modification, are permitted provided that the following conditions 
;are met:
;
;Redistributions of source code must retain the above copyright notice, 
;this list of conditions and the following disclaimer. 
;
;Redistributions in binary form must reproduce the above copyright 
;notice, this list of conditions and the following disclaimer in the 
;documentation and/or other materials provided with the distribution. 
;
;Neither the name of the Met Office nor the names of its contributors 
;may be used to endorse or promote products derived from this software 
;without specific prior written permission. 
;
;THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
;"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
;LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
;A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
;HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
;SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
;LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
;DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
;THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
;(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
;OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

PRO UNFLAGGING, qc_flags, plotting_flags, difference_values, column, sum_neighbours, label, type, $
                no_neighbours=no_neighbours, dpd=dpd, all_dewpoint_flags=all_dewpoint_flags

; NAME:
;     UNFLAGGING
;
; PURPOSE:
;     To unset flags if the neighbour checks pass certain tests
;
; CALLING SEQUENCE:
;     unflagging, qc_flags, difference_values, column, sum_neighbours,
;     label, type, no_neighbours=no_neighbours, dpd=dpd,
;     all_dewpoint_flags=all_dewpoint_flags 
;
; INPUTS:
;     qc_flags - array containing the QC flags
;
;     plotting_flags - array containing flags to show where QC has
;                      been undone
;                      = 1 for a flag set
;                      = 2 if sufficient neighbours
;                      = 3 flag undone
;
;     difference_values - array containing the offsets of the values
;                         of the station in question from the median
;                         of all the neighbours which are not mdi, in
;                         units of the Median Absolute Deviation
;
;     column - which column (which flag) to work on
;
;     sum_neighbours - how many neighbours have valid values at each hour
;
;     label - what label - for output to screen
;
;     type - what flag to undo - for output to screen
;
;     no_neighbours - keyword to also unset the flag if no neighbours
;                     are present to correlate with
;
;     dpd - keyword to so specific test for the dewpoint depression
;
;     all_dewpoint_flags - keyword array to pass all the DPD flags
;                          from all the neighbours
;
;
; METHOD:
;     Finds the locations where there are more than 3 neighbours
;     present and the relevant flag has been set in the station under
;     scrutiny.  If the values of this station are within the bounds
;     (4.5 Median Absolute Deviations ["~3 sigma"]) of the median of the
;     neighbours then undo the flag, else keep the flag.
;
;     If DPD - unset the flag if at least 2/3 of the neighbours also
;              have the DPD flag set.
;
;     For climatological check, if there are insufficient neighbours
;     to cross-correlate with (<3) then undo the flag as we cannot be
;     certain that these tentative flags (See header for
;     ClimatologicalCheck.pro) are appropriate. 
;
;
; OUTPUTS:
;
;     None - arrays returned to control script
;
; RESTRICTIONS:
;
;     
;   
; HISTORY:
;      #165 - Created to undo certain flags after the neighbour check
;             7 January 2011 RJHD
;      #167 - Adjusted to use 4.5 MADs (~3sigma) in the within bounds check 
;             12 January 2011 RJHD
;      #168 - Added plotting_flags for diagnostic plotting using outlier_plot.pro
;             13 January 2011 RJHD
;      #191 - Checked that it works with new gap check unflagging
;             procedure.  Adjustments to plotting_flags - now three levels.
;             8 March 2011 RJHD
;      #237 - SVN information added to header
;             16 November 2011 RJHD
;      #243 - Changed numerical order of FIRM and TENTATIVE FLAGS
;             21 November 2011 RJHD
;      #249 - Updated /no_neighbours unflagging to work TENTATIVE
;             flags after change at v243
;             25 November 2011 RJHD
;
;************************************************************************
;                    SVN Info
;$Rev:: 308                                           $:  Revision of last commit
;$Author:: rdunn                                      $:  Author of last commit
;$Date:: 2012-07-04 14:47:15 +0100 (Wed, 04 Jul 2012) $:  Date of last commit
;************************************************************************
;                                 START
;************************************************************************



;*************************************************
; extract the flags relevant for this value
;  As climatological and gap check have a 2 level flag system (1 or 2)
;  explicitly search for flags = 2 (tentative flagged)
;*************************************************

value_flags=qc_flags(column,*) 

pfs=WHERE(value_flags EQ 2,prior_flag_sum)

;*************************************************
; find where there are more than half of the neighbours present.
; nothing done on flagging as comparisons have to be totally clean.
;*************************************************

total_exist=WHERE(sum_neighbours GE 3, neighcnt) 

; for plotting only
plotting_flags(*)=0
anyflag=WHERE(value_flags GT 0, aflgcnt)

IF aflgcnt GE 1 THEN BEGIN
    plotting_flags(anyflag)=1
    IF neighcnt GE 1 THEN BEGIN
        nflags=WHERE(value_flags(total_exist) GT 0, nflagcnt) 
        IF nflagcnt GE 1 THEN BEGIN
            plotting_flags(total_exist[nflags])=2
        ENDIF
    ENDIF
ENDIF

IF neighcnt GE 1 THEN BEGIN
;*************************************************
; find where flag has been set when there are neighbours
;  As climatological & gap checks have a 2 level flag system
;  explicitly search for flags = 2 
;  Flag=1 - firm, Flag=2 - tentative
;*************************************************
    flags=WHERE(value_flags(total_exist) EQ 2, flagcnt) 

    value_differences=difference_values(total_exist)

;************************************
; All we are doing is unflagging the appropriate flag
; when appropriate.  All other flags remain set.
; So if this point is bad for some other reason, it will
; still be trapped 30 NOV 2010 & 7 JAN 2011 RJHD
;************************************
    IF flagcnt GE 1 THEN BEGIN

    ;*************************************************
    ; If DPD, then see if DPD has been set on the neighbours
    ;*************************************************
        IF KEYWORD_SET(dpd) THEN BEGIN
            dpd_proportion=all_dewpoint_flags[total_exist[flags]]/sum_neighbours[total_exist[flags]] 

        ;*************************************************
        ; if more than 2/3, then unset the flag
        ;*************************************************
            dpd_exist=WHERE(dpd_proportion GE 2./3., dpdcnt) 

            IF dpdcnt GE 1 THEN BEGIN

                  
                value_flags(total_exist[flags[dpd_exist]])=0
                plotting_flags(total_exist[flags[dpd_exist]])=3

                qc_flags(column,*)=value_flags
                printstring=label+" Outlier "+type+" Unflags"
                PRINT,printstring, dpdcnt," of ", prior_flag_sum, FORMAT='(A-50,I8,A-3,I8)'
            ENDIF    
            
    ;*************************************************
    ; Else, if the value is within 4.5 MADs of the median
    ;*************************************************
        ENDIF ELSE BEGIN

            within_bounds=WHERE(value_differences(flags) LE 4.5, wbc)
        
        ;*************************************************
        ; then unset the flag
        ;*************************************************
            IF wbc GE 1 THEN BEGIN
            
                value_flags(total_exist(flags(within_bounds)))=0
                ; add to array for plotting if diagnostics turned on
                plotting_flags(total_exist[flags[within_bounds]])=3
            
                qc_flags(column,*)=value_flags
                printstring=label+" Outlier "+type+" Unflags"
                PRINT,printstring, wbc, " of " ,prior_flag_sum, FORMAT='(A-50,I8,A-3,I8)'
            ENDIF
        ENDELSE

    
    ENDIF ELSE BEGIN
        printstring="No "+type+" flags concurrent with neighbours"
        PRINT, printstring, prior_flag_sum, FORMAT='(A-50,I8)'
    ENDELSE
ENDIF ELSE PRINT, "Insufficient Neighbours"

;************************************
; If there are insufficient neighbours, then unset the tentative
; climatological flag
;************************************
IF KEYWORD_SET(no_neighbours) THEN BEGIN

    value_flags=qc_flags(column,*) 

    total_exist=WHERE(sum_neighbours LT 3, tec)

    IF tec GE 1 THEN BEGIN
       
        flags=WHERE(value_flags(total_exist) EQ 2, flagcnt) ; only for TENTATIVE flags
        IF flagcnt GE 1 THEN BEGIN
            value_flags(total_exist(flags))=0
            plotting_flags(total_exist[flags])=3

            qc_flags(column,*)=value_flags
            printstring=label+" Outlier "+type+" Unflags - no neigh"
            PRINT,printstring, flagcnt, FORMAT='(A-50,I8)'
        ENDIF
    ENDIF ELSE PRINT, "No "+type+" flags concurrent with no neighbours"

ENDIF

RETURN

END
;************************************************************************
;                                 END
;************************************************************************

