;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;British Crown Copyright (c) 2012, the Met Office All rights reserved.
;
;Redistribution and use in source and binary forms, with or without 
;modification, are permitted provided that the following conditions 
;are met:
;
;Redistributions of source code must retain the above copyright notice, 
;this list of conditions and the following disclaimer. 
;
;Redistributions in binary form must reproduce the above copyright 
;notice, this list of conditions and the following disclaimer in the 
;documentation and/or other materials provided with the distribution. 
;
;Neither the name of the Met Office nor the names of its contributors 
;may be used to endorse or promote products derived from this software 
;without specific prior written permission. 
;
;THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
;"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
;LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
;A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
;HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
;SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
;LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
;DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
;THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
;(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
;OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

PRO VARIANCE_CHECK, last_year, times, full_times, h_times, full_h_times, values, full_values, mdi, fdi, qc_flags, filter_flags, $
                    columns, flagged_values, mask=mask, diagnostics, station_id, $
                    windsp=windsp, full_windspeeds=full_windspeeds

;*************************************************************
; Program undertakes a variance check that removes years with 'unusual' variance
; greater than 4 inter-quartile range pseudo-sigma values from the median value for that station 
;*************************************************************
; mean annual variance will be calculated from all data in anomaly space - only for years with >9 months of data
; KATE NEW OCT 9th 09 - and ONLY FOR data with more than 10 years present
; REMOVALS - careful comparing years with less than 6 months of data as variance may be skewed
;*************************************************************
; NEW - KATE OCT 9th 09
; CAVEAT - changes in reporting frequency and decimal resolution will change the variance - check years for 
; consistency and expand threshold for years with 'abnormal' frequency/resolution
;*************************************************************
;
; NAME:
;     VARIANCE_CHECK
;
; PURPOSE:
;     To find months of observations which have excess variance, those which
;     fall outside of the expected range in monthly medians and spreads by a long way.
;
; CALLING SEQUENCE:
;     VARIANCE_CHECK, times, full_times, values, full_values, mdi,
;     qc_flags, last_year=last_year, mask=mask, diagnostics,
;     station_id, windsp=windsp, full_windspeeds=full_windspeeds
;
;
; INPUTS:
;     last_year - keyword for the last year of observations
;
;     times - array containing the times of the observations in
;             fractions of days
;
;     full_times - array containing the uncompressed times of the
;                  observations in fractions of days
;
;     h_times - array containing the times of the observations in hours
;
;     full_h_times - array containing the uncompressed times of the
;                  observations in hours
;
;     values - array containing the variable's values
;
;     full_values - array containing the uncompressed values
;
;     mdi - missing data indicator
;
;     fdi - flagged data indicator
;
;     qc_flags - array containing all the flags
;
;     filter_flags - array containing all flags previously set on
;                    values
;
;     columns - which column in qc array to fill
;
;     flagged_values - array containing the observations which have
;                      been flagged
;
;     mask - keyword to apply the masking
;     
;     diagnostics - keyword for diagnostic plots
;
;     station_id - station identifier for plots
;
;     windsp - array containing the windspeed data (SLP only)
;
;     full_windspeeds - array containing the uncompressed windspeed
;                       data (SLP only)
;
; OUTPUTS:
;
;     Updates arrays and returns to control programme
;
; RESTRICTIONS:
;
;     
;   
; HISTORY:
;     NB - not all revisions are noted here.
;     # 33 - Initial form of the program
;             9 November 2010 RJHD
;     # 34 - Converted to be run once per variable.  Commented and
;           noted similarities with Climatological_check.pro
;             9 November 2010 RJHD
;     # 40 - Added diagnostic histogram
;             10 November 2010 RJHD
;     # 42 - Adjusted diagnostic histogram
;             11 November 2010 RJHD
;     # 50 - Adjusted diagnostic histogram (again)
;             15 November 2010 RJHD
;     # 128 - Adjusted diagnostic histogram (again).  Now shows the
;             IQR thresholds (the last ones used) on the anomalised
;             results.  Much clearer.  The IQR thresholds for the SLP
;             were brought in line to those for the temperature and
;             dewpoints.  Also updated winsorising section.
;             6 December 2010 RJHD
;     # 139 - Added diagnostic plot showing the first difference
;             series to the SLPs - to see what is being produced and flagged
;             9 December 2010 RJHD
;     # 151 - Added windspeed as a further test as to whether a storm
;             is being flagged as having excess variance.  Using
;             median w/s over all years for that month, and 6x MAD as
;             the threshold over which it's a storm.  Max w/s and min
;             SLP have to be within 24hrs of each other.
;             14 December 2010 RJHD
;     # 171 - Added filter flags to pass through the total flags set
;             for each value
;             14 January 2011 RJHD
;     # 173 - Used filtered values to set up average variances etc
;             17 January 2011 RJHD
;     # 181 - Adjusted the threshold for a storm
;             07 February 2011 RJHD
;     # 199 - Changed MAD offset for storm threshold to 4.5 (from 4.0)
;             to match a) definition of 3sigma for gaussian
;             distribution and b) neighbour unflagging
;             11 March 2011 RJHD
;     # 200 - changed last_year from optional keyword to required input
;             to remove the hardwiring of the end date
;             11 March 2011 RJHD
;     # 229 - Improved labelling of diagnostic histogram
;             28 September 2011 RJHD
;     # 231 - Improved colouring of diagnostic histogram
;             30 September 2011 RJHD
;     # 237 - SVN information added to header
;             Added fdi and flagged_value functionality
;             16 November 2011 RJHD
;     # 299 - Added hourly times for matching and changed floats to doubles
;             16 November 2011 RJHD
;
;************************************************************************
;                    SVN Info
;$Rev:: 308                                           $:  Revision of last commit
;$Author:: rdunn                                      $:  Author of last commit
;$Date:: 2012-07-04 14:47:15 +0100 (Wed, 04 Jul 2012) $:  Date of last commit
;************************************************************************
;                                 START
;************************************************************************

IF columns EQ 58 THEN BEGIN
   flagged_col=0
ENDIF ELSE IF columns EQ 59 THEN BEGIN
   flagged_col=1
ENDIF ELSE IF columns EQ 60 THEN BEGIN
   flagged_col=2
ENDIF

;**************************************************************
;**************************************************************
;;;;; THE START OF THE SIMILARITY WITH CLIMATOLOGICAL_CHECK.PRO
;**************************************************************
;**************************************************************


;**************************************************************
; Set up a duplicate of full_values - 
;       one needed for script and is amended during the run.  This is
;       now filtered using the filters_flags          (17/1/11)
; "values" array does not need to be filtered as only used for
; flagging at the end of the script anyway
;**************************************************************
untouched_full_values=full_values

MATCH, h_times, full_h_times, suba, subb

past_flags=WHERE(filter_flags GT 0, pfcnt)

IF pfcnt GE 1 THEN BEGIN
    full_values(subb(past_flags))=mdi
ENDIF

start_date = JULDAY(1, 1, 1973, 0)


;*************************************************************
; Firstly ascertain the *typical* recording precision based upon the prevalence of given 
; decimal points within the legal values of the series for the recorded parameters. This then 
; has an implication for the critical value derivation later on
;*************************************************************


REPORTING_ACCURACY, last_year, values,mdi,fdi,vres,remhist,times, vfreq

;**************************************************************
; Set up the monthly values
;**************************************************************

MONTH_SETUP,last_year,nmonths,nyears,times,full_times,monthly_segment_starts,$
  monthly_segment_ends, monthly_segment_lengths,/full


monthly_segment_lengths(nmonths-1)=744 ;31mths*24hrs
monthly_segment_ends(nmonths-1)=monthly_segment_ends(nmonths-1)+1
;
monthly_segment_starts=REFORM(monthly_segment_starts,12,nyears)
monthly_segment_ends=REFORM(monthly_segment_ends,12,nyears)
monthly_segment_lengths=REFORM(monthly_segment_lengths,12,nyears)
;
v_good_month_present_count=make_array(12,nyears,/int,value=0)	; an array to store a 1 (data present) or a 0 (missing) for each month


;**************************************************************
; Outer month loop - minimise space by considering months in turn for creating a normalised
; index set.
;**************************************************************
FOR m=0,11 DO BEGIN
   ;**************************************************************
   ; Create the set of climatologies from which to anomalise the data
   ;**************************************************************

    vclims=DBLARR(24)

    FOR hr=0,23 DO BEGIN
      ;**************************************************************
      ; Use EXECUTE to run through all hours and set up climatologies
      ;**************************************************************
        clim_mdi=STRING(hr,FORMAT='("vclm",I2.2,"=mdi")')
        Exec_run=EXECUTE(clim_mdi)
    ENDFOR
    

    FOR yr=0,nyears-1 DO BEGIN
        ndays=monthly_segment_lengths(m,yr)/24
        temparr=full_values(monthly_segment_starts(m,yr):monthly_segment_ends(m,yr))
        temparr=REFORM(temparr,24,ndays)
        
        FOR hr=0,23 DO BEGIN
            ;**************************************************************
            ; Use read in hourly values into appropriate array.
            ;**************************************************************
            FMT1="points=WHERE(((temparr("
            FMT2=",*) NE mdi) AND (temparr("
            FMT3=",*) NE fdi)) , vpc)"
            
            clim_mdi=STRING(FMT1,hr,FMT2,hr,FMT3,FORMAT='(A23,I2,A26,I2,A20)')
            IF KEYWORD_SET(diagnostics) and 1 eq 0  THEN print,clim_mdi
            Exec_run=EXECUTE(clim_mdi)

            FMT1="IF vpc GE 1 THEN vclm"
            FMT2="=[vclm"
            FMT3=",REFORM(temparr("
            FMT4=",points),vpc)]"

            clim_mdi=STRING(FMT1,hr,FMT2,hr,FMT3,hr,FMT4,FORMAT='(A21,I2.2,A6,I2.2,A16,I2,A14)')
            IF KEYWORD_SET(diagnostics) and 1 eq 0  THEN print,clim_mdi
            Exec_run=EXECUTE(clim_mdi)

            
        ENDFOR
    ENDFOR

    FOR hr=0,23 DO BEGIN
        
        FMT1="IF(hr EQ "
        FMT2=") THEN subarr=vclm"
        FMT3="(SORT(vclm"
        FMT4="))"

        clim_mdi=STRING(FMT1,hr,FMT2,hr,FMT3,hr,FMT4,FORMAT='(A9,I2,A18,I2.2,A10,I2.2,A2)')
        IF KEYWORD_SET(diagnostics) and 1 eq 0  THEN print,clim_mdi
        Exec_run=EXECUTE(clim_mdi)

        IF (n_elements(subarr) GT 100) THEN BEGIN
            ;**************************************************************
            ; calculate the 5% and 95% percentile limits
            ; Winsorise (set values <5% and >95% to the 
            ; 5% and 95% values and add to total climatologies
            ; array)
            ;**************************************************************
            pct5=FIX(FLOOR(((n_elements(subarr)-1)/100.)*5.))
            pct95=FIX(FLOOR(((n_elements(subarr)-1)/100.)*95.))
            ;**************************************************************
            ; old version, pre winsorising
            ;vclims(hr)=TOTAL(subarr(pct5+1:pct95+1))/n_elements(subarr(pct5+1:pct95+1))  ; a "mean"-like value
            ;**************************************************************
            ; Proper Winsorising code 06/12/10 RJHD
            ;**************************************************************
            subarr(0:pct5+1)=subarr(pct5+1)
            subarr(pct95+1:n_elements(subarr)-1)=subarr(pct95+1)
            vclims(hr)=TOTAL(subarr)/(n_elements(subarr)-1)
        ENDIF ELSE vclims(hr)=mdi
    ENDFOR


    ;**************************************************************
    ; Anomalise
    ; For this month, over all years, compare the hourly averages
    ;  with the hourly values, and subtract.
    ;**************************************************************

    allvs=[0] ; Arrays to contain full set of anomalies in each parameter. Saves a further loop!
  
    FOR yr=0,nyears-1 DO BEGIN
        ndays=monthly_segment_lengths(m,yr)/24
        temparr=full_values(monthly_segment_starts(m,yr):monthly_segment_ends(m,yr))
        temparr=REFORM(temparr,24,ndays)
        FOR hr=0,23 DO BEGIN
            points=WHERE(((temparr(hr,*) NE mdi) AND (temparr(hr,*) NE fdi)), vpc)
            IF (vpc GE 1) AND ((vclims(hr) NE mdi) AND (vclims(hr) NE fdi)) THEN BEGIN ; check there are no dodgey things due to sub hourly reporting frequency
                temparr(hr,points)=temparr(hr,points)-vclims(hr)
                vt=temparr(hr,points)
                vt=REFORM(vt,vpc)
                allvs=[allvs,vt]
            ENDIF ELSE temparr(hr,*)=mdi
        ENDFOR
        
        full_values(monthly_segment_starts(m,yr):monthly_segment_ends(m,yr))=temparr
        
    ENDFOR

    ;**************************************************************
    ; Ascertain the inter-quartile ranges in anomaly space - these are robust to outliers anyway so no need to winsorise
    ;**************************************************************
 
    count=N_ELEMENTS(allvs)
    IF count GE 10 THEN BEGIN 
        rnk_tmp=SORT(allvs)
        allvs=allvs(rnk_tmp)
        vinterq=(allvs(ROUND(3.*FLOAT(count)/4.))-allvs(ROUND(FLOAT(count)/4.)))/2.
        IF vinterq LE 1.5 THEN vinterq=1.5 ; Added as 1 deg to avoid problems in very stable sites 03/09/09, changed to 2deg 22/09/09
    ENDIF ELSE vinterq=-999.


    ;**************************************************************
    ; Normalise by inter-q range, and save the monthly averages
    ;**************************************************************
    monthly_vqvs=DBLARR(nyears)
    
    FOR yr=0,nyears-1 DO BEGIN
        ndays=monthly_segment_lengths(m,yr)/24
        temparr=full_values(monthly_segment_starts(m,yr):monthly_segment_ends(m,yr))
        temparr=REFORM(temparr,24,ndays)
        FOR hr=0,23 DO BEGIN
            points=WHERE(((temparr(hr,*) NE mdi) AND (temparr(hr,*) NE fdi)), vpc)
            IF (vpc GE 1) AND (vinterq NE -999.) THEN temparr(hr,points)=temparr(hr,points)/vinterq
        ENDFOR
        full_values(monthly_segment_starts(m,yr):monthly_segment_ends(m,yr))=temparr
        pts=WHERE(((temparr NE mdi) AND (temparr NE fdi)), betty)
        IF betty GE 1 THEN BEGIN
            monthly_vqvs(yr)=MEDIAN(temparr(pts))
            ;**************************************************************
            ; THIS BIT IS DIFFERENT TO CLIMATOLOGICAL CHECK
            ;**************************************************************

            IF (betty GT 30) THEN v_good_month_present_count(m,yr)=betty ; only count if enough data per month
        ENDIF
    ENDFOR



    ;**************************************************************
    ;**************************************************************
    ;;;;; THE END OF THE SIMILARITY WITH CLIMATOLOGICAL_CHECK.PRO
    ;**************************************************************
    ;**************************************************************

    ;**************************************************************
    ; get the standard deviation for month each year and take the median of all years for a station with at least 10 years of data
    ;**************************************************************
    v_variance=make_array(nyears,/double,value=mdi)
    v_yr_count=make_array(nyears,/int,value=0)
    v_median_variance=mdi
    v_IQR_variance=mdi


    ;**************************************************************
    ; create records of annual vres and vfreq for checking changes
    ;**************************************************************
    nvres=make_array(nyears,/double,value=mdi)
    nvfreq=make_array(nyears,/double,value=mdi)

    ;**************************************************************
    ; get variance for each month
    ;**************************************************************

    FOR yr=0,nyears-1 DO BEGIN
        st_yr_seg=JULDAY(m+1,1,yr+1973,0,0)-start_date
        IF (m EQ 11) THEN ed_yr_seg=JULDAY(1,1,yr+1+1973,0,0)-start_date ELSE ed_yr_seg=JULDAY(m+2,1,yr+1973,0,0)-start_date
        IF (v_good_month_present_count(m,yr) GE 30) THEN BEGIN ; 4x obs per day for 25 days

            subarr=full_values(WHERE((full_times GE st_yr_seg) AND (full_times LT ed_yr_seg)))
            subarr=REFORM(subarr(WHERE(((subarr NE mdi) AND (subarr NE fdi)))))
            v_variance(yr)=MEANABSDEV(subarr,/median) ; changed from STDDEV 3 December 2010 RJHD
            ;**************************************************************
            ; get new Resolution and reporting frequency
            ;**************************************************************
            remainders=ABS(subarr(WHERE(((subarr NE mdi) AND (subarr NE fdi)))))	
            remhist=HISTOGRAM(remainders,min=-0.05,max=0.95,binsize=0.1)
            IF TOTAL(remhist) GT 0 THEN BEGIN
                remhist=FLOAT(remhist)/FLOAT(TOTAL(remhist))
                IF remhist(0) GE 0.2 THEN BEGIN
                    IF remhist(5) GE 0.15 THEN BEGIN
                        nvres(yr)=0.5
                    ENDIF ELSE BEGIN 
                                ;**************************************************************
                                ; No secondary peak at 0.5 so reporting largely to whole numbers
                                ;**************************************************************
                        nvres(yr)=1.
                    ENDELSE
                ENDIF ELSE BEGIN 
                    nvres(yr)=0.1
                ENDELSE
            ENDIF ELSE nvres(yr)=1. ; default value
            
            ;**************************************************************
            ; sussing hour reporting frequency   
            ;**************************************************************
            subtims=full_times(WHERE((full_times GE st_yr_seg) AND (full_times LT ed_yr_seg) AND (full_values NE mdi) AND (full_values NE fdi)))
            hourrecs=FIX((subtims-DOUBLE(FLOOR(subtims)))*24)
            histo=histogram(hourrecs,binsize=1,min=0,max=23)
            hourgots=WHERE((histo GT TOTAL(histo)/48.),cthrgts)
            IF (cthrgts EQ 24) THEN BEGIN
                nvfreq(yr)=1
            ENDIF ELSE IF (cthrgts GE 12) THEN BEGIN
                nvfreq(yr)=2
            ENDIF ELSE IF (cthrgts GE 8) THEN BEGIN
                nvfreq(yr)=3
            ENDIF ELSE nvfreq(yr)=4
        ENDIF
        
    ENDFOR



       
    ;**************************************************************
    ;get median and IQR variance for the whole data series.
    ;**************************************************************

    keepems=WHERE(v_good_month_present_count(m,*) GE 100,kc)
    IF (kc GE 10) THEN BEGIN ; 1200 is 120 obs in a month times 10 years - this may fall over 
        subarr=v_variance(keepems)
        v_median_variance=MEDIAN(subarr)
        count=n_elements(subarr)
        sortedvals=subarr(SORT(subarr))
        v_IQR_variance=(sortedvals(ROUND(3.*FLOAT(count)/4.))-sortedvals(ROUND(FLOAT(count)/4.)))/2.
        IF v_IQR_variance LE 0.01 THEN v_IQR_variance=0.01 
        ; Added to avoid problems in very stable sites 03/09/09
    ENDIF

    ;**************************************************************
    ; If SLP then calculate the median & MAD windspeed & slp for the month in question
    ;**************************************************************
    IF columns EQ 60 THEN BEGIN

        temparr=[0]
        FOR yr=0,nyears-1 DO BEGIN
            temparr=[temparr,full_windspeeds(monthly_segment_starts(m,yr):monthly_segment_ends(m,yr))]
        ENDFOR
        
        temparr=temparr[1:N_ELEMENTS(temparr)-1]
        good_test=WHERE(((temparr NE mdi) AND (temparr NE fdi)), wdcnt)

        IF wdcnt GE 1 THEN BEGIN
            temparr=temparr(good_test)
            monthly_median_windspeeds=MEDIAN(temparr)
            monthly_mad_windspeeds=MEANABSDEV(temparr,/median)
        ENDIF ELSE BEGIN
            monthly_median_windspeeds=0.
            monthly_mad_windspeeds=0.
        ENDELSE

        temparr=[0]
        FOR yr=0,nyears-1 DO BEGIN
            temparr=[temparr,untouched_full_values(monthly_segment_starts(m,yr):monthly_segment_ends(m,yr))]
        ENDFOR
        
        temparr=temparr[1:N_ELEMENTS(temparr)-1]
        good_test=WHERE(((temparr NE mdi) AND (temparr NE fdi)), scnt)

        IF scnt GE 1 THEN BEGIN
            temparr=temparr(good_test)
            s_median=MEDIAN(temparr)
            s_mad=MEANABSDEV(temparr,/median)
        ENDIF ELSE BEGIN
            s_median=0.
            s_mad=0.
        ENDELSE
    ENDIF

    ;**************************************************************
    ; chuck out years where variance differs from median variance by > 6-10 IQRs
    ; for SLP - check the first difference series too - 
    ;**************************************************************

    FOR yr=0,nyears-1 DO BEGIN
        biggest_neg=0
        biggest_pos=0
        neg_counts=0
        pos_counts=0
        gvalue='GD'
        st_yr_seg=JULDAY(m+1,1,yr+1973,0,0)-start_date
        IF (m EQ 11) THEN ed_yr_seg=JULDAY(1,1,yr+1+1973,0,0)-start_date ELSE ed_yr_seg=JULDAY(m+2,1,yr+1973,0,0)-start_date
        ;**************************************************************
        ; ***KATE JUL 2010 changed 6. to 8.
        ;**************************************************************
        ; Removed and placed further down in sequence.  So that non-tested
        ; years do not change the iqr_thresh.
        ; Done to ensure diagnostic plots show the last value used, rather
        ; than the one it was reset to 28/9/11 RJHD
;        IF (columns EQ 58) OR (columns EQ 59) THEN BEGIN
;            iqr_thresh=8.
;        ENDIF ELSE IF (columns EQ 60) THEN BEGIN
;            iqr_thresh=6.
;        ENDIF

        IF (v_variance(yr) NE mdi) AND (v_variance(yr) NE fdi) AND $
           (v_good_month_present_count(m,yr) GE 120) AND $
           (v_IQR_variance NE mdi) AND (v_IQR_variance NE fdi) AND $
           (v_median_variance NE mdi) AND (v_median_variance NE fdi) THEN BEGIN 
            ;**************************************************************
            ; change later to restrict to years with ? data quantity
            ;**************************************************************

            ;**************************************************************
            ; test for shift in resolution or frequency first - if found then widen the 4IQR thresh
            ; ***KATE JUL 2010 changed 8. to 10. 
            ;**************************************************************
            IF (columns EQ 58) OR (columns EQ 59) THEN BEGIN
                iqr_thresh=8.
                IF ((nvres(yr) NE vres) OR (nvfreq(yr) NE vfreq)) THEN iqr_thresh=10.
                IF (ABS((v_variance(yr)-v_median_variance)/v_IQR_variance) GT iqr_thresh) THEN BEGIN 
                ;**************************************************************
                ; REMOVAL INITIATED
                ;**************************************************************
                    vpoints=WHERE((times GE st_yr_seg) AND (times LT ed_yr_seg),vcount)
                    IF vcount GE 1 THEN BEGIN
                        gvalue='B'
                        timerange=WHERE((times GE st_yr_seg) AND (times LT ed_yr_seg))
                        subarr=values(timerange)
                        mdiloc=WHERE(((subarr NE mdi) AND (subarr NE fdi)), mdicnt)
                        IF mdicnt GE 1 THEN BEGIN

                            ;********************************
                            ; flags copied across - so old ones not wiped
                            ;********************************
                            qc_subarr=qc_flags(columns,timerange)
                            qc_subarr(mdiloc)=1

                            IF KEYWORD_SET(mask) THEN BEGIN
                                 ;********************************
                                 ; if set to mask, then do
                                 ;********************************
                                flagged_values[flagged_col,timerange(mdiloc)]=values(timerange(mdiloc))
                                values(timerange(mdiloc))=fdi
                            ENDIF

                            qc_flags(columns,timerange)=qc_subarr
                            
                            sa_flags=WHERE(qc_subarr EQ 1, sacnt)

                            ;********************************
                            ; set filtering flags
                            ;********************************
                            IF sacnt GE 1 THEN BEGIN
                                temp_flags=filter_flags(timerange)
                                temp_flags(sa_flags)=1
                                filter_flags(timerange)=temp_flags                              
                            ENDIF

                        ENDIF
                    ENDIF
                ENDIF
            ENDIF ELSE IF (columns EQ 60) THEN BEGIN
                
                iqr_thresh=6.

                ;**************************************************************
                ; SLP different as want to test for
                ; and keep hurricane features
                ;**************************************************************
                IF ((nvres(yr) NE vres) OR (nvfreq(yr) NE vfreq)) THEN iqr_thresh=8.
                IF (ABS((v_variance(yr)-v_median_variance)/v_IQR_variance) GT iqr_thresh) THEN BEGIN 
                ;**************************************************************
                ; REMOVAL INITIATED
                ;**************************************************************
                    vpoints=WHERE((times GE st_yr_seg) AND (times LT ed_yr_seg),vcount)
                    IF vcount GE 1 THEN BEGIN
                        subarr=values(vpoints)
                        timarr=times(vpoints)
                        p=WHERE(((subarr NE mdi) AND (subarr NE mdi)), c)
                        IF c GE 1 THEN BEGIN
                            ;**************************************************************
                            ; Use windspeeds to find storms too.
                            ;  Max wind within 1 day of min SLP
                            ;  Anomaly >4 using median and MAD
                            ;**************************************************************

                            month_windsp=windsp(vpoints(p))

                            w_max=MAX(month_windsp, w_max_loc)

                            s_min=MIN(subarr(p), s_min_loc)
                            wind_storm='no'

                            IF N_ELEMENTS(month_windsp) GE 1 THEN BEGIN
                                IF ((ABS(timarr(p(w_max_loc)) - timarr(p(s_min_loc))) LE 1) AND ((w_max-monthly_median_windspeeds)/monthly_mad_windspeeds GE 4.5) AND ((s_median-s_min)/s_mad GE 4.5)) THEN $
                                  wind_storm='yes' 
                            ENDIF ELSE BEGIN
                                OPENW,99,pwd+'/spurious_stations.txt',/append
                                PRINTF,99,'Variance Check',station_id,'1973-2007',strcompress(m+1,/remove_all),label,'No windspeed to check for storms',FORMAT='(A20,A20,A10,I4,A20,A)'
                                CLOSE,99
                            ENDELSE
                            ;**************************************************************
                            ; initiate test for first difference - trying not to remove hurricanes!
                            ;**************************************************************
                            newsub=subarr(p(0:c-2))-subarr(p(1:c-1)) 
                            ;**************************************************************
                            ; first difference series created
                            ;**************************************************************
                            FOR loop=0,c-2 DO BEGIN
                                IF (newsub(loop) GT 0.0) THEN BEGIN 
                                    IF (neg_counts GT biggest_neg) THEN biggest_neg=neg_counts
                                    neg_counts=0
                                    pos_counts=pos_counts+1
                                ENDIF ELSE BEGIN
                                    IF (pos_counts GT biggest_pos) THEN biggest_pos=pos_counts
                                    pos_counts=0
                                    neg_counts=neg_counts+1
                                ENDELSE
                            ENDFOR 
                            IF ((biggest_neg LT 10) AND (biggest_pos LT 10) AND (wind_storm EQ 'no')) THEN BEGIN 
                                
                                ;**************************************************************
                                ; not a hurricane???
                                ;**************************************************************
                                gslp='B'

                                ;********************************
                                ; flags copied across - so old ones not wiped
                                ;********************************
                                mdiloc=WHERE((subarr NE mdi) AND (subarr NE fdi))

                                qc_subarr=qc_flags(columns,vpoints)
                                qc_subarr(mdiloc)=1
                                qc_flags(columns,vpoints)=qc_subarr

                                IF KEYWORD_SET(mask) THEN BEGIN
                                    ;********************************
                                    ; if set to mask, then do
                                    ;********************************
                                    flagged_values[flagged_col,vpoints(mdiloc)]=values(vpoints(mdiloc))
                                    values(vpoints(mdiloc))=fdi
                                ENDIF

                                ;********************************
                                ; set filtering flags
                                ;********************************
                                sa_flags=WHERE(qc_subarr EQ 1, sacnt)

                                IF sacnt GE 1 THEN BEGIN
                                    temp_flags=filter_flags(WHERE((times GE st_yr_seg) AND (times LT ed_yr_seg)))
                                    temp_flags(sa_flags)=1
                                    filter_flags(WHERE((times GE st_yr_seg) AND (times LT ed_yr_seg)))=temp_flags                              
                                ENDIF


                                PRINT, 'No Storm or Hurricane in ',m+1,yr+1973.
                            ENDIF ELSE BEGIN
                                PRINT, 'Storm or Hurricane in ',m+1,yr+1973.
                            ENDELSE

                            IF KEYWORD_SET(diagnostics) THEN BEGIN
                                ;********************************
                                ; do plotting
                                ;********************************
                                set_plot,'PS'
                                cd, "./", current=outdirplot
                                device,filename=outdirplot+'/images/'+station_id+'_Variance_SLP_1stdiff_'+string(yr+1973,FORMAT='(I4.4)')+'_'+string(m+1,FORMAT='(I2.2)')+'.eps'$
                                  ,/color,/encapsulate,/helvetica,/bold,/landscape,ysize=20,xsize=26
                                !P.Font=0
                                !P.Thick=4
                                !P.Multi=[0,1,3]
                                plotsym,3,1.2

                                dummy=LABEL_DATE(DATE_FORMAT=['%D/%N/%Y'])
                                
                                plot,timarr(p(0:c-2))+start_date,subarr(p(0:c-2)),min_value=-100,xstyle=1,ystyle=1,ytitle="SLP",xtitle='time', title='Variance_Check - SLP First Differenc '+string(yr+1973,FORMAT='(I4.4)')+' '+string(m+1,FORMAT='(I2.2)'),charsize=1,ythick=4,xthick=4, POSITION=[0.05, 0.65, 0.95, 0.95],psym=1,xtickformat='LABEL_DATE'
                                plot,timarr(p(0:c-2))+start_date,newsub,min_value=-100,xstyle=1,ystyle=1,ytitle="SLP First Diff",xtitle='time',charsize=1,ythick=4,xthick=4, POSITION=[0.05, 0.35, 0.95, 0.60],psym=1,xtickformat='LABEL_DATE'
                                plot,timarr(p(0:c-2))+start_date,month_windsp,min_value=-100,xstyle=1,ystyle=1,ytitle="Wind m/s",xtitle='time',charsize=1,ythick=4,xthick=4, POSITION=[0.05, 0.05, 0.95, 0.30],psym=1,xtickformat='LABEL_DATE'
                                oplot,[timarr(p(0))+start_date,timarr(p(c-2))+start_date],[monthly_median_windspeeds,monthly_median_windspeeds],min_value=-100,xstyle=1,ystyle=1,color=239,linestyle=0,xtitle='time',charsize=1,ythick=4,xthick=4, POSITION=[0.05, 0.05, 0.95, 0.30],xtickformat='LABEL_DATE'
                                oplot,[timarr(p(0))+start_date,timarr(p(c-2))+start_date],[monthly_median_windspeeds-6*monthly_mad_windspeeds,monthly_median_windspeeds-4*monthly_mad_windspeeds],min_value=-100,xstyle=1,ystyle=1,color=239,linestyle=1,xtitle='time',charsize=1,ythick=4,xthick=4, POSITION=[0.05, 0.05, 0.95, 0.30],xtickformat='LABEL_DATE'
                                oplot,[timarr(p(0))+start_date,timarr(p(c-2))+start_date],[monthly_median_windspeeds+6*monthly_mad_windspeeds,monthly_median_windspeeds+4*monthly_mad_windspeeds],min_value=-100,xstyle=1,ystyle=1,color=239,linestyle=1,xtitle='time',charsize=1,ythick=4,xthick=4, POSITION=[0.05, 0.05, 0.95, 0.30],xtickformat='LABEL_DATE'
                                device,/close
                                
                                
                            ENDIF
                        ENDIF
                    ENDIF
                ENDIF
            ENDIF
        ENDIF
    ENDFOR
    IF KEYWORD_SET(diagnostics) THEN BEGIN
        CASE columns OF
            58: BEGIN
                label="temperatures"
            END
            
            59: BEGIN
                label="dewpoints"
            END
                        
            60: BEGIN
                label="slp"
            END
        ENDCASE
        good_variances=WHERE(((v_variance NE mdi) AND (v_variance NE fdi)), gvcnt)
        IF gvcnt GE 1 THEN BEGIN
            plot_variances=(v_variance(good_variances)-v_median_variance)/v_IQR_variance


            min_val=FIX(MIN(plot_variances))-1
            max_val=CEIL(MAX(plot_variances))
            binsize=1.0
        
            hist = HISTOGRAM(plot_variances, BINSIZE=binsize,LOCATION=loc, MAX=max_val, MIN=min_val)
            hist=[0,0,0,temporary(hist),0,0,0] 
        
        
            ;******************************************
            ; adjust for padding in histogram
            ;******************************************
            min_bin=MIN(loc)
            max_bin=MAX(loc)
            binns=[min_bin-(3.*binsize),min_bin-(2.*binsize),min_bin-binsize,loc,max_bin+binsize,max_bin+(2.*binsize),max_bin+(3.*binsize)]+(binsize/2)
            set_plot,'PS'
            cd, "./", current=outdirplot
            device,filename=outdirplot+'/images/'+station_id+'_Variance_'+label+'_'+string(m+1,FORMAT='(I2.2)')+'.eps'$
              ,/color,/encapsulate,/helvetica,/bold,/landscape,ysize=20,xsize=26
            !P.Font=0
            !P.Thick=4
            !P.Multi=[0,1,1]
            plotsym,3,1.2
            plot,binns,hist,min_value=-100,xstyle=1,ystyle=1,ytitle="Frequency",xtitle='variance offset (IQR)',$
              title="Variance_Check - Diagnostic Histogram - "+label+" - month "+string(m+1,FORMAT='(I2.2)'),charsize=1,ythick=4,xthick=4,psym=10,YLog=1,YRange=[0.1,max(hist)*5], POSITION=[0.10, 0.10, 0.90, 0.90]
            oplot,[-iqr_thresh,-iqr_thresh],[0.1,max(hist)*5],psym=0,color=240,YLog=1
            oplot,[iqr_thresh,iqr_thresh],[0.1,max(hist)*5],psym=0,color=240,YLog=1
            
            ;******************************************
            ; overplot flagged bins in red
            ;******************************************
            
            high_var=WHERE(plot_variances GT iqr_thresh, hvcnt)

            IF hvcnt GE 1 THEN plot_hv=plot_variances[high_var]

            low_var=WHERE(plot_variances LT -iqr_thresh, lvcnt)

            IF lvcnt GE 1 THEN plot_lv=plot_variances[low_var]

            flag_var=1
            IF (hvcnt GE 1) AND (lvcnt GE 1) THEN flag_var=[plot_lv,plot_hv] $
              ELSE IF hvcnt GE 1 THEN flag_var=plot_hv $
              ELSE IF lvcnt GE 1 THEN flag_var=plot_lv

            IF (hvcnt GE 1) OR (lvcnt GE 1) THEN BEGIN
                hist = HISTOGRAM(flag_var, BINSIZE=binsize,LOCATION=loc, MAX=max_val, MIN=min_val)
                hist=[0,0,0,temporary(hist),0,0,0] 
                
                oplot,binns,hist,min_value=-100,xstyle=1,ystyle=1,ytitle="Frequency",xtitle='variance offset (IQR)',$
                  title="Variance_Check - Diagnostic Histogram - "+label+" - month "+string(m+1,FORMAT='(I2.2)'),charsize=1,ythick=4,xthick=4,psym=10,YLog=1,YRange=[0.1,max(hist)*5], POSITION=[0.10, 0.10, 0.90, 0.90], color=239
            ENDIF
            device,/close
        ENDIF
    ENDIF


ENDFOR

full_values=untouched_full_values

RETURN

END
;************************************************************************
;                                 END
;************************************************************************
